@@ -213,6 +213,7 @@ namespace JSC { namespace B3 {
     case VectorFloor: \
     case VectorTrunc: \
     case VectorTruncSat: \
+    case VectorRelaxedTruncSat: \
     case VectorConvert: \
     case VectorConvertLow: \
     case VectorNearest: \
@@ -233,6 +234,8 @@ namespace JSC { namespace B3 {
     case VectorRelaxedSwizzle: \
     case VectorMulByElement: \
     case VectorShiftByVector: \
+    case VectorRelaxedMAdd: \
+    case VectorRelaxedNMAdd: \
         return MACRO(SIMDValue); \
     default: \
         RELEASE_ASSERT_NOT_REACHED(); \
@@ -403,11 +406,11 @@ inline bool Value::isNegativeZero() const
 {
     if (hasDouble()) {
         double value = asDouble();
-        return !value && std::signbit(value);
+        return value == 0.0 && std::signbit(value);
     }
     if (hasFloat()) {
         float value = asFloat();
-        return !value && std::signbit(value);
+        return value == 0.0f && std::signbit(value);
     }
     return false;
 }
