@@ -28,13 +28,10 @@
 
 #include "ConsoleMessage.h"
 #include "InjectedScriptManager.h"
-#include "InspectorFrontendRouter.h"
 #include "InspectorHeapAgent.h"
 #include "ScriptArguments.h"
-#include "ScriptCallFrame.h"
-#include "ScriptCallStack.h"
 #include "ScriptCallStackFactory.h"
-#include "ScriptObject.h"
+#include <wtf/SetForScope.h>
 #include <wtf/text/StringConcatenateNumbers.h>
 
 namespace Inspector {
@@ -58,8 +55,7 @@ void InspectorConsoleAgent::didCreateFrontendAndBackend(FrontendRouter*, Backend
 
 void InspectorConsoleAgent::willDestroyFrontendAndBackend(DisconnectReason)
 {
-    String errorString;
-    disable(errorString);
+    disable();
 }
 
 void InspectorConsoleAgent::discardValues()
@@ -68,15 +64,15 @@ void InspectorConsoleAgent::discardValues()
     m_expiredConsoleMessageCount = 0;
 }
 
-void InspectorConsoleAgent::enable(ErrorString&)
+Protocol::ErrorStringOr<void> InspectorConsoleAgent::enable()
 {
     if (m_enabled)
-        return;
+        return { };
 
     m_enabled = true;
 
     if (m_expiredConsoleMessageCount) {
-        ConsoleMessage expiredMessage(MessageSource::Other, MessageType::Log, MessageLevel::Warning, makeString(m_expiredConsoleMessageCount, " console messages are not shown."));
+        ConsoleMessage expiredMessage(MessageSource::Other, MessageType::Log, MessageLevel::Warning, makeString(m_expiredConsoleMessageCount, " console messages are not shown."_s));
         expiredMessage.addToFrontend(*m_frontendDispatcher, m_injectedScriptManager, false);
     }
 
@@ -85,17 +81,40 @@ void InspectorConsoleAgent::enable(ErrorString&)
 
     for (size_t i = 0; i < messages.size(); ++i)
         messages[i]->addToFrontend(*m_frontendDispatcher, m_injectedScriptManager, false);
+
+    return { };
 }
 
-void InspectorConsoleAgent::disable(ErrorString&)
+Protocol::ErrorStringOr<void> InspectorConsoleAgent::disable()
 {
     if (!m_enabled)
-        return;
+        return { };
 
     m_enabled = false;
+
+    return { };
 }
 
-void InspectorConsoleAgent::clearMessages(ErrorString&)
+Protocol::ErrorStringOr<void> InspectorConsoleAgent::clearMessages()
+{
+    clearMessages(Inspector::Protocol::Console::ClearReason::ConsoleAPI);
+    return { };
+}
+
+bool InspectorConsoleAgent::developerExtrasEnabled() const
+{
+    return m_injectedScriptManager.inspectorEnvironment().developerExtrasEnabled();
+}
+
+void InspectorConsoleAgent::mainFrameNavigated()
+{
+    clearMessages(Inspector::Protocol::Console::ClearReason::MainFrameNavigation);
+
+    m_times.clear();
+    m_counts.clear();
+}
+
+void InspectorConsoleAgent::clearMessages(Inspector::Protocol::Console::ClearReason reason)
 {
     m_consoleMessages.clear();
     m_expiredConsoleMessageCount = 0;
@@ -103,36 +122,19 @@ void InspectorConsoleAgent::clearMessages(ErrorString&)
     m_injectedScriptManager.releaseObjectGroup("console"_s);
 
     if (m_enabled)
-        m_frontendDispatcher->messagesCleared();
-}
-
-void InspectorConsoleAgent::reset()
-{
-    ErrorString ignored;
-    clearMessages(ignored);
-
-    m_times.clear();
-    m_counts.clear();
+        m_frontendDispatcher->messagesCleared(reason);
 }
 
 void InspectorConsoleAgent::addMessageToConsole(std::unique_ptr<ConsoleMessage> message)
 {
-    if (!m_injectedScriptManager.inspectorEnvironment().developerExtrasEnabled())
-        return;
-
-    if (message->type() == MessageType::Clear) {
-        ErrorString ignored;
-        clearMessages(ignored);
-    }
+    if (message->type() == MessageType::Clear)
+        clearMessages();
 
     addConsoleMessage(WTFMove(message));
 }
 
 void InspectorConsoleAgent::startTiming(JSC::JSGlobalObject* globalObject, const String& label)
 {
-    if (!m_injectedScriptManager.inspectorEnvironment().developerExtrasEnabled())
-        return;
-
     ASSERT(!label.isNull());
     if (label.isNull())
         return;
@@ -141,16 +143,13 @@ void InspectorConsoleAgent::startTiming(JSC::JSGlobalObject* globalObject, const
 
     if (!result.isNewEntry) {
         // FIXME: Send an enum to the frontend for localization?
-        String warning = makeString("Timer \"", label, "\" already exists");
+        auto warning = makeString("Timer \""_s, ScriptArguments::truncateStringForConsoleMessage(label), "\" already exists"_s);
         addMessageToConsole(makeUnique<ConsoleMessage>(MessageSource::ConsoleAPI, MessageType::Timing, MessageLevel::Warning, warning, createScriptCallStackForConsole(globalObject, 1)));
     }
 }
 
 void InspectorConsoleAgent::logTiming(JSC::JSGlobalObject* globalObject, const String& label, Ref<ScriptArguments>&& arguments)
 {
-    if (!m_injectedScriptManager.inspectorEnvironment().developerExtrasEnabled())
-        return;
-
     ASSERT(!label.isNull());
     if (label.isNull())
         return;
@@ -160,22 +159,19 @@ void InspectorConsoleAgent::logTiming(JSC::JSGlobalObject* globalObject, const S
     auto it = m_times.find(label);
     if (it == m_times.end()) {
         // FIXME: Send an enum to the frontend for localization?
-        String warning = makeString("Timer \"", label, "\" does not exist");
+        auto warning = makeString("Timer \""_s, ScriptArguments::truncateStringForConsoleMessage(label), "\" does not exist"_s);
         addMessageToConsole(makeUnique<ConsoleMessage>(MessageSource::ConsoleAPI, MessageType::Timing, MessageLevel::Warning, warning, WTFMove(callStack)));
         return;
     }
 
     MonotonicTime startTime = it->value;
     Seconds elapsed = MonotonicTime::now() - startTime;
-    String message = makeString(label, ": ", FormattedNumber::fixedWidth(elapsed.milliseconds(), 3), "ms");
+    auto message = makeString(ScriptArguments::truncateStringForConsoleMessage(label), ": "_s, FormattedNumber::fixedWidth(elapsed.milliseconds(), 3), "ms"_s);
     addMessageToConsole(makeUnique<ConsoleMessage>(MessageSource::ConsoleAPI, MessageType::Timing, MessageLevel::Debug, message, WTFMove(arguments), WTFMove(callStack)));
 }
 
 void InspectorConsoleAgent::stopTiming(JSC::JSGlobalObject* globalObject, const String& label)
 {
-    if (!m_injectedScriptManager.inspectorEnvironment().developerExtrasEnabled())
-        return;
-
     ASSERT(!label.isNull());
     if (label.isNull())
         return;
@@ -185,14 +181,14 @@ void InspectorConsoleAgent::stopTiming(JSC::JSGlobalObject* globalObject, const
     auto it = m_times.find(label);
     if (it == m_times.end()) {
         // FIXME: Send an enum to the frontend for localization?
-        String warning = makeString("Timer \"", label, "\" does not exist");
+        auto warning = makeString("Timer \""_s, ScriptArguments::truncateStringForConsoleMessage(label), "\" does not exist"_s);
         addMessageToConsole(makeUnique<ConsoleMessage>(MessageSource::ConsoleAPI, MessageType::Timing, MessageLevel::Warning, warning, WTFMove(callStack)));
         return;
     }
 
     MonotonicTime startTime = it->value;
     Seconds elapsed = MonotonicTime::now() - startTime;
-    String message = makeString(label, ": ", FormattedNumber::fixedWidth(elapsed.milliseconds(), 3), "ms");
+    auto message = makeString(ScriptArguments::truncateStringForConsoleMessage(label), ": "_s, FormattedNumber::fixedWidth(elapsed.milliseconds(), 3), "ms"_s);
     addMessageToConsole(makeUnique<ConsoleMessage>(MessageSource::ConsoleAPI, MessageType::Timing, MessageLevel::Debug, message, WTFMove(callStack)));
 
     m_times.remove(it);
@@ -200,44 +196,35 @@ void InspectorConsoleAgent::stopTiming(JSC::JSGlobalObject* globalObject, const
 
 void InspectorConsoleAgent::takeHeapSnapshot(const String& title)
 {
-    if (!m_injectedScriptManager.inspectorEnvironment().developerExtrasEnabled())
-        return;
-
     if (!m_heapAgent)
         return;
 
-    ErrorString ignored;
-    double timestamp;
-    String snapshotData;
-    m_heapAgent->snapshot(ignored, &timestamp, &snapshotData);
+    auto result = m_heapAgent->snapshot();
+    if (!result)
+        return;
 
-    m_frontendDispatcher->heapSnapshot(timestamp, snapshotData, title.isEmpty() ? nullptr : &title);
+    auto [timestamp, snapshotData] = WTFMove(result.value());
+    m_frontendDispatcher->heapSnapshot(timestamp, snapshotData, title);
 }
 
 void InspectorConsoleAgent::count(JSC::JSGlobalObject* globalObject, const String& label)
 {
-    if (!m_injectedScriptManager.inspectorEnvironment().developerExtrasEnabled())
-        return;
-
     auto result = m_counts.add(label, 1);
     if (!result.isNewEntry)
         result.iterator->value += 1;
 
     // FIXME: Web Inspector should have a better UI for counters, but for now we just log an updated counter value.
 
-    String message = makeString(label, ": ", result.iterator->value);
+    auto message = makeString(ScriptArguments::truncateStringForConsoleMessage(label), ": "_s, result.iterator->value);
     addMessageToConsole(makeUnique<ConsoleMessage>(MessageSource::ConsoleAPI, MessageType::Log, MessageLevel::Debug, message, createScriptCallStackForConsole(globalObject, 1)));
 }
 
 void InspectorConsoleAgent::countReset(JSC::JSGlobalObject* globalObject, const String& label)
 {
-    if (!m_injectedScriptManager.inspectorEnvironment().developerExtrasEnabled())
-        return;
-
     auto it = m_counts.find(label);
     if (it == m_counts.end()) {
         // FIXME: Send an enum to the frontend for localization?
-        String warning = makeString("Counter \"", label, "\" does not exist");
+        auto warning = makeString("Counter \""_s, ScriptArguments::truncateStringForConsoleMessage(label), "\" does not exist"_s);
         addMessageToConsole(makeUnique<ConsoleMessage>(MessageSource::ConsoleAPI, MessageType::Log, MessageLevel::Warning, warning, createScriptCallStackForConsole(globalObject, 1)));
         return;
     }
@@ -256,9 +243,6 @@ static bool isGroupMessage(MessageType type)
 
 void InspectorConsoleAgent::addConsoleMessage(std::unique_ptr<ConsoleMessage> consoleMessage)
 {
-    if (!m_injectedScriptManager.inspectorEnvironment().developerExtrasEnabled())
-        return;
-
     ASSERT_ARG(consoleMessage, consoleMessage);
 
     ConsoleMessage* previousMessage = m_consoleMessages.isEmpty() ? nullptr : m_consoleMessages.last().get();
@@ -270,8 +254,12 @@ void InspectorConsoleAgent::addConsoleMessage(std::unique_ptr<ConsoleMessage> co
     } else {
         ConsoleMessage* newMessage = consoleMessage.get();
         m_consoleMessages.append(WTFMove(consoleMessage));
-        if (m_enabled)
-            newMessage->addToFrontend(*m_frontendDispatcher, m_injectedScriptManager, true);
+        if (m_enabled) {
+            auto generatePreview = !m_isAddingMessageToFrontend;
+            SetForScope isAddingMessageToFrontend(m_isAddingMessageToFrontend, true);
+
+            newMessage->addToFrontend(*m_frontendDispatcher, m_injectedScriptManager, generatePreview);
+        }
 
         if (m_consoleMessages.size() >= maximumConsoleMessages) {
             m_expiredConsoleMessageCount += expireConsoleMessagesStep;
@@ -280,15 +268,15 @@ void InspectorConsoleAgent::addConsoleMessage(std::unique_ptr<ConsoleMessage> co
     }
 }
 
-void InspectorConsoleAgent::getLoggingChannels(ErrorString&, RefPtr<JSON::ArrayOf<Protocol::Console::Channel>>& channels)
+Protocol::ErrorStringOr<Ref<JSON::ArrayOf<Protocol::Console::Channel>>> InspectorConsoleAgent::getLoggingChannels()
 {
     // Default implementation has no logging channels.
-    channels = JSON::ArrayOf<Protocol::Console::Channel>::create();
+    return JSON::ArrayOf<Protocol::Console::Channel>::create();
 }
 
-void InspectorConsoleAgent::setLoggingChannelLevel(ErrorString& errorString, const String&, const String&)
+Protocol::ErrorStringOr<void> InspectorConsoleAgent::setLoggingChannelLevel(Protocol::Console::ChannelSource, Protocol::Console::ChannelLevel)
 {
-    errorString = "Not supported"_s;
+    return makeUnexpected("Not supported"_s);
 }
 
 } // namespace Inspector
