@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2012-2017 Apple Inc. All rights reserved.
+ * Copyright (C) 2012-2022 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -39,6 +39,7 @@
 #include "LLIntData.h"
 #include "LLIntSlowPaths.h"
 #include "JSCInlines.h"
+#include "SuperSampler.h"
 #include <wtf/Assertions.h>
 #include <wtf/MathExtras.h>
 
@@ -54,7 +55,7 @@ using namespace JSC::LLInt;
 // includes.
 //
 // In addition, some JIT trampoline functions which are needed by LLInt
-// (e.g. getHostCallReturnValue, ctiOpThrowNotCaught) are also added as
+// (e.g. ctiOpThrowNotCaught) are also added as
 // bytecodes, and the CLoop will provide bytecode handlers for them.
 //
 // In the CLoop, we can only dispatch indirectly to these bytecodes
@@ -87,6 +88,10 @@ using namespace JSC::LLInt;
 // Define the opcode dispatch mechanism when using the C loop:
 //
 
+#if ENABLE(UNIFIED_AND_FREEZABLE_CONFIG_RECORD)
+using WebConfig::g_config;
+#endif
+
 // These are for building a C Loop interpreter:
 #define OFFLINE_ASM_BEGIN
 #define OFFLINE_ASM_END
@@ -125,6 +130,8 @@ using namespace JSC::LLInt;
 
 namespace JSC {
 
+class CallLinkInfo;
+
 //============================================================================
 // CLoopRegister is the storage for an emulated CPU register.
 // It defines the policy of how ints smaller than intptr_t are packed into the
@@ -157,11 +164,12 @@ public:
     ALWAYS_INLINE Opcode opcode() const { return bitwise_cast<Opcode>(m_value); }
 
     operator CallFrame*() { return bitwise_cast<CallFrame*>(m_value); }
-    operator const Instruction*() { return bitwise_cast<const Instruction*>(m_value); }
+    operator const JSInstruction*() { return bitwise_cast<const JSInstruction*>(m_value); }
     operator JSCell*() { return bitwise_cast<JSCell*>(m_value); }
     operator ProtoCallFrame*() { return bitwise_cast<ProtoCallFrame*>(m_value); }
     operator Register*() { return bitwise_cast<Register*>(m_value); }
     operator VM*() { return bitwise_cast<VM*>(m_value); }
+    operator CallLinkInfo*() { return bitwise_cast<CallLinkInfo*>(m_value); }
 
     template<typename T, typename = std::enable_if_t<sizeof(T) == sizeof(uintptr_t)>>
     ALWAYS_INLINE void operator=(T value) { m_value = bitwise_cast<uintptr_t>(value); }
@@ -222,7 +230,7 @@ static void double2Ints(double val, CLoopRegister& lo, CLoopRegister& hi)
 }
 #endif // USE(JSVALUE32_64)
 
-static void decodeResult(SlowPathReturnType result, CLoopRegister& t0, CLoopRegister& t1)
+static void decodeResult(UGPRPair result, CLoopRegister& t0, CLoopRegister& t1)
 {
     const void* t0Result;
     const void* t1Result;
@@ -280,7 +288,7 @@ JSValue CLoop::execute(OpcodeID entryOpcodeID, void* executableAddress, VM* vm,
         // initialized the opcodeMap above. This is because getCodePtr()
         // can depend on the opcodeMap.
         uint8_t* exceptionInstructions = reinterpret_cast<uint8_t*>(LLInt::exceptionInstructions());
-        for (int i = 0; i < maxOpcodeLength + 1; ++i)
+        for (unsigned i = 0; i < maxBytecodeStructLength + 1; ++i)
             exceptionInstructions[i] = llint_throw_from_slow_path_trampoline;
 
         return JSValue();
@@ -315,8 +323,9 @@ JSValue CLoop::execute(OpcodeID entryOpcodeID, void* executableAddress, VM* vm,
 
     CLoopRegister t0, t1, t2, t3, t5, sp, cfr, lr, pc;
 #if USE(JSVALUE64)
-    CLoopRegister pcBase, numberTag, notCellMask;
+    CLoopRegister numberTag, notCellMask;
 #endif
+    CLoopRegister pcBase;
     CLoopRegister metadataTable;
     CLoopDoubleRegister d0, d1;
 
@@ -336,7 +345,7 @@ JSValue CLoop::execute(OpcodeID entryOpcodeID, void* executableAddress, VM* vm,
         void* m_originalStackPointer;
     };
 
-    CLoopStack& cloopStack = vm->interpreter->cloopStack();
+    CLoopStack& cloopStack = vm->interpreter.cloopStack();
     StackPointerScope stackPointerScope(cloopStack);
 
     lr = getOpcode(llint_return_to_host);
@@ -382,18 +391,6 @@ JSValue CLoop::execute(OpcodeID entryOpcodeID, void* executableAddress, VM* vm,
 #define RECORD_OPCODE_STATS(__opcode)
 #endif
 
-#if USE(JSVALUE32_64)
-#define FETCH_OPCODE() *pc.i8p
-#else // USE(JSVALUE64)
-#define FETCH_OPCODE() *bitwise_cast<OpcodeID*>(pcBase.i8p + pc.i)
-#endif // USE(JSVALUE64)
-
-#define NEXT_INSTRUCTION() \
-    do {                         \
-        opcode = FETCH_OPCODE(); \
-        DISPATCH_OPCODE();       \
-    } while (false)
-
 #if ENABLE(COMPUTED_GOTO_OPCODES)
 
     //========================================================================
@@ -447,23 +444,6 @@ JSValue CLoop::execute(OpcodeID entryOpcodeID, void* executableAddress, VM* vm,
 #endif
         }
 
-        // In the ASM llint, getHostCallReturnValue() is a piece of glue
-        // function provided by the JIT (see jit/JITOperations.cpp).
-        // We simulate it here with a pseduo-opcode handler.
-        OFFLINE_ASM_GLUE_LABEL(getHostCallReturnValue)
-        {
-            // The part in getHostCallReturnValueWithExecState():
-            JSValue result = vm->hostCallReturnValue;
-#if USE(JSVALUE32_64)
-            t1 = result.tag();
-            t0 = result.payload();
-#else
-            t0 = JSValue::encode(result);
-#endif
-            opcode = lr.opcode();
-            DISPATCH_OPCODE();
-        }
-
 #if !ENABLE(COMPUTED_GOTO_OPCODES)
     default:
         ASSERT(false);
@@ -484,7 +464,6 @@ JSValue CLoop::execute(OpcodeID entryOpcodeID, void* executableAddress, VM* vm,
     #undef LLINT_OPCODE_ENTRY
 #endif
 
-    #undef NEXT_INSTRUCTION
     #undef DEFINE_OPCODE
     #undef CHECK_FOR_TIMEOUT
     #undef CAST
@@ -500,11 +479,48 @@ JSValue CLoop::execute(OpcodeID entryOpcodeID, void* executableAddress, VM* vm,
 // Define the opcode dispatch mechanism when using an ASM loop:
 //
 
+// We're disabling this for now because of a suspected linker issue.
+#define OFFLINE_ASM_USE_ALT_ENTRY 0
+
+#if COMPILER(CLANG)
+
+// We need an OFFLINE_ASM_BEGIN_SPACER because we'll be declaring every OFFLINE_ASM_GLOBAL_LABEL
+// as an alt entry. However, Clang will error out if the first global label is also an alt entry.
+// To work around this, we'll make OFFLINE_ASM_BEGIN emit an unused global label (which will now
+// be the first) that is not an alt entry, and insert a spacer instruction between it and the
+// actual first global label emitted by the offlineasm. Clang also requires that these 2 labels
+// not point to the same spot in memory; hence, the need for the spacer.
+//
+// For the spacer instruction, we'll choose a breakpoint instruction. However, we can
+// also just emit an unused piece of data. A breakpoint instruction is preferable.
+
+#if CPU(ARM_THUMB2)
+#define OFFLINE_ASM_BEGIN_SPACER "bkpt #0\n"
+#elif CPU(ARM64)
+#define OFFLINE_ASM_BEGIN_SPACER "brk #0xc471\n"
+#elif CPU(X86_64)
+#define OFFLINE_ASM_BEGIN_SPACER "int3\n"
+#else
+#define OFFLINE_ASM_BEGIN_SPACER ".int 0xbadbeef0\n"
+#endif
+
+#else
+#define OFFLINE_ASM_BEGIN_SPACER
+#endif // COMPILER(CLANG)
+
 // These are for building an interpreter from generated assembly code:
+
+#if OFFLINE_ASM_USE_ALT_ENTRY
+#define OFFLINE_ASM_BEGIN   asm ( \
+    OFFLINE_ASM_GLOBAL_LABEL_IMPL(jsc_llint_begin, OFFLINE_ASM_NO_ALT_ENTRY_DIRECTIVE) \
+    OFFLINE_ASM_BEGIN_SPACER
+#else
 #define OFFLINE_ASM_BEGIN   asm (
+#endif
+
 #define OFFLINE_ASM_END     );
 
-#if USE(LLINT_EMBEDDED_OPCODE_ID)
+#if ENABLE(LLINT_EMBEDDED_OPCODE_ID)
 #define EMBED_OPCODE_ID_IF_NEEDED(__opcode) ".int " __opcode##_value_string "\n"
 #else
 #define EMBED_OPCODE_ID_IF_NEEDED(__opcode)
@@ -515,33 +531,63 @@ JSValue CLoop::execute(OpcodeID entryOpcodeID, void* executableAddress, VM* vm,
     OFFLINE_ASM_OPCODE_DEBUG_LABEL(llint_##__opcode) \
     OFFLINE_ASM_LOCAL_LABEL(llint_##__opcode)
 
-#define OFFLINE_ASM_GLUE_LABEL(__opcode)   OFFLINE_ASM_LOCAL_LABEL(__opcode)
+#define OFFLINE_ASM_GLUE_LABEL(__opcode) \
+    OFFLINE_ASM_OPCODE_DEBUG_LABEL(__opcode) \
+    OFFLINE_ASM_LOCAL_LABEL(__opcode)
+
+#define OFFLINE_ASM_NO_ALT_ENTRY_DIRECTIVE(label)
+
+#if COMPILER(CLANG)
+#define OFFLINE_ASM_ALT_ENTRY_DIRECTIVE(label) \
+    ".alt_entry " SYMBOL_STRING(label) "\n"
+#else
+#define OFFLINE_ASM_ALT_ENTRY_DIRECTIVE(label)
+#endif
 
 #if CPU(ARM_THUMB2)
-#define OFFLINE_ASM_GLOBAL_LABEL(label)          \
+#define OFFLINE_ASM_GLOBAL_LABEL_IMPL(label, ALT_ENTRY) \
     ".text\n"                                    \
-    ".align 4\n"                                 \
+    ".balign 4\n"                                \
+    ALT_ENTRY(label)                             \
     ".globl " SYMBOL_STRING(label) "\n"          \
     HIDE_SYMBOL(label) "\n"                      \
     ".thumb\n"                                   \
     ".thumb_func " THUMB_FUNC_PARAM(label) "\n"  \
     SYMBOL_STRING(label) ":\n"
 #elif CPU(ARM64)
-#define OFFLINE_ASM_GLOBAL_LABEL(label)         \
+#define OFFLINE_ASM_GLOBAL_LABEL_IMPL(label, ALT_ENTRY) \
     ".text\n"                                   \
-    ".align 4\n"                                \
+    ".balign 4\n"                               \
+    ALT_ENTRY(label)                            \
     ".globl " SYMBOL_STRING(label) "\n"         \
     HIDE_SYMBOL(label) "\n"                     \
     SYMBOL_STRING(label) ":\n"
 #else
-#define OFFLINE_ASM_GLOBAL_LABEL(label)         \
+#define OFFLINE_ASM_GLOBAL_LABEL_IMPL(label, ALT_ENTRY) \
     ".text\n"                                   \
+    ALT_ENTRY(label)                            \
     ".globl " SYMBOL_STRING(label) "\n"         \
     HIDE_SYMBOL(label) "\n"                     \
     SYMBOL_STRING(label) ":\n"
 #endif
 
-#define OFFLINE_ASM_LOCAL_LABEL(label)   LOCAL_LABEL_STRING(label) ":\n"
+#if OFFLINE_ASM_USE_ALT_ENTRY
+#define OFFLINE_ASM_GLOBAL_LABEL(label) \
+    OFFLINE_ASM_GLOBAL_LABEL_IMPL(label, OFFLINE_ASM_ALT_ENTRY_DIRECTIVE)
+#else
+#define OFFLINE_ASM_GLOBAL_LABEL(label) \
+    OFFLINE_ASM_GLOBAL_LABEL_IMPL(label, OFFLINE_ASM_NO_ALT_ENTRY_DIRECTIVE)
+#endif // USE_ALT_ENTRY
+
+#if COMPILER(CLANG) && OFFLINE_ASM_USE_ALT_ENTRY
+#define OFFLINE_ASM_ALT_GLOBAL_LABEL(label) OFFLINE_ASM_GLOBAL_LABEL(label)
+#else
+#define OFFLINE_ASM_ALT_GLOBAL_LABEL(label)
+#endif
+
+#define OFFLINE_ASM_LOCAL_LABEL(label) \
+    LOCAL_LABEL_STRING(label) ":\n" \
+    OFFLINE_ASM_ALT_GLOBAL_LABEL(label)
 
 #if OS(LINUX)
 #define OFFLINE_ASM_OPCODE_DEBUG_LABEL(label)  #label ":\n"
@@ -549,8 +595,19 @@ JSValue CLoop::execute(OpcodeID entryOpcodeID, void* executableAddress, VM* vm,
 #define OFFLINE_ASM_OPCODE_DEBUG_LABEL(label)
 #endif
 
+// This works around a bug in GDB where, if the compilation unit
+// doesn't have any address range information, its line table won't
+// even be consulted. Emit {before,after}_llint_asm so that the code
+// emitted in the top level inline asm statement is within functions
+// visible to the compiler. This way, GDB can resolve a PC in the
+// llint asm code to this compilation unit and the successfully look
+// up the line number information.
+DEBUGGER_ANNOTATION_MARKER(before_llint_asm)
+
 // This is a file generated by offlineasm, which contains all of the assembly code
 // for the interpreter, as compiled from LowLevelInterpreter.asm.
 #include "LLIntAssembly.h"
 
+DEBUGGER_ANNOTATION_MARKER(after_llint_asm)
+
 #endif // ENABLE(C_LOOP)
