@@ -1,4 +1,4 @@
-# Copyright (C) 2012-2019 Apple Inc. All rights reserved.
+# Copyright (C) 2012-2021 Apple Inc. All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
 # modification, are permitted provided that the following conditions
@@ -160,6 +160,7 @@ class Immediate
 
         case type
         when :int8;    "int8_t(#{valueStr})"
+        when :int16;   "int16_t(#{valueStr})"
         when :int32;   "int32_t(#{valueStr})"
         when :int64;   "int64_t(#{valueStr})"
         when :intptr;  "intptr_t(#{valueStr})"
@@ -183,6 +184,7 @@ class Address
     def clValue(type=:intptr)
         case type
         when :int8;         int8MemRef
+        when :int16;        int16MemRef
         when :int32;        int32MemRef
         when :int64;        int64MemRef
         when :intptr;       intptrMemRef
@@ -257,9 +259,11 @@ class BaseIndex
         case type
         when :int8;       int8MemRef
         when :int32;      int32MemRef
+        when :int16;      int16MemRef
         when :int64;      int64MemRef
         when :intptr;     intptrMemRef
         when :uint8;      uint8MemRef
+        when :uint16;     uint16MemRef
         when :uint32;     uint32MemRef
         when :uint64;     uint64MemRef
         when :uintptr;    uintptrMemRef
@@ -331,6 +335,9 @@ class LabelReference
     end
     def cloopEmitLea(destination, type)
         $asm.putc "#{destination.clLValue(:voidPtr)} = CAST<void*>(&#{cLabel});"
+        if offset != 0
+            $asm.putc "#{destination.clLValue(:int8Ptr)} = #{destination.clValue(:int8Ptr)} + #{offset};"
+        end
     end
 end
 
@@ -342,7 +349,7 @@ end
 class Address
     def cloopEmitLea(destination, type)
         if destination == base
-            $asm.putc "#{destination.clLValue(:int8Ptr)} += #{offset.clValue(type)};"
+            $asm.putc "#{destination.clLValue(:int8Ptr)} = #{destination.clValue(:int8Ptr)} + #{offset.clValue(type)};"
         else
             $asm.putc "#{destination.clLValue(:int8Ptr)} = #{base.clValue(:int8Ptr)} + #{offset.clValue(type)};"
         end
@@ -386,7 +393,7 @@ end
 
 def cloopEmitOperation(operands, type, operator)
     raise unless type == :intptr || type == :uintptr || type == :int32 || type == :uint32 || \
-        type == :int64 || type == :uint64 || type == :double
+        type == :int64 || type == :uint64 || type == :double || type == :int16
     if operands.size == 3
         op1 = operands[0]
         op2 = operands[1]
@@ -401,6 +408,9 @@ def cloopEmitOperation(operands, type, operator)
     if dst.is_a? RegisterID and (type == :int32 or type == :uint32)
         truncationHeader = "(uint32_t)("
         truncationFooter = ")"
+    elsif dst.is_a? RegisterID and (type == :int16)
+        truncationHeader = "(uint16_t)("
+        truncationFooter = ")"
     else
         truncationHeader = ""
         truncationFooter = ""
@@ -552,7 +562,7 @@ end
 def cloopEmitCallSlowPath(operands)
     $asm.putc "{"
     $asm.putc "    cloopStack.setCurrentStackPointer(sp.vp());"
-    $asm.putc "    SlowPathReturnType result = #{operands[0].cLabel}(#{operands[1].clDump}, #{operands[2].clDump});"
+    $asm.putc "    UGPRPair result = #{operands[0].cLabel}(#{operands[1].clDump}, #{operands[2].clDump});"
     $asm.putc "    decodeResult(result, t0, t1);"
     $asm.putc "}"
 end
@@ -562,6 +572,22 @@ def cloopEmitCallSlowPathVoid(operands)
     $asm.putc "#{operands[0].cLabel}(#{operands[1].clDump}, #{operands[2].clDump});"
 end
 
+def cloopEmitCallSlowPath3(operands)
+    $asm.putc "{"
+    $asm.putc "    cloopStack.setCurrentStackPointer(sp.vp());"
+    $asm.putc "    UGPRPair result = #{operands[0].cLabel}(#{operands[1].clDump}, #{operands[2].clDump}, #{operands[3].clDump});"
+    $asm.putc "    decodeResult(result, t0, t1);"
+    $asm.putc "}"
+end
+
+def cloopEmitCallSlowPath4(operands)
+    $asm.putc "{"
+    $asm.putc "    cloopStack.setCurrentStackPointer(sp.vp());"
+    $asm.putc "    UGPRPair result = #{operands[0].cLabel}(#{operands[1].clDump}, #{operands[2].clDump}, #{operands[3].clDump}, #{operands[4].clDump});"
+    $asm.putc "    decodeResult(result, t0, t1);"
+    $asm.putc "}"
+end
+
 class Instruction
     def lowerC_LOOP
         case opcode
@@ -585,6 +611,8 @@ class Instruction
             cloopEmitOperation(operands, :int64, "|")
         when "orp"
             cloopEmitOperation(operands, :intptr, "|")
+        when "orh"
+            cloopEmitOperation(operands, :int16, "|")
 
         when "xori"
             cloopEmitOperation(operands, :int32, "^")
@@ -685,8 +713,8 @@ class Instruction
             cloopEmitOperation(operands, :double, "*")
 
         # Convert an int value to its double equivalent, and store it in a double register.
-        when "ci2d"
-            $asm.putc "#{operands[1].clLValue(:double)} = (double)#{operands[0].clValue(:int32)}; // ci2d"
+        when "ci2ds"
+            $asm.putc "#{operands[1].clLValue(:double)} = (double)#{operands[0].clValue(:int32)}; // ci2ds"
 
         when "bdeq"
             cloopEmitCompareAndBranch(operands, :double, "==")
@@ -732,6 +760,14 @@ class Instruction
             $asm.putc "#{operands[1].clLValue(:int64)} = #{operands[0].clValue(:int32)};"
         when "zxi2q"
             $asm.putc "#{operands[1].clLValue(:uint64)} = #{operands[0].clValue(:uint32)};"
+        when "sxb2i"
+            $asm.putc "#{operands[1].clLValue(:int32)} = #{operands[0].clValue(:int8)};"
+        when "sxh2i"
+            $asm.putc "#{operands[1].clLValue(:int32)} = #{operands[0].clValue(:int16)};"
+        when "sxb2q"
+            $asm.putc "#{operands[1].clLValue(:int64)} = #{operands[0].clValue(:int8)};"
+        when "sxh2q"
+            $asm.putc "#{operands[1].clLValue(:int64)} = #{operands[0].clValue(:int16)};"
         when "nop"
             $asm.putc "// nop"
         when "bbeq"
@@ -940,6 +976,8 @@ class Instruction
             cloopEmitCompareAndSet(operands, :int64, ">")
         when "cpgt"
             cloopEmitCompareAndSet(operands, :intptr, ">")
+        when "cdgt"
+            cloopEmitCompareAndSet(operands, :double, ">")
 
         when "cbgteq"
             cloopEmitCompareAndSet(operands, :int8, ">=")
@@ -949,6 +987,8 @@ class Instruction
             cloopEmitCompareAndSet(operands, :int64, ">=")
         when "cpgteq"
             cloopEmitCompareAndSet(operands, :intptr, ">=")
+        when "cdgteq"
+            cloopEmitCompareAndSet(operands, :double, ">=")
 
         when "cblt"
             cloopEmitCompareAndSet(operands, :int8, "<")
@@ -958,6 +998,8 @@ class Instruction
             cloopEmitCompareAndSet(operands, :int64, "<")
         when "cplt"
             cloopEmitCompareAndSet(operands, :intptr, "<")
+        when "cdlt"
+            cloopEmitCompareAndSet(operands, :double, "<")
 
         when "cblteq"
             cloopEmitCompareAndSet(operands, :int8, "<=")
@@ -967,6 +1009,8 @@ class Instruction
             cloopEmitCompareAndSet(operands, :int64, "<=")
         when "cplteq"
             cloopEmitCompareAndSet(operands, :intptr, "<=")
+        when "cdlteq"
+            cloopEmitCompareAndSet(operands, :double, "<=")
 
         when "tbs"
             cloopEmitTestSet(operands, :int8, "< 0")
@@ -1147,6 +1191,12 @@ class Instruction
         when "cloopCallSlowPathVoid"
             cloopEmitCallSlowPathVoid(operands)
 
+        when "cloopCallSlowPath3"
+            cloopEmitCallSlowPath3(operands)
+
+        when "cloopCallSlowPath4"
+            cloopEmitCallSlowPath4(operands)
+
         # For debugging only. This is used to insert instrumentation into the
         # generated LLIntAssembly.h during llint development only. Do not use
         # for production code.
@@ -1165,6 +1215,5 @@ class Instruction
     def recordMetaDataC_LOOP
         $asm.codeOrigin codeOriginString if $enableCodeOriginComments
         $asm.annotation annotation if $enableInstrAnnotations && (opcode != "cloopDo")
-        $asm.debugAnnotation codeOrigin.debugDirective if $enableDebugAnnotations
     end
 end
