@@ -58,6 +58,7 @@
 #include "HasOwnPropertyCache.h"
 #include "Heap.h"
 #include "HeapProfiler.h"
+#include "IncrementalSweeper.h"
 #include "Interpreter.h"
 #include "IntlCache.h"
 #include "JITCode.h"
@@ -239,6 +240,7 @@ VM::VM(VMType vmType, HeapType heapType, WTF::RunLoop* runLoop, bool* success)
     stringStructure.setWithoutWriteBarrier(JSString::createStructure(*this, nullptr, jsNull()));
 
     smallStrings.initializeCommonStrings(*this);
+    numericStrings.initializeSmallIntCache(*this);
 
     propertyNames = new CommonIdentifiers(*this);
     propertyNameEnumeratorStructure.setWithoutWriteBarrier(JSPropertyNameEnumerator::createStructure(*this, nullptr, jsNull()));
@@ -313,12 +315,14 @@ VM::VM(VMType vmType, HeapType heapType, WTF::RunLoop* runLoop, bool* success)
     if (UNLIKELY(Options::useProfiler())) {
         m_perBytecodeProfiler = makeUnique<Profiler::Database>(*this);
 
-        StringPrintStream pathOut;
-        const char* profilerPath = getenv("JSC_PROFILER_PATH");
-        if (profilerPath)
-            pathOut.print(profilerPath, "/");
-        pathOut.print("JSCProfile-", getCurrentProcessID(), "-", m_perBytecodeProfiler->databaseID(), ".json");
-        m_perBytecodeProfiler->registerToSaveAtExit(pathOut.toCString().data());
+        if (UNLIKELY(Options::dumpProfilerDataAtExit())) {
+            StringPrintStream pathOut;
+            const char* profilerPath = getenv("JSC_PROFILER_PATH");
+            if (profilerPath)
+                pathOut.print(profilerPath, "/");
+            pathOut.print("JSCProfile-", getCurrentProcessID(), "-", m_perBytecodeProfiler->databaseID(), ".json");
+            m_perBytecodeProfiler->registerToSaveAtExit(pathOut.toCString().data());
+        }
     }
 
     // Initialize this last, as a free way of asserting that VM initialization itself
@@ -1557,6 +1561,7 @@ template<typename Visitor>
 void VM::visitAggregateImpl(Visitor& visitor)
 {
     m_microtaskQueue.visitAggregate(visitor);
+    numericStrings.visitAggregate(visitor);
 
     visitor.append(structureStructure);
     visitor.append(structureRareDataStructure);
@@ -1622,6 +1627,18 @@ void VM::removeDebugger(Debugger& debugger)
     m_debuggers.remove(&debugger);
 }
 
+void VM::performOpportunisticallyScheduledTasks(MonotonicTime deadline)
+{
+    bool hasPendingWork;
+    {
+        JSLockHolder locker { *this };
+        hasPendingWork = deferredWorkTimer->hasAnyPendingWork();
+    }
+
+    if (!hasPendingWork)
+        heap.sweeper().doWorkUntil(*this, deadline);
+}
+
 void QueuedTask::run()
 {
     if (!m_job.isObject())
