@@ -124,13 +124,15 @@ bool BBQPlan::dumpDisassembly(CompilationContext& context, LinkBuffer& linkBuffe
             const char* airPrefix = "Air        ";
             const char* asmPrefix = "asm              ";
 
+            B3::Value* prevOrigin = nullptr;
             auto forEachInst = scopedLambda<void(B3::Air::Inst&)>([&] (B3::Air::Inst& inst) {
-                if (inst.origin && context.procedure->code().shouldPreserveB3Origins()) {
+                if (inst.origin && inst.origin != prevOrigin && context.procedure->code().shouldPreserveB3Origins()) {
                     if (String string = inst.origin->compilerConstructionSite(); !string.isNull())
                         dataLogLn("\033[1;37m", string, "\033[0m");
                     dataLog(b3Prefix);
                     inst.origin->deepDump(context.procedure.get(), WTF::dataFile());
                     dataLogLn();
+                    prevOrigin = inst.origin;
                 }
             });
 
