@@ -31,11 +31,13 @@
 #include "LLIntExceptions.h"
 #include "WasmCalleeRegistry.h"
 #include "WasmCallingConvention.h"
+#include "WasmModuleInformation.h"
 
 namespace JSC { namespace Wasm {
 
 Callee::Callee(Wasm::CompilationMode compilationMode)
     : m_compilationMode(compilationMode)
+    , m_implementationVisibility(ImplementationVisibility::Private)
 {
 }
 
@@ -72,12 +74,15 @@ inline void Callee::runWithDowncast(const Func& func)
     case CompilationMode::OMGForOSREntryMode:
         break;
 #endif
-    case CompilationMode::EmbedderEntrypointMode:
-        func(static_cast<EmbedderEntrypointCallee*>(this));
+    case CompilationMode::JSEntrypointMode:
+        func(static_cast<JSEntrypointCallee*>(this));
         break;
     case CompilationMode::JSToWasmICMode:
         func(static_cast<JSToWasmICCallee*>(this));
         break;
+    case CompilationMode::WasmToJSMode:
+        func(static_cast<WasmToJSCallee*>(this));
+        break;
     }
 }
 
@@ -150,30 +155,10 @@ void JITCallee::setEntrypoint(Wasm::Entrypoint&& entrypoint)
     CalleeRegistry::singleton().registerCallee(this);
 }
 
-ptrdiff_t JSToWasmICCallee::previousInstanceOffset(const RegisterAtOffsetList& calleeSaveRegisters)
+WasmToJSCallee::WasmToJSCallee()
+    : Callee(Wasm::CompilationMode::WasmToJSMode)
 {
-    ptrdiff_t result = calleeSaveRegisters.registerCount() * sizeof(CPURegister);
-    result = -result - sizeof(CPURegister);
-#if ASSERT_ENABLED
-    ptrdiff_t minOffset = 1;
-    for (const RegisterAtOffset& regAtOffset : calleeSaveRegisters) {
-        ptrdiff_t offset = regAtOffset.offset();
-        ASSERT(offset < 0);
-        minOffset = std::min(offset, minOffset);
-#if USE(JSVALUE32_64)
-        ASSERT(!regAtOffset.reg().isFPR()); // Because FPRs are wider than sizeof(CPURegister)
-#endif
-    }
-    ASSERT(minOffset - static_cast<ptrdiff_t>(sizeof(CPURegister)) == result);
-#endif
-    return result;
-}
-
-Wasm::Instance* JSToWasmICCallee::previousInstance(CallFrame* callFrame)
-{
-    ASSERT(callFrame->callee().asWasmCallee() == this);
-    auto* result = *bitwise_cast<Wasm::Instance**>(bitwise_cast<char*>(callFrame) + previousInstanceOffset(m_entrypoint.calleeSaveRegisters));
-    return result;
+    CalleeRegistry::singleton().registerCallee(this);
 }
 
 LLIntCallee::LLIntCallee(FunctionCodeBlockGenerator& generator, size_t index, std::pair<const Name*, RefPtr<NameSection>>&& name)
@@ -251,6 +236,43 @@ const WasmInstruction* LLIntCallee::outOfLineJumpTarget(const WasmInstruction* p
     return m_instructions->at(offset + target).ptr();
 }
 
+void OptimizingJITCallee::addCodeOrigin(unsigned firstInlineCSI, unsigned lastInlineCSI, const Wasm::ModuleInformation& info, uint32_t functionIndex)
+{
+    if (!nameSections.size())
+        nameSections.append(info.nameSection);
+    // The inline frame list is stored in postorder. For example:
+    // A { B() C() D { E() } F() } -> B C E D F A
+#if ASSERT_ENABLED
+    ASSERT(firstInlineCSI <= lastInlineCSI);
+    for (unsigned i = 0; i + 1 < codeOrigins.size(); ++i)
+        ASSERT(codeOrigins[i].lastInlineCSI <= codeOrigins[i + 1].lastInlineCSI);
+    for (unsigned i = 0; i < codeOrigins.size(); ++i)
+        ASSERT(codeOrigins[i].lastInlineCSI <= lastInlineCSI);
+    ASSERT(nameSections.size() == 1);
+    ASSERT(nameSections[0].ptr() == info.nameSection.ptr());
+#endif
+    codeOrigins.append({ firstInlineCSI, lastInlineCSI, functionIndex, 0 });
+}
+
+IndexOrName OptimizingJITCallee::getOrigin(unsigned csi, unsigned depth, bool& isInlined) const
+{
+    isInlined = false;
+    auto iter = std::lower_bound(codeOrigins.begin(), codeOrigins.end(), WasmCodeOrigin { 0, csi, 0, 0}, [&] (const auto& a, const auto& b) {
+        return b.lastInlineCSI - a.lastInlineCSI;
+    });
+    if (!iter || iter == codeOrigins.end())
+        iter = codeOrigins.begin();
+    while (iter != codeOrigins.end()) {
+        if (iter->firstInlineCSI <= csi && iter->lastInlineCSI >= csi && !(depth--)) {
+            isInlined = true;
+            return IndexOrName(iter->functionIndex, nameSections[iter->moduleIndex]->get(iter->functionIndex));
+        }
+        ++iter;
+    }
+
+    return indexOrName();
+}
+
 #if ENABLE(WEBASSEMBLY_B3JIT)
 void OptimizingJITCallee::linkExceptionHandlers(Vector<UnlinkedHandlerInfo> unlinkedExceptionHandlers, Vector<CodeLocationLabel<ExceptionHandlerPtrTag>> exceptionHandlerLocations)
 {
