@@ -26,7 +26,7 @@
 
 // Version number for shader translation API.
 // It is incremented every time the API changes.
-#define ANGLE_SH_VERSION 312
+#define ANGLE_SH_VERSION 328
 
 enum ShShaderSpec
 {
@@ -76,20 +76,31 @@ enum ShShaderOutput
     SH_MSL_METAL_OUTPUT = 0x8B4D,
 };
 
+struct ShCompileOptionsMetal
+{
+    // Direct-to-metal backend constants:
+
+    // Binding index for driver uniforms:
+    int driverUniformsBindingIndex;
+    // Binding index for default uniforms:
+    int defaultUniformsBindingIndex;
+    // Binding index for UBO's argument buffer
+    int UBOArgumentBufferBindingIndex;
+};
+
 // For ANGLE_shader_pixel_local_storage.
 // Instructs the compiler which pixel local storage configuration to generate code for.
-enum class ShPixelLocalStorageType
+enum class ShPixelLocalStorageType : uint8_t
 {
     NotSupported,
-    ImageStoreR32PackedFormats,
-    ImageStoreNativeFormats,
+    ImageLoadStore,
     FramebufferFetch,
     PixelLocalStorageEXT,  // GL_EXT_shader_pixel_local_storage.
 };
 
 // For ANGLE_shader_pixel_local_storage_coherent.
 // Instructs the compiler which fragment synchronization method to use, if any.
-enum class ShFragmentSynchronizationType
+enum class ShFragmentSynchronizationType : uint8_t
 {
     NotSupported,  // Fragments cannot be ordered or synchronized.
 
@@ -107,25 +118,21 @@ enum class ShFragmentSynchronizationType
     EnumCount = InvalidEnum,
 };
 
-// Compile options.
-struct ShCompileOptionsMetal
-{
-    // Direct-to-metal backend constants:
-
-    // Binding index for driver uniforms:
-    int driverUniformsBindingIndex;
-    // Binding index for default uniforms:
-    int defaultUniformsBindingIndex;
-    // Binding index for UBO's argument buffer
-    int UBOArgumentBufferBindingIndex;
-};
-
-struct ShCompileOptionsPLS
+struct ShPixelLocalStorageOptions
 {
     ShPixelLocalStorageType type = ShPixelLocalStorageType::NotSupported;
+
     // For ANGLE_shader_pixel_local_storage_coherent.
-    ShFragmentSynchronizationType fragmentSynchronizationType =
-        ShFragmentSynchronizationType::NotSupported;
+    ShFragmentSynchronizationType fragmentSyncType = ShFragmentSynchronizationType::NotSupported;
+
+    // ShPixelLocalStorageType::ImageLoadStore only: Can we use rgba8/rgba8i/rgba8ui image formats?
+    // Or do we need to manually pack and unpack from r32i/r32ui?
+    bool supportsNativeRGBA8ImageFormats = false;
+
+    // anglebug.com/7792 -- Metal [[raster_order_group()]] does not work for read_write textures on
+    // AMD when the render pass doesn't have a color attachment on slot 0. To work around this we
+    // attach one of the PLS textures to GL_COLOR_ATTACHMENT0, if there isn't one already.
+    bool renderPassNeedsAMDRasterOrderGroupsWorkaround = false;
 };
 
 struct ShCompileOptions
@@ -398,20 +405,31 @@ struct ShCompileOptions
     // ceil()ed instead.
     uint64_t roundOutputAfterDithering : 1;
 
-    // Even when the dividend and divisor have the same value some platforms do not return 1.0f.
-    // Need to emit different division code for such platforms.
-    uint64_t precisionSafeDivision : 1;
+    // issuetracker.google.com/274859104 add OpQuantizeToF16 instruction to cast
+    // mediump floating-point values to 16 bit. ARM compiler utilized RelaxedPrecision
+    // to minimize type case and keep a mediump float as 32 bit when assigning it with
+    // a highp floating-point value. It is possible that GLSL shader code is comparing
+    // two meiump values, but ARM compiler is comparing a 32 bit value with a 16 bit value,
+    // causing the comparison to fail.
+    uint64_t castMediumpFloatTo16Bit : 1;
 
     // anglebug.com/7527: packUnorm4x8 fails on Pixel 4 if it is not passed a highp vec4.
     // TODO(anglebug.com/7527): This workaround is currently only applied for pixel local storage.
     // We may want to apply it generally.
     uint64_t passHighpToPackUnormSnormBuiltins : 1;
 
-    // When clip and cull distances are used simultaneously, D3D11 can support up to four of each.
-    uint64_t limitSimultaneousClipAndCullDistanceUsage : 1;
+    // Use an integer uniform to pass a bitset of enabled clip distances.
+    uint64_t emulateClipDistanceState : 1;
+
+    // issuetracker.google.com/266235549 add aliased memory decoration to ssbo if the variable is
+    // not declared with "restrict" memory qualifier in GLSL
+    uint64_t aliasedSSBOUnlessRestrict : 1;
+
+    // Use fragment shaders to compute and set coverage mask based on the alpha value
+    uint64_t emulateAlphaToCoverage : 1;
 
     ShCompileOptionsMetal metal;
-    ShCompileOptionsPLS pls;
+    ShPixelLocalStorageOptions pls;
 };
 
 // The 64 bits hash function. The first parameter is the input string; the
@@ -446,6 +464,7 @@ struct ShBuiltInResources
     int NV_EGL_stream_consumer_external;
     int ARB_texture_rectangle;
     int EXT_blend_func_extended;
+    int EXT_conservative_depth;
     int EXT_draw_buffers;
     int EXT_frag_depth;
     int EXT_shader_texture_lod;
@@ -484,8 +503,10 @@ struct ShBuiltInResources
     int EXT_texture_buffer;
     int OES_sample_variables;
     int EXT_clip_cull_distance;
+    int ANGLE_clip_cull_distance;
     int EXT_primitive_bounding_box;
     int OES_primitive_bounding_box;
+    int EXT_separate_shader_objects;
     int ANGLE_base_vertex_base_instance_shader_builtin;
     int ANDROID_extension_pack_es31a;
     int KHR_blend_equation_advanced;
@@ -653,7 +674,7 @@ struct ShBuiltInResources
     // Subpixel bits used in rasterization.
     int SubPixelBits;
 
-    // APPLE_clip_distance/EXT_clip_cull_distance constant
+    // APPLE_clip_distance / EXT_clip_cull_distance / ANGLE_clip_cull_distance constants
     int MaxClipDistances;
     int MaxCullDistances;
     int MaxCombinedClipAndCullDistances;
@@ -675,7 +696,8 @@ using ShHandle = void *;
 
 namespace sh
 {
-using BinaryBlob = std::vector<uint32_t>;
+using BinaryBlob       = std::vector<uint32_t>;
+using ShaderBinaryBlob = std::vector<uint8_t>;
 
 //
 // Driver must call this first, once, before doing any other compiler operations.
@@ -765,6 +787,15 @@ const std::string &GetObjectCode(const ShHandle handle);
 // handle: Specifies the compiler
 const BinaryBlob &GetObjectBinaryBlob(const ShHandle handle);
 
+// Returns a full binary for a compiled shader, to be loaded with glShaderBinary during runtime.
+// Parameters:
+// handle: Specifies the compiler
+bool GetShaderBinary(const ShHandle handle,
+                     const char *const shaderStrings[],
+                     size_t numStrings,
+                     const ShCompileOptions &compileOptions,
+                     ShaderBinaryBlob *const binaryOut);
+
 // Returns a (original_name, hash) map containing all the user defined names in the shader,
 // including variable names, function names, struct names, and struct field names.
 // Parameters:
@@ -849,6 +880,9 @@ unsigned int GetImage2DRegisterIndex(const ShHandle handle);
 // handle: Specifies the compiler
 const std::set<std::string> *GetUsedImage2DFunctionNames(const ShHandle handle);
 
+uint8_t GetClipDistanceArraySize(const ShHandle handle);
+uint8_t GetCullDistanceArraySize(const ShHandle handle);
+bool HasClipDistanceInVertexShader(const ShHandle handle);
 bool HasDiscardInFragmentShader(const ShHandle handle);
 bool HasValidGeometryShaderInputPrimitiveType(const ShHandle handle);
 bool HasValidGeometryShaderOutputPrimitiveType(const ShHandle handle);
@@ -942,7 +976,8 @@ extern const char kDriverUniformsVarName[];
 // - 6 bits for sample count
 // - 8 bits for enabled clip planes
 // - 1 bit for whether depth should be transformed to Vulkan clip space
-// - 11 bits unused
+// - 1 bit for whether alpha to coverage is enabled
+// - 10 bits unused
 constexpr uint32_t kDriverUniformsMiscSwapXYMask                  = 0x1;
 constexpr uint32_t kDriverUniformsMiscAdvancedBlendEquationOffset = 1;
 constexpr uint32_t kDriverUniformsMiscAdvancedBlendEquationMask   = 0x1F;
@@ -952,6 +987,8 @@ constexpr uint32_t kDriverUniformsMiscEnabledClipPlanesOffset     = 12;
 constexpr uint32_t kDriverUniformsMiscEnabledClipPlanesMask       = 0xFF;
 constexpr uint32_t kDriverUniformsMiscTransformDepthOffset        = 20;
 constexpr uint32_t kDriverUniformsMiscTransformDepthMask          = 0x1;
+constexpr uint32_t kDriverUniformsMiscAlphaToCoverageOffset       = 21;
+constexpr uint32_t kDriverUniformsMiscAlphaToCoverageMask         = 0x1;
 
 // Interface block array name used for atomic counter emulation
 extern const char kAtomicCountersBlockName[];
@@ -976,8 +1013,8 @@ extern const char kInputAttachmentName[];
 
 namespace mtl
 {
-// Specialization constant to enable GL_SAMPLE_COVERAGE_VALUE emulation.
-extern const char kCoverageMaskEnabledConstName[];
+// Specialization constant to enable multisampled rendering behavior.
+extern const char kMultisampledRenderingConstName[];
 
 // Specialization constant to emulate rasterizer discard.
 extern const char kRasterizerDiscardEnabledConstName[];
