@@ -47,6 +47,7 @@ angle_toggleable_extensions = [
 angle_requestable_extensions = [
     "GL_ANGLE_base_vertex_base_instance",
     "GL_ANGLE_base_vertex_base_instance_shader_builtin",
+    "GL_ANGLE_clip_cull_distance",
     "GL_ANGLE_compressed_texture_etc",
     "GL_ANGLE_copy_texture_3d",
     "GL_ANGLE_framebuffer_multisample",
@@ -61,8 +62,12 @@ angle_requestable_extensions = [
     "GL_ANGLE_multiview_multisample",
     "GL_ANGLE_provoking_vertex",
     "GL_ANGLE_read_only_depth_stencil_feedback_loops",
+    "GL_ANGLE_renderability_validation",
     "GL_ANGLE_robust_fragment_shader_output",
     "GL_ANGLE_semaphore_fuchsia",
+    "GL_ANGLE_shader_pixel_local_storage",
+    "GL_ANGLE_shader_pixel_local_storage_coherent",
+    "GL_ANGLE_stencil_texturing",
     "GL_ANGLE_texture_compression_dxt3",
     "GL_ANGLE_texture_compression_dxt5",
     "GL_ANGLE_texture_external_update",
@@ -83,6 +88,7 @@ gles_requestable_extensions = [
     "GL_ANGLE_texture_usage",
     "GL_APPLE_clip_distance",
     "GL_ARB_sync",
+    "GL_ARM_shader_framebuffer_fetch",
     "GL_EXT_base_instance",
     "GL_EXT_blend_func_extended",
     "GL_EXT_blend_minmax",
@@ -92,7 +98,9 @@ gles_requestable_extensions = [
     "GL_EXT_color_buffer_float",
     "GL_EXT_color_buffer_half_float",
     "GL_EXT_compressed_ETC1_RGB8_sub_texture",
+    "GL_EXT_conservative_depth",
     "GL_EXT_copy_image",
+    "GL_EXT_depth_clamp",
     "GL_EXT_disjoint_timer_query",
     "GL_EXT_draw_buffers",
     "GL_EXT_draw_buffers_indexed",
@@ -113,9 +121,11 @@ gles_requestable_extensions = [
     "GL_EXT_multisampled_render_to_texture",
     "GL_EXT_multisampled_render_to_texture2",
     "GL_EXT_occlusion_query_boolean",
+    "GL_EXT_polygon_offset_clamp",
     "GL_EXT_protected_textures",
     "GL_EXT_pvrtc_sRGB",
     "GL_EXT_read_format_bgra",
+    "GL_EXT_render_snorm",
     "GL_EXT_semaphore",
     "GL_EXT_semaphore_fd",
     "GL_EXT_separate_shader_objects",
@@ -136,7 +146,9 @@ gles_requestable_extensions = [
     "GL_EXT_texture_compression_s3tc_srgb",
     "GL_EXT_texture_cube_map_array",
     "GL_EXT_texture_filter_anisotropic",
+    "GL_EXT_texture_filter_minmax",
     "GL_EXT_texture_format_BGRA8888",
+    "GL_EXT_texture_mirror_clamp_to_edge",
     "GL_EXT_texture_norm16",
     "GL_EXT_texture_rg",
     "GL_EXT_texture_sRGB_R8",
@@ -209,6 +221,7 @@ gles_requestable_extensions = [
     "GL_OES_vertex_type_10_10_10_2",
     "GL_OVR_multiview",
     "GL_OVR_multiview2",
+    "GL_QCOM_render_shared_exponent",
     "GL_QCOM_shading_rate",
     "GL_WEBGL_video_texture",
 ]
@@ -223,8 +236,7 @@ angle_es_only_extensions = [
     "GL_ANGLE_rgbx_internal_format",
     "GL_ANGLE_robust_client_memory",
     "GL_ANGLE_robust_resource_initialization",
-    "GL_ANGLE_shader_pixel_local_storage",
-    "GL_ANGLE_shader_pixel_local_storage_coherent",
+    "GL_ANGLE_shader_binary",
     "GL_ANGLE_webgl_compatibility",
     "GL_CHROMIUM_bind_generates_resource",
     "GL_CHROMIUM_bind_uniform_location",
@@ -319,6 +331,7 @@ supported_egl_extensions = [
     "EGL_ANGLE_swap_with_frame_token",
     "EGL_ANGLE_sync_control_rate",
     "EGL_ANGLE_vulkan_image",
+    "EGL_ANGLE_wait_until_work_scheduled",
     "EGL_ANGLE_window_fixed_size",
     "EGL_CHROMIUM_sync_control",
     "EGL_EXT_create_context_robustness",
