@@ -28,11 +28,11 @@
 #include <set>
 #include <sstream>
 #include <string>
+#include <thread>
 #include <unordered_map>
 #include <unordered_set>
 #include <vector>
 
-// A helper class to disallow copy and assignment operators
 namespace angle
 {
 
@@ -50,6 +50,9 @@ template <typename Key,
           class Hash = absl::container_internal::hash_default_hash<Key>,
           class Eq   = absl::container_internal::hash_default_eq<Key>>
 using HashSet = absl::flat_hash_set<Key, Hash, Eq>;
+
+// Absl has generic lookup unconditionally
+#    define ANGLE_HAS_HASH_MAP_GENERIC_LOOKUP 1
 #else
 template <typename Key,
           typename T,
@@ -57,9 +60,22 @@ template <typename Key,
           class KeyEqual = std::equal_to<Key>>
 using HashMap = std::unordered_map<Key, T, Hash, KeyEqual>;
 template <typename Key, class Hash = std::hash<Key>, class KeyEqual = std::equal_to<Key>>
-using HashSet = std::unordered_set<Key, Hash, KeyEqual>;
+using HashSet  = std::unordered_set<Key, Hash, KeyEqual>;
+#    if __cpp_lib_generic_unordered_lookup >= 201811L
+#        define ANGLE_HAS_HASH_MAP_GENERIC_LOOKUP 1
+#    else
+#        define ANGLE_HAS_HASH_MAP_GENERIC_LOOKUP 0
+#    endif
 #endif  // defined(ANGLE_USE_ABSEIL)
 
+// Forward declaration. Implementation in system_utils.h
+#if defined(ANGLE_PLATFORM_LINUX) || defined(ANGLE_PLATFORM_WINDOWS)
+using ThreadId = uint64_t;
+#else
+using ThreadId = std::thread::id;
+#endif
+
+// A helper class to disallow copy and assignment operators
 class NonCopyable
 {
   protected:
@@ -143,6 +159,7 @@ struct PerfMonitorTriplet
     FN(commandQueueSubmitCallsPerFrame)            \
     FN(vkQueueSubmitCallsTotal)                    \
     FN(vkQueueSubmitCallsPerFrame)                 \
+    FN(commandQueueWaitSemaphoresTotal)            \
     FN(renderPasses)                               \
     FN(writeDescriptorSets)                        \
     FN(flushedOutsideRenderPassCommandBuffers)     \
@@ -195,6 +212,7 @@ struct PerfMonitorTriplet
     FN(buffersGhosted)                             \
     FN(vertexArraySyncStateCalls)                  \
     FN(allocateNewBufferBlockCalls)                \
+    FN(bufferSuballocationCalls)                   \
     FN(dynamicBufferAllocations)                   \
     FN(framebufferCacheSize)
 
@@ -551,6 +569,22 @@ class MsanScopedDisableInterceptorChecks final : angle::NonCopyable
 #    define ANGLE_FORMAT_PRINTF(fmt, args)
 #endif
 
+#if defined(__clang__) || (defined(__GNUC__) && defined(__has_attribute))
+#    define ANGLE_HAS_ATTRIBUTE_CONSTRUCTOR (__has_attribute(constructor))
+#    define ANGLE_HAS_ATTRIBUTE_DESTRUCTOR (__has_attribute(destructor))
+#else
+#    define ANGLE_HAS_ATTRIBUTE_CONSTRUCTOR 0
+#    define ANGLE_HAS_ATTRIBUTE_DESTRUCTOR 0
+#endif
+
+#if ANGLE_HAS_ATTRIBUTE_CONSTRUCTOR
+#    define ANGLE_CONSTRUCTOR __attribute__((constructor))
+#endif
+
+#if ANGLE_HAS_ATTRIBUTE_DESTRUCTOR
+#    define ANGLE_DESTRUCTOR __attribute__((destructor))
+#endif
+
 ANGLE_FORMAT_PRINTF(1, 0)
 size_t FormatStringIntoVector(const char *fmt, va_list vararg, std::vector<char> &buffer);
 
