@@ -81,24 +81,24 @@ Stream &operator<<(Stream &out, CommaSeparatedListItemPrefixGenerator &gen)
 
 }  // namespace
 
-TOutputGLSLBase::TOutputGLSLBase(TInfoSinkBase &objSink,
-                                 ShArrayIndexClampingStrategy clampingStrategy,
-                                 ShHashFunction64 hashFunction,
-                                 NameMap &nameMap,
-                                 TSymbolTable *symbolTable,
-                                 sh::GLenum shaderType,
-                                 int shaderVersion,
-                                 ShShaderOutput output,
-                                 ShCompileOptions compileOptions)
-    : TIntermTraverser(true, true, true, symbolTable),
+TOutputGLSLBase::TOutputGLSLBase(TCompiler *compiler,
+                                 TInfoSinkBase &objSink,
+                                 const ShCompileOptions &compileOptions)
+    : TIntermTraverser(true, true, true, &compiler->getSymbolTable()),
       mObjSink(objSink),
       mDeclaringVariable(false),
-      mClampingStrategy(clampingStrategy),
-      mHashFunction(hashFunction),
-      mNameMap(nameMap),
-      mShaderType(shaderType),
-      mShaderVersion(shaderVersion),
-      mOutput(output),
+      mHashFunction(compiler->getHashFunction()),
+      mNameMap(compiler->getNameMap()),
+      mShaderType(compiler->getShaderType()),
+      mShaderVersion(compiler->getShaderVersion()),
+      mOutput(compiler->getOutputType()),
+      mHighPrecisionSupported(compiler->isHighPrecisionSupported()),
+      // If pixel local storage introduces new fragment outputs, we are now required to specify a
+      // location for _all_ fragment outputs, including previously valid outputs that had an
+      // implicit location of zero.
+      mAlwaysSpecifyFragOutLocation(compiler->hasPixelLocalStorageUniforms() &&
+                                    compileOptions.pls.type ==
+                                        ShPixelLocalStorageType::FramebufferFetch),
       mCompileOptions(compileOptions)
 {}
 
@@ -111,6 +111,12 @@ void TOutputGLSLBase::writeInvariantQualifier(const TType &type)
     }
 }
 
+void TOutputGLSLBase::writePreciseQualifier(const TType &type)
+{
+    TInfoSinkBase &out = objSink();
+    out << "precise ";
+}
+
 void TOutputGLSLBase::writeFloat(TInfoSinkBase &out, float f)
 {
     if ((gl::isInf(f) || gl::isNaN(f)) && mShaderVersion >= 300)
@@ -137,21 +143,20 @@ void TOutputGLSLBase::writeTriplet(Visit visit,
         out << postStr;
 }
 
-void TOutputGLSLBase::writeBuiltInFunctionTriplet(Visit visit,
-                                                  TOperator op,
-                                                  bool useEmulatedFunction)
+void TOutputGLSLBase::writeFunctionTriplet(Visit visit,
+                                           const ImmutableString &functionName,
+                                           bool useEmulatedFunction)
 {
     TInfoSinkBase &out = objSink();
     if (visit == PreVisit)
     {
-        const char *opStr(GetOperatorString(op));
         if (useEmulatedFunction)
         {
-            BuiltInFunctionEmulator::WriteEmulatedFunctionName(out, opStr);
+            BuiltInFunctionEmulator::WriteEmulatedFunctionName(out, functionName.data());
         }
         else
         {
-            out << opStr;
+            out << functionName;
         }
         out << "(";
     }
@@ -163,7 +168,7 @@ void TOutputGLSLBase::writeBuiltInFunctionTriplet(Visit visit,
 
 // Outputs what goes inside layout(), except for location and binding qualifiers, as they are
 // handled differently between GL GLSL and Vulkan GLSL.
-std::string TOutputGLSLBase::getCommonLayoutQualifiers(TIntermTyped *variable)
+std::string TOutputGLSLBase::getCommonLayoutQualifiers(TIntermSymbol *variable)
 {
     std::ostringstream out;
     CommaSeparatedListItemPrefixGenerator listItemPrefix;
@@ -171,23 +176,29 @@ std::string TOutputGLSLBase::getCommonLayoutQualifiers(TIntermTyped *variable)
     const TType &type                       = variable->getType();
     const TLayoutQualifier &layoutQualifier = type.getLayoutQualifier();
 
-    if (type.getQualifier() == EvqFragmentOut || type.getQualifier() == EvqVertexIn ||
-        IsVarying(type.getQualifier()))
+    if (type.getQualifier() == EvqFragDepth)
     {
-        if (type.getQualifier() == EvqFragmentOut && layoutQualifier.index >= 0)
+        ASSERT(layoutQualifier.depth != EdUnspecified);
+        out << listItemPrefix << getDepthString(layoutQualifier.depth);
+    }
+
+    if (type.getQualifier() == EvqFragmentOut || type.getQualifier() == EvqFragmentInOut)
+    {
+        if (layoutQualifier.index >= 0)
         {
             out << listItemPrefix << "index = " << layoutQualifier.index;
         }
-    }
-
-    if (type.getQualifier() == EvqFragmentOut)
-    {
-        if (layoutQualifier.yuv == true)
+        if (layoutQualifier.yuv)
         {
             out << listItemPrefix << "yuv";
         }
     }
 
+    if (type.getQualifier() == EvqFragmentInOut && layoutQualifier.noncoherent)
+    {
+        out << listItemPrefix << "noncoherent";
+    }
+
     if (IsImage(type.getBasicType()))
     {
         if (layoutQualifier.imageInternalFormat != EiifUnspecified)
@@ -206,7 +217,8 @@ std::string TOutputGLSLBase::getCommonLayoutQualifiers(TIntermTyped *variable)
     return out.str();
 }
 
-// Outputs what comes after in/out/uniform/buffer storage qualifier.
+// Outputs memory qualifiers applied to images, buffers and its fields, as well as image function
+// arguments.
 std::string TOutputGLSLBase::getMemoryQualifiers(const TType &type)
 {
     std::ostringstream out;
@@ -214,50 +226,44 @@ std::string TOutputGLSLBase::getMemoryQualifiers(const TType &type)
     const TMemoryQualifier &memoryQualifier = type.getMemoryQualifier();
     if (memoryQualifier.readonly)
     {
-        ASSERT(IsImage(type.getBasicType()) || IsStorageBuffer(type.getQualifier()));
         out << "readonly ";
     }
 
     if (memoryQualifier.writeonly)
     {
-        ASSERT(IsImage(type.getBasicType()) || IsStorageBuffer(type.getQualifier()));
         out << "writeonly ";
     }
 
     if (memoryQualifier.coherent)
     {
-        ASSERT(IsImage(type.getBasicType()) || IsStorageBuffer(type.getQualifier()));
         out << "coherent ";
     }
 
     if (memoryQualifier.restrictQualifier)
     {
-        ASSERT(IsImage(type.getBasicType()) || IsStorageBuffer(type.getQualifier()));
         out << "restrict ";
     }
 
     if (memoryQualifier.volatileQualifier)
     {
-        ASSERT(IsImage(type.getBasicType()) || IsStorageBuffer(type.getQualifier()));
         out << "volatile ";
     }
 
     return out.str();
 }
 
-void TOutputGLSLBase::writeLayoutQualifier(TIntermTyped *variable)
+void TOutputGLSLBase::writeLayoutQualifier(TIntermSymbol *variable)
 {
     const TType &type = variable->getType();
 
-    if (!NeedsToWriteLayoutQualifier(type))
+    if (!needsToWriteLayoutQualifier(type))
     {
         return;
     }
 
     if (type.getBasicType() == EbtInterfaceBlock)
     {
-        const TInterfaceBlock *interfaceBlock = type.getInterfaceBlock();
-        declareInterfaceBlockLayout(interfaceBlock);
+        declareInterfaceBlockLayout(type);
         return;
     }
 
@@ -267,12 +273,13 @@ void TOutputGLSLBase::writeLayoutQualifier(TIntermTyped *variable)
 
     CommaSeparatedListItemPrefixGenerator listItemPrefix;
 
-    if (type.getQualifier() == EvqFragmentOut || type.getQualifier() == EvqVertexIn ||
+    if (IsFragmentOutput(type.getQualifier()) || type.getQualifier() == EvqVertexIn ||
         IsVarying(type.getQualifier()))
     {
-        if (layoutQualifier.location >= 0)
+        if (layoutQualifier.location >= 0 ||
+            (mAlwaysSpecifyFragOutLocation && IsFragmentOutput(type.getQualifier())))
         {
-            out << listItemPrefix << "location = " << layoutQualifier.location;
+            out << listItemPrefix << "location = " << std::max(layoutQualifier.location, 0);
         }
     }
 
@@ -295,7 +302,9 @@ void TOutputGLSLBase::writeLayoutQualifier(TIntermTyped *variable)
 
 void TOutputGLSLBase::writeFieldLayoutQualifier(const TField *field)
 {
-    if (!field->type()->isMatrix() && !field->type()->isStructureContainingMatrices())
+    TLayoutQualifier layoutQualifier = field->type()->getLayoutQualifier();
+    if (!field->type()->isMatrix() && !field->type()->isStructureContainingMatrices() &&
+        layoutQualifier.imageInternalFormat == EiifUnspecified)
     {
         return;
     }
@@ -303,21 +312,30 @@ void TOutputGLSLBase::writeFieldLayoutQualifier(const TField *field)
     TInfoSinkBase &out = objSink();
 
     out << "layout(";
-    switch (field->type()->getLayoutQualifier().matrixPacking)
+    CommaSeparatedListItemPrefixGenerator listItemPrefix;
+    if (field->type()->isMatrix() || field->type()->isStructureContainingMatrices())
     {
-        case EmpUnspecified:
-        case EmpColumnMajor:
-            // Default matrix packing is column major.
-            out << "column_major";
-            break;
+        switch (layoutQualifier.matrixPacking)
+        {
+            case EmpUnspecified:
+            case EmpColumnMajor:
+                // Default matrix packing is column major.
+                out << listItemPrefix << "column_major";
+                break;
 
-        case EmpRowMajor:
-            out << "row_major";
-            break;
+            case EmpRowMajor:
+                out << listItemPrefix << "row_major";
+                break;
 
-        default:
-            UNREACHABLE();
-            break;
+            default:
+                UNREACHABLE();
+                break;
+        }
+    }
+    // EXT_shader_pixel_local_storage.
+    if (layoutQualifier.imageInternalFormat != EiifUnspecified)
+    {
+        out << listItemPrefix << getImageInternalFormatString(layoutQualifier.imageInternalFormat);
     }
     out << ") ";
 }
@@ -336,7 +354,7 @@ void TOutputGLSLBase::writeQualifier(TQualifier qualifier, const TType &type, co
 const char *TOutputGLSLBase::mapQualifierToString(TQualifier qualifier)
 {
     if (sh::IsGLSL410OrOlder(mOutput) && mShaderVersion >= 300 &&
-        (mCompileOptions & SH_REMOVE_INVARIANT_AND_CENTROID_FOR_ESSL3) != 0)
+        mCompileOptions.removeInvariantAndCentroidForESSL3)
     {
         switch (qualifier)
         {
@@ -348,6 +366,12 @@ const char *TOutputGLSLBase::mapQualifierToString(TQualifier qualifier)
                 return "smooth in";
             case EvqCentroidOut:
                 return "smooth out";
+            case EvqNoPerspectiveCentroid:
+                return "noperspective";
+            case EvqNoPerspectiveCentroidIn:
+                return "noperspective in";
+            case EvqNoPerspectiveCentroidOut:
+                return "noperspective out";
             default:
                 break;
         }
@@ -366,10 +390,48 @@ const char *TOutputGLSLBase::mapQualifierToString(TQualifier qualifier)
                 break;
         }
     }
-    return sh::getQualifierString(qualifier);
+
+    switch (qualifier)
+    {
+        // gl_ClipDistance / gl_CullDistance require different qualifiers based on shader type.
+        case EvqClipDistance:
+        case EvqCullDistance:
+            return (sh::IsGLSL130OrNewer(mOutput) || mShaderVersion > 100)
+                       ? (mShaderType == GL_FRAGMENT_SHADER ? "in" : "out")
+                       : "varying";
+
+        case EvqFragDepth:
+            return "out";
+
+        // gl_LastFragColor / gl_LastFragData have no qualifiers.
+        case EvqLastFragData:
+        case EvqLastFragColor:
+            return nullptr;
+
+        default:
+            return sh::getQualifierString(qualifier);
+    }
 }
 
-void TOutputGLSLBase::writeVariableType(const TType &type, const TSymbol *symbol)
+namespace
+{
+
+constexpr char kIndent[]      = "                    ";  // 10x2 spaces
+constexpr int kIndentWidth    = 2;
+constexpr int kMaxIndentLevel = sizeof(kIndent) / kIndentWidth;
+
+}  // namespace
+
+const char *TOutputGLSLBase::getIndentPrefix(int extraIndentation)
+{
+    int indentDepth = std::min(kMaxIndentLevel, getCurrentBlockDepth() + extraIndentation);
+    ASSERT(indentDepth >= 0);
+    return kIndent + (kMaxIndentLevel - indentDepth) * kIndentWidth;
+}
+
+void TOutputGLSLBase::writeVariableType(const TType &type,
+                                        const TSymbol *symbol,
+                                        bool isFunctionArgument)
 {
     TQualifier qualifier = type.getQualifier();
     TInfoSinkBase &out   = objSink();
@@ -377,13 +439,23 @@ void TOutputGLSLBase::writeVariableType(const TType &type, const TSymbol *symbol
     {
         writeInvariantQualifier(type);
     }
+    if (type.isPrecise())
+    {
+        writePreciseQualifier(type);
+    }
     if (qualifier != EvqTemporary && qualifier != EvqGlobal)
     {
         writeQualifier(qualifier, type, symbol);
     }
+    if (isFunctionArgument)
+    {
+        // Function arguments are the only place (other than image/SSBO/field declaration) where
+        // memory qualifiers can appear.
+        out << getMemoryQualifiers(type);
+    }
 
-    // Declare the struct if we have not done so already.
-    if (type.getBasicType() == EbtStruct && !structDeclared(type.getStruct()))
+    // Declare the struct.
+    if (type.isStructSpecifier())
     {
         const TStructure *structure = type.getStruct();
 
@@ -391,8 +463,7 @@ void TOutputGLSLBase::writeVariableType(const TType &type, const TSymbol *symbol
     }
     else if (type.getBasicType() == EbtInterfaceBlock)
     {
-        const TInterfaceBlock *interfaceBlock = type.getInterfaceBlock();
-        declareInterfaceBlock(interfaceBlock);
+        declareInterfaceBlock(type);
     }
     else
     {
@@ -410,12 +481,16 @@ void TOutputGLSLBase::writeFunctionParameters(const TFunction *func)
     {
         const TVariable *param = func->getParam(i);
         const TType &type      = param->getType();
-        writeVariableType(type, param);
+        writeVariableType(type, param, true);
 
         if (param->symbolType() != SymbolType::Empty)
+        {
             out << " " << hashName(param);
+        }
         if (type.isArray())
+        {
             out << ArrayString(type);
+        }
 
         // Put a comma if this is not the last argument.
         if (i != paramCount - 1)
@@ -585,62 +660,8 @@ bool TOutputGLSLBase::visitBinary(Visit visit, TIntermBinary *node)
             break;
 
         case EOpIndexDirect:
-            writeTriplet(visit, nullptr, "[", "]");
-            break;
         case EOpIndexIndirect:
-            if (node->getAddIndexClamp())
-            {
-                if (visit == InVisit)
-                {
-                    if (mClampingStrategy == SH_CLAMP_WITH_CLAMP_INTRINSIC)
-                        out << "[int(clamp(float(";
-                    else
-                        out << "[webgl_int_clamp(";
-                }
-                else if (visit == PostVisit)
-                {
-                    TIntermTyped *left = node->getLeft();
-                    TType leftType     = left->getType();
-
-                    if (mClampingStrategy == SH_CLAMP_WITH_CLAMP_INTRINSIC)
-                        out << "), 0.0, float(";
-                    else
-                        out << ", 0, ";
-
-                    if (leftType.isUnsizedArray())
-                    {
-                        // For runtime-sized arrays in ESSL 3.10 we need to call the length method
-                        // to get the length to clamp against. See ESSL 3.10 section 4.1.9. Note
-                        // that a runtime-sized array expression is guaranteed not to have side
-                        // effects, so it's fine to add the expression to the output twice.
-                        ASSERT(mShaderVersion >= 310);
-                        ASSERT(!left->hasSideEffects());
-                        left->traverse(this);
-                        out << ".length() - 1";
-                    }
-                    else
-                    {
-                        int maxSize;
-                        if (leftType.isArray())
-                        {
-                            maxSize = static_cast<int>(leftType.getOutermostArraySize()) - 1;
-                        }
-                        else
-                        {
-                            maxSize = leftType.getNominalSize() - 1;
-                        }
-                        out << maxSize;
-                    }
-                    if (mClampingStrategy == SH_CLAMP_WITH_CLAMP_INTRINSIC)
-                        out << ")))]";
-                    else
-                        out << ")]";
-                }
-            }
-            else
-            {
-                writeTriplet(visit, nullptr, "[", "]");
-            }
+            writeTriplet(visit, nullptr, "[", "]");
             break;
         case EOpIndexDirectStruct:
             if (visit == InVisit)
@@ -786,69 +807,10 @@ bool TOutputGLSLBase::visitUnary(Visit visit, TIntermUnary *node)
             postString = ").length())";
             break;
 
-        case EOpRadians:
-        case EOpDegrees:
-        case EOpSin:
-        case EOpCos:
-        case EOpTan:
-        case EOpAsin:
-        case EOpAcos:
-        case EOpAtan:
-        case EOpSinh:
-        case EOpCosh:
-        case EOpTanh:
-        case EOpAsinh:
-        case EOpAcosh:
-        case EOpAtanh:
-        case EOpExp:
-        case EOpLog:
-        case EOpExp2:
-        case EOpLog2:
-        case EOpSqrt:
-        case EOpInversesqrt:
-        case EOpAbs:
-        case EOpSign:
-        case EOpFloor:
-        case EOpTrunc:
-        case EOpRound:
-        case EOpRoundEven:
-        case EOpCeil:
-        case EOpFract:
-        case EOpIsnan:
-        case EOpIsinf:
-        case EOpFloatBitsToInt:
-        case EOpFloatBitsToUint:
-        case EOpIntBitsToFloat:
-        case EOpUintBitsToFloat:
-        case EOpPackSnorm2x16:
-        case EOpPackUnorm2x16:
-        case EOpPackHalf2x16:
-        case EOpUnpackSnorm2x16:
-        case EOpUnpackUnorm2x16:
-        case EOpUnpackHalf2x16:
-        case EOpPackUnorm4x8:
-        case EOpPackSnorm4x8:
-        case EOpUnpackUnorm4x8:
-        case EOpUnpackSnorm4x8:
-        case EOpLength:
-        case EOpNormalize:
-        case EOpDFdx:
-        case EOpDFdy:
-        case EOpFwidth:
-        case EOpTranspose:
-        case EOpDeterminant:
-        case EOpInverse:
-        case EOpAny:
-        case EOpAll:
-        case EOpLogicalNotComponentWise:
-        case EOpBitfieldReverse:
-        case EOpBitCount:
-        case EOpFindLSB:
-        case EOpFindMSB:
-            writeBuiltInFunctionTriplet(visit, node->getOp(), node->getUseEmulatedFunction());
-            return true;
         default:
-            UNREACHABLE();
+            writeFunctionTriplet(visit, node->getFunction()->name(),
+                                 node->getUseEmulatedFunction());
+            return true;
     }
 
     writeTriplet(visit, preString, nullptr, postString);
@@ -885,7 +847,7 @@ bool TOutputGLSLBase::visitIfElse(Visit visit, TIntermIfElse *node)
 
     if (node->getFalseBlock())
     {
-        out << "else\n";
+        out << getIndentPrefix() << "else\n";
         visitCodeBlock(node->getFalseBlock());
     }
     return false;
@@ -928,6 +890,9 @@ bool TOutputGLSLBase::visitBlock(Visit visit, TIntermBlock *node)
     {
         TIntermNode *curNode = *iter;
         ASSERT(curNode != nullptr);
+
+        out << getIndentPrefix(curNode->getAsCaseNode() ? -1 : 0);
+
         curNode->traverse(this);
 
         if (isSingleStatement(curNode))
@@ -937,7 +902,7 @@ bool TOutputGLSLBase::visitBlock(Visit visit, TIntermBlock *node)
     // Scope the blocks except when at the global scope.
     if (getCurrentTraversalDepth() > 0)
     {
-        out << "}\n";
+        out << getIndentPrefix(-1) << "}\n";
     }
     return false;
 }
@@ -952,12 +917,13 @@ bool TOutputGLSLBase::visitFunctionDefinition(Visit visit, TIntermFunctionDefini
     return false;
 }
 
-bool TOutputGLSLBase::visitInvariantDeclaration(Visit visit, TIntermInvariantDeclaration *node)
+bool TOutputGLSLBase::visitGlobalQualifierDeclaration(Visit visit,
+                                                      TIntermGlobalQualifierDeclaration *node)
 {
     TInfoSinkBase &out = objSink();
     ASSERT(visit == PreVisit);
     const TIntermSymbol *symbol = node->getSymbol();
-    out << "invariant " << hashName(&symbol->variable());
+    out << (node->isPrecise() ? "precise " : "invariant ") << hashName(&symbol->variable());
     return false;
 }
 
@@ -966,7 +932,7 @@ void TOutputGLSLBase::visitFunctionPrototype(TIntermFunctionPrototype *node)
     TInfoSinkBase &out = objSink();
 
     const TType &type = node->getType();
-    writeVariableType(type, node->getFunction());
+    writeVariableType(type, node->getFunction(), false);
     if (type.isArray())
         out << ArrayString(type);
 
@@ -980,87 +946,30 @@ void TOutputGLSLBase::visitFunctionPrototype(TIntermFunctionPrototype *node)
 bool TOutputGLSLBase::visitAggregate(Visit visit, TIntermAggregate *node)
 {
     bool visitChildren = true;
-    TInfoSinkBase &out = objSink();
-    switch (node->getOp())
+    if (node->getOp() == EOpConstruct)
     {
-        case EOpCallFunctionInAST:
-        case EOpCallInternalRawFunction:
-        case EOpCallBuiltInFunction:
-            // Function call.
-            if (visit == PreVisit)
-            {
-                if (node->getOp() == EOpCallBuiltInFunction)
-                {
-                    out << translateTextureFunction(node->getFunction()->name());
-                }
-                else
-                {
-                    out << hashFunctionNameIfNeeded(node->getFunction());
-                }
-                out << "(";
-            }
-            else if (visit == InVisit)
-                out << ", ";
-            else
-                out << ")";
-            break;
-        case EOpConstruct:
-            writeConstructorTriplet(visit, node->getType());
-            break;
+        writeConstructorTriplet(visit, node->getType());
+    }
+    else
+    {
+        // Function call.
+        ImmutableString functionName = node->getFunction()->name();
+        if (visit == PreVisit)
+        {
+            // No raw function is expected.
+            ASSERT(node->getOp() != EOpCallInternalRawFunction);
 
-        case EOpEqualComponentWise:
-        case EOpNotEqualComponentWise:
-        case EOpLessThanComponentWise:
-        case EOpGreaterThanComponentWise:
-        case EOpLessThanEqualComponentWise:
-        case EOpGreaterThanEqualComponentWise:
-        case EOpMod:
-        case EOpModf:
-        case EOpPow:
-        case EOpAtan:
-        case EOpMin:
-        case EOpMax:
-        case EOpClamp:
-        case EOpMix:
-        case EOpStep:
-        case EOpSmoothstep:
-        case EOpFrexp:
-        case EOpLdexp:
-        case EOpDistance:
-        case EOpDot:
-        case EOpCross:
-        case EOpFaceforward:
-        case EOpReflect:
-        case EOpRefract:
-        case EOpMulMatrixComponentWise:
-        case EOpOuterProduct:
-        case EOpBitfieldExtract:
-        case EOpBitfieldInsert:
-        case EOpUaddCarry:
-        case EOpUsubBorrow:
-        case EOpUmulExtended:
-        case EOpImulExtended:
-        case EOpBarrier:
-        case EOpMemoryBarrier:
-        case EOpMemoryBarrierAtomicCounter:
-        case EOpMemoryBarrierBuffer:
-        case EOpMemoryBarrierImage:
-        case EOpMemoryBarrierShared:
-        case EOpGroupMemoryBarrier:
-        case EOpAtomicAdd:
-        case EOpAtomicMin:
-        case EOpAtomicMax:
-        case EOpAtomicAnd:
-        case EOpAtomicOr:
-        case EOpAtomicXor:
-        case EOpAtomicExchange:
-        case EOpAtomicCompSwap:
-        case EOpEmitVertex:
-        case EOpEndPrimitive:
-            writeBuiltInFunctionTriplet(visit, node->getOp(), node->getUseEmulatedFunction());
-            break;
-        default:
-            UNREACHABLE();
+            if (node->getOp() == EOpCallFunctionInAST)
+            {
+                functionName = hashFunctionNameIfNeeded(node->getFunction());
+            }
+            else
+            {
+                functionName =
+                    translateTextureFunction(node->getFunction()->name(), mCompileOptions);
+            }
+        }
+        writeFunctionTriplet(visit, functionName, node->getUseEmulatedFunction());
     }
     return visitChildren;
 }
@@ -1073,12 +982,24 @@ bool TOutputGLSLBase::visitDeclaration(Visit visit, TIntermDeclaration *node)
     if (visit == PreVisit)
     {
         const TIntermSequence &sequence = *(node->getSequence());
-        TIntermTyped *variable          = sequence.front()->getAsTyped();
-        writeLayoutQualifier(variable);
-        TIntermSymbol *symbolNode = variable->getAsSymbolNode();
-        writeVariableType(variable->getType(), symbolNode ? &symbolNode->variable() : nullptr);
-        if (variable->getAsSymbolNode() == nullptr ||
-            variable->getAsSymbolNode()->variable().symbolType() != SymbolType::Empty)
+        TIntermTyped *decl              = sequence.front()->getAsTyped();
+        TIntermSymbol *symbolNode       = decl->getAsSymbolNode();
+        if (symbolNode == nullptr)
+        {
+            ASSERT(decl->getAsBinaryNode() && decl->getAsBinaryNode()->getOp() == EOpInitialize);
+            symbolNode = decl->getAsBinaryNode()->getLeft()->getAsSymbolNode();
+        }
+        ASSERT(symbolNode);
+
+        if (symbolNode->getName() != "gl_ClipDistance" &&
+            symbolNode->getName() != "gl_CullDistance")
+        {
+            // gl_Clip/CullDistance re-declaration doesn't need layout.
+            writeLayoutQualifier(symbolNode);
+        }
+
+        writeVariableType(symbolNode->getType(), &symbolNode->variable(), false);
+        if (symbolNode->variable().symbolType() != SymbolType::Empty)
         {
             out << " ";
         }
@@ -1173,6 +1094,7 @@ void TOutputGLSLBase::visitCodeBlock(TIntermBlock *node)
     TInfoSinkBase &out = objSink();
     if (node != nullptr)
     {
+        out << getIndentPrefix();
         node->traverse(this);
         // Single statements not part of a sequence need to be terminated
         // with semi-colon.
@@ -1221,6 +1143,13 @@ void TOutputGLSLBase::visitPreprocessorDirective(TIntermPreprocessorDirective *n
 
 ImmutableString TOutputGLSLBase::getTypeName(const TType &type)
 {
+    if (type.getBasicType() == EbtSamplerVideoWEBGL)
+    {
+        // TODO(http://anglebug.com/3889): translate SamplerVideoWEBGL into different token
+        // when necessary (e.g. on Android devices)
+        return ImmutableString("sampler2D");
+    }
+
     return GetTypeName(type, mHashFunction, &mNameMap);
 }
 
@@ -1252,17 +1181,6 @@ ImmutableString TOutputGLSLBase::hashFunctionNameIfNeeded(const TFunction *func)
     }
 }
 
-bool TOutputGLSLBase::structDeclared(const TStructure *structure) const
-{
-    ASSERT(structure);
-    if (structure->symbolType() == SymbolType::Empty)
-    {
-        return false;
-    }
-
-    return (mDeclaredStructs.count(structure->uniqueId().get()) > 0);
-}
-
 void TOutputGLSLBase::declareStruct(const TStructure *structure)
 {
     TInfoSinkBase &out = objSink();
@@ -1277,25 +1195,39 @@ void TOutputGLSLBase::declareStruct(const TStructure *structure)
     const TFieldList &fields = structure->fields();
     for (size_t i = 0; i < fields.size(); ++i)
     {
-        const TField *field = fields[i];
-        if (writeVariablePrecision(field->type()->getPrecision()))
+        out << getIndentPrefix(1);
+        const TField *field    = fields[i];
+        const TType &fieldType = *field->type();
+        if (writeVariablePrecision(fieldType.getPrecision()))
+        {
             out << " ";
-        out << getTypeName(*field->type()) << " " << hashFieldName(field);
-        if (field->type()->isArray())
-            out << ArrayString(*field->type());
+        }
+        if (fieldType.isPrecise())
+        {
+            writePreciseQualifier(fieldType);
+        }
+        out << getTypeName(fieldType) << " " << hashFieldName(field);
+        if (fieldType.isArray())
+        {
+            out << ArrayString(fieldType);
+        }
         out << ";\n";
     }
-    out << "}";
-
-    if (structure->symbolType() != SymbolType::Empty)
-    {
-        mDeclaredStructs.insert(structure->uniqueId().get());
-    }
+    out << getIndentPrefix() << "}";
 }
 
-void TOutputGLSLBase::declareInterfaceBlockLayout(const TInterfaceBlock *interfaceBlock)
+void TOutputGLSLBase::declareInterfaceBlockLayout(const TType &type)
 {
-    TInfoSinkBase &out = objSink();
+    // 4.4.5 Uniform and Shader Storage Block Layout Qualifiers in GLSL 4.5 spec.
+    // Layout qualifiers can be used for uniform and shader storage blocks,
+    // but not for non-block uniform declarations.
+    if (IsShaderIoBlock(type.getQualifier()))
+    {
+        return;
+    }
+
+    const TInterfaceBlock *interfaceBlock = type.getInterfaceBlock();
+    TInfoSinkBase &out                    = objSink();
 
     out << "layout(";
 
@@ -1333,26 +1265,96 @@ void TOutputGLSLBase::declareInterfaceBlockLayout(const TInterfaceBlock *interfa
     out << ") ";
 }
 
-void TOutputGLSLBase::declareInterfaceBlock(const TInterfaceBlock *interfaceBlock)
+const char *getVariableInterpolation(TQualifier qualifier)
 {
-    TInfoSinkBase &out = objSink();
+    switch (qualifier)
+    {
+        case EvqSmoothOut:
+            return "smooth out ";
+        case EvqFlatOut:
+            return "flat out ";
+        case EvqNoPerspectiveOut:
+            return "noperspective out ";
+        case EvqCentroidOut:
+            return "centroid out ";
+        case EvqSampleOut:
+            return "sample out ";
+        case EvqNoPerspectiveCentroidOut:
+            return "noperspective centroid out ";
+        case EvqNoPerspectiveSampleOut:
+            return "noperspective sample out ";
+        case EvqSmoothIn:
+            return "smooth in ";
+        case EvqFlatIn:
+            return "flat in ";
+        case EvqNoPerspectiveIn:
+            return "noperspective in ";
+        case EvqCentroidIn:
+            return "centroid in ";
+        case EvqSampleIn:
+            return "sample in ";
+        case EvqNoPerspectiveCentroidIn:
+            return "noperspective centroid in ";
+        case EvqNoPerspectiveSampleIn:
+            return "noperspective sample in ";
+        default:
+            break;
+    }
+    return nullptr;
+}
+
+void TOutputGLSLBase::declareInterfaceBlock(const TType &type)
+{
+    const TInterfaceBlock *interfaceBlock = type.getInterfaceBlock();
+    TInfoSinkBase &out                    = objSink();
 
     out << hashName(interfaceBlock) << "{\n";
     const TFieldList &fields = interfaceBlock->fields();
     for (const TField *field : fields)
     {
-        writeFieldLayoutQualifier(field);
+        out << getIndentPrefix(1);
+        if (!IsShaderIoBlock(type.getQualifier()) && type.getQualifier() != EvqPatchIn &&
+            type.getQualifier() != EvqPatchOut)
+        {
+            writeFieldLayoutQualifier(field);
+        }
 
-        if (writeVariablePrecision(field->type()->getPrecision()))
+        const TType &fieldType = *field->type();
+
+        out << getMemoryQualifiers(fieldType);
+        if (writeVariablePrecision(fieldType.getPrecision()))
             out << " ";
-        out << getTypeName(*field->type()) << " " << hashFieldName(field);
-        if (field->type()->isArray())
-            out << ArrayString(*field->type());
+        if (fieldType.isInvariant())
+        {
+            writeInvariantQualifier(fieldType);
+        }
+        if (fieldType.isPrecise())
+        {
+            writePreciseQualifier(fieldType);
+        }
+
+        const char *qualifier = getVariableInterpolation(fieldType.getQualifier());
+        if (qualifier != nullptr)
+            out << qualifier;
+
+        out << getTypeName(fieldType) << " " << hashFieldName(field);
+
+        if (fieldType.isArray())
+            out << ArrayString(fieldType);
         out << ";\n";
     }
     out << "}";
 }
 
+void WritePragma(TInfoSinkBase &out, const ShCompileOptions &compileOptions, const TPragma &pragma)
+{
+    if (!compileOptions.flattenPragmaSTDGLInvariantAll)
+    {
+        if (pragma.stdgl.invariantAll)
+            out << "#pragma STDGL invariant(all)\n";
+    }
+}
+
 void WriteGeometryShaderLayoutQualifiers(TInfoSinkBase &out,
                                          sh::TLayoutPrimitiveType inputPrimitive,
                                          int invocations,
@@ -1401,28 +1403,89 @@ void WriteGeometryShaderLayoutQualifiers(TInfoSinkBase &out,
     }
 }
 
+void WriteTessControlShaderLayoutQualifiers(TInfoSinkBase &out, int inputVertices)
+{
+    if (inputVertices != 0)
+    {
+        out << "layout (vertices = " << inputVertices << ") out;\n";
+    }
+}
+
+void WriteTessEvaluationShaderLayoutQualifiers(TInfoSinkBase &out,
+                                               sh::TLayoutTessEvaluationType inputPrimitive,
+                                               sh::TLayoutTessEvaluationType inputVertexSpacing,
+                                               sh::TLayoutTessEvaluationType inputOrdering,
+                                               sh::TLayoutTessEvaluationType inputPoint)
+{
+    if (inputPrimitive != EtetUndefined)
+    {
+        out << "layout (";
+        out << getTessEvaluationShaderTypeString(inputPrimitive);
+        if (inputVertexSpacing != EtetUndefined)
+        {
+            out << ", " << getTessEvaluationShaderTypeString(inputVertexSpacing);
+        }
+        if (inputOrdering != EtetUndefined)
+        {
+            out << ", " << getTessEvaluationShaderTypeString(inputOrdering);
+        }
+        if (inputPoint != EtetUndefined)
+        {
+            out << ", " << getTessEvaluationShaderTypeString(inputPoint);
+        }
+        out << ") in;\n";
+    }
+}
+
 // If SH_SCALARIZE_VEC_AND_MAT_CONSTRUCTOR_ARGS is enabled, layout qualifiers are spilled whenever
 // variables with specified layout qualifiers are copied. Additional checks are needed against the
 // type and storage qualifier of the variable to verify that layout qualifiers have to be outputted.
 // TODO (mradev): Fix layout qualifier spilling in ScalarizeVecAndMatConstructorArgs and remove
 // NeedsToWriteLayoutQualifier.
-bool NeedsToWriteLayoutQualifier(const TType &type)
+bool TOutputGLSLBase::needsToWriteLayoutQualifier(const TType &type)
 {
-    if (type.getBasicType() == EbtInterfaceBlock)
-    {
-        return true;
-    }
-
     const TLayoutQualifier &layoutQualifier = type.getLayoutQualifier();
 
-    if ((type.getQualifier() == EvqFragmentOut || type.getQualifier() == EvqVertexIn ||
-         IsVarying(type.getQualifier())) &&
-        layoutQualifier.location >= 0)
+    if (type.getBasicType() == EbtInterfaceBlock)
+    {
+        if (type.getQualifier() == EvqPixelLocalEXT)
+        {
+            // We only use per-member EXT_shader_pixel_local_storage formats, so the PLS interface
+            // block will never have a layout qualifier.
+            ASSERT(layoutQualifier.imageInternalFormat == EiifUnspecified);
+            return false;
+        }
+        return true;
+    }
+
+    if (IsFragmentOutput(type.getQualifier()) || type.getQualifier() == EvqVertexIn ||
+        IsVarying(type.getQualifier()))
+    {
+        if (layoutQualifier.location >= 0 ||
+            (mAlwaysSpecifyFragOutLocation && IsFragmentOutput(type.getQualifier())))
+        {
+            return true;
+        }
+    }
+
+    if (type.getQualifier() == EvqFragDepth && layoutQualifier.depth != EdUnspecified)
     {
         return true;
     }
 
-    if (type.getQualifier() == EvqFragmentOut && layoutQualifier.yuv == true)
+    if (type.getQualifier() == EvqFragmentOut || type.getQualifier() == EvqFragmentInOut)
+    {
+        if (layoutQualifier.index >= 0)
+        {
+            return true;
+        }
+        if (layoutQualifier.yuv)
+        {
+            return true;
+        }
+    }
+
+    if (type.getQualifier() == EvqFragmentInOut && layoutQualifier.noncoherent)
     {
         return true;
     }
@@ -1439,4 +1502,64 @@ bool NeedsToWriteLayoutQualifier(const TType &type)
     return false;
 }
 
+void EmitEarlyFragmentTestsGLSL(const TCompiler &compiler, TInfoSinkBase &sink)
+{
+    if (compiler.isEarlyFragmentTestsSpecified())
+    {
+        sink << "layout (early_fragment_tests) in;\n";
+    }
+}
+
+void EmitWorkGroupSizeGLSL(const TCompiler &compiler, TInfoSinkBase &sink)
+{
+    if (compiler.isComputeShaderLocalSizeDeclared())
+    {
+        const sh::WorkGroupSize &localSize = compiler.getComputeShaderLocalSize();
+        sink << "layout (local_size_x=" << localSize[0] << ", local_size_y=" << localSize[1]
+             << ", local_size_z=" << localSize[2] << ") in;\n";
+    }
+}
+
+void EmitMultiviewGLSL(const TCompiler &compiler,
+                       const ShCompileOptions &compileOptions,
+                       const TExtension extension,
+                       const TBehavior behavior,
+                       TInfoSinkBase &sink)
+{
+    ASSERT(behavior != EBhUndefined);
+    if (behavior == EBhDisable)
+        return;
+
+    const bool isVertexShader = (compiler.getShaderType() == GL_VERTEX_SHADER);
+    if (compileOptions.initializeBuiltinsForInstancedMultiview)
+    {
+        // Emit ARB_shader_viewport_layer_array/NV_viewport_array2 in a vertex shader if the
+        // SH_SELECT_VIEW_IN_NV_GLSL_VERTEX_SHADER option is set and the
+        // OVR_multiview(2) extension is requested.
+        if (isVertexShader && compileOptions.selectViewInNvGLSLVertexShader)
+        {
+            sink << "#if defined(GL_ARB_shader_viewport_layer_array)\n"
+                 << "#extension GL_ARB_shader_viewport_layer_array : require\n"
+                 << "#elif defined(GL_NV_viewport_array2)\n"
+                 << "#extension GL_NV_viewport_array2 : require\n"
+                 << "#endif\n";
+        }
+    }
+    else
+    {
+        sink << "#extension GL_OVR_multiview";
+        if (extension == TExtension::OVR_multiview2)
+        {
+            sink << "2";
+        }
+        sink << " : " << GetBehaviorString(behavior) << "\n";
+
+        const auto &numViews = compiler.getNumViews();
+        if (isVertexShader && numViews != -1)
+        {
+            sink << "layout(num_views=" << numViews << ") in;\n";
+        }
+    }
+}
+
 }  // namespace sh
