@@ -185,6 +185,7 @@ void InitBuiltInResources(ShBuiltInResources *resources)
     resources->NV_EGL_stream_consumer_external                = 0;
     resources->ARB_texture_rectangle                          = 0;
     resources->EXT_blend_func_extended                        = 0;
+    resources->EXT_conservative_depth                         = 0;
     resources->EXT_draw_buffers                               = 0;
     resources->EXT_frag_depth                                 = 0;
     resources->EXT_shader_texture_lod                         = 0;
@@ -222,6 +223,7 @@ void InitBuiltInResources(ShBuiltInResources *resources)
     resources->EXT_texture_buffer                             = 0;
     resources->OES_sample_variables                           = 0;
     resources->EXT_clip_cull_distance                         = 0;
+    resources->ANGLE_clip_cull_distance                       = 0;
     resources->KHR_blend_equation_advanced                    = 0;
 
     resources->MaxClipDistances                = 8;
@@ -462,6 +464,18 @@ const BinaryBlob &GetObjectBinaryBlob(const ShHandle handle)
     return infoSink.obj.getBinary();
 }
 
+bool GetShaderBinary(const ShHandle handle,
+                     const char *const shaderStrings[],
+                     size_t numStrings,
+                     const ShCompileOptions &compileOptions,
+                     ShaderBinaryBlob *const binaryOut)
+{
+    TCompiler *compiler = GetCompilerFromHandle(handle);
+    ASSERT(compiler);
+
+    return compiler->getShaderBinary(handle, shaderStrings, numStrings, compileOptions, binaryOut);
+}
+
 const std::map<std::string, std::string> *GetNameHashingMap(const ShHandle handle)
 {
     TCompiler *compiler = GetCompilerFromHandle(handle);
@@ -731,6 +745,37 @@ const std::set<std::string> *GetUsedImage2DFunctionNames(const ShHandle handle)
 #endif  // ANGLE_ENABLE_HLSL
 }
 
+uint8_t GetClipDistanceArraySize(const ShHandle handle)
+{
+    ASSERT(handle);
+    TShHandleBase *base = static_cast<TShHandleBase *>(handle);
+    TCompiler *compiler = base->getAsCompiler();
+    ASSERT(compiler);
+
+    return compiler->getClipDistanceArraySize();
+}
+
+uint8_t GetCullDistanceArraySize(const ShHandle handle)
+{
+    ASSERT(handle);
+    TShHandleBase *base = static_cast<TShHandleBase *>(handle);
+    TCompiler *compiler = base->getAsCompiler();
+    ASSERT(compiler);
+
+    return compiler->getCullDistanceArraySize();
+}
+
+bool HasClipDistanceInVertexShader(const ShHandle handle)
+{
+    ASSERT(handle);
+
+    TShHandleBase *base = static_cast<TShHandleBase *>(handle);
+    TCompiler *compiler = base->getAsCompiler();
+    ASSERT(compiler);
+
+    return compiler->getShaderType() == GL_VERTEX_SHADER && compiler->hasClipDistance();
+}
+
 bool HasDiscardInFragmentShader(const ShHandle handle)
 {
     ASSERT(handle);
@@ -1022,6 +1067,10 @@ const char *InterpolationTypeToString(InterpolationType type)
             return "flat";
         case InterpolationType::INTERPOLATION_NOPERSPECTIVE:
             return "noperspective";
+        case InterpolationType::INTERPOLATION_NOPERSPECTIVE_CENTROID:
+            return "noperspective centroid";
+        case InterpolationType::INTERPOLATION_NOPERSPECTIVE_SAMPLE:
+            return "noperspective sample";
         default:
             return "invalid";
     }
