@@ -19,6 +19,7 @@
 #include "compiler/translator/IntermNode.h"
 #include "compiler/translator/OutputSPIRV.h"
 #include "compiler/translator/StaticType.h"
+#include "compiler/translator/tree_ops/DeclarePerVertexBlocks.h"
 #include "compiler/translator/tree_ops/MonomorphizeUnsupportedFunctions.h"
 #include "compiler/translator/tree_ops/RecordConstantPrecision.h"
 #include "compiler/translator/tree_ops/RemoveAtomicCounterBuiltins.h"
@@ -29,7 +30,6 @@
 #include "compiler/translator/tree_ops/RewriteDfdy.h"
 #include "compiler/translator/tree_ops/RewriteStructSamplers.h"
 #include "compiler/translator/tree_ops/SeparateStructFromUniformDeclarations.h"
-#include "compiler/translator/tree_ops/vulkan/DeclarePerVertexBlocks.h"
 #include "compiler/translator/tree_ops/vulkan/EmulateAdvancedBlendEquations.h"
 #include "compiler/translator/tree_ops/vulkan/EmulateDithering.h"
 #include "compiler/translator/tree_ops/vulkan/EmulateFragColorData.h"
@@ -159,8 +159,8 @@ bool DeclareDefaultUniforms(TCompiler *compiler,
     TLayoutQualifier layoutQualifier = TLayoutQualifier::Create();
     layoutQualifier.blockStorage     = EbsStd140;
     const TVariable *uniformBlock    = DeclareInterfaceBlock(
-           root, symbolTable, uniformList, EvqUniform, layoutQualifier, TMemoryQualifier::Create(), 0,
-           ImmutableString(kDefaultUniformNames[shaderType]), ImmutableString(""));
+        root, symbolTable, uniformList, EvqUniform, layoutQualifier, TMemoryQualifier::Create(), 0,
+        ImmutableString(kDefaultUniformNames[shaderType]), ImmutableString(""));
 
     // Create a map from the uniform variables to new variables that reference the fields of the
     // block.
@@ -633,7 +633,6 @@ bool HasFramebufferFetch(const TExtensionBehavior &extBehavior,
 {
     return IsExtensionEnabled(extBehavior, TExtension::EXT_shader_framebuffer_fetch) ||
            IsExtensionEnabled(extBehavior, TExtension::EXT_shader_framebuffer_fetch_non_coherent) ||
-           IsExtensionEnabled(extBehavior, TExtension::ARM_shader_framebuffer_fetch) ||
            IsExtensionEnabled(extBehavior, TExtension::NV_shader_framebuffer_fetch) ||
            (compileOptions.pls.type == ShPixelLocalStorageType::FramebufferFetch &&
             IsExtensionEnabled(extBehavior, TExtension::ANGLE_shader_pixel_local_storage));
@@ -641,7 +640,7 @@ bool HasFramebufferFetch(const TExtensionBehavior &extBehavior,
 }  // anonymous namespace
 
 TranslatorVulkan::TranslatorVulkan(sh::GLenum type, ShShaderSpec spec)
-    : TCompiler(type, spec, SH_GLSL_450_CORE_OUTPUT)
+    : TCompiler(type, spec, SH_SPIRV_VULKAN_OUTPUT)
 {}
 
 bool TranslatorVulkan::translateImpl(TIntermBlock *root,
@@ -977,7 +976,8 @@ bool TranslatorVulkan::translateImpl(TIntermBlock *root,
             {
                 if (getShaderVersion() == 100)
                 {
-                    if (!ReplaceLastFragData(this, root, &getSymbolTable(), &mUniforms))
+                    if (!ReplaceLastFrag(this, root, &getSymbolTable(), &mUniforms,
+                                         FramebufferFetchReplaceTarget::LastFragData))
                     {
                         return false;
                     }
@@ -991,6 +991,19 @@ bool TranslatorVulkan::translateImpl(TIntermBlock *root,
                 }
             }
 
+            // Translating GL_ARM_shader_framebuffer_fetch deals with a separate variable
+            // than translating the GL_EXT_shader_framebuffer_fetch extension. So they need
+            // to be handled in separate passes despite their similarities.
+            if (IsExtensionEnabled(getExtensionBehavior(),
+                                   TExtension::ARM_shader_framebuffer_fetch))
+            {
+                if (!ReplaceLastFrag(this, root, &getSymbolTable(), &mUniforms,
+                                     FramebufferFetchReplaceTarget::LastFragColor))
+                {
+                    return false;
+                }
+            }
+
             // Emulate gl_FragColor and gl_FragData with normal output variables.
             if (!EmulateFragColorData(this, root, &getSymbolTable()))
             {
@@ -1003,9 +1016,8 @@ bool TranslatorVulkan::translateImpl(TIntermBlock *root,
             // attachment variable then create a new one.
             if (getAdvancedBlendEquations().any() &&
                 compileOptions.addAdvancedBlendEquationsEmulation &&
-                !EmulateAdvancedBlendEquations(this, compileOptions, root, &getSymbolTable(),
-                                               driverUniforms, &mUniforms,
-                                               getAdvancedBlendEquations()))
+                !EmulateAdvancedBlendEquations(this, root, &getSymbolTable(), driverUniforms,
+                                               &mUniforms, getAdvancedBlendEquations()))
             {
                 return false;
             }
