@@ -220,6 +220,7 @@ MSG kInsufficientBufferSize = "Insufficient buffer size.";
 MSG kInsufficientParams = "More parameters are required than were provided.";
 MSG kInsufficientVertexBufferSize = "Vertex buffer is not big enough for the draw call";
 MSG kIntegerOverflow = "Integer overflow.";
+MSG kInternalErrorFormatNotFound = "Internal error: unknown internal format.";
 MSG kInternalFormatRequiresTexture2D = "internalformat is an ETC1 or PVRTC1 format.";
 MSG kInternalFormatRequiresTexture2DArray = "internalformat is an ETC2/EAC format and target is not GL_TEXTURE_2D_ARRAY.";
 MSG kInternalFormatRequiresTexture2DArrayS3TC = "internalformat is an S3TC format and target is not GL_TEXTURE_2D_ARRAY.";
@@ -501,21 +502,23 @@ MSG kPLSInvalidLoadOperation = "Invalid pixel local storage Load Operation: 0x%0
 MSG kPLSInvalidStoreOperation = "Invalid pixel local storage Store Operation: 0x%04X.";
 MSG kPLSInvalidTextureType = "Invalid pixel local storage texture type.";
 MSG kPLSKeepingMemorylessPlane = "Load Operation GL_LOAD_OP_LOAD_ANGLE is invalid for memoryless planes.";
+MSG kPLSLoadOpsNULL = "<loadops> cannot be null.";
 MSG kPLSMaxColorAttachmentsExceded = "Framebuffer cannot have images attached to color attachment points on or after COLOR_ATTACHMENT0 + MAX_COLOR_ATTACHMENTS_WITH_ACTIVE_PIXEL_LOCAL_STORAGE_ANGLE.";
 MSG kPLSMaxCombinedDrawBuffersAndPlanesExceded = "Framebuffer cannot have images attached to color attachment points on or after COLOR_ATTACHMENT0 + MAX_COMBINED_DRAW_BUFFERS_AND_PIXEL_LOCAL_STORAGE_PLANES_ANGLE - <n>.";
 MSG kPLSMismatchedBackingTextureSizes = "Mismatched pixel local storage backing texture sizes.";
 MSG kPLSMultisamplingEnabled = "Attempted to begin pixel local storage with a multisampled framebuffer.";
 MSG kPLSNNotEqualActivePlanes = "<n> != ACTIVE_PIXEL_LOCAL_STORAGE_PLANES_ANGLE";
 MSG kPLSNoAttachmentsNoTextureBacked = "Draw framebuffer has no attachments and no enabled, texture-backed pixel local storage planes.";
-MSG kPLSNullLoadOps = "loadops cannot null.";
+MSG kPLSInterruptOverflow = "Pixel local storage does not support more than 255 nested interruptions.";
+MSG kPLSInterrupted = "Pixel local storage on the draw framebuffer is interrupted.";
+MSG kPLSNotInterrupted = "Pixel local storage on the draw framebuffer is not interrupted.";
+MSG kPLSParamsNULL = "<params> cannot be null.";
 MSG kPLSPlaneLessThanZero = "Plane cannot be less than 0.";
 MSG kPLSPlaneOutOfRange = "Plane must be less than GL_MAX_PIXEL_LOCAL_STORAGE_PLANES_ANGLE.";
 MSG kPLSPlanesLessThanOne = "Planes must be greater than 0.";
 MSG kPLSPlanesOutOfRange = "Planes must be less than or equal to GL_MAX_PIXEL_LOCAL_STORAGE_PLANES_ANGLE.";
 MSG kPLSRasterizerDiscardEnabled = "Attempted to begin pixel local storage with GL_RASTERIZER_DISCARD enabled.";
 MSG kPLSReservedDrawBufferInUse = "When beginning pixel local storage, glDrawBuffers must all be GL_NONE at indices greater than or equal to: min(GL_MAX_COLOR_ATTACHMENTS_WITH_ACTIVE_PLS_ANGLE, GL_MAX_COMBINED_DRAW_BUFFERS_AND_PLS_PLANES_ANGLE - <n>).";
-MSG kPLSSampleAlphaToCoverageEnabled = "Attempted to begin pixel local storage with GL_SAMPLE_ALPHA_TO_COVERAGE enabled.";
-MSG kPLSSampleCoverageEnabled = "Attempted to begin pixel local storage with GL_SAMPLE_COVERAGE enabled.";
 MSG kPointSizeArrayExtensionNotEnabled = "GL_OES_point_size_array not enabled.";
 MSG kProgramDoesNotExist = "Program doesn't exist.";
 MSG kProgramInterfaceMustBeProgramOutput = "programInterface must be set to GL_PROGRAM_OUTPUT.";
