@@ -144,8 +144,8 @@ class TextureState final : private angle::NonCopyable
     const SamplerState &getSamplerState() const { return mSamplerState; }
     GLenum getUsage() const { return mUsage; }
     bool hasProtectedContent() const { return mHasProtectedContent; }
+    bool renderabilityValidation() const { return mRenderabilityValidation; }
     GLenum getDepthStencilTextureMode() const { return mDepthStencilTextureMode; }
-    bool isStencilMode() const { return mDepthStencilTextureMode == GL_STENCIL_INDEX; }
 
     bool hasBeenBoundAsImage() const { return mHasBeenBoundAsImage; }
     bool is3DTextureAndHasBeenBoundAs2DImage() const { return mIs3DAndHasBeenBoundAs2DImage; }
@@ -157,6 +157,15 @@ class TextureState final : private angle::NonCopyable
     const ImageDesc &getBaseLevelDesc() const;
     const ImageDesc &getLevelZeroDesc() const;
 
+    // This helper is used by backends that require special setup to read stencil data
+    bool isStencilMode() const
+    {
+        const GLenum format =
+            getImageDesc(getBaseImageTarget(), getEffectiveBaseLevel()).format.info->format;
+        return (format == GL_DEPTH_STENCIL) ? (mDepthStencilTextureMode == GL_STENCIL_INDEX)
+                                            : (format == GL_STENCIL_INDEX);
+    }
+
     // GLES1 emulation: For GL_OES_draw_texture
     void setCrop(const Rectangle &rect);
     const Rectangle &getCrop() const;
@@ -235,6 +244,8 @@ class TextureState final : private angle::NonCopyable
     // GL_EXT_protected_textures
     bool mHasProtectedContent;
 
+    bool mRenderabilityValidation;
+
     std::vector<ImageDesc> mImageDescs;
 
     // GLES1 emulation: Texture crop rectangle
@@ -343,6 +354,8 @@ class Texture final : public RefCountObject<TextureID>,
     void setProtectedContent(Context *context, bool hasProtectedContent);
     bool hasProtectedContent() const override;
 
+    void setRenderabilityValidation(Context *context, bool renderabilityValidation);
+
     const TextureState &getState() const { return mState; }
 
     void setBorderColor(const Context *context, const ColorGeneric &color);
@@ -626,6 +639,7 @@ class Texture final : public RefCountObject<TextureID>,
         DIRTY_BIT_BASE_LEVEL,
         DIRTY_BIT_MAX_LEVEL,
         DIRTY_BIT_DEPTH_STENCIL_TEXTURE_MODE,
+        DIRTY_BIT_RENDERABILITY_VALIDATION_ANGLE,
 
         // Image state
         DIRTY_BIT_BOUND_AS_IMAGE,
