@@ -408,6 +408,8 @@ void SerializeRasterizerState(JsonSerializer *json, const gl::RasterizerState &r
     json->addScalar("PolygonOffsetFill", rasterizerState.polygonOffsetFill);
     json->addScalar("PolygonOffsetFactor", rasterizerState.polygonOffsetFactor);
     json->addScalar("PolygonOffsetUnits", rasterizerState.polygonOffsetUnits);
+    json->addScalar("PolygonOffsetClamp", rasterizerState.polygonOffsetClamp);
+    json->addScalar("DepthClamp", rasterizerState.depthClamp);
     json->addScalar("PointDrawMode", rasterizerState.pointDrawMode);
     json->addScalar("MultiSample", rasterizerState.multiSample);
     json->addScalar("RasterizerDiscard", rasterizerState.rasterizerDiscard);
@@ -574,6 +576,8 @@ void SerializeContextState(JsonSerializer *json, const gl::State &state)
     SerializeRectangle(json, "Viewport", state.getViewport());
     json->addScalar("Near", state.getNearPlane());
     json->addScalar("Far", state.getFarPlane());
+    json->addString("ClipOrigin", ToString(state.getClipOrigin()));
+    json->addString("ClipDepthMode", ToString(state.getClipDepthMode()));
     SerializeResourceID(json, "ReadFramebufferID", state.getReadFramebuffer());
     SerializeResourceID(json, "DrawFramebufferID", state.getDrawFramebuffer());
     json->addScalar("RenderbufferID", state.getRenderbufferId().value);
@@ -957,7 +961,9 @@ void SerializeShader(const gl::Context *context,
     GroupScope group(json, "Shader", id);
     SerializeShaderState(json, shader->getState());
     json->addScalar("Handle", shader->getHandle().value);
-    json->addScalar("RefCount", shader->getRefCount());
+    // TODO: implement MEC context validation only after all contexts have been initialized
+    // http://anglebug.com/8029
+    // json->addScalar("RefCount", shader->getRefCount());
     json->addScalar("FlaggedForDeletion", shader->isFlaggedForDeletion());
     // Do not serialize mType because it is already serialized in SerializeShaderState.
     json->addString("InfoLogString", shader->getInfoLogString());
@@ -1092,7 +1098,9 @@ void SerializeProgram(JsonSerializer *json,
     SerializeProgramAliasedBindings(json, program->getFragmentOutputIndexes());
     json->addScalar("IsLinked", program->isLinked());
     json->addScalar("IsFlaggedForDeletion", program->isFlaggedForDeletion());
-    json->addScalar("RefCount", program->getRefCount());
+    // TODO: implement MEC context validation only after all contexts have been initialized
+    // http://anglebug.com/8029
+    // json->addScalar("RefCount", program->getRefCount());
     json->addScalar("ID", program->id().value);
 
     // Serialize uniforms.
