@@ -153,9 +153,6 @@ class ProgramD3DMetadata final : angle::NonCopyable
     int getRendererMajorShaderModel() const;
     bool usesBroadcast(const gl::State &data) const;
     bool usesSecondaryColor() const;
-    bool usesClipDistance() const;
-    bool usesCullDistance() const;
-    bool usesFragDepth() const;
     bool usesPointCoord() const;
     bool usesFragCoord() const;
     bool usesPointSize() const;
@@ -170,7 +167,11 @@ class ProgramD3DMetadata final : angle::NonCopyable
     bool usesSystemValuePointSize() const;
     bool usesMultipleFragmentOuts() const;
     bool usesCustomOutVars() const;
+    bool usesSampleMask() const;
     const ShaderD3D *getFragmentShader() const;
+    FragDepthUsage getFragDepthUsage() const;
+    uint8_t getClipDistanceArraySize() const;
+    uint8_t getCullDistanceArraySize() const;
 
   private:
     const int mRendererMajorShaderModel;
@@ -325,7 +326,7 @@ class ProgramD3D : public ProgramImpl
         return mAttribLocationToD3DSemantic;
     }
 
-    void updateCachedInputLayout(Serial associatedSerial, const gl::State &state);
+    void updateCachedInputLayout(UniqueSerial associatedSerial, const gl::State &state);
     void updateCachedOutputLayout(const gl::Context *context, const gl::Framebuffer *framebuffer);
     void updateCachedComputeImage2DBindLayout(const gl::Context *context);
 
@@ -411,20 +412,24 @@ class ProgramD3D : public ProgramImpl
     class PixelExecutable
     {
       public:
-        PixelExecutable(const std::vector<GLenum> &outputSignature,
+        PixelExecutable(const std::pair<bool, const std::vector<GLenum>> &outputSignature,
                         ShaderExecutableD3D *shaderExecutable);
         ~PixelExecutable();
 
-        bool matchesSignature(const std::vector<GLenum> &signature) const
+        bool matchesSignature(const std::pair<bool, const std::vector<GLenum>> &signature) const
         {
             return mOutputSignature == signature;
         }
 
-        const std::vector<GLenum> &outputSignature() const { return mOutputSignature; }
+        const std::pair<bool, const std::vector<GLenum>> &outputSignature() const
+        {
+            return mOutputSignature;
+        }
+
         ShaderExecutableD3D *shaderExecutable() const { return mShaderExecutable; }
 
       private:
-        std::vector<GLenum> mOutputSignature;
+        const std::pair<bool, const std::vector<GLenum>> mOutputSignature;
         ShaderExecutableD3D *mShaderExecutable;
     };
 
@@ -549,7 +554,8 @@ class ProgramD3D : public ProgramImpl
     gl::ShaderMap<std::string> mShaderHLSL;
     gl::ShaderMap<CompilerWorkaroundsD3D> mShaderWorkarounds;
 
-    bool mUsesFragDepth;
+    FragDepthUsage mFragDepthUsage;
+    bool mUsesSampleMask;
     bool mHasANGLEMultiviewEnabled;
     bool mUsesVertexID;
     bool mUsesViewID;
@@ -576,7 +582,7 @@ class ProgramD3D : public ProgramImpl
     gl::ShaderMap<gl::RangeUI> mUsedAtomicCounterRange;
 
     // Cache for pixel shader output layout to save reallocations.
-    std::vector<GLenum> mPixelShaderOutputLayoutCache;
+    std::pair<bool, std::vector<GLenum>> mPixelShaderOutputLayoutCache;
     Optional<size_t> mCachedPixelExecutableIndex;
 
     AttribIndexArray mAttribLocationToD3DSemantic;
@@ -608,7 +614,7 @@ class ProgramD3D : public ProgramImpl
     static unsigned int issueSerial();
     static unsigned int mCurrentSerial;
 
-    Serial mCurrentVertexArrayStateSerial;
+    UniqueSerial mCurrentVertexArrayStateSerial;
 };
 }  // namespace rx
 
