@@ -114,8 +114,6 @@ void ShaderD3D::uncompile()
     // set by compileToHLSL
     mCompilerOutputType = SH_ESSL_OUTPUT;
 
-    mUsesClipDistance            = false;
-    mUsesCullDistance            = false;
     mUsesMultipleRenderTargets   = false;
     mUsesFragColor               = false;
     mUsesFragData                = false;
@@ -125,14 +123,20 @@ void ShaderD3D::uncompile()
     mUsesHelperInvocation        = false;
     mUsesPointSize               = false;
     mUsesPointCoord              = false;
+    mUsesSampleID                = false;
+    mUsesSamplePosition          = false;
+    mUsesSampleMaskIn            = false;
+    mUsesSampleMask              = false;
     mUsesDepthRange              = false;
-    mUsesFragDepth               = false;
     mHasANGLEMultiviewEnabled    = false;
     mUsesVertexID                = false;
     mUsesViewID                  = false;
     mUsesDiscardRewriting        = false;
     mUsesNestedBreak             = false;
     mRequiresIEEEStrictCompiling = false;
+    mFragDepthUsage              = FragDepthUsage::Unused;
+    mClipDistanceSize            = 0;
+    mCullDistanceSize            = 0;
 
     mDebugInfo.clear();
 }
@@ -291,12 +295,7 @@ std::shared_ptr<WaitableCompileEvent> ShaderD3D::compile(const gl::Context *cont
     }
     if (extensions.shaderPixelLocalStorageANGLE)
     {
-        options->pls.type = mRenderer->getNativePixelLocalStorageType();
-        if (extensions.shaderPixelLocalStorageCoherentANGLE)
-        {
-            options->pls.fragmentSynchronizationType =
-                ShFragmentSynchronizationType::RasterizerOrderViews_D3D;
-        }
+        options->pls = mRenderer->getNativePixelLocalStorageOptions();
     }
 
     auto postTranslateFunctor = [this](gl::ShCompilerInstance *compiler, std::string *infoLog) {
@@ -305,20 +304,21 @@ std::shared_ptr<WaitableCompileEvent> ShaderD3D::compile(const gl::Context *cont
 
         const std::string &translatedSource = mState.getTranslatedSource();
 
-        mUsesClipDistance = translatedSource.find("GL_USES_CLIP_DISTANCE") != std::string::npos;
-        mUsesCullDistance = translatedSource.find("GL_USES_CULL_DISTANCE") != std::string::npos;
         mUsesMultipleRenderTargets = translatedSource.find("GL_USES_MRT") != std::string::npos;
         mUsesFragColor      = translatedSource.find("GL_USES_FRAG_COLOR") != std::string::npos;
         mUsesFragData       = translatedSource.find("GL_USES_FRAG_DATA") != std::string::npos;
         mUsesSecondaryColor = translatedSource.find("GL_USES_SECONDARY_COLOR") != std::string::npos;
         mUsesFragCoord      = translatedSource.find("GL_USES_FRAG_COORD") != std::string::npos;
         mUsesFrontFacing    = translatedSource.find("GL_USES_FRONT_FACING") != std::string::npos;
+        mUsesSampleID       = translatedSource.find("GL_USES_SAMPLE_ID") != std::string::npos;
+        mUsesSamplePosition = translatedSource.find("GL_USES_SAMPLE_POSITION") != std::string::npos;
+        mUsesSampleMaskIn   = translatedSource.find("GL_USES_SAMPLE_MASK_IN") != std::string::npos;
+        mUsesSampleMask     = translatedSource.find("GL_USES_SAMPLE_MASK_OUT") != std::string::npos;
         mUsesHelperInvocation =
             translatedSource.find("GL_USES_HELPER_INVOCATION") != std::string::npos;
         mUsesPointSize  = translatedSource.find("GL_USES_POINT_SIZE") != std::string::npos;
         mUsesPointCoord = translatedSource.find("GL_USES_POINT_COORD") != std::string::npos;
         mUsesDepthRange = translatedSource.find("GL_USES_DEPTH_RANGE") != std::string::npos;
-        mUsesFragDepth  = translatedSource.find("GL_USES_FRAG_DEPTH") != std::string::npos;
         mHasANGLEMultiviewEnabled =
             translatedSource.find("GL_ANGLE_MULTIVIEW_ENABLED") != std::string::npos;
         mUsesVertexID = translatedSource.find("GL_USES_VERTEX_ID") != std::string::npos;
@@ -331,6 +331,20 @@ std::shared_ptr<WaitableCompileEvent> ShaderD3D::compile(const gl::Context *cont
 
         ShHandle compilerHandle = compiler->getHandle();
 
+        if (translatedSource.find("GL_USES_FRAG_DEPTH_GREATER") != std::string::npos)
+        {
+            mFragDepthUsage = FragDepthUsage::Greater;
+        }
+        else if (translatedSource.find("GL_USES_FRAG_DEPTH_LESS") != std::string::npos)
+        {
+            mFragDepthUsage = FragDepthUsage::Less;
+        }
+        else if (translatedSource.find("GL_USES_FRAG_DEPTH") != std::string::npos)
+        {
+            mFragDepthUsage = FragDepthUsage::Any;
+        }
+        mClipDistanceSize   = sh::GetClipDistanceArraySize(compilerHandle);
+        mCullDistanceSize   = sh::GetCullDistanceArraySize(compilerHandle);
         mUniformRegisterMap = GetUniformRegisterMap(sh::GetUniformRegisterMap(compilerHandle));
         mReadonlyImage2DRegisterIndex = sh::GetReadonlyImage2DRegisterIndex(compilerHandle);
         mImage2DRegisterIndex         = sh::GetImage2DRegisterIndex(compilerHandle);
