@@ -425,15 +425,18 @@ angle::Result Context9::popDebugGroup(const gl::Context *context)
 angle::Result Context9::syncState(const gl::Context *context,
                                   const gl::State::DirtyBits &dirtyBits,
                                   const gl::State::DirtyBits &bitMask,
+                                  const gl::State::ExtendedDirtyBits &extendedDirtyBits,
+                                  const gl::State::ExtendedDirtyBits &extendedBitMask,
                                   gl::Command command)
 {
-    mRenderer->getStateManager()->syncState(mState, dirtyBits);
+    mRenderer->getStateManager()->syncState(mState, dirtyBits, extendedDirtyBits);
     return angle::Result::Continue;
 }
 
 GLint Context9::getGPUDisjoint()
 {
-    return mRenderer->getGPUDisjoint();
+    // Disjoint timer queries are not supported.
+    return false;
 }
 
 GLint64 Context9::getTimestamp()
@@ -467,9 +470,9 @@ const gl::Limitations &Context9::getNativeLimitations() const
     return mRenderer->getNativeLimitations();
 }
 
-ShPixelLocalStorageType Context9::getNativePixelLocalStorageType() const
+const ShPixelLocalStorageOptions &Context9::getNativePixelLocalStorageOptions() const
 {
-    return mRenderer->getNativePixelLocalStorageType();
+    return mRenderer->getNativePixelLocalStorageOptions();
 }
 
 angle::Result Context9::dispatchCompute(const gl::Context *context,
