@@ -1327,8 +1327,17 @@ angle::Result FramebufferGL::syncState(const gl::Context *context,
                                                  mState.getDefaultLayers());
                 break;
             case Framebuffer::DIRTY_BIT_FLIP_Y:
-                functions->framebufferParameteri(GL_FRAMEBUFFER, GL_FRAMEBUFFER_FLIP_Y_MESA,
-                                                 gl::ConvertToGLBoolean(mState.getFlipY()));
+                ASSERT(functions->framebufferParameteri || functions->framebufferParameteriMESA);
+                if (functions->framebufferParameteri)
+                {
+                    functions->framebufferParameteri(GL_FRAMEBUFFER, GL_FRAMEBUFFER_FLIP_Y_MESA,
+                                                     gl::ConvertToGLBoolean(mState.getFlipY()));
+                }
+                else
+                {
+                    functions->framebufferParameteriMESA(GL_FRAMEBUFFER, GL_FRAMEBUFFER_FLIP_Y_MESA,
+                                                         gl::ConvertToGLBoolean(mState.getFlipY()));
+                }
                 break;
             default:
             {
@@ -1388,6 +1397,12 @@ void FramebufferGL::syncClearState(const gl::Context *context, GLbitfield mask)
     StateManagerGL *stateManager      = GetStateManagerGL(context);
     const angle::FeaturesGL &features = GetFeaturesGL(context);
 
+    // Clip origin must not affect scissor box but some drivers flip it for clear ops.
+    if (context->getState().isScissorTestEnabled())
+    {
+        stateManager->setClipControl(ClipOrigin::LowerLeft, ClipDepthMode::NegativeOneToOne);
+    }
+
     if (features.doesSRGBClearsOnLinearFramebufferAttachments.enabled &&
         (mask & GL_COLOR_BUFFER_BIT) != 0 && !isDefault())
     {
@@ -1416,6 +1431,12 @@ void FramebufferGL::syncClearBufferState(const gl::Context *context,
     StateManagerGL *stateManager      = GetStateManagerGL(context);
     const angle::FeaturesGL &features = GetFeaturesGL(context);
 
+    // Clip origin must not affect scissor box but some drivers flip it for clear ops.
+    if (context->getState().isScissorTestEnabled())
+    {
+        stateManager->setClipControl(ClipOrigin::LowerLeft, ClipDepthMode::NegativeOneToOne);
+    }
+
     if (features.doesSRGBClearsOnLinearFramebufferAttachments.enabled && buffer == GL_COLOR &&
         !isDefault())
     {
