@@ -112,6 +112,12 @@ ContextNULL::ContextNULL(const gl::State &state,
     mCaps = GenerateMinimumCaps(maxClientVersion, mExtensions);
 
     InitMinimumTextureCapsMap(maxClientVersion, mExtensions, &mTextureCaps);
+
+    if (mExtensions.shaderPixelLocalStorageANGLE)
+    {
+        mPLSOptions.type             = ShPixelLocalStorageType::FramebufferFetch;
+        mPLSOptions.fragmentSyncType = ShFragmentSynchronizationType::Automatic;
+    }
 }
 
 ContextNULL::~ContextNULL() {}
@@ -368,6 +374,8 @@ angle::Result ContextNULL::popDebugGroup(const gl::Context *context)
 angle::Result ContextNULL::syncState(const gl::Context *context,
                                      const gl::State::DirtyBits &dirtyBits,
                                      const gl::State::DirtyBits &bitMask,
+                                     const gl::State::ExtendedDirtyBits &extendedDirtyBits,
+                                     const gl::State::ExtendedDirtyBits &extendedBitMask,
                                      gl::Command command)
 {
     return angle::Result::Continue;
@@ -408,11 +416,9 @@ const gl::Limitations &ContextNULL::getNativeLimitations() const
     return mLimitations;
 }
 
-ShPixelLocalStorageType ContextNULL::getNativePixelLocalStorageType() const
+const ShPixelLocalStorageOptions &ContextNULL::getNativePixelLocalStorageOptions() const
 {
-    return getNativeExtensions().shaderPixelLocalStorageANGLE
-               ? ShPixelLocalStorageType::FramebufferFetch
-               : ShPixelLocalStorageType::NotSupported;
+    return mPLSOptions;
 }
 
 CompilerImpl *ContextNULL::createCompiler()
