@@ -9,11 +9,9 @@
 #include "libANGLE/renderer/vulkan/ProgramExecutableVk.h"
 
 #include "common/string_utils.h"
-#include "libANGLE/renderer/glslang_wrapper_utils.h"
 #include "libANGLE/renderer/vulkan/BufferVk.h"
 #include "libANGLE/renderer/vulkan/DisplayVk.h"
 #include "libANGLE/renderer/vulkan/FramebufferVk.h"
-#include "libANGLE/renderer/vulkan/GlslangWrapperVk.h"
 #include "libANGLE/renderer/vulkan/ProgramPipelineVk.h"
 #include "libANGLE/renderer/vulkan/ProgramVk.h"
 #include "libANGLE/renderer/vulkan/TextureVk.h"
@@ -66,7 +64,8 @@ void SaveShaderInterfaceVariableXfbInfo(const ShaderInterfaceVariableXfbInfo &xf
     }
 }
 
-bool ValidateTransformedSpirV(const gl::ShaderBitSet &linkedShaderStages,
+bool ValidateTransformedSpirV(const ContextVk *contextVk,
+                              const gl::ShaderBitSet &linkedShaderStages,
                               const ShaderInterfaceVariableInfoMap &variableInfoMap,
                               const gl::ShaderMap<angle::spirv::Blob> &spirvBlobs)
 {
@@ -74,16 +73,18 @@ bool ValidateTransformedSpirV(const gl::ShaderBitSet &linkedShaderStages,
 
     for (gl::ShaderType shaderType : linkedShaderStages)
     {
-        GlslangSpirvOptions options;
+        SpvTransformOptions options;
         options.shaderType                = shaderType;
         options.negativeViewportSupported = false;
         options.removeDebugInfo           = true;
         options.isLastPreFragmentStage    = shaderType == lastPreFragmentStage;
         options.isTransformFeedbackStage  = shaderType == lastPreFragmentStage;
+        options.useSpirvVaryingPrecisionFixer =
+            contextVk->getFeatures().varyingsRequireMatchingPrecisionInSpirv.enabled;
 
         angle::spirv::Blob transformed;
-        if (GlslangWrapperVk::TransformSpirV(options, variableInfoMap, spirvBlobs[shaderType],
-                                             &transformed) != angle::Result::Continue)
+        if (SpvTransformSpirvCode(options, variableInfoMap, spirvBlobs[shaderType], &transformed) !=
+            angle::Result::Continue)
         {
             return false;
         }
@@ -133,6 +134,9 @@ void SetupDefaultPipelineState(const ContextVk *contextVk,
     graphicsPipelineDescOut->setRenderPassSampleCount(1);
     graphicsPipelineDescOut->setRenderPassFramebufferFetchMode(glExecutable.usesFramebufferFetch());
 
+    graphicsPipelineDescOut->setVertexShaderComponentTypes(
+        glExecutable.getNonBuiltinAttribLocationsMask(), glExecutable.getAttributesTypeMask());
+
     const std::vector<sh::ShaderVariable> &outputVariables   = glExecutable.getOutputVariables();
     const std::vector<gl::VariableLocation> &outputLocations = glExecutable.getOutputLocations();
 
@@ -187,8 +191,10 @@ void GetPipelineCacheData(ContextVk *contextVk,
                           angle::MemoryBuffer *cacheDataOut)
 {
     ASSERT(pipelineCache.valid() || contextVk->getState().isGLES1() ||
-           !contextVk->getFeatures().warmUpPipelineCacheAtLink.enabled);
-    if (!pipelineCache.valid())
+           !contextVk->getFeatures().warmUpPipelineCacheAtLink.enabled ||
+           !contextVk->getFeatures().hasEffectivePipelineCacheSerialization.enabled);
+    if (!pipelineCache.valid() ||
+        !contextVk->getFeatures().hasEffectivePipelineCacheSerialization.enabled)
     {
         return;
     }
@@ -217,6 +223,17 @@ void GetPipelineCacheData(ContextVk *contextVk,
         cacheDataOut->clear();
     }
 }
+
+vk::SpecializationConstants MakeSpecConsts(ProgramTransformOptions transformOptions,
+                                           const vk::GraphicsPipelineDesc &desc)
+{
+    vk::SpecializationConstants specConsts;
+
+    specConsts.surfaceRotation = transformOptions.surfaceRotation;
+    specConsts.dither          = desc.getEmulatedDitherControl();
+
+    return specConsts;
+}
 }  // namespace
 
 DefaultUniformBlock::DefaultUniformBlock() = default;
@@ -248,7 +265,8 @@ angle::Result ShaderInfo::initShaders(ContextVk *contextVk,
     // will naturally be validated.  This improves GLES1 test run times.
     if (!contextVk->getState().isGLES1())
     {
-        ASSERT(ValidateTransformedSpirV(linkedShaderStages, variableInfoMap, mSpirvBlobs));
+        ASSERT(
+            ValidateTransformedSpirV(contextVk, linkedShaderStages, variableInfoMap, mSpirvBlobs));
     }
 
     mIsInitialized = true;
@@ -319,7 +337,7 @@ angle::Result ProgramInfo::initProgram(ContextVk *contextVk,
     gl::ShaderMap<angle::spirv::Blob> transformedSpirvBlobs;
     angle::spirv::Blob &transformedSpirvBlob = transformedSpirvBlobs[shaderType];
 
-    GlslangSpirvOptions options;
+    SpvTransformOptions options;
     options.shaderType               = shaderType;
     options.removeDebugInfo          = !contextVk->getFeatures().retainSPIRVDebugInfo.enabled;
     options.isLastPreFragmentStage   = isLastPreFragmentStage;
@@ -337,8 +355,11 @@ angle::Result ProgramInfo::initProgram(ContextVk *contextVk,
     options.validate =
         !(contextVk->getState().isGLES1() && contextVk->getRenderer()->getEnableValidationLayers());
 
-    ANGLE_TRY(GlslangWrapperVk::TransformSpirV(options, variableInfoMap, originalSpirvBlob,
-                                               &transformedSpirvBlob));
+    options.useSpirvVaryingPrecisionFixer =
+        contextVk->getFeatures().varyingsRequireMatchingPrecisionInSpirv.enabled;
+
+    ANGLE_TRY(
+        SpvTransformSpirvCode(options, variableInfoMap, originalSpirvBlob, &transformedSpirvBlob));
     ANGLE_TRY(vk::InitShaderModule(contextVk, &mShaders[shaderType].get(),
                                    transformedSpirvBlob.data(),
                                    transformedSpirvBlob.size() * sizeof(uint32_t)));
@@ -383,7 +404,6 @@ void ProgramExecutableVk::resetLayout(ContextVk *contextVk)
     }
     mImmutableSamplersMaxDescriptorCount = 1;
     mImmutableSamplerIndexMap.clear();
-    mPipelineLayout.reset();
 
     mDescriptorSets.fill(VK_NULL_HANDLE);
     mNumDefaultUniformDescriptors = 0;
@@ -401,21 +421,29 @@ void ProgramExecutableVk::resetLayout(ContextVk *contextVk)
     // Initialize with an invalid BufferSerial
     mCurrentDefaultUniformBufferSerial = vk::BufferSerial();
 
-    for (ProgramInfo &programInfo : mGraphicsProgramInfos)
-    {
-        programInfo.release(contextVk);
-    }
-    mComputeProgramInfo.release(contextVk);
-
     for (CompleteGraphicsPipelineCache &pipelines : mCompleteGraphicsPipelines)
     {
         pipelines.release(contextVk);
     }
+    for (ShadersGraphicsPipelineCache &pipelines : mShadersGraphicsPipelines)
+    {
+        pipelines.release(contextVk);
+    }
     for (vk::PipelineHelper &pipeline : mComputePipelines)
     {
         pipeline.release(contextVk);
     }
 
+    // Program infos and pipeline layout must be released after pipelines are; they might be having
+    // pending jobs that are referencing them.
+    for (ProgramInfo &programInfo : mGraphicsProgramInfos)
+    {
+        programInfo.release(contextVk);
+    }
+    mComputeProgramInfo.release(contextVk);
+
+    mPipelineLayout.reset();
+
     contextVk->onProgramExecutableReset(this);
 }
 
@@ -455,9 +483,32 @@ angle::Result ProgramExecutableVk::initializePipelineCache(
     ANGLE_VK_TRY(contextVk, mPipelineCache.init(contextVk->getDevice(), pipelineCacheCreateInfo));
 
     // Merge the pipeline cache into RendererVk's.
-    // TODO(http://anglebug.com/7369): When VK_EXT_graphics_pipeline_library is supported, do not
-    // merge into RendererVk's cache.  |mPipelineCache| will continue to be used in that case.
-    return contextVk->getRenderer()->mergeIntoPipelineCache(mPipelineCache);
+    if (contextVk->getFeatures().mergeProgramPipelineCachesToGlobalCache.enabled)
+    {
+        ANGLE_TRY(contextVk->getRenderer()->mergeIntoPipelineCache(mPipelineCache));
+    }
+
+    return angle::Result::Continue;
+}
+
+angle::Result ProgramExecutableVk::ensurePipelineCacheInitialized(ContextVk *contextVk)
+{
+    if (!mPipelineCache.valid())
+    {
+        VkPipelineCacheCreateInfo pipelineCacheCreateInfo = {};
+        pipelineCacheCreateInfo.sType = VK_STRUCTURE_TYPE_PIPELINE_CACHE_CREATE_INFO;
+
+        if (contextVk->getFeatures().supportsPipelineCreationCacheControl.enabled)
+        {
+            pipelineCacheCreateInfo.flags |=
+                VK_PIPELINE_CACHE_CREATE_EXTERNALLY_SYNCHRONIZED_BIT_EXT;
+        }
+
+        ANGLE_VK_TRY(contextVk,
+                     mPipelineCache.init(contextVk->getDevice(), pipelineCacheCreateInfo));
+    }
+
+    return angle::Result::Continue;
 }
 
 std::unique_ptr<rx::LinkEvent> ProgramExecutableVk::load(ContextVk *contextVk,
@@ -500,8 +551,9 @@ std::unique_ptr<rx::LinkEvent> ProgramExecutableVk::load(ContextVk *contextVk,
                 {
                     LoadShaderInterfaceVariableXfbInfo(stream, &xfb);
                 }
-                info.builtinIsInput          = stream->readBool();
-                info.builtinIsOutput         = stream->readBool();
+                info.useRelaxedPrecision     = stream->readBool();
+                info.varyingIsInput          = stream->readBool();
+                info.varyingIsOutput         = stream->readBool();
                 info.attributeComponentCount = stream->readInt<uint8_t>();
                 info.attributeLocationCount  = stream->readInt<uint8_t>();
                 info.isDuplicate             = stream->readBool();
@@ -616,8 +668,9 @@ void ProgramExecutableVk::save(ContextVk *contextVk,
                 {
                     SaveShaderInterfaceVariableXfbInfo(xfb, stream);
                 }
-                stream->writeBool(info.builtinIsInput);
-                stream->writeBool(info.builtinIsOutput);
+                stream->writeBool(info.useRelaxedPrecision);
+                stream->writeBool(info.varyingIsInput);
+                stream->writeBool(info.varyingIsOutput);
                 stream->writeInt(info.attributeComponentCount);
                 stream->writeInt(info.attributeLocationCount);
                 stream->writeBool(info.isDuplicate);
@@ -697,22 +750,11 @@ angle::Result ProgramExecutableVk::warmUpPipelineCache(ContextVk *contextVk,
         return angle::Result::Continue;
     }
 
-    // Initialize the pipeline cache
-    ASSERT(!mPipelineCache.valid());
-
-    VkPipelineCacheCreateInfo pipelineCacheCreateInfo = {};
-    pipelineCacheCreateInfo.sType = VK_STRUCTURE_TYPE_PIPELINE_CACHE_CREATE_INFO;
-
-    if (contextVk->getFeatures().supportsPipelineCreationCacheControl.enabled)
-    {
-        pipelineCacheCreateInfo.flags |= VK_PIPELINE_CACHE_CREATE_EXTERNALLY_SYNCHRONIZED_BIT_EXT;
-    }
-
-    ANGLE_VK_TRY(contextVk, mPipelineCache.init(contextVk->getDevice(), pipelineCacheCreateInfo));
+    ANGLE_TRY(ensurePipelineCacheInitialized(contextVk));
 
     // No synchronization necessary when accessing the program executable's cache as there is no
     // access to it from other threads at this point.
-    PipelineCacheAccess pipelineCache;
+    vk::PipelineCacheAccess pipelineCache;
     pipelineCache.init(&mPipelineCache, nullptr);
 
     // Create a set of pipelines.  Ideally, that would be the entire set of possible pipelines so
@@ -728,7 +770,12 @@ angle::Result ProgramExecutableVk::warmUpPipelineCache(ContextVk *contextVk,
                                              glExecutable, &pipeline));
 
         // Merge the cache with RendererVk's
-        return contextVk->getRenderer()->mergeIntoPipelineCache(mPipelineCache);
+        if (contextVk->getFeatures().mergeProgramPipelineCachesToGlobalCache.enabled)
+        {
+            ANGLE_TRY(contextVk->getRenderer()->mergeIntoPipelineCache(mPipelineCache));
+        }
+
+        return angle::Result::Continue;
     }
 
     const vk::GraphicsPipelineDesc *descPtr = nullptr;
@@ -762,17 +809,30 @@ angle::Result ProgramExecutableVk::warmUpPipelineCache(ContextVk *contextVk,
         surfaceRotationVariations.push_back(true);
     }
 
+    // Only build the shaders subset of the pipeline if VK_EXT_graphics_pipeline_library is
+    // supported, especially since the vertex input and fragment output state set up here is
+    // completely bogus.
+    vk::GraphicsPipelineSubset subset =
+        contextVk->getFeatures().supportsGraphicsPipelineLibrary.enabled
+            ? vk::GraphicsPipelineSubset::Shaders
+            : vk::GraphicsPipelineSubset::Complete;
+
     for (bool rotation : surfaceRotationVariations)
     {
         transformOptions.surfaceRotation = rotation;
 
-        ANGLE_TRY(createGraphicsPipelineImpl(
-            contextVk, transformOptions, vk::GraphicsPipelineSubset::Complete, &pipelineCache,
-            PipelineSource::WarmUp, graphicsPipelineDesc, glExecutable, &descPtr, &pipeline));
+        ANGLE_TRY(createGraphicsPipelineImpl(contextVk, transformOptions, subset, &pipelineCache,
+                                             PipelineSource::WarmUp, graphicsPipelineDesc,
+                                             glExecutable, &descPtr, &pipeline));
     }
 
     // Merge the cache with RendererVk's
-    return contextVk->getRenderer()->mergeIntoPipelineCache(mPipelineCache);
+    if (contextVk->getFeatures().mergeProgramPipelineCachesToGlobalCache.enabled)
+    {
+        ANGLE_TRY(contextVk->getRenderer()->mergeIntoPipelineCache(mPipelineCache));
+    }
+
+    return angle::Result::Continue;
 }
 
 void ProgramExecutableVk::addInterfaceBlockDescriptorSetDesc(
@@ -1084,7 +1144,7 @@ angle::Result ProgramExecutableVk::createGraphicsPipelineImpl(
     ContextVk *contextVk,
     ProgramTransformOptions transformOptions,
     vk::GraphicsPipelineSubset pipelineSubset,
-    PipelineCacheAccess *pipelineCache,
+    vk::PipelineCacheAccess *pipelineCache,
     PipelineSource source,
     const vk::GraphicsPipelineDesc &desc,
     const gl::ProgramExecutable &glExecutable,
@@ -1099,19 +1159,31 @@ angle::Result ProgramExecutableVk::createGraphicsPipelineImpl(
 
     // Set specialization constants.  These are also a part of GraphicsPipelineDesc, so that a
     // change in specialization constants also results in a new pipeline.
-    vk::SpecializationConstants specConsts;
-    specConsts.surfaceRotation = transformOptions.surfaceRotation;
-    specConsts.dither          = desc.getEmulatedDitherControl();
+    vk::SpecializationConstants specConsts = MakeSpecConsts(transformOptions, desc);
 
     // Pull in a compatible RenderPass.
-    vk::RenderPass *compatibleRenderPass = nullptr;
+    const vk::RenderPass *compatibleRenderPass = nullptr;
     ANGLE_TRY(contextVk->getRenderPassCache().getCompatibleRenderPass(
         contextVk, desc.getRenderPassDesc(), &compatibleRenderPass));
 
-    CompleteGraphicsPipelineCache &pipelines = mCompleteGraphicsPipelines[programIndex];
-    return shaderProgram->createGraphicsPipeline(contextVk, &pipelines, pipelineCache,
-                                                 *compatibleRenderPass, getPipelineLayout(), source,
-                                                 desc, specConsts, descPtrOut, pipelineOut);
+    if (pipelineSubset == vk::GraphicsPipelineSubset::Complete)
+    {
+        CompleteGraphicsPipelineCache &pipelines = mCompleteGraphicsPipelines[programIndex];
+        return shaderProgram->createGraphicsPipeline(
+            contextVk, &pipelines, pipelineCache, *compatibleRenderPass, getPipelineLayout(),
+            source, desc, specConsts, descPtrOut, pipelineOut);
+    }
+    else
+    {
+        // Vertex input and fragment output subsets are independent of shaders, and are not created
+        // through the program executable.
+        ASSERT(pipelineSubset == vk::GraphicsPipelineSubset::Shaders);
+
+        ShadersGraphicsPipelineCache &pipelines = mShadersGraphicsPipelines[programIndex];
+        return shaderProgram->createGraphicsPipeline(
+            contextVk, &pipelines, pipelineCache, *compatibleRenderPass, getPipelineLayout(),
+            source, desc, specConsts, descPtrOut, pipelineOut);
+    }
 }
 
 angle::Result ProgramExecutableVk::getGraphicsPipeline(ContextVk *contextVk,
@@ -1132,7 +1204,18 @@ angle::Result ProgramExecutableVk::getGraphicsPipeline(ContextVk *contextVk,
     *descPtrOut  = nullptr;
     *pipelineOut = nullptr;
 
-    mCompleteGraphicsPipelines[programIndex].getPipeline(desc, descPtrOut, pipelineOut);
+    if (pipelineSubset == vk::GraphicsPipelineSubset::Complete)
+    {
+        mCompleteGraphicsPipelines[programIndex].getPipeline(desc, descPtrOut, pipelineOut);
+    }
+    else
+    {
+        // Vertex input and fragment output subsets are independent of shaders, and are not created
+        // through the program executable.
+        ASSERT(pipelineSubset == vk::GraphicsPipelineSubset::Shaders);
+
+        mShadersGraphicsPipelines[programIndex].getPipeline(desc, descPtrOut, pipelineOut);
+    }
 
     return angle::Result::Continue;
 }
@@ -1140,7 +1223,7 @@ angle::Result ProgramExecutableVk::getGraphicsPipeline(ContextVk *contextVk,
 angle::Result ProgramExecutableVk::createGraphicsPipeline(
     ContextVk *contextVk,
     vk::GraphicsPipelineSubset pipelineSubset,
-    PipelineCacheAccess *pipelineCache,
+    vk::PipelineCacheAccess *pipelineCache,
     PipelineSource source,
     const vk::GraphicsPipelineDesc &desc,
     const gl::ProgramExecutable &glExecutable,
@@ -1149,13 +1232,66 @@ angle::Result ProgramExecutableVk::createGraphicsPipeline(
 {
     ProgramTransformOptions transformOptions = getTransformOptions(contextVk, desc, glExecutable);
 
-    return createGraphicsPipelineImpl(contextVk, transformOptions, pipelineSubset, pipelineCache,
-                                      source, desc, glExecutable, descPtrOut, pipelineOut);
+    // When creating monolithic pipelines, the renderer's pipeline cache is used as passed in.
+    // When creating the shaders subset of pipelines, the program's own pipeline cache is used.
+    vk::PipelineCacheAccess perProgramPipelineCache;
+    const bool useProgramPipelineCache = pipelineSubset == vk::GraphicsPipelineSubset::Shaders;
+    if (useProgramPipelineCache)
+    {
+        ANGLE_TRY(ensurePipelineCacheInitialized(contextVk));
+
+        perProgramPipelineCache.init(&mPipelineCache, nullptr);
+        pipelineCache = &perProgramPipelineCache;
+    }
+
+    ANGLE_TRY(createGraphicsPipelineImpl(contextVk, transformOptions, pipelineSubset, pipelineCache,
+                                         source, desc, glExecutable, descPtrOut, pipelineOut));
+
+    if (useProgramPipelineCache &&
+        contextVk->getFeatures().mergeProgramPipelineCachesToGlobalCache.enabled)
+    {
+        ANGLE_TRY(contextVk->getRenderer()->mergeIntoPipelineCache(mPipelineCache));
+    }
+
+    return angle::Result::Continue;
+}
+
+angle::Result ProgramExecutableVk::linkGraphicsPipelineLibraries(
+    ContextVk *contextVk,
+    vk::PipelineCacheAccess *pipelineCache,
+    const vk::GraphicsPipelineDesc &desc,
+    const gl::ProgramExecutable &glExecutable,
+    vk::PipelineHelper *vertexInputPipeline,
+    vk::PipelineHelper *shadersPipeline,
+    vk::PipelineHelper *fragmentOutputPipeline,
+    const vk::GraphicsPipelineDesc **descPtrOut,
+    vk::PipelineHelper **pipelineOut)
+{
+    ProgramTransformOptions transformOptions = getTransformOptions(contextVk, desc, glExecutable);
+    const uint8_t programIndex               = GetGraphicsProgramIndex(transformOptions);
+
+    ANGLE_TRY(mCompleteGraphicsPipelines[programIndex].linkLibraries(
+        contextVk, pipelineCache, desc, getPipelineLayout(), vertexInputPipeline, shadersPipeline,
+        fragmentOutputPipeline, descPtrOut, pipelineOut));
+
+    // If monolithic pipelines are preferred over libraries, create a task so that it can be created
+    // asynchronously.
+    if (contextVk->getFeatures().preferMonolithicPipelinesOverLibraries.enabled)
+    {
+        vk::SpecializationConstants specConsts = MakeSpecConsts(transformOptions, desc);
+
+        mGraphicsProgramInfos[programIndex]
+            .getShaderProgram()
+            ->createMonolithicPipelineCreationTask(contextVk, pipelineCache, desc,
+                                                   getPipelineLayout(), specConsts, *pipelineOut);
+    }
+
+    return angle::Result::Continue;
 }
 
 angle::Result ProgramExecutableVk::getOrCreateComputePipeline(
     ContextVk *contextVk,
-    PipelineCacheAccess *pipelineCache,
+    vk::PipelineCacheAccess *pipelineCache,
     PipelineSource source,
     const gl::ProgramExecutable &glExecutable,
     vk::PipelineHelper **pipelineOut)
@@ -1241,9 +1377,8 @@ angle::Result ProgramExecutableVk::createPipelineLayout(
 
     // Decide if we should use dynamic or fixed descriptor types.
     VkPhysicalDeviceLimits limits = contextVk->getRenderer()->getPhysicalDeviceProperties().limits;
-    uint32_t totalDynamicUniformBufferCount = numActiveUniformBufferDescriptors +
-                                              mNumDefaultUniformDescriptors +
-                                              kReservedDriverUniformBindingCount;
+    uint32_t totalDynamicUniformBufferCount =
+        numActiveUniformBufferDescriptors + mNumDefaultUniformDescriptors;
     if (totalDynamicUniformBufferCount <= limits.maxDescriptorSetUniformBuffersDynamic)
     {
         mUniformBufferDescriptorType = VK_DESCRIPTOR_TYPE_UNIFORM_BUFFER_DYNAMIC;
@@ -1331,6 +1466,47 @@ angle::Result ProgramExecutableVk::createPipelineLayout(
     return angle::Result::Continue;
 }
 
+void ProgramExecutableVk::resolvePrecisionMismatch(const gl::ProgramMergedVaryings &mergedVaryings)
+{
+    for (const gl::ProgramVaryingRef &mergedVarying : mergedVaryings)
+    {
+        if (!mergedVarying.frontShader || !mergedVarying.backShader)
+        {
+            continue;
+        }
+
+        GLenum frontPrecision = mergedVarying.frontShader->precision;
+        GLenum backPrecision  = mergedVarying.backShader->precision;
+        if (frontPrecision == backPrecision)
+        {
+            continue;
+        }
+
+        ASSERT(frontPrecision >= GL_LOW_FLOAT && frontPrecision <= GL_HIGH_INT);
+        ASSERT(backPrecision >= GL_LOW_FLOAT && backPrecision <= GL_HIGH_INT);
+
+        if (frontPrecision > backPrecision)
+        {
+            // The output is higher precision than the input
+            ShaderInterfaceVariableInfo &info = mVariableInfoMap.getMutable(
+                mergedVarying.frontShaderStage, ShaderVariableType::Varying,
+                mergedVarying.frontShader->mappedName);
+            info.varyingIsOutput     = true;
+            info.useRelaxedPrecision = true;
+        }
+        else
+        {
+            // The output is lower precision than the input, adjust the input
+            ASSERT(backPrecision > frontPrecision);
+            ShaderInterfaceVariableInfo &info = mVariableInfoMap.getMutable(
+                mergedVarying.backShaderStage, ShaderVariableType::Varying,
+                mergedVarying.backShader->mappedName);
+            info.varyingIsInput      = true;
+            info.useRelaxedPrecision = true;
+        }
+    }
+}
+
 angle::Result ProgramExecutableVk::getOrAllocateDescriptorSet(
     vk::Context *context,
     UpdateDescriptorSetsBuilder *updateBuilder,
@@ -1348,7 +1524,7 @@ angle::Result ProgramExecutableVk::getOrAllocateDescriptorSet(
     if (*newSharedCacheKeyOut != nullptr)
     {
         // Cache miss. A new cache entry has been created.
-        descriptorSetDesc.updateDescriptorSet(updateBuilder, mDescriptorSets[setIndex]);
+        descriptorSetDesc.updateDescriptorSet(context, updateBuilder, mDescriptorSets[setIndex]);
     }
     else
     {
@@ -1430,7 +1606,8 @@ angle::Result ProgramExecutableVk::updateTexturesDescriptorSet(
         ANGLE_TRY(fullDesc.updateFullActiveTextures(context, mVariableInfoMap, executable, textures,
                                                     samplers, emulateSeamfulCubeMapSampling,
                                                     pipelineType, newSharedCacheKey));
-        fullDesc.updateDescriptorSet(updateBuilder, mDescriptorSets[DescriptorSetIndex::Texture]);
+        fullDesc.updateDescriptorSet(context, updateBuilder,
+                                     mDescriptorSets[DescriptorSetIndex::Texture]);
     }
     else
     {
