@@ -73,8 +73,9 @@ angle::Result RenderbufferVk::setStorageImpl(const gl::Context *context,
 
     if (mImage == nullptr)
     {
-        mImage     = new vk::ImageHelper();
-        mOwnsImage = true;
+        mImage              = new vk::ImageHelper();
+        mOwnsImage          = true;
+        mImageSiblingSerial = {};
         mImageObserverBinding.bind(mImage);
         mImageViews.init(renderer);
     }
@@ -126,7 +127,8 @@ angle::Result RenderbufferVk::setStorageImpl(const gl::Context *context,
                                    gl::LevelIndex(0), 1, 1, robustInit, false));
 
     VkMemoryPropertyFlags flags = VK_MEMORY_PROPERTY_DEVICE_LOCAL_BIT;
-    ANGLE_TRY(mImage->initMemory(contextVk, false, renderer->getMemoryProperties(), flags));
+    ANGLE_TRY(mImage->initMemory(contextVk, false, renderer->getMemoryProperties(), flags,
+                                 vk::MemoryAllocationType::RenderBufferStorageImage));
 
     // If multisampled render to texture, an implicit multisampled image is created which is used as
     // the color or depth/stencil attachment.  At the end of the render pass, this image is
@@ -140,12 +142,13 @@ angle::Result RenderbufferVk::setStorageImpl(const gl::Context *context,
             *mImage, robustInit));
 
         mRenderTarget.init(&mMultisampledImage, &mMultisampledImageViews, mImage, &mImageViews,
-                           gl::LevelIndex(0), 0, 1, RenderTargetTransience::MultisampledTransient);
+                           mImageSiblingSerial, gl::LevelIndex(0), 0, 1,
+                           RenderTargetTransience::MultisampledTransient);
     }
     else
     {
-        mRenderTarget.init(mImage, &mImageViews, nullptr, nullptr, gl::LevelIndex(0), 0, 1,
-                           RenderTargetTransience::Default);
+        mRenderTarget.init(mImage, &mImageViews, nullptr, nullptr, mImageSiblingSerial,
+                           gl::LevelIndex(0), 0, 1, RenderTargetTransience::Default);
     }
 
     return angle::Result::Continue;
@@ -178,11 +181,14 @@ angle::Result RenderbufferVk::setStorageEGLImageTarget(const gl::Context *contex
     ContextVk *contextVk = vk::GetImpl(context);
     RendererVk *renderer = contextVk->getRenderer();
 
+    ANGLE_TRY(contextVk->getShareGroup()->lockDefaultContextsPriority(contextVk));
+
     releaseAndDeleteImage(contextVk);
 
-    ImageVk *imageVk = vk::GetImpl(image);
-    mImage           = imageVk->getImage();
-    mOwnsImage       = false;
+    ImageVk *imageVk    = vk::GetImpl(image);
+    mImage              = imageVk->getImage();
+    mOwnsImage          = false;
+    mImageSiblingSerial = imageVk->generateSiblingSerial();
     mImageObserverBinding.bind(mImage);
     mImageViews.init(renderer);
 
@@ -205,8 +211,9 @@ angle::Result RenderbufferVk::setStorageEGLImageTarget(const gl::Context *contex
         ANGLE_TRY(contextVk->onEGLImageQueueChange());
     }
 
-    mRenderTarget.init(mImage, &mImageViews, nullptr, nullptr, imageVk->getImageLevel(),
-                       imageVk->getImageLayer(), 1, RenderTargetTransience::Default);
+    mRenderTarget.init(mImage, &mImageViews, nullptr, nullptr, mImageSiblingSerial,
+                       imageVk->getImageLevel(), imageVk->getImageLayer(), 1,
+                       RenderTargetTransience::Default);
 
     return angle::Result::Continue;
 }
@@ -286,6 +293,8 @@ void RenderbufferVk::releaseOwnershipOfImage(const gl::Context *context)
 {
     ContextVk *contextVk = vk::GetImpl(context);
 
+    ASSERT(!mImageSiblingSerial.valid());
+
     mOwnsImage = false;
     releaseAndDeleteImage(contextVk);
 }
@@ -314,18 +323,22 @@ void RenderbufferVk::releaseImage(ContextVk *contextVk)
 
     if (mImage && mOwnsImage)
     {
-        mImage->releaseImageFromShareContexts(renderer, contextVk);
+        mImage->releaseImageFromShareContexts(renderer, contextVk, mImageSiblingSerial);
         mImage->releaseStagedUpdates(renderer);
     }
     else
     {
+        if (mImage)
+        {
+            mImage->finalizeImageLayoutInShareContexts(renderer, contextVk, mImageSiblingSerial);
+        }
         mImage = nullptr;
         mImageObserverBinding.bind(nullptr);
     }
 
     if (mMultisampledImage.valid())
     {
-        mMultisampledImage.releaseImageFromShareContexts(renderer, contextVk);
+        mMultisampledImage.releaseImageFromShareContexts(renderer, contextVk, mImageSiblingSerial);
     }
 }
 
