@@ -26,6 +26,7 @@
 #include "libANGLE/Caps.h"
 #include "libANGLE/renderer/vulkan/CommandProcessor.h"
 #include "libANGLE/renderer/vulkan/DebugAnnotatorVk.h"
+#include "libANGLE/renderer/vulkan/MemoryTracking.h"
 #include "libANGLE/renderer/vulkan/QueryVk.h"
 #include "libANGLE/renderer/vulkan/ResourceVk.h"
 #include "libANGLE/renderer/vulkan/UtilsVk.h"
@@ -58,31 +59,6 @@ class Format;
 static constexpr size_t kMaxExtensionNames = 400;
 using ExtensionNameList                    = angle::FixedVector<const char *, kMaxExtensionNames>;
 
-// Process GPU memory reports
-class MemoryReport final : angle::NonCopyable
-{
-  public:
-    MemoryReport();
-    void processCallback(const VkDeviceMemoryReportCallbackDataEXT &callbackData, bool logCallback);
-    void logMemoryReportStats() const;
-
-  private:
-    struct MemorySizes
-    {
-        VkDeviceSize allocatedMemory;
-        VkDeviceSize allocatedMemoryMax;
-        VkDeviceSize importedMemory;
-        VkDeviceSize importedMemoryMax;
-    };
-    mutable std::mutex mMemoryReportMutex;
-    VkDeviceSize mCurrentTotalAllocatedMemory;
-    VkDeviceSize mMaxTotalAllocatedMemory;
-    angle::HashMap<VkObjectType, MemorySizes> mSizesPerType;
-    VkDeviceSize mCurrentTotalImportedMemory;
-    VkDeviceSize mMaxTotalImportedMemory;
-    angle::HashMap<uint64_t, int> mUniqueIDCounts;
-};
-
 // Information used to accurately skip known synchronization issues in ANGLE.
 struct SkippedSyncvalMessage
 {
@@ -92,30 +68,31 @@ struct SkippedSyncvalMessage
     bool isDueToNonConformantCoherentFramebufferFetch = false;
 };
 
-// Used to designate memory allocation type for tracking purposes.
-enum class MemoryAllocationType
+class ImageMemorySuballocator : angle::NonCopyable
 {
-    Unspecified    = 0,
-    Image          = 1,
-    ImageExternal  = 2,
-    Buffer         = 3,
-    BufferExternal = 4,
+  public:
+    ImageMemorySuballocator();
+    ~ImageMemorySuballocator();
 
-    InvalidEnum = 5,
-    EnumCount   = InvalidEnum,
-};
+    void destroy(RendererVk *renderer);
 
-constexpr const char *kMemoryAllocationTypeMessage[] = {
-    "Unspecified", "Image", "ImageExternal", "Buffer", "BufferExternal", "Invalid",
-};
-constexpr const uint32_t kMemoryAllocationTypeCount =
-    static_cast<uint32_t>(MemoryAllocationType::EnumCount);
+    // Allocates memory for the image and binds it.
+    VkResult allocateAndBindMemory(RendererVk *renderer,
+                                   Image *image,
+                                   VkMemoryPropertyFlags requiredFlags,
+                                   VkMemoryPropertyFlags preferredFlags,
+                                   MemoryAllocationType memoryAllocationType,
+                                   Allocation *allocationOut,
+                                   VkMemoryPropertyFlags *memoryFlagsOut,
+                                   uint32_t *memoryTypeIndexOut,
+                                   VkDeviceSize *sizeOut);
 
-// Used to select the severity for memory allocation logs.
-enum class MemoryLogSeverity
-{
-    INFO,
-    WARN,
+    // Maps the memory to initialize with non-zero value.
+    VkResult mapMemoryAndInitWithNonZeroValue(RendererVk *renderer,
+                                              Allocation *allocation,
+                                              VkDeviceSize size,
+                                              int value,
+                                              VkMemoryPropertyFlags flags);
 };
 }  // namespace vk
 
@@ -167,14 +144,16 @@ class WaitableCompressEvent
 class OneOffCommandPool : angle::NonCopyable
 {
   public:
+    OneOffCommandPool();
+    void init(vk::ProtectionType protectionType);
     angle::Result getCommandBuffer(vk::Context *context,
-                                   bool hasProtectedContent,
                                    vk::PrimaryCommandBuffer *commandBufferOut);
     void releaseCommandBuffer(const QueueSerial &submitQueueSerial,
                               vk::PrimaryCommandBuffer &&primary);
     void destroy(VkDevice device);
 
   private:
+    vk::ProtectionType mProtectionType;
     std::mutex mMutex;
     vk::CommandPool mCommandPool;
     struct PendingOneOffCommands
@@ -185,76 +164,6 @@ class OneOffCommandPool : angle::NonCopyable
     std::deque<PendingOneOffCommands> mPendingCommands;
 };
 
-// Memory tracker for allocations and deallocations, which is used in RendererVk.
-class MemoryAllocationTracker : angle::NonCopyable
-{
-  public:
-    MemoryAllocationTracker(RendererVk *renderer);
-    void initMemoryTrackers();
-
-    // Collect information regarding memory allocations and deallocations.
-    void onMemoryAllocImpl(vk::MemoryAllocationType allocType,
-                           VkDeviceSize size,
-                           uint32_t memoryTypeIndex,
-                           void *handle);
-    void onMemoryDeallocImpl(vk::MemoryAllocationType allocType,
-                             VkDeviceSize size,
-                             uint32_t memoryTypeIndex,
-                             void *handle);
-
-    // Memory allocation statistics functions.
-    VkDeviceSize getActiveMemoryAllocationsSize(uint32_t allocTypeIndex) const;
-    VkDeviceSize getActiveHeapMemoryAllocationsSize(uint32_t allocTypeIndex,
-                                                    uint32_t heapIndex) const;
-
-    uint64_t getActiveMemoryAllocationsCount(uint32_t allocTypeIndex) const;
-    uint64_t getActiveHeapMemoryAllocationsCount(uint32_t allocTypeIndex, uint32_t heapIndex) const;
-
-    // Pending memory allocation information is used for logging in case of an unsuccessful
-    // allocation. It is cleared in onMemoryAlloc().
-    VkDeviceSize getPendingMemoryAllocationSize() const;
-    vk::MemoryAllocationType getPendingMemoryAllocationType() const;
-    uint32_t getPendingMemoryTypeIndex() const;
-
-    void resetPendingMemoryAlloc();
-    void setPendingMemoryAlloc(vk::MemoryAllocationType allocType,
-                               VkDeviceSize size,
-                               uint32_t memoryTypeIndex);
-
-  private:
-    // Pointer to parent renderer object.
-    RendererVk *const mRenderer;
-
-    // For tracking the overall memory allocation sizes and counts per memory allocation type.
-    std::array<std::atomic<VkDeviceSize>, vk::kMemoryAllocationTypeCount>
-        mActiveMemoryAllocationsSize;
-    std::array<std::atomic<uint64_t>, vk::kMemoryAllocationTypeCount> mActiveMemoryAllocationsCount;
-
-    // Memory allocation data per memory heap. To update the data, a mutex is used.
-    std::mutex mMemoryAllocationMutex;
-
-    using PerHeapMemoryAllocationSizeVector  = std::vector<VkDeviceSize>;
-    using PerHeapMemoryAllocationCountVector = std::vector<uint64_t>;
-
-    std::array<PerHeapMemoryAllocationSizeVector, vk::kMemoryAllocationTypeCount>
-        mActivePerHeapMemoryAllocationsSize;
-    std::array<PerHeapMemoryAllocationCountVector, vk::kMemoryAllocationTypeCount>
-        mActivePerHeapMemoryAllocationsCount;
-
-    // Pending memory allocation information is used for logging in case of an allocation error.
-    // It includes the size and type of the last attempted allocation, which are cleared after
-    // the allocation is successful.
-    std::atomic<VkDeviceSize> mPendingMemoryAllocationSize;
-    std::atomic<vk::MemoryAllocationType> mPendingMemoryAllocationType;
-    std::atomic<uint32_t> mPendingMemoryTypeIndex;
-
-    // Additional information regarding memory allocation with debug layers enabled, including
-    // allocation ID and a record of all active allocations.
-    uint64_t mMemoryAllocationID;
-    using MemoryAllocInfoMap = angle::HashMap<vk::MemoryAllocInfoMapKey, vk::MemoryAllocationInfo>;
-    angle::HashMap<angle::BacktraceInfo, MemoryAllocInfoMap> mMemoryAllocationRecord;
-};
-
 class RendererVk : angle::NonCopyable
 {
   public:
@@ -280,7 +189,7 @@ class RendererVk : angle::NonCopyable
     gl::Version getMaxSupportedESVersion() const;
     gl::Version getMaxConformantESVersion() const;
 
-    uint32_t getApiVersion() const { return mApiVersion; }
+    uint32_t getDeviceVersion();
     VkInstance getInstance() const { return mInstance; }
     VkPhysicalDevice getPhysicalDevice() const { return mPhysicalDevice; }
     const VkPhysicalDeviceProperties &getPhysicalDeviceProperties() const
@@ -303,7 +212,11 @@ class RendererVk : angle::NonCopyable
     const VkPhysicalDeviceFeatures2KHR &getEnabledFeatures() const { return mEnabledFeatures; }
     VkDevice getDevice() const { return mDevice; }
 
+    bool isVulkan11Instance() const;
+    bool isVulkan11Device() const;
+
     const vk::Allocator &getAllocator() const { return mAllocator; }
+    vk::ImageMemorySuballocator &getImageMemorySuballocator() { return mImageMemorySuballocator; }
 
     angle::Result selectPresentQueueForSurface(DisplayVk *displayVk,
                                                VkSurfaceKHR surface,
@@ -338,10 +251,6 @@ class RendererVk : angle::NonCopyable
     uint32_t getMaxVertexAttribDivisor() const { return mMaxVertexAttribDivisor; }
     VkDeviceSize getMaxVertexAttribStride() const { return mMaxVertexAttribStride; }
 
-    VkDeviceSize getMinImportedHostPointerAlignment() const
-    {
-        return mMinImportedHostPointerAlignment;
-    }
     uint32_t getDefaultUniformBufferSize() const { return mDefaultUniformBufferSize; }
 
     angle::vk::ICD getEnabledICD() const { return mEnabledICD; }
@@ -363,6 +272,10 @@ class RendererVk : angle::NonCopyable
                                     const VkFormatFeatureFlags featureBits) const;
 
     bool isAsyncCommandQueueEnabled() const { return mFeatures.asyncCommandQueue.enabled; }
+    bool isAsyncCommandBufferResetEnabled() const
+    {
+        return mFeatures.asyncCommandBufferReset.enabled;
+    }
 
     ANGLE_INLINE egl::ContextPriority getDriverPriority(egl::ContextPriority priority)
     {
@@ -374,38 +287,38 @@ class RendererVk : angle::NonCopyable
 
     // This command buffer should be submitted immediately via queueSubmitOneOff.
     angle::Result getCommandBufferOneOff(vk::Context *context,
-                                         bool hasProtectedContent,
+                                         vk::ProtectionType protectionType,
                                          vk::PrimaryCommandBuffer *commandBufferOut)
     {
-        return mOneOffCommandPool.getCommandBuffer(context, hasProtectedContent, commandBufferOut);
-    }
-
-    void resetOutsideRenderPassCommandBuffer(vk::OutsideRenderPassCommandBuffer &&commandBuffer)
-    {
-        mOutsideRenderPassCommandBufferRecycler.resetCommandBuffer(std::move(commandBuffer));
-    }
-
-    void resetRenderPassCommandBuffer(vk::RenderPassCommandBuffer &&commandBuffer)
-    {
-        mRenderPassCommandBufferRecycler.resetCommandBuffer(std::move(commandBuffer));
+        return mOneOffCommandPoolMap[protectionType].getCommandBuffer(context, commandBufferOut);
     }
 
     // Fire off a single command buffer immediately with default priority.
     // Command buffer must be allocated with getCommandBufferOneOff and is reclaimed.
     angle::Result queueSubmitOneOff(vk::Context *context,
                                     vk::PrimaryCommandBuffer &&primary,
-                                    bool hasProtectedContent,
+                                    vk::ProtectionType protectionType,
                                     egl::ContextPriority priority,
-                                    const vk::Semaphore *waitSemaphore,
+                                    VkSemaphore waitSemaphore,
                                     VkPipelineStageFlags waitSemaphoreStageMasks,
                                     const vk::Fence *fence,
                                     vk::SubmitPolicy submitPolicy,
                                     QueueSerial *queueSerialOut);
 
+    angle::Result queueSubmitWaitSemaphore(vk::Context *context,
+                                           egl::ContextPriority priority,
+                                           const vk::Semaphore &waitSemaphore,
+                                           VkPipelineStageFlags waitSemaphoreStageMasks,
+                                           QueueSerial submitQueueSerial);
+
     template <typename... ArgsT>
     void collectGarbage(const vk::ResourceUse &use, ArgsT... garbageIn)
     {
-        if (hasUnfinishedUse(use))
+        if (hasResourceUseFinished(use))
+        {
+            DestroyGarbage(mDevice, garbageIn...);
+        }
+        else
         {
             std::vector<vk::GarbageObject> sharedGarbage;
             CollectGarbage(&sharedGarbage, garbageIn...);
@@ -414,27 +327,43 @@ class RendererVk : angle::NonCopyable
                 collectGarbage(use, std::move(sharedGarbage));
             }
         }
+    }
+
+    void collectAllocationGarbage(const vk::ResourceUse &use, vk::Allocation &allocationGarbageIn)
+    {
+        if (!allocationGarbageIn.valid())
+        {
+            return;
+        }
+
+        if (hasResourceUseFinished(use))
+        {
+            allocationGarbageIn.destroy(getAllocator());
+        }
         else
         {
-            DestroyGarbage(mDevice, garbageIn...);
+            std::vector<vk::GarbageObject> sharedGarbage;
+            CollectGarbage(&sharedGarbage, &allocationGarbageIn);
+            if (!sharedGarbage.empty())
+            {
+                collectGarbage(use, std::move(sharedGarbage));
+            }
         }
     }
 
     void collectGarbage(const vk::ResourceUse &use, vk::GarbageList &&sharedGarbage)
     {
-        if (!sharedGarbage.empty())
+        ASSERT(!sharedGarbage.empty());
+        vk::SharedGarbage garbage(use, std::move(sharedGarbage));
+        if (!hasResourceUseSubmitted(use))
         {
-            vk::SharedGarbage garbage(use, std::move(sharedGarbage));
-            if (hasUnsubmittedUse(use))
-            {
-                std::unique_lock<std::mutex> lock(mGarbageMutex);
-                mPendingSubmissionGarbage.push(std::move(garbage));
-            }
-            else if (!garbage.destroyIfComplete(this))
-            {
-                std::unique_lock<std::mutex> lock(mGarbageMutex);
-                mSharedGarbage.push(std::move(garbage));
-            }
+            std::unique_lock<std::mutex> lock(mGarbageMutex);
+            mPendingSubmissionGarbage.push(std::move(garbage));
+        }
+        else if (!garbage.destroyIfComplete(this))
+        {
+            std::unique_lock<std::mutex> lock(mGarbageMutex);
+            mSharedGarbage.push(std::move(garbage));
         }
     }
 
@@ -442,21 +371,7 @@ class RendererVk : angle::NonCopyable
                                      vk::BufferSuballocation &&suballocation,
                                      vk::Buffer &&buffer)
     {
-        if (hasUnfinishedUse(use))
-        {
-            std::unique_lock<std::mutex> lock(mGarbageMutex);
-            if (hasUnsubmittedUse(use))
-            {
-                mPendingSubmissionSuballocationGarbage.emplace(use, std::move(suballocation),
-                                                               std::move(buffer));
-            }
-            else
-            {
-                mSuballocationGarbageSizeInBytes += suballocation.getSize();
-                mSuballocationGarbage.emplace(use, std::move(suballocation), std::move(buffer));
-            }
-        }
-        else
+        if (hasResourceUseFinished(use))
         {
             // mSuballocationGarbageDestroyed is atomic, so we dont need mGarbageMutex to
             // protect it.
@@ -464,6 +379,20 @@ class RendererVk : angle::NonCopyable
             buffer.destroy(mDevice);
             suballocation.destroy(this);
         }
+        else
+        {
+            std::unique_lock<std::mutex> lock(mGarbageMutex);
+            if (hasResourceUseSubmitted(use))
+            {
+                mSuballocationGarbageSizeInBytes += suballocation.getSize();
+                mSuballocationGarbage.emplace(use, std::move(suballocation), std::move(buffer));
+            }
+            else
+            {
+                mPendingSubmissionSuballocationGarbage.emplace(use, std::move(suballocation),
+                                                               std::move(buffer));
+            }
+        }
     }
 
     angle::Result getPipelineCache(vk::PipelineCacheAccess *pipelineCacheOut);
@@ -498,17 +427,33 @@ class RendererVk : angle::NonCopyable
         }
     }
 
-    angle::Result waitForQueueSerialToBeSubmitted(vk::Context *context,
-                                                  const QueueSerial &queueSerial)
+    angle::Result waitForResourceUseToBeSubmittedToDevice(vk::Context *context,
+                                                          const vk::ResourceUse &use)
     {
         // This is only needed for async submission code path. For immediate submission, it is a nop
         // since everything is submitted immediately.
         if (isAsyncCommandQueueEnabled())
         {
+            ASSERT(mCommandProcessor.hasResourceUseEnqueued(use));
+            return mCommandProcessor.waitForResourceUseToBeSubmitted(context, use);
+        }
+        // This ResourceUse must have been submitted.
+        ASSERT(mCommandQueue.hasResourceUseSubmitted(use));
+        return angle::Result::Continue;
+    }
+
+    angle::Result waitForQueueSerialToBeSubmittedToDevice(vk::Context *context,
+                                                          const QueueSerial &queueSerial)
+    {
+        // This is only needed for async submission code path. For immediate submission, it is a nop
+        // since everything is submitted immediately.
+        if (isAsyncCommandQueueEnabled())
+        {
+            ASSERT(mCommandProcessor.hasQueueSerialEnqueued(queueSerial));
             return mCommandProcessor.waitForQueueSerialToBeSubmitted(context, queueSerial);
         }
         // This queueSerial must have been submitted.
-        ASSERT(!mCommandQueue.hasUnsubmittedUse(vk::ResourceUse(queueSerial)));
+        ASSERT(mCommandQueue.hasQueueSerialSubmitted(queueSerial));
         return angle::Result::Continue;
     }
 
@@ -520,11 +465,6 @@ class RendererVk : angle::NonCopyable
 
     egl::Display *getDisplay() const { return mDisplay; }
 
-    VkResult getLastPresentResult(VkSwapchainKHR swapchain)
-    {
-        return mCommandProcessor.getLastPresentResult(swapchain);
-    }
-
     bool enableDebugUtils() const { return mEnableDebugUtils; }
     bool angleDebuggerMode() const { return mAngleDebuggerMode; }
 
@@ -548,13 +488,16 @@ class RendererVk : angle::NonCopyable
     void cleanupPendingSubmissionGarbage();
 
     angle::Result submitCommands(vk::Context *context,
-                                 bool hasProtectedContent,
+                                 vk::ProtectionType protectionType,
                                  egl::ContextPriority contextPriority,
-                                 std::vector<VkSemaphore> &&waitSemaphores,
-                                 std::vector<VkPipelineStageFlags> &&waitSemaphoreStageMasks,
                                  const vk::Semaphore *signalSemaphore,
-                                 vk::SecondaryCommandPools *commandPools,
-                                 const QueueSerial &submitSerialOut);
+                                 const QueueSerial &submitQueueSerial);
+
+    angle::Result submitPriorityDependency(vk::Context *context,
+                                           vk::ProtectionTypes protectionTypes,
+                                           egl::ContextPriority srcContextPriority,
+                                           egl::ContextPriority dstContextPriority,
+                                           SerialIndex index);
 
     void handleDeviceLost();
     angle::Result finishResourceUse(vk::Context *context, const vk::ResourceUse &use);
@@ -563,38 +506,46 @@ class RendererVk : angle::NonCopyable
                                                             const vk::ResourceUse &use,
                                                             uint64_t timeout,
                                                             VkResult *result);
-    angle::Result finish(vk::Context *context, bool hasProtectedContent);
     angle::Result checkCompletedCommands(vk::Context *context);
+    angle::Result retireFinishedCommands(vk::Context *context);
 
+    angle::Result flushWaitSemaphores(vk::ProtectionType protectionType,
+                                      egl::ContextPriority priority,
+                                      std::vector<VkSemaphore> &&waitSemaphores,
+                                      std::vector<VkPipelineStageFlags> &&waitSemaphoreStageMasks);
     angle::Result flushRenderPassCommands(vk::Context *context,
-                                          bool hasProtectedContent,
+                                          vk::ProtectionType protectionType,
+                                          egl::ContextPriority priority,
                                           const vk::RenderPass &renderPass,
                                           vk::RenderPassCommandBufferHelper **renderPassCommands);
     angle::Result flushOutsideRPCommands(
         vk::Context *context,
-        bool hasProtectedContent,
+        vk::ProtectionType protectionType,
+        egl::ContextPriority priority,
         vk::OutsideRenderPassCommandBufferHelper **outsideRPCommands);
 
-    VkResult queuePresent(vk::Context *context,
-                          egl::ContextPriority priority,
-                          const VkPresentInfoKHR &presentInfo);
+    void queuePresent(vk::Context *context,
+                      egl::ContextPriority priority,
+                      const VkPresentInfoKHR &presentInfo,
+                      vk::SwapchainStatus *swapchainStatus);
+
+    // Only useful if async submission is enabled
+    angle::Result waitForPresentToBeSubmitted(vk::SwapchainStatus *swapchainStatus);
 
     angle::Result getOutsideRenderPassCommandBufferHelper(
         vk::Context *context,
-        vk::CommandPool *commandPool,
+        vk::SecondaryCommandPool *commandPool,
         vk::SecondaryCommandMemoryAllocator *commandsAllocator,
         vk::OutsideRenderPassCommandBufferHelper **commandBufferHelperOut);
     angle::Result getRenderPassCommandBufferHelper(
         vk::Context *context,
-        vk::CommandPool *commandPool,
+        vk::SecondaryCommandPool *commandPool,
         vk::SecondaryCommandMemoryAllocator *commandsAllocator,
         vk::RenderPassCommandBufferHelper **commandBufferHelperOut);
 
     void recycleOutsideRenderPassCommandBufferHelper(
-        VkDevice device,
         vk::OutsideRenderPassCommandBufferHelper **commandBuffer);
-    void recycleRenderPassCommandBufferHelper(VkDevice device,
-                                              vk::RenderPassCommandBufferHelper **commandBuffer);
+    void recycleRenderPassCommandBufferHelper(vk::RenderPassCommandBufferHelper **commandBuffer);
 
     // Process GPU memory reports
     void processMemoryReportCallback(const VkDeviceMemoryReportCallbackDataEXT &callbackData)
@@ -693,7 +644,8 @@ class RendererVk : angle::NonCopyable
         return getFeatures().preferLinearFilterForYUV.enabled ? VK_FILTER_LINEAR : defaultFilter;
     }
 
-    angle::Result allocateQueueSerialIndex(SerialIndex *indexOut, Serial *serialOut);
+    angle::Result allocateScopedQueueSerialIndex(vk::ScopedQueueSerialIndex *indexOut);
+    angle::Result allocateQueueSerialIndex(SerialIndex *serialIndexOut);
     size_t getLargestQueueSerialIndexEverAllocated() const
     {
         return mQueueSerialIndexAllocator.getLargestIndexEverAllocated();
@@ -704,10 +656,13 @@ class RendererVk : angle::NonCopyable
                              size_t count,
                              RangedSerialFactory *rangedSerialFactory);
 
-    // The ResourceUse still have unfinished queue serial by vulkan.
-    bool hasUnfinishedUse(const vk::ResourceUse &use) const;
-    // The ResourceUse still have queue serial not yet submitted to vulkan.
-    bool hasUnsubmittedUse(const vk::ResourceUse &use) const;
+    // Return true if all serials in ResourceUse have been submitted.
+    bool hasResourceUseSubmitted(const vk::ResourceUse &use) const;
+    bool hasQueueSerialSubmitted(const QueueSerial &queueSerial) const;
+    Serial getLastSubmittedSerial(SerialIndex index) const;
+    // Return true if all serials in ResourceUse have been finished.
+    bool hasResourceUseFinished(const vk::ResourceUse &use) const;
+    bool hasQueueSerialFinished(const QueueSerial &queueSerial) const;
 
     // Memory statistics can be updated on allocation and deallocation.
     template <typename HandleT>
@@ -730,17 +685,52 @@ class RendererVk : angle::NonCopyable
                                                      reinterpret_cast<void *>(handle));
     }
 
-    // Memory statistics are logged when handling a context error.
-    void logMemoryStatsOnError();
-
     MemoryAllocationTracker *getMemoryAllocationTracker() { return &mMemoryAllocationTracker; }
 
+    void requestAsyncCommandsAndGarbageCleanup(vk::Context *context);
+
+    // Static function to get Vulkan object type name.
+    static const char *GetVulkanObjectTypeName(VkObjectType type);
+
   private:
     angle::Result initializeDevice(DisplayVk *displayVk, uint32_t queueFamilyIndex);
     void ensureCapsInitialized() const;
     void initializeValidationMessageSuppressions();
 
     void queryDeviceExtensionFeatures(const vk::ExtensionNameList &deviceExtensionNames);
+    void appendDeviceExtensionFeaturesNotPromoted(const vk::ExtensionNameList &deviceExtensionNames,
+                                                  VkPhysicalDeviceFeatures2KHR *deviceFeatures,
+                                                  VkPhysicalDeviceProperties2 *deviceProperties);
+    void appendDeviceExtensionFeaturesPromotedTo11(
+        const vk::ExtensionNameList &deviceExtensionNames,
+        VkPhysicalDeviceFeatures2KHR *deviceFeatures,
+        VkPhysicalDeviceProperties2 *deviceProperties);
+    void appendDeviceExtensionFeaturesPromotedTo12(
+        const vk::ExtensionNameList &deviceExtensionNames,
+        VkPhysicalDeviceFeatures2KHR *deviceFeatures,
+        VkPhysicalDeviceProperties2 *deviceProperties);
+    void appendDeviceExtensionFeaturesPromotedTo13(
+        const vk::ExtensionNameList &deviceExtensionNames,
+        VkPhysicalDeviceFeatures2KHR *deviceFeatures,
+        VkPhysicalDeviceProperties2 *deviceProperties);
+
+    angle::Result enableInstanceExtensions(DisplayVk *displayVk,
+                                           const VulkanLayerVector &enabledInstanceLayerNames,
+                                           const char *wsiExtension,
+                                           bool canLoadDebugUtils);
+    angle::Result enableDeviceExtensions(DisplayVk *displayVk,
+                                         const VulkanLayerVector &enabledDeviceLayerNames);
+
+    void enableDeviceExtensionsNotPromoted(const vk::ExtensionNameList &deviceExtensionNames);
+    void enableDeviceExtensionsPromotedTo11(const vk::ExtensionNameList &deviceExtensionNames);
+    void enableDeviceExtensionsPromotedTo12(const vk::ExtensionNameList &deviceExtensionNames);
+    void enableDeviceExtensionsPromotedTo13(const vk::ExtensionNameList &deviceExtensionNames);
+
+    void initInstanceExtensionEntryPoints();
+    void initDeviceExtensionEntryPoints();
+    // Initialize extension entry points from core ones if needed
+    void initializeInstanceExtensionEntryPointsFromCore() const;
+    void initializeDeviceExtensionEntryPointsFromCore() const;
 
     void initFeatures(DisplayVk *display, const vk::ExtensionNameList &extensions);
     void appBasedFeatureOverrides(DisplayVk *display, const vk::ExtensionNameList &extensions);
@@ -766,7 +756,7 @@ class RendererVk : angle::NonCopyable
 
     template <typename CommandBufferHelperT, typename RecyclerT>
     angle::Result getCommandBufferImpl(vk::Context *context,
-                                       vk::CommandPool *commandPool,
+                                       vk::SecondaryCommandPool *commandPool,
                                        vk::SecondaryCommandMemoryAllocator *commandsAllocator,
                                        RecyclerT *recycler,
                                        CommandBufferHelperT **commandBufferHelperOut);
@@ -783,7 +773,18 @@ class RendererVk : angle::NonCopyable
     mutable ShPixelLocalStorageOptions mNativePLSOptions;
     mutable angle::FeaturesVk mFeatures;
 
-    uint32_t mApiVersion;
+    // The instance and device versions.  The instance version is the one from the Vulkan loader,
+    // while the device version comes from VkPhysicalDeviceProperties::apiVersion.  With instance
+    // version 1.0, only device version 1.0 can be used.  If instance version is at least 1.1, any
+    // device version (even higher than that) can be used.  Some extensions have been promoted to
+    // Vulkan 1.1 or higher, but the version check must be done against the instance or device
+    // version, depending on whether it's an instance or device extension.
+    //
+    // Note that mDeviceVersion is technically redundant with mPhysicalDeviceProperties.apiVersion,
+    // but ANGLE may use a smaller version with problematic ICDs.
+    uint32_t mInstanceVersion;
+    uint32_t mDeviceVersion;
+
     VkInstance mInstance;
     bool mEnableValidationLayers;
     // True if ANGLE is enabling the VK_EXT_debug_utils extension.
@@ -796,8 +797,13 @@ class RendererVk : angle::NonCopyable
     angle::vk::ICD mEnabledICD;
     VkDebugUtilsMessengerEXT mDebugUtilsMessenger;
     VkPhysicalDevice mPhysicalDevice;
+
     VkPhysicalDeviceProperties mPhysicalDeviceProperties;
+    VkPhysicalDeviceVulkan11Properties mPhysicalDevice11Properties;
+
     VkPhysicalDeviceFeatures mPhysicalDeviceFeatures;
+    VkPhysicalDeviceVulkan11Features mPhysicalDevice11Features;
+
     VkPhysicalDeviceLineRasterizationFeaturesEXT mLineRasterizationFeatures;
     VkPhysicalDeviceProvokingVertexFeaturesEXT mProvokingVertexFeatures;
     VkPhysicalDeviceVertexAttributeDivisorFeaturesEXT mVertexAttributeDivisorFeatures;
@@ -808,7 +814,6 @@ class RendererVk : angle::NonCopyable
     VkPhysicalDeviceShaderSubgroupExtendedTypesFeaturesKHR mSubgroupExtendedTypesFeatures;
     VkPhysicalDeviceDeviceMemoryReportFeaturesEXT mMemoryReportFeatures;
     VkDeviceDeviceMemoryReportCreateInfoEXT mMemoryReportCallback;
-    VkPhysicalDeviceExternalMemoryHostPropertiesEXT mExternalMemoryHostProperties;
     VkPhysicalDeviceShaderFloat16Int8FeaturesKHR mShaderFloat16Int8Features;
     VkPhysicalDeviceDepthStencilResolvePropertiesKHR mDepthStencilResolveProperties;
     VkPhysicalDeviceMultisampledRenderToSingleSampledFeaturesGOOGLEX
@@ -822,12 +827,11 @@ class RendererVk : angle::NonCopyable
     VkPhysicalDeviceDriverPropertiesKHR mDriverProperties;
     VkPhysicalDeviceCustomBorderColorFeaturesEXT mCustomBorderColorFeatures;
     VkPhysicalDeviceProtectedMemoryFeatures mProtectedMemoryFeatures;
-    VkPhysicalDeviceProtectedMemoryProperties mProtectedMemoryProperties;
     VkPhysicalDeviceHostQueryResetFeaturesEXT mHostQueryResetFeatures;
+    VkPhysicalDeviceDepthClipEnableFeaturesEXT mDepthClipEnableFeatures;
     VkPhysicalDeviceDepthClipControlFeaturesEXT mDepthClipControlFeatures;
     VkPhysicalDevicePrimitivesGeneratedQueryFeaturesEXT mPrimitivesGeneratedQueryFeatures;
     VkPhysicalDevicePrimitiveTopologyListRestartFeaturesEXT mPrimitiveTopologyListRestartFeatures;
-    VkPhysicalDeviceBlendOperationAdvancedFeaturesEXT mBlendOperationAdvancedFeatures;
     VkPhysicalDeviceSamplerYcbcrConversionFeatures mSamplerYcbcrConversionFeatures;
     VkPhysicalDevicePipelineCreationCacheControlFeaturesEXT mPipelineCreationCacheControlFeatures;
     VkPhysicalDeviceExtendedDynamicStateFeaturesEXT mExtendedDynamicStateFeatures;
@@ -841,14 +845,14 @@ class RendererVk : angle::NonCopyable
     VkPhysicalDevicePipelineProtectedAccessFeaturesEXT mPipelineProtectedAccessFeatures;
     VkPhysicalDeviceRasterizationOrderAttachmentAccessFeaturesEXT
         mRasterizationOrderAttachmentAccessFeatures;
-    VkPhysicalDeviceSwapchainMaintenance1FeaturesEXT mSwapchainMaintenance1FeaturesEXT;
+    VkPhysicalDeviceSwapchainMaintenance1FeaturesEXT mSwapchainMaintenance1Features;
     VkPhysicalDeviceDrmPropertiesEXT mDrmProperties;
+
     angle::PackedEnumBitSet<gl::ShadingRate, uint8_t> mSupportedFragmentShadingRates;
     std::vector<VkQueueFamilyProperties> mQueueFamilyProperties;
     uint32_t mMaxVertexAttribDivisor;
     uint32_t mCurrentQueueFamilyIndex;
     VkDeviceSize mMaxVertexAttribStride;
-    VkDeviceSize mMinImportedHostPointerAlignment;
     uint32_t mDefaultUniformBufferSize;
     VkDevice mDevice;
     AtomicSerialFactory mShaderSerialFactory;
@@ -882,6 +886,10 @@ class RendererVk : angle::NonCopyable
     mutable angle::FormatMap<VkFormatProperties> mFormatProperties;
 
     vk::Allocator mAllocator;
+
+    // Used to allocate memory for images using VMA, utilizing suballocation.
+    vk::ImageMemorySuballocator mImageMemorySuballocator;
+
     vk::MemoryProperties mMemoryProperties;
     VkDeviceSize mPreferredLargeHeapBlockSize;
 
@@ -928,7 +936,7 @@ class RendererVk : angle::NonCopyable
     uint32_t mGarbageCollectionFlushThreshold;
 
     // Only used for "one off" command buffers.
-    OneOffCommandPool mOneOffCommandPool;
+    angle::PackedEnumMap<vk::ProtectionType, OneOffCommandPool> mOneOffCommandPoolMap;
 
     // Synchronous Command Queue
     vk::CommandQueue mCommandQueue;
@@ -937,11 +945,9 @@ class RendererVk : angle::NonCopyable
     vk::CommandProcessor mCommandProcessor;
 
     // Command buffer pool management.
-    vk::CommandBufferRecycler<vk::OutsideRenderPassCommandBuffer,
-                              vk::OutsideRenderPassCommandBufferHelper>
+    vk::CommandBufferRecycler<vk::OutsideRenderPassCommandBufferHelper>
         mOutsideRenderPassCommandBufferRecycler;
-    vk::CommandBufferRecycler<vk::RenderPassCommandBuffer, vk::RenderPassCommandBufferHelper>
-        mRenderPassCommandBufferRecycler;
+    vk::CommandBufferRecycler<vk::RenderPassCommandBufferHelper> mRenderPassCommandBufferRecycler;
 
     SamplerCache mSamplerCache;
     SamplerYcbcrConversionCache mYuvConversionCache;
@@ -1003,22 +1009,77 @@ ANGLE_INLINE void RendererVk::reserveQueueSerials(SerialIndex index,
     mQueueSerialFactory[index].reserve(rangedSerialFactory, count);
 }
 
-ANGLE_INLINE bool RendererVk::hasUnfinishedUse(const vk::ResourceUse &use) const
-{
-    return mCommandQueue.hasUnfinishedUse(use);
-}
-
-ANGLE_INLINE bool RendererVk::hasUnsubmittedUse(const vk::ResourceUse &use) const
+ANGLE_INLINE bool RendererVk::hasResourceUseSubmitted(const vk::ResourceUse &use) const
 {
     if (isAsyncCommandQueueEnabled())
     {
-        return mCommandProcessor.hasUnsubmittedUse(use);
+        return mCommandProcessor.hasResourceUseEnqueued(use);
     }
     else
     {
-        return mCommandQueue.hasUnsubmittedUse(use);
+        return mCommandQueue.hasResourceUseSubmitted(use);
     }
 }
+
+ANGLE_INLINE bool RendererVk::hasQueueSerialSubmitted(const QueueSerial &queueSerial) const
+{
+    if (isAsyncCommandQueueEnabled())
+    {
+        return mCommandProcessor.hasQueueSerialEnqueued(queueSerial);
+    }
+    else
+    {
+        return mCommandQueue.hasQueueSerialSubmitted(queueSerial);
+    }
+}
+
+ANGLE_INLINE Serial RendererVk::getLastSubmittedSerial(SerialIndex index) const
+{
+    if (isAsyncCommandQueueEnabled())
+    {
+        return mCommandProcessor.getLastEnqueuedSerial(index);
+    }
+    else
+    {
+        return mCommandQueue.getLastSubmittedSerial(index);
+    }
+}
+
+ANGLE_INLINE bool RendererVk::hasResourceUseFinished(const vk::ResourceUse &use) const
+{
+    return mCommandQueue.hasResourceUseFinished(use);
+}
+
+ANGLE_INLINE bool RendererVk::hasQueueSerialFinished(const QueueSerial &queueSerial) const
+{
+    return mCommandQueue.hasQueueSerialFinished(queueSerial);
+}
+
+ANGLE_INLINE angle::Result RendererVk::waitForPresentToBeSubmitted(
+    vk::SwapchainStatus *swapchainStatus)
+{
+    if (isAsyncCommandQueueEnabled())
+    {
+        return mCommandProcessor.waitForPresentToBeSubmitted(swapchainStatus);
+    }
+    ASSERT(!swapchainStatus->isPending);
+    return angle::Result::Continue;
+}
+
+ANGLE_INLINE void RendererVk::requestAsyncCommandsAndGarbageCleanup(vk::Context *context)
+{
+    mCommandProcessor.requestCommandsAndGarbageCleanup();
+}
+
+ANGLE_INLINE angle::Result RendererVk::checkCompletedCommands(vk::Context *context)
+{
+    return mCommandQueue.checkAndCleanupCompletedCommands(context);
+}
+
+ANGLE_INLINE angle::Result RendererVk::retireFinishedCommands(vk::Context *context)
+{
+    return mCommandQueue.retireFinishedCommands(context);
+}
 }  // namespace rx
 
 #endif  // LIBANGLE_RENDERER_VULKAN_RENDERERVK_H_
