@@ -155,7 +155,7 @@ angle::Result SyncHelper::getStatus(Context *context, ContextVk *contextVk, bool
     // Submit commands if it was deferred on the context that issued the sync object
     ANGLE_TRY(submitSyncIfDeferred(contextVk, RenderPassClosureReason::SyncObjectClientWait));
 
-    if (!renderer->hasUnfinishedUse(mUse))
+    if (renderer->hasResourceUseFinished(mUse))
     {
         *signaled = true;
     }
@@ -163,7 +163,7 @@ angle::Result SyncHelper::getStatus(Context *context, ContextVk *contextVk, bool
     {
         // Do immediate check in case it actually already finished.
         ANGLE_TRY(renderer->checkCompletedCommands(context));
-        *signaled = !renderer->hasUnfinishedUse(mUse);
+        *signaled = renderer->hasResourceUseFinished(mUse);
     }
     return angle::Result::Continue;
 }
@@ -176,7 +176,7 @@ angle::Result SyncHelper::submitSyncIfDeferred(ContextVk *contextVk, RenderPassC
         return angle::Result::Continue;
     }
 
-    if (!contextVk->getRenderer()->hasUnsubmittedUse(mUse))
+    if (contextVk->getRenderer()->hasResourceUseSubmitted(mUse))
     {
         return angle::Result::Continue;
     }
@@ -201,7 +201,7 @@ angle::Result SyncHelper::submitSyncIfDeferred(ContextVk *contextVk, RenderPassC
     }
     // Note mUse could still be invalid here if it is inserted on a fresh created context, i.e.,
     // fence is tracking nothing and is finished when inserted..
-    ASSERT(!contextVk->getRenderer()->hasUnsubmittedUse(mUse));
+    ASSERT(contextVk->getRenderer()->hasResourceUseSubmitted(mUse));
 
     return angle::Result::Continue;
 }
@@ -279,8 +279,8 @@ angle::Result SyncHelperNativeFence::initializeWithFd(ContextVk *contextVk, int
     // signal it is in the graphics pipeline at the time we export the fd. Thus we need to
     // EnsureSubmitted here.
     ANGLE_TRY(renderer->queueSubmitOneOff(contextVk, vk::PrimaryCommandBuffer(),
-                                          contextVk->hasProtectedContent(),
-                                          contextVk->getPriority(), nullptr, 0, &fence.get(),
+                                          contextVk->getProtectionType(), contextVk->getPriority(),
+                                          VK_NULL_HANDLE, 0, &fence.get(),
                                           vk::SubmitPolicy::EnsureSubmitted, &queueSerialOut));
 
     VkFenceGetFdInfoKHR fenceGetFdInfo = {};
@@ -377,7 +377,7 @@ angle::Result SyncHelperNativeFence::getStatus(Context *context,
     // We've got a serial, check if the serial is still in use
     if (mUse.valid())
     {
-        *signaled = !context->getRenderer()->hasUnfinishedUse(mUse);
+        *signaled = context->getRenderer()->hasResourceUseFinished(mUse);
         return angle::Result::Continue;
     }
 
