@@ -21,10 +21,17 @@ namespace rx
 
 enum class ImageMipLevels
 {
-    EnabledLevels = 0,
-    FullMipChain  = 1,
+    EnabledLevels                 = 0,
+    FullMipChainForGenerateMipmap = 1,
+    FullMipChain                  = 2,
 
-    InvalidEnum = 2,
+    InvalidEnum = 3,
+};
+
+enum class TextureUpdateResult
+{
+    ImageUnaffected,
+    ImageRespecified,
 };
 
 class TextureVk : public TextureImpl, public angle::ObserverInterface
@@ -181,8 +188,20 @@ class TextureVk : public TextureImpl, public angle::ObserverInterface
                                         bool fixedSampleLocations) override;
 
     angle::Result initializeContents(const gl::Context *context,
+                                     GLenum binding,
                                      const gl::ImageIndex &imageIndex) override;
 
+    angle::Result initializeContentsWithBlack(const gl::Context *context,
+                                              GLenum binding,
+                                              const gl::ImageIndex &imageIndex);
+
+    GLint getRequiredExternalTextureImageUnits([[maybe_unused]] const gl::Context *context) override
+    {
+        // For now, we assume that only one image unit is needed to support
+        // external GL textures in the Vulkan backend.
+        return 1;
+    }
+
     const vk::ImageHelper &getImage() const
     {
         ASSERT(mImage && mImage->valid());
@@ -195,16 +214,20 @@ class TextureVk : public TextureImpl, public angle::ObserverInterface
         return *mImage;
     }
 
-    void retainBufferViews(vk::ResourceUseList *resourceUseList)
+    void retainBufferViews(vk::CommandBufferHelperCommon *commandBufferHelper)
     {
-        mBufferViews.retain(resourceUseList);
+        commandBufferHelper->retainResource(&mBufferViews);
     }
 
+    bool isImmutable() { return mState.getImmutableFormat(); }
+    bool imageValid() const { return (mImage && mImage->valid()); }
+
     void releaseOwnershipOfImage(const gl::Context *context);
 
     const vk::ImageView &getReadImageView(vk::Context *context,
                                           GLenum srgbDecode,
-                                          bool texelFetchStaticUse) const;
+                                          bool texelFetchStaticUse,
+                                          bool samplerExternal2DY2YEXT) const;
 
     // A special view for cube maps as a 2D array, used with shaders that do texelFetch() and for
     // seamful cube map emulation.
@@ -223,18 +246,46 @@ class TextureVk : public TextureImpl, public angle::ObserverInterface
                                       const gl::ImageUnit &binding,
                                       const vk::ImageView **imageViewOut);
 
-    const vk::SamplerHelper &getSampler() const
+    const vk::SamplerHelper &getSampler(bool isSamplerExternalY2Y) const
     {
+        if (isSamplerExternalY2Y)
+        {
+            ASSERT(mY2YSampler.valid());
+            return mY2YSampler.get();
+        }
         ASSERT(mSampler.valid());
         return mSampler.get();
     }
 
+    void resetSampler()
+    {
+        mSampler.reset();
+        mY2YSampler.reset();
+    }
+
     // Normally, initialize the image with enabled mipmap level counts.
     angle::Result ensureImageInitialized(ContextVk *contextVk, ImageMipLevels mipLevels);
 
     vk::ImageOrBufferViewSubresourceSerial getImageViewSubresourceSerial(
-        const gl::SamplerState &samplerState) const;
+        const gl::SamplerState &samplerState) const
+    {
+        if (samplerState.getSRGBDecode() == GL_DECODE_EXT)
+        {
+            ASSERT(getImageViewSubresourceSerialImpl(GL_DECODE_EXT) ==
+                   mCachedImageViewSubresourceSerialSRGBDecode);
+            return mCachedImageViewSubresourceSerialSRGBDecode;
+        }
+        else
+        {
+            ASSERT(getImageViewSubresourceSerialImpl(GL_SKIP_DECODE_EXT) ==
+                   mCachedImageViewSubresourceSerialSkipDecode);
+            return mCachedImageViewSubresourceSerialSkipDecode;
+        }
+    }
+
     vk::ImageOrBufferViewSubresourceSerial getBufferViewSerial() const;
+    vk::ImageOrBufferViewSubresourceSerial getStorageImageViewSerial(
+        const gl::ImageUnit &binding) const;
 
     GLenum getColorReadFormat(const gl::Context *context) override;
     GLenum getColorReadType(const gl::Context *context) override;
@@ -267,7 +318,7 @@ class TextureVk : public TextureImpl, public angle::ObserverInterface
     }
 
     angle::Result ensureMutable(ContextVk *contextVk);
-    angle::Result ensureRenderable(ContextVk *contextVk);
+    angle::Result ensureRenderable(ContextVk *contextVk, TextureUpdateResult *updateResultOut);
 
     bool getAndResetImmutableSamplerDirtyState()
     {
@@ -276,6 +327,17 @@ class TextureVk : public TextureImpl, public angle::ObserverInterface
         return isDirty;
     }
 
+    angle::Result onLabelUpdate(const gl::Context *context) override;
+
+    void onNewDescriptorSet(const vk::SharedDescriptorSetCacheKey &sharedCacheKey)
+    {
+        mDescriptorSetCacheManager.addKey(sharedCacheKey);
+    }
+
+    // Check if the texture is consistently specified. Used for flushing mutable textures.
+    bool isMutableTextureConsistentlySpecifiedForFlush();
+    bool isMipImageDescDefined(gl::TextureTarget textureTarget, size_t level);
+
   private:
     // Transform an image index from the frontend into one that can be used on the backing
     // ImageHelper, taking into account mipmap or cube face offsets
@@ -285,6 +347,8 @@ class TextureVk : public TextureImpl, public angle::ObserverInterface
 
     // Get the layer count for views.
     uint32_t getImageViewLayerCount() const;
+    // Get the level count for views.
+    uint32_t getImageViewLevelCount() const;
 
     void releaseAndDeleteImageAndViews(ContextVk *contextVk);
     angle::Result ensureImageAllocated(ContextVk *contextVk, const vk::Format &format);
@@ -294,7 +358,8 @@ class TextureVk : public TextureImpl, public angle::ObserverInterface
                         const vk::Format &format,
                         uint32_t imageLevelOffset,
                         uint32_t imageLayerOffset,
-                        bool selfOwned);
+                        bool selfOwned,
+                        UniqueSerial siblingSerial);
 
     vk::ImageViewHelper &getImageViews()
     {
@@ -430,11 +495,7 @@ class TextureVk : public TextureImpl, public angle::ObserverInterface
     angle::Result reinitImageAsRenderable(ContextVk *contextVk,
                                           const vk::Format &format,
                                           gl::TexLevelMask skipLevelsMask);
-    angle::Result initImageViews(ContextVk *contextVk,
-                                 const angle::Format &format,
-                                 const bool sized,
-                                 uint32_t levelCount,
-                                 uint32_t layerCount);
+    angle::Result initImageViews(ContextVk *contextVk, uint32_t levelCount);
     void initSingleLayerRenderTargets(ContextVk *contextVk,
                                       GLuint layerCount,
                                       gl::LevelIndex levelIndexGL,
@@ -451,6 +512,14 @@ class TextureVk : public TextureImpl, public angle::ObserverInterface
     // Flush image's staged updates for all levels and layers.
     angle::Result flushImageStagedUpdates(ContextVk *contextVk);
 
+    // For various reasons, the underlying image may need to be respecified.  For example because
+    // base/max level changed, usage/create flags have changed, the format needs modification to
+    // become renderable, generate mipmap is adding levels, etc.  This function is called by
+    // syncState and getAttachmentRenderTarget.  The latter calls this function to be able to sync
+    // the texture's image while an attached framebuffer is being synced.  Note that we currently
+    // sync framebuffers before textures so that the deferred clear optimization works.
+    angle::Result respecifyImageStorageIfNecessary(ContextVk *contextVk, gl::Command source);
+
     const gl::InternalFormat &getImplementationSizedFormat(const gl::Context *context) const;
     const vk::Format &getBaseLevelFormat(RendererVk *renderer) const;
     // Queues a flush of any modified image attributes. The image will be reallocated with its new
@@ -458,7 +527,8 @@ class TextureVk : public TextureImpl, public angle::ObserverInterface
     angle::Result respecifyImageStorage(ContextVk *contextVk);
 
     // Update base and max levels, and re-create image if needed.
-    angle::Result maybeUpdateBaseMaxLevels(ContextVk *contextVk, bool *didRespecifyOut);
+    angle::Result maybeUpdateBaseMaxLevels(ContextVk *contextVk,
+                                           TextureUpdateResult *changeResultOut);
 
     bool isFastUnpackPossible(const vk::Format &vkFormat, size_t offset) const;
 
@@ -475,30 +545,38 @@ class TextureVk : public TextureImpl, public angle::ObserverInterface
     }
 
     angle::Result refreshImageViews(ContextVk *contextVk);
-    bool shouldDecodeSRGB(vk::Context *context, GLenum srgbDecode, bool texelFetchStaticUse) const;
+    bool shouldDecodeSRGB(vk::Context *contextVk,
+                          GLenum srgbDecode,
+                          bool texelFetchStaticUse) const;
     void initImageUsageFlags(ContextVk *contextVk, angle::FormatID actualFormatID);
     void handleImmutableSamplerTransition(const vk::ImageHelper *previousImage,
                                           const vk::ImageHelper *nextImage);
 
     vk::ImageAccess getRequiredImageAccess() const { return mRequiredImageAccess; }
-    bool imageHasActualImageFormat(angle::FormatID actualFormatID) const;
 
     void stageSelfAsSubresourceUpdates(ContextVk *contextVk);
 
+    vk::ImageOrBufferViewSubresourceSerial getImageViewSubresourceSerialImpl(
+        GLenum srgbDecode) const;
+
+    void updateCachedImageViewSerials();
+
+    angle::Result updateTextureLabel(ContextVk *contextVk);
+
     bool mOwnsImage;
+    // Generated from ImageVk if EGLImage target, or from throw-away generator if Surface target.
+    UniqueSerial mImageSiblingSerial;
+
     bool mRequiresMutableStorage;
     vk::ImageAccess mRequiredImageAccess;
     bool mImmutableSamplerDirty;
 
-    gl::TextureType mImageNativeType;
-
-    // The layer offset to apply when converting from a frontend texture layer to a texture layer in
-    // mImage. Used when this texture sources a cube map face or 3D texture layer from an EGL image.
-    uint32_t mImageLayerOffset;
-
-    // The level offset to apply when converting from a frontend texture level to texture level in
-    // mImage.
-    uint32_t mImageLevelOffset;
+    // Only valid if this texture is an "EGLImage target" and the associated EGL Image was
+    // originally sourced from an OpenGL texture. Such EGL Images can be a slice of the underlying
+    // resource. The layer and level offsets are used to track the location of the slice.
+    gl::TextureType mEGLImageNativeType;
+    uint32_t mEGLImageLayerOffset;
+    uint32_t mEGLImageLevelOffset;
 
     // If multisampled rendering to texture, an intermediate multisampled image is created for use
     // as renderpass color attachment.  An array of images and image views are used based on the
@@ -544,6 +622,9 @@ class TextureVk : public TextureImpl, public angle::ObserverInterface
     // |mSampler| contains the relevant Vulkan sampler states representing the OpenGL Texture
     // sampling states for the Texture.
     vk::SamplerBinding mSampler;
+    // |mY2YSampler| contains a version of mSampler that is meant for use with
+    // __samplerExternal2DY2YEXT (i.e., skipping conversion of YUV to RGB).
+    vk::SamplerBinding mY2YSampler;
 
     // The created vkImage usage flag.
     VkImageUsageFlags mImageUsageFlags;
@@ -570,6 +651,13 @@ class TextureVk : public TextureImpl, public angle::ObserverInterface
     // Saved between updates.
     gl::LevelIndex mCurrentBaseLevel;
     gl::LevelIndex mCurrentMaxLevel;
+
+    // Cached subresource indexes.
+    vk::ImageOrBufferViewSubresourceSerial mCachedImageViewSubresourceSerialSRGBDecode;
+    vk::ImageOrBufferViewSubresourceSerial mCachedImageViewSubresourceSerialSkipDecode;
+
+    // Manages the texture descriptor set cache that created with this texture
+    vk::DescriptorSetCacheManager mDescriptorSetCacheManager;
 };
 
 }  // namespace rx
