@@ -13,6 +13,14 @@ Pixel 6 (ARM based) specific script that measures the following for each restric
 - CPU power per frame
 - GPU memory per frame
 
+Setup:
+
+  autoninja -C out/<config> angle_trace_perf_tests angle_apks
+  adb install -r --force-queryable ./out/<config>/apks/AngleLibraries.apk
+  adb install -r out/<config>/angle_trace_tests_apk/angle_trace_tests-debug.apk
+  (cd out/<config>; ../../src/tests/run_angle_android_test.py angle_trace_tests \
+   --verbose --local-output --verbose-logging --max-steps-performed 1 --log=debug)
+
 Recommended command to run:
 
   python3 restricted_trace_perf.py --fixedtime 10 --power --output-tag android.$(date '+%Y%m%d') --loop-count 5
@@ -195,12 +203,11 @@ def run_trace(trace, args):
     if args.maxsteps != '':
         adb_command += '--max-steps-performed\ ' + args.maxsteps + '\ '
     if args.fixedtime != '':
-        adb_command += '--fixed-test-time\ ' + args.fixedtime + '\ '
+        adb_command += '--fixed-test-time-with-warmup\ ' + args.fixedtime + '\ '
     if args.minimizegpuwork:
         adb_command += '--minimize-gpu-work\ '
     adb_command += '--verbose\ '
-    adb_command += '--verbose-logging\ '
-    adb_command += '--warmup-trials\ 1\"\ '
+    adb_command += '--verbose-logging\"\ '
     adb_command += '-e org.chromium.native_test.NativeTestInstrumentationTestRunner.ShardNanoTimeout "1000000000000000000" '
     adb_command += '-e org.chromium.native_test.NativeTestInstrumentationTestRunner.NativeTestActivity com.android.angle.test.AngleUnitTestActivity '
     adb_command += 'com.android.angle.test/org.chromium.build.gtest_apk.NativeTestInstrumentationTestRunner'
@@ -571,7 +578,10 @@ def main():
     parser.add_argument('-f', '--filter', help='Trace filter. Defaults to all.', default='*')
     parser.add_argument('-l', '--log', help='Logging level.', default=DEFAULT_LOG_LEVEL)
     parser.add_argument(
-        '--renderer', help='Which renderer to use: native, vulkan, or both.', default='both')
+        '--renderer',
+        help='Which renderer to use: native, vulkan (via ANGLE), or default (' +
+        'GLES driver selected by system). Providing no option will run twice, native and vulkan',
+        default='both')
     parser.add_argument(
         '--walltimeonly',
         help='Limit output to just wall time',
@@ -702,12 +712,21 @@ def main():
             run_adb_command(
                 'shell settings put global angle_gl_driver_selection_pkgs com.android.angle.test')
             run_adb_command('shell settings put global angle_gl_driver_selection_values native')
-        else:
+        elif renderer == "vulkan":
             # Force the settings to ANGLE
             run_adb_command('shell settings put global angle_debug_package org.chromium.angle')
             run_adb_command(
                 'shell settings put global angle_gl_driver_selection_pkgs com.android.angle.test')
             run_adb_command('shell settings put global angle_gl_driver_selection_values angle')
+        elif renderer == "default":
+            logging.info('Deleting Android settings for forcing selection of GLES driver, ' +
+                         'allowing system to load the default')
+            run_adb_command('shell settings delete global angle_debug_package')
+            run_adb_command('shell settings delete global angle_gl_driver_selection_pkgs')
+            run_adb_command('shell settings delete global angle_gl_driver_selection_values')
+        else:
+            logging.error('Unsupported renderer {}'.format(renderer))
+            exit()
 
         for i in range(int(args.loop_count)):
             print("\nStarting run %i with %s at %s\n" %
