@@ -24,7 +24,6 @@ void UpdateResourceMap(GLuint *resourceMap, GLuint id, GLsizei readBufferOffset)
 
 DecompressCallback gDecompressCallback;
 DeleteCallback gDeleteCallback;
-std::string gBinaryDataDir = ".";
 
 void DeleteBinaryData()
 {
@@ -134,9 +133,9 @@ void UpdateUniformLocation(GLuint program, const char *name, GLint location, GLi
     {
         programLocations.resize(location + count, 0);
     }
+    GLuint mappedProgramID = gShaderProgramMap[program];
     for (GLint arrayIndex = 0; arrayIndex < count; ++arrayIndex)
     {
-        GLuint mappedProgramID = gShaderProgramMap[program];
         programLocations[location + arrayIndex] =
             glGetUniformLocation(mappedProgramID, name) + arrayIndex;
     }
@@ -156,7 +155,8 @@ void UpdateUniformBlockIndex(GLuint program, const char *name, GLuint index)
 void UniformBlockBinding(GLuint program, GLuint uniformblockIndex, GLuint binding)
 {
     glUniformBlockBinding(gShaderProgramMap[program],
-                          gUniformBlockIndexes[program][uniformblockIndex], binding);
+                          gUniformBlockIndexes[gShaderProgramMap[program]][uniformblockIndex],
+                          binding);
 }
 
 void UpdateCurrentProgram(GLuint program)
@@ -167,6 +167,7 @@ void UpdateCurrentProgram(GLuint program)
 uint8_t *gBinaryData;
 uint8_t *gReadBuffer;
 uint8_t *gClientArrays[kMaxClientArrays];
+GLuint *gResourceIDBuffer;
 SyncResourceMap gSyncMap;
 ContextMap gContextMap;
 GLuint gShareContextId;
@@ -193,6 +194,10 @@ SurfaceMap gSurfaceMap;
 GLeglImageOES *gEGLImageMap2;
 EGLSurface *gSurfaceMap2;
 EGLContext *gContextMap2;
+GLsync *gSyncMap2;
+EGLSync *gEGLSyncMap;
+
+std::string gBinaryDataDir = ".";
 
 void SetBinaryDataDecompressCallback(DecompressCallback decompressCallback,
                                      DeleteCallback deleteCallback)
@@ -219,6 +224,71 @@ GLuint *AllocateZeroedUints(size_t count)
     return AllocateZeroedValues<GLuint>(count);
 }
 
+void InitializeReplay4(const char *binaryDataFileName,
+                       size_t maxClientArraySize,
+                       size_t readBufferSize,
+                       size_t resourceIDBufferSize,
+                       GLuint contextId,
+                       uint32_t maxBuffer,
+                       uint32_t maxContext,
+                       uint32_t maxFenceNV,
+                       uint32_t maxFramebuffer,
+                       uint32_t maxImage,
+                       uint32_t maxMemoryObject,
+                       uint32_t maxProgramPipeline,
+                       uint32_t maxQuery,
+                       uint32_t maxRenderbuffer,
+                       uint32_t maxSampler,
+                       uint32_t maxSemaphore,
+                       uint32_t maxShaderProgram,
+                       uint32_t maxSurface,
+                       uint32_t maxSync,
+                       uint32_t maxTexture,
+                       uint32_t maxTransformFeedback,
+                       uint32_t maxVertexArray,
+                       GLuint maxEGLSyncID)
+{
+    InitializeReplay3(binaryDataFileName, maxClientArraySize, readBufferSize, resourceIDBufferSize,
+                      contextId, maxBuffer, maxContext, maxFenceNV, maxFramebuffer, maxImage,
+                      maxMemoryObject, maxProgramPipeline, maxQuery, maxRenderbuffer, maxSampler,
+                      maxSemaphore, maxShaderProgram, maxSurface, maxSync, maxTexture,
+                      maxTransformFeedback, maxVertexArray);
+    gEGLSyncMap = AllocateZeroedValues<EGLSync>(maxEGLSyncID);
+}
+
+void InitializeReplay3(const char *binaryDataFileName,
+                       size_t maxClientArraySize,
+                       size_t readBufferSize,
+                       size_t resourceIDBufferSize,
+                       GLuint contextId,
+                       uint32_t maxBuffer,
+                       uint32_t maxContext,
+                       uint32_t maxFenceNV,
+                       uint32_t maxFramebuffer,
+                       uint32_t maxImage,
+                       uint32_t maxMemoryObject,
+                       uint32_t maxProgramPipeline,
+                       uint32_t maxQuery,
+                       uint32_t maxRenderbuffer,
+                       uint32_t maxSampler,
+                       uint32_t maxSemaphore,
+                       uint32_t maxShaderProgram,
+                       uint32_t maxSurface,
+                       uint32_t maxSync,
+                       uint32_t maxTexture,
+                       uint32_t maxTransformFeedback,
+                       uint32_t maxVertexArray)
+{
+    InitializeReplay2(binaryDataFileName, maxClientArraySize, readBufferSize, contextId, maxBuffer,
+                      maxContext, maxFenceNV, maxFramebuffer, maxImage, maxMemoryObject,
+                      maxProgramPipeline, maxQuery, maxRenderbuffer, maxSampler, maxSemaphore,
+                      maxShaderProgram, maxSurface, maxTexture, maxTransformFeedback,
+                      maxVertexArray);
+
+    gSyncMap2         = AllocateZeroedValues<GLsync>(maxSync);
+    gResourceIDBuffer = AllocateZeroedUints(resourceIDBufferSize);
+}
+
 void InitializeReplay2(const char *binaryDataFileName,
                        size_t maxClientArraySize,
                        size_t readBufferSize,
@@ -307,6 +377,7 @@ void FinishReplay()
         delete[] clientArray;
     }
     delete[] gReadBuffer;
+    delete[] gResourceIDBuffer;
 
     delete[] gBufferMap;
     delete[] gContextMap2;
@@ -322,6 +393,7 @@ void FinishReplay()
     delete[] gSamplerMap;
     delete[] gSemaphoreMap;
     delete[] gSurfaceMap2;
+    delete[] gSyncMap2;
     delete[] gTransformFeedbackMap;
     delete[] gVertexArrayMap;
 
@@ -333,6 +405,19 @@ void SetValidateSerializedStateCallback(ValidateSerializedStateCallback callback
     gValidateSerializedStateCallback = callback;
 }
 
+std::vector<std::string> gTraceFiles;
+std::string gTraceGzPath;
+
+void SetTraceInfo(const std::vector<std::string> &traceFiles)
+{
+    gTraceFiles = traceFiles;
+}
+
+void SetTraceGzPath(const std::string &traceGzPath)
+{
+    gTraceGzPath = traceGzPath;
+}
+
 void UpdateClientArrayPointer(int arrayIndex, const void *data, uint64_t size)
 {
     memcpy(gClientArrays[arrayIndex], data, static_cast<size_t>(size));
@@ -353,6 +438,11 @@ void UpdateClientBufferDataWithOffset(GLuint bufferID,
     memcpy(reinterpret_cast<void *>(dest), source, size);
 }
 
+void UpdateResourceIDBuffer(int resourceIndex, GLuint id)
+{
+    gResourceIDBuffer[resourceIndex] = id;
+}
+
 void UpdateBufferID(GLuint id, GLsizei readBufferOffset)
 {
     UpdateResourceMap(gBufferMap, id, readBufferOffset);
@@ -497,6 +587,11 @@ void FenceSync(GLenum condition, GLbitfield flags, uintptr_t fenceSync)
     gSyncMap[fenceSync] = glFenceSync(condition, flags);
 }
 
+void FenceSync2(GLenum condition, GLbitfield flags, uintptr_t fenceSync)
+{
+    gSyncMap2[fenceSync] = glFenceSync(condition, flags);
+}
+
 void CreateEGLImage(EGLDisplay dpy,
                     EGLContext ctx,
                     EGLenum target,
@@ -519,6 +614,16 @@ void CreateEGLImageKHR(EGLDisplay dpy,
     gEGLImageMap2[imageID]       = eglCreateImageKHR(dpy, ctx, target, clientBuffer, attrib_list);
 }
 
+void CreateEGLSyncKHR(EGLDisplay dpy, EGLenum type, const EGLint *attrib_list, GLuint syncID)
+{
+    gEGLSyncMap[syncID] = eglCreateSyncKHR(dpy, type, attrib_list);
+}
+
+void CreateEGLSync(EGLDisplay dpy, EGLenum type, const EGLAttrib *attrib_list, GLuint syncID)
+{
+    gEGLSyncMap[syncID] = eglCreateSync(dpy, type, attrib_list);
+}
+
 void CreatePbufferSurface(EGLDisplay dpy,
                           EGLConfig config,
                           const EGLint *attrib_list,
@@ -539,6 +644,11 @@ void CreateContext(GLuint contextID)
     gContextMap2[contextID] = context;
 }
 
+void SetCurrentContextID(GLuint id)
+{
+    gContextMap2[id] = eglGetCurrentContext();
+}
+
 ANGLE_REPLAY_EXPORT PFNEGLCREATEIMAGEPROC r_eglCreateImage;
 ANGLE_REPLAY_EXPORT PFNEGLCREATEIMAGEKHRPROC r_eglCreateImageKHR;
 ANGLE_REPLAY_EXPORT PFNEGLDESTROYIMAGEPROC r_eglDestroyImage;
