@@ -997,7 +997,7 @@
 
 #if !defined(ENABLE_IMAGE_ANALYSIS_ENHANCEMENTS) \
     && ((PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 130000) \
-    || ((PLATFORM(IOS) || PLATFORM(MACCATALYST)) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 160000))
+    || (PLATFORM(IOS) || PLATFORM(MACCATALYST)))
 #define ENABLE_IMAGE_ANALYSIS_ENHANCEMENTS 1
 #endif
 
@@ -1006,3 +1006,8 @@
     || PLATFORM(VISION)
 #define ENABLE_ACCESSIBILITY_ANIMATION_CONTROL 1
 #endif
+
+#if !defined(ENABLE_ALTERNATIVE_TEXT_REQUIRES_AUTOMATIC_SPELLING_CORRECTION) \
+    && (PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED < 140000)
+#define ENABLE_ALTERNATIVE_TEXT_REQUIRES_AUTOMATIC_SPELLING_CORRECTION 1
+#endif
