@@ -87,7 +87,7 @@
 #define ENABLE_APPLE_PAY_INSTALLMENTS 1
 #endif
 
-#if !defined(ENABLE_APPLE_PAY_LATER) && ((PLATFORM(IOS) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 160000)  || PLATFORM(VISION))
+#if !defined(ENABLE_APPLE_PAY_LATER) && (PLATFORM(IOS)  || PLATFORM(VISION))
 #define ENABLE_APPLE_PAY_LATER 1
 #endif
 
@@ -139,7 +139,7 @@
 
 #endif // !PLATFORM(MAC) && !PLATFORM(MACCATALYST) && !PLATFORM(WATCHOS) && !PLATFORM(APPLETV)
 
-#if !defined(ENABLE_APPLE_PAY_SESSION_V11) && ((PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 110300) || (PLATFORM(IOS) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 140500) || PLATFORM(VISION))
+#if !defined(ENABLE_APPLE_PAY_SESSION_V11) && (PLATFORM(MAC) || PLATFORM(IOS) || PLATFORM(VISION))
 #define ENABLE_APPLE_PAY_SESSION_V11 1
 #endif
 
@@ -151,7 +151,7 @@
 #define ENABLE_APPLE_PAY_UPDATE_SHIPPING_METHODS_WHEN_CHANGING_LINE_ITEMS 1
 #endif
 
-#if !defined(ENABLE_APPLE_PAY_AMS_UI) && ((PLATFORM(MAC) && __MAC_OS_X_VERSION_MAX_ALLOWED >= 120300) || (PLATFORM(IOS) && __IPHONE_OS_VERSION_MAX_ALLOWED >= 150400) || PLATFORM(VISION))
+#if !defined(ENABLE_APPLE_PAY_AMS_UI) && (PLATFORM(MAC) || PLATFORM(IOS) || PLATFORM(VISION))
 #define ENABLE_APPLE_PAY_AMS_UI 1
 #endif
 
@@ -199,7 +199,7 @@
 #define ENABLE_CACHE_PARTITIONING 1
 #endif
 
-#if !defined(ENABLE_CFPREFS_DIRECT_MODE) && (PLATFORM(IOS_FAMILY) || PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 110000)
+#if !defined(ENABLE_CFPREFS_DIRECT_MODE)
 #define ENABLE_CFPREFS_DIRECT_MODE 1
 #endif
 
@@ -211,7 +211,8 @@
 #define ENABLE_CONTENT_EXTENSIONS 1
 #endif
 
-#if !defined(ENABLE_CONTENT_FILTERING) && !(PLATFORM(WATCHOS) && PLATFORM(IOS_FAMILY_SIMULATOR) && __WATCH_OS_VERSION_MIN_REQUIRED < 70000) && !PLATFORM(APPLETV)
+// FIXME: Why not APPLETV?
+#if !defined(ENABLE_CONTENT_FILTERING) && !PLATFORM(APPLETV)
 #define ENABLE_CONTENT_FILTERING 1
 #endif
 
@@ -263,7 +264,7 @@
 #define ENABLE_DATALIST_ELEMENT 1
 #endif
 
-#if !defined(ENABLE_DATA_DETECTION) && (PLATFORM(MAC) || PLATFORM(IOS) || (PLATFORM(MACCATALYST) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 150000) || PLATFORM(VISION))
+#if !defined(ENABLE_DATA_DETECTION) && (PLATFORM(MAC) || PLATFORM(IOS) || PLATFORM(MACCATALYST) || PLATFORM(VISION))
 #define ENABLE_DATA_DETECTION 1
 #endif
 
@@ -320,16 +321,10 @@
 #endif
 
 #if !defined(ENABLE_GPU_PROCESS_BY_DEFAULT)
-#if PLATFORM(MAC) \
-    || ((PLATFORM(IOS) || PLATFORM(MACCATALYST)) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 150000) \
-    || (PLATFORM(WATCHOS) && __WATCH_OS_VERSION_MIN_REQUIRED >= 80000) \
-    || (PLATFORM(APPLETV) && __TV_OS_VERSION_MIN_REQUIRED >= 150000) \
-    || PLATFORM(VISION)
 #define ENABLE_GPU_PROCESS_BY_DEFAULT 1
 #endif
-#endif
 
-#if !defined(ENABLE_GPU_DRIVER_PREWARMING) && PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101600 && __MAC_OS_X_VERSION_MIN_REQUIRED < 140000
+#if !defined(ENABLE_GPU_DRIVER_PREWARMING) && PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED < 140000
 #define ENABLE_GPU_DRIVER_PREWARMING 1
 #endif
 
@@ -409,15 +404,15 @@
 #define ENABLE_MEDIA_CONTROLS_SCRIPT 1
 #endif
 
-#if !defined(ENABLE_MEDIA_CONTROLS_CONTEXT_MENUS) && (PLATFORM(MAC) || ((PLATFORM(IOS) || PLATFORM(MACCATALYST) || PLATFORM(VISION)) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 150000))
+#if !defined(ENABLE_MEDIA_CONTROLS_CONTEXT_MENUS) && (PLATFORM(MAC) || PLATFORM(IOS) || PLATFORM(MACCATALYST) || PLATFORM(VISION))
 #define ENABLE_MEDIA_CONTROLS_CONTEXT_MENUS 1
 #endif
 
-#if !defined(ENABLE_MEDIA_SESSION_COORDINATOR) && PLATFORM(MAC) && __MAC_OS_X_VERSION_MAX_ALLOWED >= 120000
+#if !defined(ENABLE_MEDIA_SESSION_COORDINATOR) && PLATFORM(MAC)
 #define ENABLE_MEDIA_SESSION_COORDINATOR 1
 #endif
 
-#if !defined(ENABLE_MEDIA_SESSION_PLAYLIST) && PLATFORM(MAC) && __MAC_OS_X_VERSION_MAX_ALLOWED >= 120000
+#if !defined(ENABLE_MEDIA_SESSION_PLAYLIST) && PLATFORM(MAC)
 #define ENABLE_MEDIA_SESSION_PLAYLIST 1
 #endif
 
@@ -445,7 +440,7 @@
 #define ENABLE_MEDIA_RECORDER 1
 #endif
 
-#if !defined(ENABLE_MEDIA_USAGE) && ((PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 110000) || (PLATFORM(MACCATALYST) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 140000))
+#if !defined(ENABLE_MEDIA_USAGE) && (PLATFORM(MAC) || PLATFORM(MACCATALYST))
 #define ENABLE_MEDIA_USAGE 1
 #endif
 
@@ -540,7 +535,7 @@
 #endif
 
 
-#if !defined(ENABLE_POST_EDITING_GRAMMAR_CHECKING) && ((PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 130000) || (PLATFORM(IOS) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 160000) || PLATFORM(VISION))
+#if !defined(ENABLE_POST_EDITING_GRAMMAR_CHECKING) && ((PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 130000) || PLATFORM(IOS) || PLATFORM(VISION))
 #define ENABLE_POST_EDITING_GRAMMAR_CHECKING 1
 #endif
 
@@ -565,8 +560,8 @@
 #endif
 
 #if ((PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 130000) \
-    || (PLATFORM(IOS) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 160000) \
-    || (PLATFORM(MACCATALYST) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 160000) \
+    || PLATFORM(IOS) \
+    || PLATFORM(MACCATALYST) \
     || PLATFORM(VISION))
 #define ENABLE_JS_COOKIE_CHECKING 1
 #endif
@@ -583,7 +578,7 @@
 #define ENABLE_REVEAL 1
 #endif
 
-#if !defined(ENABLE_ROUTING_ARBITRATION) && (PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 110000)
+#if !defined(ENABLE_ROUTING_ARBITRATION) && PLATFORM(MAC)
 #define ENABLE_ROUTING_ARBITRATION 1
 #endif
 
@@ -685,7 +680,7 @@
 #define HAVE_PIP_CONTROLLER 1
 #endif
 
-#if !defined(ENABLE_VP9) && ((PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 110000) || PLATFORM(IOS) || PLATFORM(IOS_SIMULATOR) || PLATFORM(VISION))
+#if !defined(ENABLE_VP9) && (PLATFORM(MAC) || PLATFORM(IOS) || PLATFORM(IOS_SIMULATOR) || PLATFORM(VISION))
 #define ENABLE_VP9 1
 #endif
 
@@ -761,23 +756,24 @@
 #define ENABLE_WIRELESS_PLAYBACK_TARGET_AVAILABILITY_API 1
 #endif
 
-#if !defined(ENABLE_WKPDFVIEW) && (PLATFORM(IOS) || (PLATFORM(MACCATALYST) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 140000) || PLATFORM(VISION))
+#if !defined(ENABLE_WKPDFVIEW) && (PLATFORM(IOS) || PLATFORM(MACCATALYST) || PLATFORM(VISION))
 #define ENABLE_WKPDFVIEW 1
 #endif
 
-#if !defined(ENABLE_VORBIS) && (PLATFORM(MAC) && __MAC_OS_X_VERSION_MAX_ALLOWED >= 110300 || PLATFORM(IOS) || PLATFORM(IOS_SIMULATOR) || PLATFORM(VISION))
+#if !defined(ENABLE_VORBIS) && (PLATFORM(MAC) || PLATFORM(IOS) || PLATFORM(VISION))
 #define ENABLE_VORBIS 1
 #endif
 
-#if !defined(ENABLE_SET_WEBCONTENT_PROCESS_INFORMATION_IN_NETWORK_PROCESS) && PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 120000
+#if !defined(ENABLE_SET_WEBCONTENT_PROCESS_INFORMATION_IN_NETWORK_PROCESS) && PLATFORM(MAC)
 #define ENABLE_SET_WEBCONTENT_PROCESS_INFORMATION_IN_NETWORK_PROCESS 1
 #endif
 
-#if !defined(ENABLE_WEBM_FORMAT_READER) && PLATFORM(MAC) && __MAC_OS_X_VERSION_MAX_ALLOWED >= 110300
+#if !defined(ENABLE_WEBM_FORMAT_READER) && PLATFORM(MAC)
 #define ENABLE_WEBM_FORMAT_READER 1
 #endif
 
-#if !defined(ENABLE_OPUS) && (PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 120000 || PLATFORM(IOS) || PLATFORM(IOS_SIMULATOR) || PLATFORM(VISION))
+// FIXME: Is it intentional that tvOS and macCatalyst are omitted?
+#if !defined(ENABLE_OPUS) && (PLATFORM(MAC) || PLATFORM(IOS) || PLATFORM(VISION))
 #define ENABLE_OPUS 1
 #endif
 
@@ -785,18 +781,17 @@
 #define ENABLE_SEPARATED_MODEL 1
 #endif
 
-#if !defined(ENABLE_NOTIFY_FILTERING) && PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 120000
+#if !defined(ENABLE_NOTIFY_FILTERING) && PLATFORM(MAC)
 #define ENABLE_NOTIFY_FILTERING 1
 #endif
 
-#if !defined(ENABLE_APP_PRIVACY_REPORT) && PLATFORM(IOS_FAMILY) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 150000
+// FIXME: Should this be disabled on tvOS and watchOS?
+#if !defined(ENABLE_APP_PRIVACY_REPORT) && PLATFORM(IOS_FAMILY) && !PLATFORM(APPLETV) && !PLATFORM(WATCHOS)
 #define ENABLE_APP_PRIVACY_REPORT 1
 #endif
 
-#if (!defined(ENABLE_APP_HIGHLIGHTS) \
-    && ((PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 120000) \
-        || (PLATFORM(IOS) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 150000) \
-        || PLATFORM(VISION)))
+#if !defined(ENABLE_APP_HIGHLIGHTS) \
+    && (PLATFORM(MAC) || PLATFORM(IOS) || PLATFORM(VISION))
 #define ENABLE_APP_HIGHLIGHTS 1
 #endif
 
@@ -832,7 +827,7 @@
 #define ENABLE_ARKIT_INLINE_PREVIEW 1
 #endif
 
-#if (ENABLE(ARKIT_INLINE_PREVIEW_MAC) && __MAC_OS_X_VERSION_MAX_ALLOWED >= 120300) || (ENABLE(ARKIT_INLINE_PREVIEW_IOS) && __IPHONE_OS_VERSION_MAX_ALLOWED >= 150400)
+#if ENABLE(ARKIT_INLINE_PREVIEW_MAC) || ENABLE(ARKIT_INLINE_PREVIEW_IOS)
 #define ENABLE_ARKIT_INLINE_PREVIEW_CAMERA_TRANSFORM 1
 #define ENABLE_ARKIT_INLINE_PREVIEW_ANIMATIONS_CONTROL 1
 #define ENABLE_ARKIT_INLINE_PREVIEW_AUDIO_CONTROL 1
@@ -852,7 +847,7 @@
 
 #if !defined(ENABLE_WEBCONTENT_CRASH_TESTING) && USE(APPLE_INTERNAL_SDK) \
     && ((PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 130000) \
-        || (PLATFORM(IOS) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 160000) \
+        || PLATFORM(IOS) \
         || PLATFORM(VISION)) \
     && HAVE(CHIRP_SPI)
 #define ENABLE_WEBCONTENT_CRASH_TESTING 1
@@ -866,7 +861,7 @@
 #define ENABLE_MOBILE_GESTALT_DEVICE_NAME 1
 #endif
 
-#if !defined(ENABLE_LOCKDOWN_MODE_TELEMETRY) && PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED > 120000
+#if !defined(ENABLE_LOCKDOWN_MODE_TELEMETRY) && PLATFORM(MAC)
 #define ENABLE_LOCKDOWN_MODE_TELEMETRY 1
 #endif
 
