@@ -105,10 +105,7 @@
 #endif
 #endif
 
-#if PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED < 120000
-#define HAVE_MISSING_STD_FILESYSTEM_PATH_CONSTRUCTOR 1
-#define HAVE_CPP20_INCOMPATIBLE_INTERNAL_HEADERS 1
-#elif PLATFORM(PLAYSTATION)
+#if PLATFORM(PLAYSTATION)
 #define HAVE_MISSING_STD_FILESYSTEM_PATH_CONSTRUCTOR 1
 #endif
 
@@ -244,7 +241,7 @@
 #if __has_include(<System/pthread_machdep.h>)
 #define HAVE_FAST_TLS 1
 
-#if (PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 120000) \
+#if PLATFORM(MAC) \
     || ((PLATFORM(IOS) || PLATFORM(VISION)) && !PLATFORM(IOS_FAMILY_SIMULATOR) && !PLATFORM(MACCATALYST))
 #define HAVE_SIMPLIFIED_FAST_TLS_BASE 1
 #endif
@@ -283,7 +280,8 @@
 #define HAVE_PARENTAL_CONTROLS_WITH_UNBLOCK_HANDLER 1
 #endif
 
-#if PLATFORM(COCOA) && (!PLATFORM(APPLETV) && (!PLATFORM(WATCHOS) || !PLATFORM(IOS_FAMILY_SIMULATOR) || __WATCH_OS_VERSION_MIN_REQUIRED >= 70000))
+// FIXME: Why not APPLETV?
+#if PLATFORM(COCOA) && !PLATFORM(APPLETV)
 #define HAVE_PARENTAL_CONTROLS 1
 #endif
 
@@ -335,30 +333,23 @@
 #define HAVE_IOSURFACE_SET_OWNERSHIP 1
 #endif
 
-#if (PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 120000) \
+#if PLATFORM(MAC) \
     || (PLATFORM(IOS_FAMILY) && !PLATFORM(IOS_FAMILY_SIMULATOR))
 // FIXME: Is this really unavailable in the iOS Simulator?
 #define HAVE_IOSURFACE_SET_OWNERSHIP_IDENTITY 1
 #endif
 
-#if (PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 120000) \
+#if PLATFORM(MAC) \
     || (PLATFORM(IOS) || PLATFORM(MACCATALYST) || PLATFORM(VISION))
 #define HAVE_NETWORK_CONNECTION_PRIVACY_STANCE 1
 #endif
 
-#if (PLATFORM(MAC) && __MAC_OS_X_VERSION_MAX_ALLOWED >= 110300) \
-    || (((PLATFORM(IOS) && !PLATFORM(IOS_FAMILY_SIMULATOR)) || PLATFORM(MACCATALYST)) && __IPHONE_OS_VERSION_MAX_ALLOWED >= 140500) \
-    || (PLATFORM(WATCHOS) && !PLATFORM(IOS_FAMILY_SIMULATOR) && __WATCH_OS_VERSION_MAX_ALLOWED >= 70500) \
-    || (PLATFORM(APPLETV) && !PLATFORM(IOS_FAMILY_SIMULATOR) && __TV_OS_VERSION_MAX_ALLOWED >= 140500) \
-    || (PLATFORM(VISION) && !PLATFORM(IOS_FAMILY_SIMULATOR))
+#if PLATFORM(COCOA) && !PLATFORM(IOS_FAMILY_SIMULATOR)
 #define HAVE_TASK_IDENTITY_TOKEN 1
 #endif
 
 #if (PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 130000) \
-    || (((PLATFORM(IOS) && !PLATFORM(IOS_FAMILY_SIMULATOR)) || PLATFORM(MACCATALYST)) && __IPHONE_OS_VERSION_MAX_ALLOWED >= 160000) \
-    || ((PLATFORM(WATCHOS) && !PLATFORM(IOS_FAMILY_SIMULATOR)) && __WATCH_OS_VERSION_MAX_ALLOWED >= 90000) \
-    || ((PLATFORM(APPLETV) && !PLATFORM(IOS_FAMILY_SIMULATOR)) && __TV_OS_VERSION_MAX_ALLOWED >= 160000) \
-    || (PLATFORM(VISION) && !PLATFORM(IOS_FAMILY_SIMULATOR))
+    || (PLATFORM(IOS_FAMILY) && !PLATFORM(IOS_FAMILY_SIMULATOR))
 #define HAVE_CG_CONTEXT_SET_OWNER_IDENTITY 1
 #endif
 
@@ -366,11 +357,6 @@
 #define HAVE_NS_ACTIVITY 1
 #endif
 
-#if (PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 120000) \
-    || (PLATFORM(IOS_FAMILY) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 150000)
-#define HAVE_SEC_TRUST_SET_CLIENT_AUDIT_TOKEN 1
-#endif
-
 #if PLATFORM(MAC)
 #define HAVE_TOUCH_BAR 1
 #endif
@@ -395,11 +381,7 @@
 #define HAVE_AVCONTENTKEYSESSION 1
 #endif
 
-#if (PLATFORM(MAC) && __MAC_OS_X_VERSION_MAX_ALLOWED >= 101504) \
-    || (((PLATFORM(IOS) && !PLATFORM(IOS_FAMILY_SIMULATOR)) || PLATFORM(MACCATALYST)) && __IPHONE_OS_VERSION_MAX_ALLOWED >= 130400) \
-    || (PLATFORM(WATCHOS) && !PLATFORM(IOS_FAMILY_SIMULATOR) && __WATCH_OS_VERSION_MAX_ALLOWED >= 60400) \
-    || (PLATFORM(APPLETV) && !PLATFORM(IOS_FAMILY_SIMULATOR) && __TV_OS_VERSION_MAX_ALLOWED >= 130400) \
-    || (PLATFORM(VISION) && !PLATFORM(IOS_FAMILY_SIMULATOR))
+#if !PLATFORM(IOS_FAMILY_SIMULATOR)
 #define HAVE_AVCONTENTKEYREPORTGROUP 1
 #endif
 
@@ -409,11 +391,7 @@
 #define HAVE_AVCONTENTKEYSESSIONWILLOUTPUTBEOBSCURED 1
 #endif
 
-#if (PLATFORM(MAC) && __MAC_OS_X_VERSION_MAX_ALLOWED >= 110300) \
-    || (((PLATFORM(IOS) && !PLATFORM(IOS_FAMILY_SIMULATOR)) || PLATFORM(MACCATALYST)) && __IPHONE_OS_VERSION_MAX_ALLOWED >= 140500) \
-    || (PLATFORM(WATCHOS) && !PLATFORM(IOS_FAMILY_SIMULATOR) && __WATCH_OS_VERSION_MAX_ALLOWED >= 70400) \
-    || (PLATFORM(APPLETV) && !PLATFORM(IOS_FAMILY_SIMULATOR) && __TV_OS_VERSION_MAX_ALLOWED >= 140500) \
-    || (PLATFORM(VISION) && !PLATFORM(IOS_FAMILY_SIMULATOR))
+#if PLATFORM(COCOA) && !PLATFORM(IOS_FAMILY_SIMULATOR)
 #define HAVE_AVCONTENTKEYSPECIFIER 1
 #endif
 
@@ -430,10 +408,6 @@
 #define HAVE_OS_DARK_MODE_SUPPORT 1
 #endif
 
-#if PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED < 110000
-#define HAVE_CG_FONT_RENDERING_GET_FONT_SMOOTHING_DISABLED 1
-#endif
-
 #if PLATFORM(MAC)
 #define HAVE_READ_ONLY_SYSTEM_VOLUME 1
 #endif
@@ -469,11 +443,6 @@
 #define HAVE_ALTERNATIVE_SERVICE 1
 #endif
 
-#if PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED < 120000
-#define HAVE_SAFARI_FOR_WEBKIT_DEVELOPMENT_REQUIRING_EXTRA_SYMBOLS 1
-#define HAVE_NSURLSESSION_EFFECTIVE_CONFIGURATION_OBJECT 1
-#endif
-
 #if PLATFORM(MAC)
 #define HAVE_CSCHECKFIXDISABLE 1
 #endif
@@ -490,21 +459,21 @@
 #define HAVE_UI_WK_DOCUMENT_CONTEXT 1
 #endif
 
-#if (PLATFORM(IOS) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 130400) || PLATFORM(VISION)
+#if PLATFORM(IOS) || PLATFORM(VISION)
 #define HAVE_UI_POINTER_INTERACTION 1
 #endif
 
-#if (PLATFORM(IOS) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 130400) || PLATFORM(MACCATALYST) || PLATFORM(VISION)
+#if PLATFORM(IOS) || PLATFORM(MACCATALYST) || PLATFORM(VISION)
 #define HAVE_UIKIT_WITH_MOUSE_SUPPORT 1
 #define HAVE_UI_PARALLAX_TRANSITION_GESTURE_RECOGNIZER 1
 #endif
 
-#if ((PLATFORM(IOS) || PLATFORM(MACCATALYST)) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 160000) || PLATFORM(VISION)
+#if PLATFORM(IOS) || PLATFORM(MACCATALYST) || PLATFORM(VISION)
 #define HAVE_UIKIT_HOVER_EVENT_PROTOCOL 1
 #define HAVE_UIKIT_BAR_BUTTON_LAYOUT_CUSTOMIZATION 1
 #endif
 
-#if PLATFORM(IOS) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 130400 || PLATFORM(VISION)
+#if PLATFORM(IOS) || PLATFORM(VISION)
 #define HAVE_MOUSE_DEVICE_OBSERVATION 1
 #endif
 
@@ -532,14 +501,10 @@
 #define HAVE_DEVICE_MANAGEMENT 1
 #endif
 
-#if (PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 120000) || PLATFORM(IOS_FAMILY)
+#if PLATFORM(MAC) || PLATFORM(IOS_FAMILY)
 #define HAVE_NSURLSESSION_WEBSOCKET 1
 #endif
 
-#if (PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 120000) || PLATFORM(IOS_FAMILY)
-#define HAVE_NWPARAMETERS_TRACKER_API 1
-#endif
-
 #if PLATFORM(COCOA) && !PLATFORM(MACCATALYST)
 #define HAVE_AVPLAYER_RESOURCE_CONSERVATION_LEVEL 1
 #endif
@@ -548,10 +513,6 @@
 #define HAVE_AVPLAYER_SUPRESSES_AUDIO_RENDERING 1
 #endif
 
-#if (PLATFORM(MAC) && __MAC_OS_X_VERSION_MAX_ALLOWED < 101404)
-#define HAVE_VIDEO_PERFORMANCE_METRICS 1
-#endif
-
 #if PLATFORM(IOS) || PLATFORM(MAC) || PLATFORM(VISION)
 #define HAVE_APP_SSO 1
 #endif
@@ -604,10 +565,15 @@
 #define HAVE_GCEXTENDEDGAMEPAD_HOME_BUTTON 1
 #endif
 
-#if (PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 120000)
+#if PLATFORM(MAC)
 #define HAVE_GCCONTROLLER_HID_DEVICE_CHECK 1
 #endif
 
+// Newer versions no longer square continuous haptics (rdar://110338126).
+#if (PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED < 140000)
+#define HAVE_GCCONTROLLER_REQUIRING_HAPTICS_SQUARING 1
+#endif
+
 #if PLATFORM(MAC)
 #define HAVE_INCREMENTAL_PDF_APIS 1
 #endif
@@ -616,7 +582,7 @@
 #define HAVE_ALTERNATE_ICONS 1
 #endif
 
-#if (PLATFORM(IOS) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 140000) || PLATFORM(VISION)
+#if PLATFORM(IOS) || PLATFORM(VISION)
 #define HAVE_UICONTEXTMENU_LOCATION 1
 #endif
 
@@ -637,7 +603,7 @@
 #define HAVE_DD_HIGHLIGHT_CREATE_WITH_SCALE 1
 #endif
 
-#if (PLATFORM(IOS) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 130400) || PLATFORM(VISION)
+#if PLATFORM(IOS) || PLATFORM(VISION)
 #define HAVE_UISCENE_BASED_VIEW_SERVICE_STATE_NOTIFICATIONS 1
 #endif
 
@@ -665,7 +631,7 @@
 #define HAVE_PASSKIT_NEW_BUTTON_TYPES 1
 #endif
 
-#if (PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 120000) || PLATFORM(IOS) || PLATFORM(VISION)
+#if PLATFORM(MAC) || PLATFORM(IOS) || PLATFORM(VISION)
 #define HAVE_PASSKIT_RECURRING_SUMMARY_ITEM 1
 #define HAVE_PASSKIT_DEFERRED_SUMMARY_ITEM 1
 #define HAVE_PASSKIT_SHIPPING_METHOD_DATE_COMPONENTS_RANGE 1
@@ -674,7 +640,7 @@
 #define HAVE_PASSKIT_UPDATE_SHIPPING_METHODS_WHEN_CHANGING_SUMMARY_ITEMS 1
 #endif
 
-#if (PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 130000) || (PLATFORM(IOS) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 160000) || PLATFORM(VISION)
+#if (PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 130000) || PLATFORM(IOS) || PLATFORM(VISION)
 #define HAVE_PASSKIT_MODULARIZATION 1
 #define HAVE_PASSKIT_AUTOMATIC_RELOAD_SUMMARY_ITEM 1
 #define HAVE_PASSKIT_RECURRING_PAYMENTS 1
@@ -691,7 +657,7 @@
 #define HAVE_PASSKIT_MAC_HELPER_TEMP 1
 #endif
 
-#if (PLATFORM(MAC) && __MAC_OS_X_VERSION_MAX_ALLOWED >= 120300) || (PLATFORM(IOS) && __IPHONE_OS_VERSION_MAX_ALLOWED >= 150400) || PLATFORM(VISION)
+#if PLATFORM(MAC) || PLATFORM(IOS) || PLATFORM(VISION)
 #define HAVE_PASSKIT_DEFAULT_SHIPPING_METHOD 1
 #endif
 
@@ -752,7 +718,7 @@
 #define HAVE_UI_EVENT_ATTRIBUTION 1
 #endif
 
-#if (PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 120000) \
+#if PLATFORM(MAC) \
     || PLATFORM(IOS_FAMILY)
 #define HAVE_NETWORK_LOADER 1
 #define HAVE_SEC_TRUST_COPY_CERTIFICATE_CHAIN 1
@@ -760,16 +726,15 @@
 #define HAVE_NSURL_EMPTY_PUNYCODE_CHECK 1
 #endif
 
-#if PLATFORM(MACCATALYST) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 140000
+#if PLATFORM(MACCATALYST)
 #define HAVE_CATALYST_USER_INTERFACE_IDIOM_AND_SCALE_FACTOR 1
 #endif
 
-#if ((PLATFORM(IOS) || PLATFORM(MACCATALYST)) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 140000) \
-    || PLATFORM(VISION)
+#if PLATFORM(IOS) || PLATFORM(MACCATALYST) || PLATFORM(VISION)
 #define HAVE_PENCILKIT_TEXT_INPUT 1
 #endif
 
-#if (PLATFORM(IOS) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 140000) || PLATFORM(VISION)
+#if PLATFORM(IOS) || PLATFORM(VISION)
 #define HAVE_STYLUS_DEVICE_OBSERVATION 1
 #endif
 
@@ -779,7 +744,7 @@
 #endif
 #endif
 
-#if ((PLATFORM(IOS) || PLATFORM(MACCATALYST)) && __IPHONE_OS_VERSION_MAX_ALLOWED >= 140300) || (PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 130000) || PLATFORM(VISION)
+#if PLATFORM(IOS) || PLATFORM(MACCATALYST) || (PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 130000) || PLATFORM(VISION)
 #if !defined(HAVE_AVSPEECHSYNTHESIS_SYSTEMVOICE)
 #define HAVE_AVSPEECHSYNTHESIS_SYSTEMVOICE 1
 #endif
@@ -792,7 +757,7 @@
 #endif
 
 #if COMPILER(GCC_COMPATIBLE) && defined(__has_attribute)
-#if __has_attribute(objc_direct) && (!PLATFORM(MAC) || __MAC_OS_X_VERSION_MIN_REQUIRED >= 120000)
+#if __has_attribute(objc_direct)
 #if !defined(HAVE_NS_DIRECT_SUPPORT)
 #define HAVE_NS_DIRECT_SUPPORT 1
 #endif
@@ -825,7 +790,7 @@
 #define HAVE_AVCAPTUREDEVICE 1
 #endif
 
-#if PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED > 110000 && defined __has_include && __has_include(<CoreFoundation/CFPriv.h>)
+#if PLATFORM(MAC) && defined __has_include && __has_include(<CoreFoundation/CFPriv.h>)
 #define HAVE_SANDBOX_MESSAGE_FILTERING 1
 #endif
 
@@ -837,20 +802,16 @@
 #define HAVE_APPLE_LOW_POWER_MODE_SUPPORT 1
 #endif
 
-#if (PLATFORM(MAC) && __MAC_OS_X_VERSION_MAX_ALLOWED >= 110200) \
-    || (PLATFORM(IOS) && __IPHONE_OS_VERSION_MAX_ALLOWED >= 140400) \
-    || (PLATFORM(WATCHOS) && __WATCH_OS_VERSION_MAX_ALLOWED >= 70300) \
-    || (PLATFORM(APPLETV) && __TV_OS_VERSION_MAX_ALLOWED >= 140400) \
-    || PLATFORM(VISION)
+#if PLATFORM(COCOA)
 #define HAVE_LIBXSLT_FIX_FOR_RADAR_71864140 1
 #endif
 
-#if (PLATFORM(IOS) && __IPHONE_OS_VERSION_MAX_ALLOWED >= 140400) || PLATFORM(VISION)
+#if PLATFORM(IOS) || PLATFORM(VISION)
 #define HAVE_ASC_AUTH_UI 1
 #endif
 
 #if PLATFORM(IOS) \
-    || (PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 120000) \
+    || PLATFORM(MAC) \
     || PLATFORM(VISION)
 #define HAVE_ASC_WEBKIT_SPI 1
 #endif
@@ -861,15 +822,13 @@
 #endif
 #endif
 
-#if PLATFORM(MAC) && __MAC_OS_X_VERSION_MAX_ALLOWED >= 110300
+#if PLATFORM(MAC)
 #if !defined(HAVE_MT_PLUGIN_SAMPLE_CURSOR_PLAYABLE_HORIZON)
 #define HAVE_MT_PLUGIN_SAMPLE_CURSOR_PLAYABLE_HORIZON 1
 #endif
 #endif
 
-#if (PLATFORM(MAC) && __MAC_OS_X_VERSION_MAX_ALLOWED >= 110200) \
-    || (PLATFORM(IOS) && __IPHONE_OS_VERSION_MAX_ALLOWED >= 140400) \
-    || PLATFORM(VISION)
+#if PLATFORM(MAC) || PLATFORM(IOS) || PLATFORM(VISION)
 #define HAVE_AVCONTENTKEYREQUEST_PENDING_PROTECTION_STATUS 1
 #endif
 
@@ -881,13 +840,6 @@
 #define HAVE_AVCONTENTKEYREQUEST_COMPATABILITIY_MODE 1
 #endif
 
-#if (PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 120000) \
-    || PLATFORM(IOS_FAMILY)
-#define HAVE_LEVEL_3_SYSTEM_FONT_WIDTH_VALUES 1
-#else
-#define HAVE_LEVEL_2_SYSTEM_FONT_WIDTH_VALUES 1
-#endif
-
 #if __has_include(<AVFoundation/AVPlayerInterstitialEventController.h>)
 #if !defined(HAVE_AVFOUNDATION_INTERSTITIAL_EVENTS)
 #define HAVE_AVFOUNDATION_INTERSTITIAL_EVENTS 1
@@ -898,7 +850,7 @@
 #define HAVE_PEPPER_UI_CORE 1
 #endif
 
-#if PLATFORM(WATCHOS) && __WATCH_OS_VERSION_MIN_REQUIRED >= 80000
+#if PLATFORM(WATCHOS)
 #define HAVE_QUICKBOARD_CONTROLLER 1
 #define HAVE_PUIC_BUTTON_TYPE_PILL 1
 #endif
@@ -920,13 +872,13 @@
 #define HAVE_CORE_GRAPHICS_CONIC_GRADIENTS 1
 #endif
 
-#if PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 120000
+#if PLATFORM(MAC)
 #define HAVE_STATIC_FONT_REGISTRY 1
 #define HAVE_DATA_DETECTORS_MAC_ACTION 1
 #define HAVE_NSSCROLLVIEW_SEPARATOR_TRACKING_ADAPTER 1
 #endif
 
-#if (PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 120000) \
+#if PLATFORM(MAC) \
     || PLATFORM(IOS) \
     || PLATFORM(VISION)
 // FIXME: Does this intend to exclude macCatalyst?
@@ -953,17 +905,7 @@
 #define HAVE_VISION 1
 #endif
 
-#if (PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 110000 && __MAC_OS_X_VERSION_MAX_ALLOWED >= 110300) \
-    || (PLATFORM(IOS_FAMILY) && __IPHONE_OS_VERSION_MAX_ALLOWED >= 140500) \
-    || (PLATFORM(APPLETV) && __TV_OS_VERSION_MAX_ALLOWED >= 140500) \
-    || (PLATFORM(WATCHOS) && __WATCH_OS_VERSION_MAX_ALLOWED >= 70400)
-#define HAVE_VM_FLAGS_PERMANENT 1
-#endif
-
-#if (PLATFORM(MAC) && __MAC_OS_X_VERSION_MAX_ALLOWED >= 120300) \
-    || (PLATFORM(IOS_FAMILY) && __IPHONE_OS_VERSION_MAX_ALLOWED >= 150300) \
-    || (PLATFORM(APPLETV) && __TV_OS_VERSION_MAX_ALLOWED >= 150300) \
-    || (PLATFORM(WATCHOS) && __WATCH_OS_VERSION_MAX_ALLOWED >= 80300)
+#if PLATFORM(COCOA)
 #define HAVE_RSA_PSS_OID 1
 #endif
 
@@ -971,7 +913,7 @@
 #define HAVE_VIDEO_RESTRICTED_DECODING 1
 #endif
 
-#if (PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 120000) \
+#if PLATFORM(MAC) \
     || PLATFORM(IOS_FAMILY)
 #define HAVE_CFNETWORK_NSURLSESSION_ATTRIBUTED_BUNDLE_IDENTIFIER 1
 #define HAVE_CFNETWORK_NSURLSESSION_HSTS_WITH_UNTRUSTED_ROOT 1
@@ -984,7 +926,7 @@
 
 #if ((PLATFORM(IOS) || PLATFORM(VISION)) && !PLATFORM(IOS_SIMULATOR)) \
     || PLATFORM(MACCATALYST) \
-    || (PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 120000)
+    || PLATFORM(MAC)
 #define HAVE_ASV_INLINE_PREVIEW 1
 #endif
 
@@ -997,7 +939,7 @@
 #endif
 
 #if (PLATFORM(IOS) || PLATFORM(MACCATALYST)) \
-    || (PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 120000) \
+    || PLATFORM(MAC) \
     || PLATFORM(VISION)
 #define HAVE_UPDATE_WEB_ACCESSIBILITY_SETTINGS 1
 #endif
@@ -1007,7 +949,7 @@
 #define HAVE_WOFF_SUPPORT 1
 #endif
 
-#if (PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 120000) \
+#if PLATFORM(MAC) \
     || (PLATFORM(IOS) || PLATFORM(MACCATALYST) || PLATFORM(VISION))
 #if !defined(HAVE_VK_IMAGE_ANALYSIS)
 #define HAVE_VK_IMAGE_ANALYSIS 1
@@ -1031,16 +973,12 @@
 #define HAVE_PER_APP_ACCESSIBILITY_PREFERENCES 1
 #endif
 
-#if !PLATFORM(MAC) || __MAC_OS_X_VERSION_MIN_REQUIRED >= 120000
-#define HAVE_APP_SPECIFIC_ACCESSIBILITY_SETTINGS 1
-#endif
-
 #if (PLATFORM(IOS_FAMILY) && !PLATFORM(IOS_FAMILY_SIMULATOR)) || PLATFORM(MACCATALYST) || PLATFORM(MAC)
 #define HAVE_PAC_SHARED_REGION_ID 1
 #endif
 
 #if !defined(HAVE_QUICKLOOK_PREVIEW_ITEM_DATA_PROVIDER) \
-    && (PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 120000)
+    && PLATFORM(MAC)
 #define HAVE_QUICKLOOK_PREVIEW_ITEM_DATA_PROVIDER 1
 #define HAVE_QUICKLOOK_ITEM_PREVIEW_OPTIONS 1
 #define HAVE_QUICKLOOK_PREVIEW_ACTIVITY 1
@@ -1054,32 +992,23 @@
 #define HAVE_SYMPTOMS_FRAMEWORK 1
 #endif
 
-#if !defined(HAVE_GROUP_ACTIVITIES) && PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 120000
+#if !defined(HAVE_GROUP_ACTIVITIES) && PLATFORM(MAC)
 #define HAVE_GROUP_ACTIVITIES 1
 #endif
 
-#if PLATFORM(IOS_FAMILY) || (PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 120000)
+#if PLATFORM(IOS_FAMILY) || PLATFORM(MAC)
 #define HAVE_WEBGL_COMPATIBLE_METAL 1
 #endif
 
-#if PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 120000
+#if PLATFORM(MAC)
 #define HAVE_CMPHOTO_TILE_DECODER_AVAILABLE 1
 #endif
 
-#if (PLATFORM(MAC) && __MAC_OS_X_VERSION_MAX_ALLOWED >= 110300 \
-    || (PLATFORM(IOS) && __IPHONE_OS_VERSION_MAX_ALLOWED >= 140500) \
-    || (PLATFORM(WATCHOS) && __WATCH_OS_VERSION_MAX_ALLOWED >= 70400) \
-    || (PLATFORM(APPLETV) && __TV_OS_VERSION_MAX_ALLOWED >= 140500) \
-    || (PLATFORM(MACCATALYST) && __IPHONE_OS_VERSION_MAX_ALLOWED >= 140500)) \
-    || PLATFORM(VISION)
-#define HAVE_AVSAMPLEBUFFERRENDERSYNCHRONIZER_RATEATHOSTTIME 1
-#endif
-
 #if (PLATFORM(IOS) || PLATFORM(VISION)) && !PLATFORM(IOS_FAMILY_SIMULATOR)
 #define HAVE_SYSTEM_STATUS 1
 #endif
 
-#if (PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 120000) \
+#if PLATFORM(MAC) \
     || (PLATFORM(IOS_FAMILY) && !PLATFORM(IOS_FAMILY_SIMULATOR))
 #define HAVE_CORE_ANIMATION_FIX_FOR_RADAR_78402035 1
 #endif
@@ -1102,34 +1031,25 @@
 #endif
 #endif
 
-#if (PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 120000) \
+#if PLATFORM(MAC) \
     || (PLATFORM(IOS_FAMILY) && !PLATFORM(IOS_FAMILY_SIMULATOR))
 #define HAVE_CVBUFFERCOPYATTACHMENTS 1
 #endif
 
-#if (PLATFORM(IOS_FAMILY) && !PLATFORM(MACCATALYST) && __IPHONE_OS_VERSION_MAX_ALLOWED >= 150400) \
-    || (PLATFORM(APPLETV) && __TV_OS_VERSION_MAX_ALLOWED >= 150400) \
-    || (PLATFORM(WATCHOS) && __WATCH_OS_VERSION_MAX_ALLOWED >= 80400)
+#if PLATFORM(IOS_FAMILY) && !PLATFORM(MACCATALYST)
 #define HAVE_AUDIOSESSION_PROCESSASSERTION 1
 #endif
 
-#if (PLATFORM(MAC) \
-    || ((PLATFORM(IOS) || PLATFORM(MACCATALYST)) && __IPHONE_OS_VERSION_MAX_ALLOWED >= 160000) \
-    || (PLATFORM(WATCHOS) && __WATCH_OS_VERSION_MAX_ALLOWED >= 90000) \
-    || (PLATFORM(APPLETV) && __TV_OS_VERSION_MAX_ALLOWED >= 160000) \
-    || PLATFORM(VISION))
-#define HAVE_NSTEXTLIST_MARKER_FORMATS 1
-#endif
-
-#if (PLATFORM(IOS) && __IPHONE_OS_VERSION_MAX_ALLOWED >= 160000) \
-    || (PLATFORM(WATCHOS) && __WATCH_OS_VERSION_MAX_ALLOWED >= 90000) \
-    || (PLATFORM(APPLETV) && __TV_OS_VERSION_MAX_ALLOWED >= 160000) \
+// FIXME: Why no catalyst?
+#if PLATFORM(IOS) \
+    || PLATFORM(WATCHOS) \
+    || PLATFORM(APPLETV) \
     || PLATFORM(VISION)
 #define HAVE_CORE_LOCATION_WEBSITE_IDENTIFIERS 1
 #define HAVE_UIKIT_BAR_BUTTON_LAYOUT_CUSTOMIZATION 1
 #endif
 
-#if (PLATFORM(IOS) || PLATFORM(MACCATALYST) || PLATFORM(VISION)) && __IPHONE_OS_VERSION_MAX_ALLOWED >= 160000
+#if PLATFORM(IOS) || PLATFORM(MACCATALYST) || PLATFORM(VISION)
 #if !defined(HAVE_TEXT_INTERACTION_WITH_CONTEXT_MENU_INTERACTION)
 #define HAVE_TEXT_INTERACTION_WITH_CONTEXT_MENU_INTERACTION 1
 #endif
@@ -1140,8 +1060,8 @@
 
 #if PLATFORM(COCOA)
 #define HAVE_CORE_LOCATION 1
-// kCLAuthorizationStatusAuthorizedWhenInUse is only available on macOS as part of the internal SDK.
-#if PLATFORM(IOS_FAMILY) || (defined __has_include && __has_include(<CoreFoundation/CFPriv.h>))
+// kCLAuthorizationStatusAuthorizedWhenInUse is SPI on macOS until 14.0.
+#if PLATFORM(IOS_FAMILY) || (defined __has_include && __has_include(<CoreFoundation/CFPriv.h>)) || __MAC_OS_X_VERSION_MIN_REQUIRED >= 140000
 #define HAVE_CORE_LOCATION_AUTHORIZED_WHEN_IN_USE 1
 #endif
 #endif
@@ -1158,7 +1078,7 @@
 #define HAVE_SHAPE_DETECTION_API_IMPLEMENTATION 1
 #endif
 
-#if HAVE(WEBGPU_IMPLEMENTATION) && ((PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 130000) || (PLATFORM(IOS) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 160000) || PLATFORM(VISION))
+#if HAVE(WEBGPU_IMPLEMENTATION) && ((PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 130000) || PLATFORM(IOS) || PLATFORM(VISION))
 #define HAVE_COREVIDEO_METAL_SUPPORT COREVIDEO_SUPPORTS_METAL
 #endif
 
@@ -1170,13 +1090,6 @@
 #define HAVE_UNIFORM_TYPE_IDENTIFIERS_FRAMEWORK 1
 #endif
 
-#if PLATFORM(COCOA)
-#define HAVE_AV_DELEGATING_PLAYBACK_COORDINATOR 1
-#endif
-#if PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED < 120000
-#undef HAVE_AV_DELEGATING_PLAYBACK_COORDINATOR
-#endif
-
 #if (PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 130000)
 #define HAVE_SCREEN_CAPTURE_KIT 1
 #endif
@@ -1186,17 +1099,17 @@
 #endif
 
 #if ((PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 130000) \
-    || (PLATFORM(IOS) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 160000) \
-    || (PLATFORM(MACCATALYST) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 160000) \
+    || PLATFORM(IOS) \
+    || PLATFORM(MACCATALYST) \
     || PLATFORM(VISION))
 #define HAVE_SANDBOX_STATE_FLAGS 1
 #endif
 
-#if ((PLATFORM(IOS) || PLATFORM(MACCATALYST) || PLATFORM(VISION)) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 160000)
+#if PLATFORM(IOS) || PLATFORM(MACCATALYST) || PLATFORM(VISION)
 #define HAVE_UIFINDINTERACTION 1
 #endif
 
-#if PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 120000
+#if PLATFORM(MAC)
 #define HAVE_NSSCREEN_SAFE_AREA 1
 #endif
 
@@ -1204,87 +1117,62 @@
 #define HAVE_CFAUTORELEASE 1
 #endif
 
-#if PLATFORM(MAC) && __MAC_OS_X_VERSION_MAX_ALLOWED >= 120200
+#if PLATFORM(MAC)
 #define HAVE_PLATFORM_SCROLL_MOMENTUM_INTERRUPTION_REASON 1
 #endif
 
-#if (PLATFORM(MAC) && __MAC_OS_X_VERSION_MAX_ALLOWED >= 120300) \
-    || (PLATFORM(IOS_FAMILY) && __IPHONE_OS_VERSION_MAX_ALLOWED >= 150400) \
-    || (PLATFORM(APPLETV) && __TV_OS_VERSION_MAX_ALLOWED >= 150400) \
-    || (PLATFORM(WATCHOS) && __WATCH_OS_VERSION_MAX_ALLOWED >= 80500)
-#define HAVE_CFNETWORK_DISABLE_CACHE_SPI 1
-#endif
-
-#if ((PLATFORM(IOS) || PLATFORM(MACCATALYST) || PLATFORM(VISION)) && __IPHONE_OS_VERSION_MAX_ALLOWED >= 150400)
+#if PLATFORM(IOS) || PLATFORM(MACCATALYST) || PLATFORM(VISION)
 #define HAVE_UIACTIVITYTYPE_SHAREPLAY 1
 #endif
 
-#if (PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 130000) || (PLATFORM(IOS) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 160000) || (PLATFORM(WATCHOS) && __WATCH_OS_VERSION_MIN_REQUIRED >= 90000) || PLATFORM(VISION)
+#if (PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 130000) || PLATFORM(IOS) || PLATFORM(WATCHOS) || PLATFORM(VISION)
 #define HAVE_SAFE_BROWSING_RESULT_DETAILS 1
 #endif
 
 #if (PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 130000) \
-    || (PLATFORM(MACCATALYST) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 160000) \
-    || (PLATFORM(IOS) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 160000) \
-    || (PLATFORM(APPLETV) && __TV_OS_VERSION_MAX_ALLOWED >= 160000) \
-    || (PLATFORM(WATCHOS) && __WATCH_OS_VERSION_MIN_REQUIRED >= 90000) \
-    || PLATFORM(VISION)
+    || PLATFORM(IOS_FAMILY)
 #define HAVE_MEMORY_ATTRIBUTION_VM_SHARE_SUPPORT 1
 #define HAVE_PROHIBIT_PRIVACY_PROXY 1
 #endif
 
 #if (PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 130000) \
-    || (PLATFORM(MACCATALYST) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 160000) \
-    || (PLATFORM(IOS) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 160000) \
-    || (PLATFORM(APPLETV) && __TV_OS_VERSION_MAX_ALLOWED >= 160000) \
-    || (PLATFORM(WATCHOS) && __WATCH_OS_VERSION_MIN_REQUIRED >= 90000) \
-    || PLATFORM(VISION)
+    || PLATFORM(IOS_FAMILY)
 #define HAVE_MACH_MEMORY_ENTRY_OWNERSHIP_IDENTITY_TOKEN_SUPPORT 1
 #endif
 
-#if (PLATFORM(IOS) && __IPHONE_OS_VERSION_MAX_ALLOWED >= 160000) || PLATFORM(VISION)
+#if PLATFORM(IOS) || PLATFORM(VISION)
 #define HAVE_SKADNETWORK_v4 1
 #endif
 
-#if (PLATFORM(IOS) && __IPHONE_OS_VERSION_MAX_ALLOWED >= 160000) \
-    || (PLATFORM(WATCHOS) && __WATCH_OS_VERSION_MAX_ALLOWED >= 90000) \
-    || (PLATFORM(APPLETV) && __TV_OS_VERSION_MAX_ALLOWED >= 160000) \
+// FIXME: Why no catalyst?
+#if PLATFORM(IOS) \
+    || PLATFORM(WATCHOS) \
+    || PLATFORM(APPLETV) \
     || PLATFORM(VISION)
 #define HAVE_IMAGEIO_CREATE_UNPREMULTIPLIED_PNG 1
 #endif
 
 #if !defined(HAVE_UI_CONTEXT_MENU_PREVIEW_ITEM_IDENTIFIER) \
-    && (((PLATFORM(IOS) || PLATFORM(MACCATALYST)) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 160000) \
-    || (PLATFORM(WATCHOS) && __WATCH_OS_VERSION_MIN_REQUIRED >= 90000) \
-    || (PLATFORM(APPLETV) && __TV_OS_VERSION_MIN_REQUIRED >= 160000) \
-    || PLATFORM(VISION))
+    && PLATFORM(IOS_FAMILY)
 #define HAVE_UI_CONTEXT_MENU_PREVIEW_ITEM_IDENTIFIER 1
 #endif
 
-#if (PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED < 120000) || (PLATFORM(MACCATALYST) && __IPHONE_OS_VERSION_MIN_REQUIRED < 150000)
-#define HAVE_SYSTEM_HTTP_CONTENT_FILTERING 1
-#endif
-
 #if (PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 130000) \
-    || (PLATFORM(MACCATALYST) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 160000) \
-    || (PLATFORM(IOS) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 160000) \
-    || (PLATFORM(APPLETV) && __TV_OS_VERSION_MAX_ALLOWED >= 160000) \
-    || (PLATFORM(WATCHOS) && __WATCH_OS_VERSION_MIN_REQUIRED >= 90000) \
-    || PLATFORM(VISION)
+    || PLATFORM(IOS_FAMILY)
 #define HAVE_CORE_TEXT_FIX_FOR_RADAR_93925620 1
 #endif
 
-#if (PLATFORM(IOS) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 160000) || PLATFORM(VISION)
+#if PLATFORM(IOS) || PLATFORM(VISION)
 #define HAVE_DDRESULT_DISABLE_URL_SCHEME_CHECKING 1
 #endif
 
 #if ((PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 130000) \
-    || (PLATFORM(IOS) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 160000) \
+    || PLATFORM(IOS) \
     || PLATFORM(VISION))
 #define HAVE_SYSTEM_CONTENT_LS_DATABASE 1
 #endif
 
-#if (PLATFORM(IOS) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 160000) || PLATFORM(VISION)
+#if PLATFORM(IOS) || PLATFORM(VISION)
 #if !defined(HAVE_UIKIT_RESIZABLE_WINDOWS)
 #define HAVE_UIKIT_RESIZABLE_WINDOWS 1
 #endif
@@ -1293,11 +1181,11 @@
 #endif
 #endif
 
-#if !defined(HAVE_AUDIO_COMPONENT_SERVER_REGISTRATIONS) && PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 120000
+#if !defined(HAVE_AUDIO_COMPONENT_SERVER_REGISTRATIONS) && PLATFORM(MAC)
 #define HAVE_AUDIO_COMPONENT_SERVER_REGISTRATIONS 1
 #endif
 
-#if (PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 130000)
+#if PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 130000
 #define HAVE_SHARING_SERVICE_PICKER_POPOVER_SPI 1
 #endif
 
@@ -1305,22 +1193,18 @@
 #define HAVE_ADDITIONAL_APPLE_CAMERA_SERVICE 1
 #endif
 
-#if ((PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 130000) \
-    || (PLATFORM(IOS) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 160000) \
-    || PLATFORM(VISION))
+#if (PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 130000) \
+    || PLATFORM(IOS) \
+    || PLATFORM(VISION)
 #define HAVE_APPLE_PUSH_SERVICE_URL_TOKEN_SUPPORT 1
 #endif
 
 #if (PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 130000) \
-    || (PLATFORM(MACCATALYST) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 160000) \
-    || (PLATFORM(IOS) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 160000) \
-    || (PLATFORM(APPLETV) && __TV_OS_VERSION_MAX_ALLOWED >= 160000) \
-    || (PLATFORM(WATCHOS) && __WATCH_OS_VERSION_MIN_REQUIRED >= 90000) \
-    || PLATFORM(VISION)
+    || PLATFORM(IOS_FAMILY)
 #define HAVE_AVIF 1
 #endif
 
-#if PLATFORM(IOS_SIMULATOR) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 160000
+#if PLATFORM(IOS_SIMULATOR)
 #define HAVE_CG_IMAGE_SOURCE_AVIF_IMAGE_TYPES_BUG 1
 #endif
 
@@ -1329,10 +1213,7 @@
 #endif
 
 #if (PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 130000) \
-    || ((PLATFORM(IOS) || PLATFORM(MACCATALYST)) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 160000) \
-    || (PLATFORM(WATCHOS) && __WATCH_OS_VERSION_MIN_REQUIRED >= 90000) \
-    || (PLATFORM(APPLETV) && __TV_OS_VERSION_MIN_REQUIRED >= 160000) \
-    || PLATFORM(VISION)
+    || PLATFORM(IOS_FAMILY)
 #if !defined(HAVE_AVSAMPLEBUFFERDISPLAYLAYER_COPYDISPLAYEDPIXELBUFFER)
 #define HAVE_AVSAMPLEBUFFERDISPLAYLAYER_COPYDISPLAYEDPIXELBUFFER 1
 #endif
@@ -1346,8 +1227,9 @@
 #define HAVE_NSPREVIEWREPRESENTINGACTIVITYITEM 1
 #endif
 
-#if (PLATFORM(IOS) && __IPHONE_OS_VERSION_MIN_REQUIRED < 160000) \
-    || PLATFORM(WATCHOS) \
+// FIXME: Surely if we don't have the compiler service on iOS,
+// we must not have it on watchOS & tvOS & visionOS either?
+#if PLATFORM(WATCHOS) \
     || PLATFORM(APPLETV) \
     || PLATFORM(VISION)
 #define HAVE_AGX_COMPILER_SERVICE 1
@@ -1357,9 +1239,9 @@
 #define HAVE_GPU_AVAILABILITY_CHECK 1
 #endif
 
-#if (!defined(HAVE_LOCKDOWN_MODE_PDF_ADDITIONS) && \
+#if !defined(HAVE_LOCKDOWN_MODE_PDF_ADDITIONS) && \
     (PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 130000 \
-    || PLATFORM(IOS) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 160000) \
+    || PLATFORM(IOS) \
     || PLATFORM(VISION))
 #define HAVE_LOCKDOWN_MODE_PDF_ADDITIONS 1
 #endif
@@ -1369,10 +1251,7 @@
 #endif
 
 #if (PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 130000) \
-    || ((PLATFORM(IOS) || PLATFORM(MACCATALYST)) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 160000) \
-    || (PLATFORM(WATCHOS) && __WATCH_OS_VERSION_MIN_REQUIRED >= 90000) \
-    || (PLATFORM(APPLETV) && __TV_OS_VERSION_MIN_REQUIRED >= 160000) \
-    || PLATFORM(VISION)
+    || PLATFORM(IOS_FAMILY)
 #define HAVE_DYLD_DLOPEN_IMAGE_HEADER_SPI 1
 #endif
 
@@ -1384,7 +1263,7 @@
 #define HAVE_SHARED_REGION_SPI 1
 #endif
 
-#if PLATFORM(IOS_FAMILY) && !PLATFORM(MACCATALYST) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 160000
+#if PLATFORM(IOS_FAMILY) && !PLATFORM(MACCATALYST)
 #define HAVE_MEDIAEXPERIENCE_AVSYSTEMCONTROLLER 1
 #endif
 
@@ -1395,8 +1274,7 @@
 #endif
 
 #if (PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 130000) \
-    || ((PLATFORM(IOS) || PLATFORM(MACCATALYST)) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 160000) \
-    || PLATFORM(VISION)
+    || PLATFORM(IOS) || PLATFORM(MACCATALYST) || PLATFORM(VISION)
 #define HAVE_CONTINUITY_CAMERA 1
 #endif
 
@@ -1408,10 +1286,7 @@
 
 #if !defined(HAVE_CGSTYLE_CREATE_SHADOW2) \
     && ((PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 130000) \
-    || ((PLATFORM(IOS) || PLATFORM(MACCATALYST)) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 160000) \
-    || (PLATFORM(WATCHOS) && __WATCH_OS_VERSION_MIN_REQUIRED >= 90000) \
-    || (PLATFORM(APPLETV) && __TV_OS_VERSION_MIN_REQUIRED >= 160000) \
-    || PLATFORM(VISION))
+    || PLATFORM(IOS_FAMILY))
 #define HAVE_CGSTYLE_CREATE_SHADOW2 1
 #endif
 
@@ -1431,7 +1306,7 @@
 #define HAVE_MEDIAPLAYBACKD 1
 #endif
 
-#if (PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 120000) || PLATFORM(IOS_FAMILY)
+#if PLATFORM(MAC) || PLATFORM(IOS_FAMILY)
 #define HAVE_AVCAPTUREDEVICE_MINFOCUSLENGTH 1
 #endif
 
@@ -1461,10 +1336,10 @@
 #define HAVE_SETUSEIOSURFACEFORTILES 1
 #endif
 
-#if ((PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 140000) \
-    || ((PLATFORM(IOS) || PLATFORM(MACCATALYST)) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 170000) \
-    || (PLATFORM(WATCHOS) && __WATCH_OS_VERSION_MIN_REQUIRED >= 100000) \
-    || (PLATFORM(APPLETV) && __TV_OS_VERSION_MIN_REQUIRED >= 170000) \
+#if ((PLATFORM(MAC) && __MAC_OS_X_VERSION_MAX_ALLOWED >= 140000) \
+    || ((PLATFORM(IOS) || PLATFORM(MACCATALYST)) && __IPHONE_OS_VERSION_MAX_ALLOWED >= 170000) \
+    || (PLATFORM(WATCHOS) && __WATCH_OS_VERSION_MAX_ALLOWED >= 100000) \
+    || (PLATFORM(APPLETV) && __TV_OS_VERSION_MAX_ALLOWED >= 170000) \
     || PLATFORM(VISION))
 #if __has_include(<Network/proxy_config.h>)
 #define HAVE_NW_PROXY_CONFIG 1
@@ -1543,6 +1418,13 @@
 #define HAVE_REDESIGNED_TEXT_CURSOR 1
 #endif
 
+#if !defined(HAVE_AUTOCORRECTION_ENHANCEMENTS) \
+    && ((PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 140000) \
+    || ((PLATFORM(IOS) || PLATFORM(MACCATALYST)) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 170000) \
+    || PLATFORM(VISION))
+#define HAVE_AUTOCORRECTION_ENHANCEMENTS 1
+#endif
+
 #if !defined(HAVE_WEB_PRIVACY_FRAMEWORK) && __has_include(<WebPrivacy/WebPrivacy.h>)
 #define HAVE_WEB_PRIVACY_FRAMEWORK 1
 #endif
@@ -1558,3 +1440,11 @@
 #if PLATFORM(COCOA)
 #define HAVE_HEIC 1
 #endif
+
+#if (PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 130000)
+#define HAVE_LS_SERVER_CONNECTION_STATUS_RELEASE_NOTIFICATIONS_MASK 1
+#endif
+
+#if PLATFORM(MAC) && __MAC_OS_X_VERSION_MAX_ALLOWED >= 140000
+#define HAVE_NSVIEW_CLIPSTOBOUNDS_API 1
+#endif
