@@ -77,9 +77,11 @@ public:
     WTF_EXPORT_PRIVATE static URL fileURLWithFileSystemPath(StringView);
 
     WTF_EXPORT_PRIVATE String strippedForUseAsReferrer() const;
-
     WTF_EXPORT_PRIVATE String strippedForUseAsReferrerWithExplicitPort() const;
 
+    // Similar to strippedForUseAsReferrer except we also remove the query component.
+    WTF_EXPORT_PRIVATE String strippedForUseAsReport() const;
+
     // Makes a deep copy. Helpful only if you need to use a URL on another
     // thread. Since the underlying StringImpl objects are immutable, there's
     // no other reason to ever prefer isolatedCopy() over plain old assignment.
@@ -140,13 +142,13 @@ public:
     // Returns true if the current URL's protocol is the same as the null-
     // terminated ASCII argument. The argument must be lower-case.
     WTF_EXPORT_PRIVATE bool protocolIs(StringView) const;
+    bool protocolIsAbout() const { return protocolIs("about"_s); }
     bool protocolIsBlob() const { return protocolIs("blob"_s); }
     bool protocolIsData() const { return protocolIs("data"_s); }
-    WTF_EXPORT_PRIVATE bool protocolIsAbout() const;
+    bool protocolIsFile() const { return protocolIs("file"_s); }
     WTF_EXPORT_PRIVATE bool protocolIsJavaScript() const;
-    WTF_EXPORT_PRIVATE bool protocolIsInFTPFamily() const;
-    bool protocolIsInHTTPFamily() const;
-    WTF_EXPORT_PRIVATE bool isLocalFile() const;
+    bool protocolIsInFTPFamily() const { return protocolIs("ftp"_s) || protocolIs("ftps"_s); }
+    bool protocolIsInHTTPFamily() const { return m_protocolIsInHTTPFamily; }
     bool cannotBeABaseURL() const { return m_cannotBeABaseURL; }
 
     WTF_EXPORT_PRIVATE bool isAboutBlank() const;
@@ -250,9 +252,6 @@ static_assert(sizeof(URL) == sizeof(String) + 8 * sizeof(unsigned), "URL should
 bool operator==(const URL&, const URL&);
 bool operator==(const URL&, const String&);
 bool operator==(const String&, const URL&);
-bool operator!=(const URL&, const URL&);
-bool operator!=(const URL&, const String&);
-bool operator!=(const String&, const URL&);
 
 WTF_EXPORT_PRIVATE bool equalIgnoringFragmentIdentifier(const URL&, const URL&);
 WTF_EXPORT_PRIVATE bool protocolHostAndPortAreEqual(const URL&, const URL&);
@@ -262,6 +261,7 @@ WTF_EXPORT_PRIVATE bool isEqualIgnoringQueryAndFragments(const URL&, const URL&)
 
 // Returns the parameters that were removed (including duplicates), in the order that they appear in the URL.
 WTF_EXPORT_PRIVATE Vector<String> removeQueryParameters(URL&, const HashSet<String>&);
+WTF_EXPORT_PRIVATE Vector<String> removeQueryParameters(URL&, Function<bool(const String&)>&&);
 
 WTF_EXPORT_PRIVATE const URL& aboutBlankURL();
 WTF_EXPORT_PRIVATE const URL& aboutSrcDocURL();
@@ -273,7 +273,6 @@ WTF_EXPORT_PRIVATE const URL& aboutSrcDocURL();
 
 WTF_EXPORT_PRIVATE bool protocolIs(StringView url, ASCIILiteral protocol);
 WTF_EXPORT_PRIVATE bool protocolIsJavaScript(StringView url);
-WTF_EXPORT_PRIVATE bool protocolIsInFTPFamily(StringView url);
 WTF_EXPORT_PRIVATE bool protocolIsInHTTPFamily(StringView url);
 
 WTF_EXPORT_PRIVATE std::optional<uint16_t> defaultPortForProtocol(StringView protocol);
@@ -317,21 +316,6 @@ inline bool operator==(const String& a, const URL& b)
     return a == b.string();
 }
 
-inline bool operator!=(const URL& a, const URL& b)
-{
-    return a.string() != b.string();
-}
-
-inline bool operator!=(const URL& a, const String& b)
-{
-    return a.string() != b;
-}
-
-inline bool operator!=(const String& a, const URL& b)
-{
-    return a != b.string();
-}
-
 inline URL::URL(HashTableDeletedValueType)
     : m_string(HashTableDeletedValue)
 {
@@ -372,11 +356,6 @@ inline bool URL::hasFragmentIdentifier() const
     return m_isValid && m_string.length() > m_queryEnd;
 }
 
-inline bool URL::protocolIsInHTTPFamily() const
-{
-    return m_protocolIsInHTTPFamily;
-}
-
 inline unsigned URL::pathEnd() const
 {
     return m_pathEnd;
