@@ -36,11 +36,12 @@
 
 #if ENABLE(MEDIA_STREAM)
 
-#include "Document.h"
-#include "Frame.h"
+#include "AudioSession.h"
+#include "DocumentInlines.h"
 #include "JSDOMPromiseDeferred.h"
 #include "JSMediaStream.h"
 #include "JSOverconstrainedError.h"
+#include "LocalFrame.h"
 #include "Logging.h"
 #include "MediaConstraints.h"
 #include "PlatformMediaSessionManager.h"
@@ -52,18 +53,20 @@
 
 namespace WebCore {
 
-Ref<UserMediaRequest> UserMediaRequest::create(Document& document, MediaStreamRequest&& request, DOMPromiseDeferred<IDLInterface<MediaStream>>&& promise)
+Ref<UserMediaRequest> UserMediaRequest::create(Document& document, MediaStreamRequest&& request, TrackConstraints&& audioConstraints, TrackConstraints&& videoConstraints, DOMPromiseDeferred<IDLInterface<MediaStream>>&& promise)
 {
-    auto result = adoptRef(*new UserMediaRequest(document, WTFMove(request), WTFMove(promise)));
+    auto result = adoptRef(*new UserMediaRequest(document, WTFMove(request), WTFMove(audioConstraints), WTFMove(videoConstraints), WTFMove(promise)));
     result->suspendIfNeeded();
     return result;
 }
 
-UserMediaRequest::UserMediaRequest(Document& document, MediaStreamRequest&& request, DOMPromiseDeferred<IDLInterface<MediaStream>>&& promise)
+UserMediaRequest::UserMediaRequest(Document& document, MediaStreamRequest&& request, TrackConstraints&& audioConstraints, TrackConstraints&& videoConstraints, DOMPromiseDeferred<IDLInterface<MediaStream>>&& promise)
     : ActiveDOMObject(document)
     , m_identifier(UserMediaRequestIdentifier::generate())
     , m_promise(makeUniqueRef<DOMPromiseDeferred<IDLInterface<MediaStream>>>(WTFMove(promise)))
     , m_request(WTFMove(request))
+    , m_audioConstraints(WTFMove(audioConstraints))
+    , m_videoConstraints(WTFMove(videoConstraints))
 {
 }
 
@@ -75,117 +78,29 @@ UserMediaRequest::~UserMediaRequest()
 
 SecurityOrigin* UserMediaRequest::userMediaDocumentOrigin() const
 {
-    if (!m_scriptExecutionContext)
-        return nullptr;
-    return m_scriptExecutionContext->securityOrigin();
+    auto* context = scriptExecutionContext();
+    return context ? context->securityOrigin() : nullptr;
 }
 
 SecurityOrigin* UserMediaRequest::topLevelDocumentOrigin() const
 {
-    if (!m_scriptExecutionContext)
-        return nullptr;
-    return &m_scriptExecutionContext->topOrigin();
-}
-
-static bool hasInvalidGetDisplayMediaConstraint(const MediaConstraints& constraints)
-{
-    // https://w3c.github.io/mediacapture-screen-share/#navigator-additions
-    // 1. Let constraints be the method's first argument.
-    // 2. For each member present in constraints whose value, value, is a dictionary, run the following steps:
-    //     1. If value contains a member named advanced, return a promise rejected with a newly created TypeError.
-    //     2. If value contains a member which in turn is a dictionary containing a member named either min or
-    //        exact, return a promise rejected with a newly created TypeError.
-    if (!constraints.isValid)
-        return false;
-
-    if (!constraints.advancedConstraints.isEmpty())
-        return true;
-
-    bool invalid = false;
-    constraints.mandatoryConstraints.filter([&invalid] (const MediaConstraint& constraint) mutable {
-        switch (constraint.constraintType()) {
-        case MediaConstraintType::Width:
-        case MediaConstraintType::Height: {
-            auto& intConstraint = downcast<IntConstraint>(constraint);
-            int value;
-            invalid = intConstraint.getExact(value) || intConstraint.getMin(value);
-            break;
-        }
-
-        case MediaConstraintType::AspectRatio:
-        case MediaConstraintType::FrameRate: {
-            auto& doubleConstraint = downcast<DoubleConstraint>(constraint);
-            double value;
-            invalid = doubleConstraint.getExact(value) || doubleConstraint.getMin(value);
-            break;
-        }
-
-        case MediaConstraintType::DisplaySurface:
-        case MediaConstraintType::LogicalSurface: {
-            auto& boolConstraint = downcast<BooleanConstraint>(constraint);
-            bool value;
-            invalid = boolConstraint.getExact(value);
-            break;
-        }
-
-        case MediaConstraintType::FacingMode:
-        case MediaConstraintType::DeviceId:
-        case MediaConstraintType::GroupId: {
-            auto& stringConstraint = downcast<StringConstraint>(constraint);
-            Vector<String> values;
-            invalid = stringConstraint.getExact(values);
-            break;
-        }
-
-        case MediaConstraintType::SampleRate:
-        case MediaConstraintType::SampleSize:
-        case MediaConstraintType::Volume:
-        case MediaConstraintType::EchoCancellation:
-            // Ignored.
-            break;
-
-        case MediaConstraintType::Unknown:
-            ASSERT_NOT_REACHED();
-            break;
-        }
-
-        return invalid;
-    });
-
-    return invalid;
+    auto* context = scriptExecutionContext();
+    return context ? &context->topOrigin() : nullptr;
 }
 
 void UserMediaRequest::start()
 {
-    ASSERT(m_scriptExecutionContext);
-    if (!m_scriptExecutionContext) {
+    auto* context = scriptExecutionContext();
+    ASSERT(context);
+    if (!context) {
         deny(MediaAccessDenialReason::UserMediaDisabled);
         return;
     }
 
-    if (m_request.type == MediaStreamRequest::Type::DisplayMedia) {
-        if (hasInvalidGetDisplayMediaConstraint(m_request.videoConstraints)) {
-            deny(MediaAccessDenialReason::IllegalConstraint);
-            return;
-        }
-    }
-
-    // https://w3c.github.io/mediacapture-main/getusermedia.html#dom-mediadevices-getusermedia()
-    // 1. Let constraints be the method's first argument.
-    // 2. Let requestedMediaTypes be the set of media types in constraints with either a dictionary
-    //    value or a value of "true".
-    // 3. If requestedMediaTypes is the empty set, return a promise rejected with a TypeError. The word
-    //    "optional" occurs in the WebIDL due to WebIDL rules, but the argument must be supplied in order
-    //    for the call to succeed.
-    if (!m_request.audioConstraints.isValid && !m_request.videoConstraints.isValid) {
-        deny(MediaAccessDenialReason::NoConstraints);
-        return;
-    }
-
     // 4. If the current settings object's responsible document is NOT allowed to use the feature indicated by
     //    attribute name allowusermedia, return a promise rejected with a DOMException object whose name
     //    attribute has the value SecurityError.
-    auto& document = downcast<Document>(*m_scriptExecutionContext);
+    auto& document = downcast<Document>(*context);
     auto* controller = UserMediaController::from(document.page());
     if (!controller) {
         deny(MediaAccessDenialReason::UserMediaDisabled);
@@ -198,23 +113,27 @@ void UserMediaRequest::start()
     // 6.10 Permission Failure: Reject p with a new DOMException object whose name attribute has
     //      the value NotAllowedError.
 
-    OptionSet<UserMediaController::CaptureType> types;
-    UserMediaController::BlockedCaller caller;
-    if (m_request.type == MediaStreamRequest::Type::DisplayMedia) {
-        types.add(UserMediaController::CaptureType::Display);
-        caller = UserMediaController::BlockedCaller::GetDisplayMedia;
-    } else {
-        if (m_request.audioConstraints.isValid)
-            types.add(UserMediaController::CaptureType::Microphone);
-        if (m_request.videoConstraints.isValid)
-            types.add(UserMediaController::CaptureType::Camera);
-        caller = UserMediaController::BlockedCaller::GetUserMedia;
-    }
-    auto access = controller->canCallGetUserMedia(document, types);
-    if (access != UserMediaController::GetUserMediaAccess::CanCall) {
-        deny(MediaAccessDenialReason::PermissionDenied);
-        controller->logGetUserMediaDenial(document, access, caller);
-        return;
+    switch (m_request.type) {
+    case MediaStreamRequest::Type::DisplayMedia:
+    case MediaStreamRequest::Type::DisplayMediaWithAudio:
+        if (!isFeaturePolicyAllowedByDocumentAndAllOwners(FeaturePolicy::Type::DisplayCapture, document)) {
+            deny(MediaAccessDenialReason::PermissionDenied);
+            controller->logGetDisplayMediaDenial(document);
+            return;
+        }
+        break;
+    case MediaStreamRequest::Type::UserMedia:
+        if (m_request.audioConstraints.isValid && !isFeaturePolicyAllowedByDocumentAndAllOwners(FeaturePolicy::Type::Microphone, document)) {
+            deny(MediaAccessDenialReason::PermissionDenied);
+            controller->logGetUserMediaDenial(document);
+            return;
+        }
+        if (m_request.videoConstraints.isValid && !isFeaturePolicyAllowedByDocumentAndAllOwners(FeaturePolicy::Type::Camera, document)) {
+            deny(MediaAccessDenialReason::PermissionDenied);
+            controller->logGetUserMediaDenial(document);
+            return;
+        }
+        break;
     }
 
     PlatformMediaSessionManager::sharedManager().prepareToSendUserMediaPermissionRequest();
@@ -231,28 +150,30 @@ static inline bool isMediaStreamCorrectlyStarted(const MediaStream& stream)
     });
 }
 
-void UserMediaRequest::allow(CaptureDevice&& audioDevice, CaptureDevice&& videoDevice, String&& deviceIdentifierHashSalt, CompletionHandler<void()>&& completionHandler)
+void UserMediaRequest::allow(CaptureDevice&& audioDevice, CaptureDevice&& videoDevice, MediaDeviceHashSalts&& deviceIdentifierHashSalt, CompletionHandler<void()>&& completionHandler)
 {
     RELEASE_LOG(MediaStream, "UserMediaRequest::allow %s %s", audioDevice ? audioDevice.persistentId().utf8().data() : "", videoDevice ? videoDevice.persistentId().utf8().data() : "");
     m_allowCompletionHandler = WTFMove(completionHandler);
     queueTaskKeepingObjectAlive(*this, TaskSource::UserInteraction, [this, audioDevice = WTFMove(audioDevice), videoDevice = WTFMove(videoDevice), deviceIdentifierHashSalt = WTFMove(deviceIdentifierHashSalt)]() mutable {
-        auto callback = [this, protector = makePendingActivity(*this)](RefPtr<MediaStreamPrivate>&& privateStream) mutable {
+        auto callback = [this, protector = makePendingActivity(*this)](auto privateStreamOrError) mutable {
             auto scopeExit = makeScopeExit([completionHandler = WTFMove(m_allowCompletionHandler)]() mutable {
                 completionHandler();
             });
             if (isContextStopped())
                 return;
 
-            if (!privateStream) {
+            if (!privateStreamOrError.has_value()) {
                 RELEASE_LOG(MediaStream, "UserMediaRequest::allow failed to create media stream!");
+                scriptExecutionContext()->addConsoleMessage(MessageSource::JS, MessageLevel::Error, privateStreamOrError.error());
                 deny(MediaAccessDenialReason::HardwareError);
                 return;
             }
+            auto privateStream = WTFMove(privateStreamOrError).value();
 
-            auto& document = downcast<Document>(*m_scriptExecutionContext);
+            auto& document = downcast<Document>(*scriptExecutionContext());
             privateStream->monitorOrientation(document.orientationNotifier());
 
-            auto stream = MediaStream::create(document, privateStream.releaseNonNull());
+            auto stream = MediaStream::create(document, WTFMove(privateStream));
             stream->startProducingData();
 
             if (!isMediaStreamCorrectlyStarted(stream)) {
@@ -260,17 +181,27 @@ void UserMediaRequest::allow(CaptureDevice&& audioDevice, CaptureDevice&& videoD
                 return;
             }
 
+            if (auto* audioTrack = stream->getFirstAudioTrack()) {
+#if USE(AUDIO_SESSION)
+                AudioSession::sharedSession().tryToSetActive(true);
+#endif
+                if (std::holds_alternative<MediaTrackConstraints>(m_audioConstraints))
+                    audioTrack->setConstraints(std::get<MediaTrackConstraints>(WTFMove(m_audioConstraints)));
+            }
+            if (auto* videoTrack = stream->getFirstVideoTrack()) {
+                if (std::holds_alternative<MediaTrackConstraints>(m_videoConstraints))
+                    videoTrack->setConstraints(std::get<MediaTrackConstraints>(WTFMove(m_videoConstraints)));
+            }
+
             ASSERT(document.isCapturing());
-            stream->document()->setHasCaptureMediaStreamTrack();
+            document.setHasCaptureMediaStreamTrack();
             m_promise->resolve(WTFMove(stream));
         };
 
         auto& document = downcast<Document>(*scriptExecutionContext());
-        document.setDeviceIDHashSalt(deviceIdentifierHashSalt);
-
         RealtimeMediaSourceCenter::singleton().createMediaStream(document.logger(), WTFMove(callback), WTFMove(deviceIdentifierHashSalt), WTFMove(audioDevice), WTFMove(videoDevice), m_request);
 
-        if (!m_scriptExecutionContext)
+        if (!scriptExecutionContext())
             return;
 
 #if ENABLE(WEB_RTC)
@@ -282,15 +213,11 @@ void UserMediaRequest::allow(CaptureDevice&& audioDevice, CaptureDevice&& videoD
 
 void UserMediaRequest::deny(MediaAccessDenialReason reason, const String& message)
 {
-    if (!m_scriptExecutionContext)
+    if (!scriptExecutionContext())
         return;
 
     ExceptionCode code;
     switch (reason) {
-    case MediaAccessDenialReason::IllegalConstraint:
-        RELEASE_LOG(MediaStream, "UserMediaRequest::deny - invalid constraints");
-        code = TypeError;
-        break;
     case MediaAccessDenialReason::NoConstraints:
         RELEASE_LOG(MediaStream, "UserMediaRequest::deny - no constraints");
         code = TypeError;
@@ -333,7 +260,7 @@ void UserMediaRequest::deny(MediaAccessDenialReason reason, const String& messag
 
 void UserMediaRequest::stop()
 {
-    auto& document = downcast<Document>(*m_scriptExecutionContext);
+    auto& document = downcast<Document>(*scriptExecutionContext());
     if (auto* controller = UserMediaController::from(document.page()))
         controller->cancelUserMediaAccessRequest(*this);
 }
@@ -345,25 +272,7 @@ const char* UserMediaRequest::activeDOMObjectName() const
 
 Document* UserMediaRequest::document() const
 {
-    return downcast<Document>(m_scriptExecutionContext);
-}
-
-void UserMediaRequest::mediaStreamDidFail(RealtimeMediaSource::Type type)
-{
-    RELEASE_LOG(MediaStream, "UserMediaRequest::mediaStreamDidFail");
-    const char* typeDescription = "";
-    switch (type) {
-    case RealtimeMediaSource::Type::Audio:
-        typeDescription = "audio";
-        break;
-    case RealtimeMediaSource::Type::Video:
-        typeDescription = "video";
-        break;
-    case RealtimeMediaSource::Type::None:
-        typeDescription = "unknown";
-        break;
-    }
-    m_promise->reject(NotReadableError, makeString("Failed starting capture of a "_s, typeDescription, " track"_s));
+    return downcast<Document>(scriptExecutionContext());
 }
 
 } // namespace WebCore
