@@ -57,10 +57,8 @@ static inline void fillRTCRTPStreamStats(RTCStatsReport::RtpStreamStats& stats,
     if (gst_structure_get_uint(structure, "ssrc", &value))
         stats.ssrc = value;
 
-    if (const char* kind = gst_structure_get_string(structure, "kind")) {
+    if (const char* kind = gst_structure_get_string(structure, "kind"))
         stats.kind = String::fromLatin1(kind);
-        stats.mediaType = stats.kind;
-    }
 }
 
 static inline void fillRTCCodecStats(RTCStatsReport::CodecStats& stats, const GstStructure* structure)
@@ -287,10 +285,6 @@ static inline void fillRTCCandidateStats(RTCStatsReport::IceCandidateStats& stat
 
     auto candidateType = String::fromLatin1(gst_structure_get_string(structure, "candidate-type"));
     stats.candidateType = iceCandidateType(candidateType);
-
-    uint64_t priority;
-    if (gst_structure_get_uint64(structure, "priority", &priority))
-        stats.priority = priority;
 }
 
 static inline void fillRTCCandidatePairStats(RTCStatsReport::IceCandidatePairStats& stats, const GstStructure* structure)
@@ -302,7 +296,7 @@ static inline void fillRTCCandidatePairStats(RTCStatsReport::IceCandidatePairSta
 
     // FIXME
     // stats.transportId =
-    // stats.state =
+    stats.state = RTCStatsReport::IceCandidatePairState::Succeeded;
     // stats.priority =
     // stats.nominated =
     // stats.writable =
@@ -403,14 +397,14 @@ static gboolean fillReportCallback(GQuark, const GValue* value, gpointer userDat
         break;
     case GST_WEBRTC_STATS_LOCAL_CANDIDATE:
     case GST_WEBRTC_STATS_REMOTE_CANDIDATE:
-        if (webkitGstCheckVersion(1, 21, 0)) {
+        if (webkitGstCheckVersion(1, 22, 0)) {
             RTCStatsReport::IceCandidateStats stats;
             fillRTCCandidateStats(stats, statsType, structure);
             report.set<IDLDOMString, IDLDictionary<RTCStatsReport::IceCandidateStats>>(stats.id, WTFMove(stats));
         }
         break;
     case GST_WEBRTC_STATS_CANDIDATE_PAIR:
-        if (webkitGstCheckVersion(1, 21, 0)) {
+        if (webkitGstCheckVersion(1, 22, 0)) {
             RTCStatsReport::IceCandidatePairStats stats;
             fillRTCCandidatePairStats(stats, structure);
             report.set<IDLDOMString, IDLDictionary<RTCStatsReport::IceCandidatePairStats>>(stats.id, WTFMove(stats));
@@ -478,6 +472,8 @@ void GStreamerStatsCollector::getStats(CollectorCallback&& callback, GstPad* pad
     }, holder, reinterpret_cast<GDestroyNotify>(destroyCallbackHolder)));
 }
 
-}; // namespace WebCore
+#undef GST_CAT_DEFAULT
+
+} // namespace WebCore
 
 #endif // ENABLE(WEB_RTC) && USE(GSTREAMER_WEBRTC)
