@@ -93,6 +93,9 @@ static std::optional<Exception> checkImageUsability(ScriptExecutionContext& cont
         return { };
     },
     [] (const RefPtr<SVGImageElement>& imageElement) -> std::optional<Exception> {
+        if (imageElement->renderingTaintsOrigin())
+            return Exception { SecurityError, "Image element is tainted"_s };
+
         auto* image = imageElement->cachedImage() ? imageElement->cachedImage()->image() : nullptr;
         if (!image)
             return Exception { InvalidStateError,  "Image element has no data"_s };
@@ -498,7 +501,7 @@ ExceptionOr<Ref<WebCodecsVideoFrame>> WebCodecsVideoFrame::clone(ScriptExecution
 
     auto clone = adoptRef(*new WebCodecsVideoFrame(context, WebCodecsVideoFrameData { m_data }));
 
-    clone->m_colorSpace = colorSpace();
+    clone->m_colorSpace = &colorSpace();
     clone->m_codedRect = codedRect();
     clone->m_visibleRect = visibleRect();
     clone->m_isDetached = m_isDetached;
@@ -526,7 +529,6 @@ void WebCodecsVideoFrame::close()
 
     m_codedRect = nullptr;
     m_visibleRect = nullptr;
-    m_colorSpace = nullptr;
 }
 
 DOMRectReadOnly* WebCodecsVideoFrame::codedRect() const
@@ -563,12 +565,12 @@ void WebCodecsVideoFrame::setVisibleRect(const DOMRectInit& rect)
     m_data.visibleHeight = rect.height;
 }
 
-VideoColorSpace* WebCodecsVideoFrame::colorSpace() const
+VideoColorSpace& WebCodecsVideoFrame::colorSpace() const
 {
-    if (!m_colorSpace && m_data.internalFrame)
-        m_colorSpace = VideoColorSpace::create(m_data.internalFrame->colorSpace());
+    if (!m_colorSpace)
+        m_colorSpace = m_data.internalFrame ? VideoColorSpace::create(m_data.internalFrame->colorSpace()) : VideoColorSpace::create();
 
-    return m_colorSpace.get();
+    return *m_colorSpace.get();
 }
 
 } // namespace WebCore
