@@ -26,86 +26,98 @@
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
-#ifndef AccessibilityTable_h
-#define AccessibilityTable_h
+#pragma once
 
 #include "AccessibilityRenderObject.h"
 #include <wtf/Forward.h>
 
 namespace WebCore {
 
-class AccessibilityTableCell;
+class AccessibilityTableRow;
 class HTMLTableElement;
-class RenderTableSection;
-    
+
 class AccessibilityTable : public AccessibilityRenderObject {
 public:
     static Ref<AccessibilityTable> create(RenderObject*);
+    static Ref<AccessibilityTable> create(Node&);
     virtual ~AccessibilityTable();
 
     void init() final;
 
     AccessibilityRole roleValue() const final;
     virtual bool isAriaTable() const { return false; }
-    
-    void addChildren() override;
-    void clearChildren() final;
-    
-    const AccessibilityChildrenVector& columns();
-    const AccessibilityChildrenVector& rows();
-    
-    virtual bool supportsSelectedRows() { return false; }
-    unsigned columnCount();
-    unsigned rowCount();
-    int tableLevel() const final;
-    
-    String title() const final;
-    
-    // all the cells in the table
-    void cells(AccessibilityChildrenVector&);
-    AccessibilityTableCell* cellForColumnAndRow(unsigned column, unsigned row);
-    
-    void columnHeaders(AccessibilityChildrenVector&);
-    void rowHeaders(AccessibilityChildrenVector&);
-    void visibleRows(AccessibilityChildrenVector&);
-    
-    // an object that contains, as children, all the objects that act as headers
-    AccessibilityObject* headerContainer();
 
-    // isExposableThroughAccessibility() is whether it is exposed as an AccessibilityTable to the platform.
-    bool isExposableThroughAccessibility() const;
-    
-    int ariaColumnCount() const;
-    int ariaRowCount() const;
+    void addChildren() final;
+    void clearChildren() final;
+    void updateChildrenRoles();
+
+    AccessibilityChildrenVector columns() override;
+    AccessibilityChildrenVector rows() override;
+
+    unsigned columnCount() final;
+    unsigned rowCount() final;
+    int tableLevel() const final;
+
+    String title() const final;
+
+    // all the cells in the table
+    AccessibilityChildrenVector cells() override;
+    AXCoreObject* cellForColumnAndRow(unsigned column, unsigned row) override;
+
+    AccessibilityChildrenVector columnHeaders() override;
+    AccessibilityChildrenVector rowHeaders() override;
+    AccessibilityChildrenVector visibleRows() override;
+
+    // Returns an object that contains, as children, all the objects that act as headers.
+    AXCoreObject* headerContainer() override;
+
+    bool isTable() const final { return true; }
+    // Returns whether it is exposed as an AccessibilityTable to the platform.
+    bool isExposable() const final { return m_isExposable; }
+    void recomputeIsExposable();
+
+    int axColumnCount() const final;
+    int axRowCount() const final;
+
+    // Cell indexes are assigned during child creation, so make sure children are up-to-date.
+    void ensureCellIndexesUpToDate() { updateChildrenIfNecessary(); }
+    Vector<Vector<AXID>> cellSlots() final;
+    void setCellSlotsDirty();
 
 protected:
     explicit AccessibilityTable(RenderObject*);
+    explicit AccessibilityTable(Node&);
 
     AccessibilityChildrenVector m_rows;
     AccessibilityChildrenVector m_columns;
+    // 2D matrix of the cells assigned to each "slot" in this table.
+    // ("Slot" as defined here: https://html.spec.whatwg.org/multipage/tables.html#concept-slots)
+    Vector<Vector<AXID>> m_cellSlots;
 
     RefPtr<AccessibilityObject> m_headerContainer;
-    bool m_isExposableThroughAccessibility;
+    bool m_isExposable;
 
-    bool hasARIARole() const;
+    // Used in type checking function is<AccessibilityTable>.
+    bool isAccessibilityTableInstance() const final { return true; }
 
-    // isTable is whether it's an AccessibilityTable object.
-    bool isTable() const final { return true; }
-    // isDataTable is whether it is exposed as an AccessibilityTable because the heuristic
-    // think this "looks" like a data-based table (instead of a table used for layout).
-    bool isDataTable() const final;
     bool computeAccessibilityIsIgnored() const final;
 
+    void addRow(AccessibilityTableRow&, unsigned, unsigned& maxColumnCount);
+
 private:
     virtual bool computeIsTableExposableThroughAccessibility() const;
     void titleElementText(Vector<AccessibilityText>&) const final;
     HTMLTableElement* tableElement() const;
-    void addChildrenFromSection(RenderTableSection*, unsigned& maxColumnCount);
-    void addTableCellChild(AccessibilityObject*, HashSet<AccessibilityObject*>& appendedRows, unsigned& columnCount);
+
+    void ensureRow(unsigned);
+    void ensureRowAndColumn(unsigned /* rowIndex */, unsigned /* columnIndex */);
+
+    bool hasNonTableARIARole() const;
+    // isDataTable is whether it is exposed as an AccessibilityTable because the heuristic
+    // think this "looks" like a data-based table (instead of a table used for layout).
+    bool isDataTable() const;
 };
 
 } // namespace WebCore 
 
-SPECIALIZE_TYPE_TRAITS_ACCESSIBILITY(AccessibilityTable, isTable())
-
-#endif // AccessibilityTable_h
+SPECIALIZE_TYPE_TRAITS_ACCESSIBILITY(AccessibilityTable, isAccessibilityTableInstance())
