@@ -29,24 +29,25 @@
 #include "config.h"
 #include "AccessibilityTableRow.h"
 
-#include "AXObjectCache.h"
 #include "AccessibilityTable.h"
 #include "AccessibilityTableCell.h"
 #include "HTMLNames.h"
-#include "HTMLTableRowElement.h"
 #include "RenderObject.h"
-#include "RenderTableCell.h"
-#include "RenderTableRow.h"
 
 namespace WebCore {
-    
+
 using namespace HTMLNames;
-    
+
 AccessibilityTableRow::AccessibilityTableRow(RenderObject* renderer)
     : AccessibilityRenderObject(renderer)
 {
 }
 
+AccessibilityTableRow::AccessibilityTableRow(Node& node)
+    : AccessibilityRenderObject(node)
+{
+}
+
 AccessibilityTableRow::~AccessibilityTableRow() = default;
 
 Ref<AccessibilityTableRow> AccessibilityTableRow::create(RenderObject* renderer)
@@ -54,6 +55,11 @@ Ref<AccessibilityTableRow> AccessibilityTableRow::create(RenderObject* renderer)
     return adoptRef(*new AccessibilityTableRow(renderer));
 }
 
+Ref<AccessibilityTableRow> AccessibilityTableRow::create(Node& node)
+{
+    return adoptRef(*new AccessibilityTableRow(node));
+}
+
 AccessibilityRole AccessibilityTableRow::determineAccessibilityRole()
 {
     if (!isTableRow())
@@ -113,19 +119,19 @@ AccessibilityTable* AccessibilityTableRow::parentTable() const
 AXCoreObject* AccessibilityTableRow::headerObject()
 {
     const auto& rowChildren = children();
-    if (!rowChildren.size())
+    if (rowChildren.isEmpty())
         return nullptr;
     
-    auto* cell = rowChildren[0].get();
-    if (!is<AccessibilityTableCell>(cell) || !cell->node() || !cell->node()->hasTagName(thTag))
+    auto* firstCell = rowChildren[0].get();
+    if (!firstCell || !firstCell->node() || !firstCell->node()->hasTagName(thTag))
         return nullptr;
-    
+
     // Verify that the row header is not part of an entire row of headers.
     // In that case, it is unlikely this is a row header.
     for (const auto& child : rowChildren) {
         // We found a non-header cell, so this is not an entire row of headers -- return the original header cell.
         if (child->node() && !child->node()->hasTagName(thTag))
-            return cell;
+            return firstCell;
     }
     return nullptr;
 }
