@@ -237,7 +237,12 @@ static RefPtr<CSSCalcExpressionNode> createCSS(const CalcExpressionNode& node, c
             return CSSCalcOperationNode::createHypot(WTFMove(children));
         }
         case CalcOperator::Mod:
-        case CalcOperator::Rem:
+        case CalcOperator::Rem: {
+            auto children = createCSS(operationChildren, style);
+            if (children.size() != 2)
+                return nullptr;
+            return CSSCalcOperationNode::createStep(op, WTFMove(children));
+        }
         case CalcOperator::Round:
         case CalcOperator::Nearest:
         case CalcOperator::ToZero:
