@@ -532,7 +532,7 @@ void BorderPainter::clipBorderSidePolygon(PaintingContext& paintingContext, cons
     if (firstEdgeMatches == secondEdgeMatches) {
         bool wasAntialiased = paintingContext.context.shouldAntialias();
         paintingContext.context.setShouldAntialias(!firstEdgeMatches);
-        paintingContext.context.clipPath(Path::polygonPathFromPoints(quad), WindRule::NonZero);
+        paintingContext.context.clipPath(Path(quad), WindRule::NonZero);
         paintingContext.context.setShouldAntialias(wasAntialiased);
         return;
     }
@@ -547,7 +547,7 @@ void BorderPainter::clipBorderSidePolygon(PaintingContext& paintingContext, cons
     };
     bool wasAntialiased = paintingContext.context.shouldAntialias();
     paintingContext.context.setShouldAntialias(!firstEdgeMatches);
-    paintingContext.context.clipPath(Path::polygonPathFromPoints(firstQuad), WindRule::NonZero);
+    paintingContext.context.clipPath(Path(firstQuad), WindRule::NonZero);
 
     Vector<FloatPoint> secondQuad = {
         quad[0],
@@ -558,7 +558,7 @@ void BorderPainter::clipBorderSidePolygon(PaintingContext& paintingContext, cons
     };
     // Antialiasing affects the second side.
     paintingContext.context.setShouldAntialias(!secondEdgeMatches);
-    paintingContext.context.clipPath(Path::polygonPathFromPoints(secondQuad), WindRule::NonZero);
+    paintingContext.context.clipPath(Path(secondQuad), WindRule::NonZero);
 
     paintingContext.context.setShouldAntialias(wasAntialiased);
 }
@@ -820,7 +820,7 @@ void BorderPainter::drawLineForBoxSide(PaintingContext& paintingContext, const F
         paintingContext.context.setFillColor(color);
         bool wasAntialiased = paintingContext.context.shouldAntialias();
         paintingContext.context.setShouldAntialias(antialias);
-        paintingContext.context.fillPath(Path::polygonPathFromPoints(quad));
+        paintingContext.context.fillPath(Path(quad));
         paintingContext.context.setShouldAntialias(wasAntialiased);
 
         paintingContext.context.setStrokeStyle(oldStrokeStyle);
