@@ -34,13 +34,15 @@
 namespace WebCore {
 
 struct SameSizeAsElementRareData : NodeRareData {
+    unsigned short m_childIndex;
+    int m_tabIndex;
     IntPoint savedLayerScrollPosition;
     Vector<std::unique_ptr<ElementAnimationRareData>> animationRareData;
-    void* pointers[14];
+    void* pointers[16];
     void* intersectionObserverData;
     void* typedOMData[2];
     void* resizeObserverData;
-    void* resizeObserverSize;
+    Markable<LayoutUnit, LayoutUnitMarkableTraits> lastRemembedSize[2];
     ExplicitlySetAttrElementsMap explicitlySetAttrElementsMap;
 };
 
