@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2016-2020 Apple Inc. All rights reserved.
+ * Copyright (C) 2016-2021 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -27,11 +27,15 @@
 #include "StaticRange.h"
 
 #include "ContainerNode.h"
-#include "Range.h"
+#include "JSNode.h"
 #include "Text.h"
+#include "WebCoreOpaqueRootInlines.h"
+#include <wtf/IsoMallocInlines.h>
 
 namespace WebCore {
 
+WTF_MAKE_ISO_ALLOCATED_IMPL(StaticRange);
+
 StaticRange::StaticRange(SimpleRange&& range)
     : SimpleRange(WTFMove(range))
 {
@@ -42,6 +46,12 @@ Ref<StaticRange> StaticRange::create(SimpleRange&& range)
     return adoptRef(*new StaticRange(WTFMove(range)));
 }
 
+Ref<StaticRange> StaticRange::create(const SimpleRange& range)
+{
+    auto copiedRange = range;
+    return create(WTFMove(copiedRange));
+}
+
 static bool isDocumentTypeOrAttr(Node& node)
 {
     // Before calling nodeType, do two fast non-virtual checks that cover almost all normal nodes, but are false for DocumentType and Attr.
@@ -67,4 +77,10 @@ ExceptionOr<Ref<StaticRange>> StaticRange::create(Init&& init)
     return create({ { init.startContainer.releaseNonNull(), init.startOffset }, { init.endContainer.releaseNonNull(), init.endOffset } });
 }
 
+void StaticRange::visitNodesConcurrently(JSC::AbstractSlotVisitor& visitor) const
+{
+    addWebCoreOpaqueRoot(visitor, start.container.get());
+    addWebCoreOpaqueRoot(visitor, end.container.get());
 }
+
+} // namespace WebCore
