@@ -85,9 +85,10 @@ static bool markersHaveIdenticalDescription(const Vector<RenderedDocumentMarker*
 }
 
 AlternativeTextController::AlternativeTextController(Document& document)
-    : m_timer(*this, &AlternativeTextController::timerFired)
+    : m_timer(&document, *this, &AlternativeTextController::timerFired)
     , m_document(document)
 {
+    m_timer.suspendIfNeeded();
 }
 
 AlternativeTextController::~AlternativeTextController()
@@ -98,10 +99,16 @@ AlternativeTextController::~AlternativeTextController()
 void AlternativeTextController::startAlternativeTextUITimer(AlternativeTextType type)
 {
     const Seconds correctionPanelTimerInterval { 300_ms };
-#if ENABLE(ALTERNATIVE_TEXT_REQUIRES_AUTOMATIC_SPELLING_CORRECTION)
-    if (!isAutomaticSpellingCorrectionEnabled())
+
+    if (!isAutomaticSpellingCorrectionEnabled()) {
+#if !ENABLE(ALTERNATIVE_TEXT_REQUIRES_AUTOMATIC_SPELLING_CORRECTION)
+        // Exclude correction & reversion bubbles which have accept on dismiss behavior.
+        if (type == AlternativeTextType::Correction || type == AlternativeTextType::Reversion)
+            return;
+#else
         return;
 #endif
+    }
 
     // If type is PanelTypeReversion, then the new range has been set. So we shouldn't clear it.
     if (type == AlternativeTextType::Correction)
@@ -112,7 +119,7 @@ void AlternativeTextController::startAlternativeTextUITimer(AlternativeTextType
 
 void AlternativeTextController::stopAlternativeTextUITimer()
 {
-    m_timer.stop();
+    m_timer.cancel();
     m_rangeWithAlternative = std::nullopt;
 }
 
@@ -412,16 +419,16 @@ void AlternativeTextController::respondToChangedSelection(const VisibleSelection
     }
 }
 
-#if USE(APPLE_INTERNAL_SDK)
-#include <WebKitAdditions/AlternativeTextControllerAdditions.cpp>
-#else
 static inline void removeCorrectionIndicatorMarkers(Document& document)
 {
+#if HAVE(AUTOCORRECTION_ENHANCEMENTS)
+    document.markers().dismissMarkers(DocumentMarker::CorrectionIndicator);
+#else
     document.markers().removeMarkers(DocumentMarker::CorrectionIndicator);
-}
 #endif
+}
 
-void AlternativeTextController::respondToAppliedEditing(CompositeEditCommand* command)
+void AlternativeTextController::respondToAppliedEditing(Document& document, CompositeEditCommand* command)
 {
     if (command->isTopLevelCommand() && !command->shouldRetainAutocorrectionIndicator())
         removeCorrectionIndicatorMarkers(m_document);
@@ -430,6 +437,13 @@ void AlternativeTextController::respondToAppliedEditing(CompositeEditCommand* co
     m_originalStringForLastDeletedAutocorrection = String();
 
     dismiss(ReasonForDismissingAlternativeText::Ignored);
+
+#if HAVE(AUTOCORRECTION_ENHANCEMENTS) && PLATFORM(IOS_FAMILY)
+    if (!command->shouldRetainAutocorrectionIndicator())
+        document.markers().dismissMarkers(DocumentMarker::CorrectionIndicator);
+#else
+    UNUSED_PARAM(document);
+#endif
 }
 
 void AlternativeTextController::respondToUnappliedEditing(EditCommandComposition* command)
