@@ -27,13 +27,16 @@
 #include "HTMLLabelElement.h"
 
 #include "Document.h"
-#include "ElementIterator.h"
 #include "Event.h"
 #include "EventNames.h"
 #include "FormListedElement.h"
+#include "FrameSelection.h"
 #include "HTMLFormControlElement.h"
 #include "HTMLNames.h"
+#include "LocalFrame.h"
+#include "MouseEvent.h"
 #include "SelectionRestorationMode.h"
+#include "TypedElementDescendantIteratorInlines.h"
 #include <wtf/IsoMallocInlines.h>
 #include <wtf/SetForScope.h>
 
@@ -65,6 +68,11 @@ Ref<HTMLLabelElement> HTMLLabelElement::create(const QualifiedName& tagName, Doc
     return adoptRef(*new HTMLLabelElement(tagName, document));
 }
 
+Ref<HTMLLabelElement> HTMLLabelElement::create(Document& document)
+{
+    return adoptRef(*new HTMLLabelElement(labelTag, document));
+}
+
 RefPtr<HTMLElement> HTMLLabelElement::control() const
 {
     auto& controlId = attributeWithoutSynchronization(forAttr);
@@ -135,6 +143,14 @@ bool HTMLLabelElement::isEventTargetedAtInteractiveDescendants(Event& event) con
 void HTMLLabelElement::defaultEventHandler(Event& event)
 {
     if (event.type() == eventNames().clickEvent && !m_processingClick) {
+        // If text of label element is selected, do not pass the event to control element.
+        // Note: a click event may be not a mouse event if created by document.createEvent().
+        if (event.isMouseEvent() && !downcast<MouseEvent>(event).isSimulated()) {
+            auto* frame = document().frame();
+            if (frame && frame->selection().selection().isRange())
+                return;
+        }
+
         auto control = this->control();
 
         // If we can't find a control or if the control received the click
