@@ -40,6 +40,7 @@
 #include "RenderLineBreak.h"
 #include "RenderListItem.h"
 #include "RenderListMarker.h"
+#include "RenderStyleSetters.h"
 #include "RenderTable.h"
 #include "RenderView.h"
 #include "TextUtil.h"
@@ -184,18 +185,14 @@ UniqueRef<Layout::Box> BoxTree::createLayoutBox(RenderObject& renderer)
     if (is<RenderText>(renderer)) {
         auto& textRenderer = downcast<RenderText>(renderer);
 
-        if (is<RenderCounter>(textRenderer) && textRenderer.preferredLogicalWidthsDirty()) {
-            // This ensures that InlineTextBox always has up-to-date counter text.
-            // Counter content is updated through preferred width computation.
-            downcast<RenderCounter>(textRenderer).updateCounter();
-        }
-
         auto style = RenderStyle::createAnonymousStyleWithDisplay(textRenderer.style(), DisplayType::Inline);
         auto isCombinedText = is<RenderCombineText>(textRenderer) && downcast<RenderCombineText>(textRenderer).isCombined();
         auto text = style.textSecurity() == TextSecurity::None
             ? (isCombinedText ? textRenderer.originalText() : textRenderer.text())
             : RenderBlock::updateSecurityDiscCharacters(style, isCombinedText ? textRenderer.originalText() : textRenderer.text());
-        return makeUniqueRef<Layout::InlineTextBox>(text, isCombinedText, textRenderer.canUseSimplifiedTextMeasuring(), textRenderer.canUseSimpleFontCodePath(), WTFMove(style), WTFMove(firstLineStyle));
+        auto canUseSimpleFontCodePath = textRenderer.canUseSimpleFontCodePath();
+        auto canUseSimplifiedTextMeasuring = canUseSimpleFontCodePath && Layout::TextUtil::canUseSimplifiedTextMeasuring(text, style, firstLineStyle.get());
+        return makeUniqueRef<Layout::InlineTextBox>(text, isCombinedText, canUseSimplifiedTextMeasuring, canUseSimpleFontCodePath, WTFMove(style), WTFMove(firstLineStyle));
     }
 
     auto& renderElement = downcast<RenderElement>(renderer);
@@ -225,8 +222,9 @@ void BoxTree::buildTreeForInlineContent()
                 return existingChildBox->removeFromParent();
             return createLayoutBox(childRenderer);
         };
-        appendChild(childLayoutBox(), childRenderer);
+        insertChild(childLayoutBox(), childRenderer, childRenderer.previousSibling());
     }
+    m_renderers.shrinkToFit();
 }
 
 void BoxTree::buildTreeForFlexContent()
@@ -234,18 +232,20 @@ void BoxTree::buildTreeForFlexContent()
     for (auto& flexItemRenderer : childrenOfType<RenderElement>(m_rootRenderer)) {
         auto style = RenderStyle::clone(flexItemRenderer.style());
         auto flexItem = makeUniqueRef<Layout::ElementBox>(elementAttributes(flexItemRenderer), WTFMove(style));
-        appendChild(WTFMove(flexItem), flexItemRenderer);
+        insertChild(WTFMove(flexItem), flexItemRenderer, flexItemRenderer.previousSibling());
     }
+    m_renderers.shrinkToFit();
 }
 
-void BoxTree::appendChild(UniqueRef<Layout::Box> childBox, RenderObject& childRenderer)
+void BoxTree::insertChild(UniqueRef<Layout::Box> childBox, RenderObject& childRenderer, const RenderObject* beforeChild)
 {
     auto& parentBox = layoutBoxForRenderer(*childRenderer.parent());
+    auto* beforeChildBox = beforeChild ? &layoutBoxForRenderer(*beforeChild) : nullptr;
 
     m_renderers.append(&childRenderer);
 
     childRenderer.setLayoutBox(childBox);
-    parentBox.appendChild(WTFMove(childBox));
+    parentBox.insertChild(WTFMove(childBox), beforeChildBox);
 }
 
 void BoxTree::updateStyle(const RenderBoxModelObject& renderer)
@@ -264,6 +264,42 @@ void BoxTree::updateStyle(const RenderBoxModelObject& renderer)
     }
 }
 
+void BoxTree::updateContent(const RenderText& textRenderer)
+{
+    auto& inlineTextBox = downcast<Layout::InlineTextBox>(layoutBoxForRenderer(textRenderer));
+    auto& style = inlineTextBox.style();
+    auto isCombinedText = is<RenderCombineText>(textRenderer) && downcast<RenderCombineText>(textRenderer).isCombined();
+    auto text = style.textSecurity() == TextSecurity::None ? (isCombinedText ? textRenderer.originalText() : textRenderer.text()) : RenderBlock::updateSecurityDiscCharacters(style, isCombinedText ? textRenderer.originalText() : textRenderer.text());
+    auto canUseSimpleFontCodePath = textRenderer.canUseSimpleFontCodePath();
+    auto canUseSimplifiedTextMeasuring = canUseSimpleFontCodePath && Layout::TextUtil::canUseSimplifiedTextMeasuring(text, style, &inlineTextBox.firstLineStyle());
+
+    inlineTextBox.updateContent(text, canUseSimpleFontCodePath, canUseSimplifiedTextMeasuring);
+}
+
+const Layout::Box& BoxTree::insert(const RenderElement& parent, RenderObject& child, const RenderObject* beforeChild)
+{
+    UNUSED_PARAM(parent);
+
+    insertChild(createLayoutBox(child), child, beforeChild);
+    if (!m_boxToRendererMap.isEmpty())
+        m_boxToRendererMap.add(*child.layoutBox(), child);
+    return layoutBoxForRenderer(child);
+}
+
+UniqueRef<Layout::Box> BoxTree::remove(const RenderElement& parent, RenderObject& child)
+{
+    UNUSED_PARAM(parent);
+    ASSERT(child.layoutBox());
+
+    auto* layoutBox = child.layoutBox();
+
+    m_boxToRendererMap = { };
+    child.clearLayoutBox();
+    // FIXME: Move over to WeakListHashSet if this turns out to be too expensive.
+    m_renderers.removeFirst(&child);
+    return layoutBox->removeFromParent();
+}
+
 const Layout::ElementBox& BoxTree::rootLayoutBox() const
 {
     return *m_rootRenderer.layoutBox();
@@ -327,8 +363,8 @@ Layout::InitialContainingBlock& BoxTree::initialContainingBlock()
 #if ENABLE(TREE_DEBUGGING)
 void showInlineContent(TextStream& stream, const InlineContent& inlineContent, size_t depth)
 {
-    auto& lines = inlineContent.lines;
-    auto& boxes = inlineContent.boxes;
+    auto& lines = inlineContent.displayContent().lines;
+    auto& boxes = inlineContent.displayContent().boxes;
 
     for (size_t lineIndex = 0, boxIndex = 0; lineIndex < lines.size() && boxIndex < boxes.size(); ++lineIndex) {
         auto addSpacing = [&](auto& streamToUse) {
@@ -383,7 +419,7 @@ void showInlineContent(TextStream& stream, const InlineContent& inlineContent, s
                     runStream << "Generic inline level box";
                 runStream << " at (" << box.left() << "," << box.top() << ") size " << box.width() << "x" << box.height();
                 if (box.isText())
-                    runStream << " run(" << box.text()->start() << ", " << box.text()->end() << ")";
+                    runStream << " run(" << box.text().start() << ", " << box.text().end() << ")";
                 runStream << " renderer->(" << &inlineContent.rendererForLayoutBox(box.layoutBox()) << ")";
                 runStream.nextLine();
             }
