@@ -30,19 +30,19 @@
 #include "AXObjectCache.h"
 #include "Chrome.h"
 #include "ChromeClient.h"
-#include "Document.h"
+#include "DocumentInlines.h"
 #include "Editing.h"
 #include "Editor.h"
 #include "EditorClient.h"
 #include "Element.h"
+#include "ElementRareData.h"
 #include "ElementTraversal.h"
 #include "Event.h"
 #include "EventHandler.h"
 #include "EventNames.h"
-#include "Frame.h"
+#include "FocusOptions.h"
 #include "FrameSelection.h"
 #include "FrameTree.h"
-#include "FrameView.h"
 #include "HTMLAreaElement.h"
 #include "HTMLImageElement.h"
 #include "HTMLInputElement.h"
@@ -52,10 +52,13 @@
 #include "HTMLTextAreaElement.h"
 #include "HitTestResult.h"
 #include "KeyboardEvent.h"
+#include "LocalFrame.h"
+#include "LocalFrameView.h"
 #include "Page.h"
 #include "Range.h"
 #include "RenderWidget.h"
 #include "ScrollAnimator.h"
+#include "SelectionRestorationMode.h"
 #include "Settings.h"
 #include "ShadowRoot.h"
 #include "SpatialNavigation.h"
@@ -67,6 +70,23 @@ namespace WebCore {
 
 using namespace HTMLNames;
 
+static bool isOpenPopoverWithInvoker(const Node* node)
+{
+    RefPtr popover = dynamicDowncast<HTMLElement>(node);
+    return popover && popover->isPopoverShowing() && popover->popoverData()->invoker();
+}
+
+static const HTMLFormControlElement* invokerForPopoverShowingState(const Node* node)
+{
+    RefPtr invoker = dynamicDowncast<HTMLFormControlElement>(node);
+    if (!invoker)
+        return nullptr;
+    HTMLElement* popover = invoker->popoverTargetElement();
+    if (popover && popover->isPopoverShowing() && popover->popoverData()->invoker() == invoker)
+        return invoker.get();
+    return nullptr;
+}
+
 static inline bool hasCustomFocusLogic(const Element& element)
 {
     return is<HTMLElement>(element) && downcast<HTMLElement>(element).hasCustomFocusLogic();
@@ -90,6 +110,7 @@ public:
     WEBCORE_EXPORT static FocusNavigationScope scopeOf(Node&);
     static FocusNavigationScope scopeOwnedByScopeOwner(Element&);
     static FocusNavigationScope scopeOwnedByIFrame(HTMLFrameOwnerElement&);
+    static FocusNavigationScope scopeOwnedByPopoverInvoker(const HTMLFormControlElement&);
 
     Node* firstNodeInScope() const;
     Node* lastNodeInScope() const;
@@ -109,6 +130,7 @@ private:
 
     explicit FocusNavigationScope(TreeScope&);
     explicit FocusNavigationScope(HTMLSlotElement&, SlotKind);
+    explicit FocusNavigationScope(Element&);
 
     RefPtr<ContainerNode> m_treeScopeRootNode;
     RefPtr<HTMLSlotElement> m_slotElement;
@@ -120,14 +142,20 @@ Node* FocusNavigationScope::firstChildInScope(const Node& node) const
 {
     if (is<Element>(node) && isFocusScopeOwner(downcast<Element>(node)))
         return nullptr;
-    return node.firstChild();
+    auto* first = node.firstChild();
+    while (isOpenPopoverWithInvoker(first))
+        first = first->nextSibling();
+    return first;
 }
 
 Node* FocusNavigationScope::lastChildInScope(const Node& node) const
 {
     if (is<Element>(node) && isFocusScopeOwner(downcast<Element>(node)))
         return nullptr;
-    return node.lastChild();
+    auto* last = node.lastChild();
+    while (isOpenPopoverWithInvoker(last))
+        last = last->previousSibling();
+    return last;
 }
 
 Node* FocusNavigationScope::parentInScope(const Node& node) const
@@ -159,7 +187,12 @@ Node* FocusNavigationScope::nextSiblingInScope(const Node& node) const
         }
         return nullptr;
     }
-    return node.nextSibling();
+    if (m_treeScopeRootNode == &node)
+        return nullptr;
+    auto* next = node.nextSibling();
+    while (isOpenPopoverWithInvoker(next))
+        next = next->nextSibling();
+    return next;
 }
 
 Node* FocusNavigationScope::previousSiblingInScope(const Node& node) const
@@ -171,7 +204,12 @@ Node* FocusNavigationScope::previousSiblingInScope(const Node& node) const
         }
         return nullptr;
     }
-    return node.previousSibling();
+    if (m_treeScopeRootNode == &node)
+        return nullptr;
+    auto* previous = node.previousSibling();
+    while (isOpenPopoverWithInvoker(previous))
+        previous = previous->previousSibling();
+    return previous;
 }
 
 Node* FocusNavigationScope::firstNodeInScope() const
@@ -180,7 +218,7 @@ Node* FocusNavigationScope::firstNodeInScope() const
         auto* assigneNodes = m_slotElement->assignedNodes();
         if (m_slotKind == SlotKind::Assigned) {
             ASSERT(assigneNodes);
-            return assigneNodes->first();
+            return assigneNodes->first().get();
         }
         ASSERT(m_slotKind == SlotKind::Fallback);
         return m_slotElement->firstChild();
@@ -195,7 +233,7 @@ Node* FocusNavigationScope::lastNodeInScope() const
         auto* assigneNodes = m_slotElement->assignedNodes();
         if (m_slotKind == SlotKind::Assigned) {
             ASSERT(assigneNodes);
-            return assigneNodes->last();
+            return assigneNodes->last().get();
         }
         ASSERT(m_slotKind == SlotKind::Fallback);
         return m_slotElement->lastChild();
@@ -239,6 +277,11 @@ FocusNavigationScope::FocusNavigationScope(HTMLSlotElement& slotElement, SlotKin
 {
 }
 
+FocusNavigationScope::FocusNavigationScope(Element& element)
+    : m_treeScopeRootNode(&element)
+{
+}
+
 Element* FocusNavigationScope::owner() const
 {
     if (m_slotElement)
@@ -247,7 +290,11 @@ Element* FocusNavigationScope::owner() const
     ASSERT(m_treeScopeRootNode);
     if (is<ShadowRoot>(*m_treeScopeRootNode))
         return downcast<ShadowRoot>(*m_treeScopeRootNode).host();
-    if (Frame* frame = m_treeScopeRootNode->document().frame())
+    if (isOpenPopoverWithInvoker(m_treeScopeRootNode.get())) {
+        RELEASE_ASSERT(is<HTMLElement>(m_treeScopeRootNode.get()));
+        return downcast<HTMLElement>(*m_treeScopeRootNode).popoverData()->invoker();
+    }
+    if (auto* frame = m_treeScopeRootNode->document().frame())
         return frame->ownerElement();
     return nullptr;
 }
@@ -265,6 +312,8 @@ FocusNavigationScope FocusNavigationScope::scopeOf(Node& startingNode)
         }
         if (is<ShadowRoot>(currentNode))
             return FocusNavigationScope(downcast<ShadowRoot>(*currentNode));
+        if (isOpenPopoverWithInvoker(currentNode.get()))
+            return FocusNavigationScope(downcast<Element>(*currentNode));
         parentNode = currentNode->parentNode();
         // The scope of a fallback content of a HTMLSlotElement is the slot element
         // but the scope of a HTMLSlotElement is its parent scope.
@@ -287,9 +336,17 @@ FocusNavigationScope FocusNavigationScope::scopeOwnedByScopeOwner(Element& eleme
 
 FocusNavigationScope FocusNavigationScope::scopeOwnedByIFrame(HTMLFrameOwnerElement& frame)
 {
-    ASSERT(frame.contentFrame());
-    ASSERT(frame.contentFrame()->document());
-    return FocusNavigationScope(*frame.contentFrame()->document());
+    ASSERT(is<LocalFrame>(frame.contentFrame()));
+    ASSERT(downcast<LocalFrame>(frame.contentFrame())->document());
+    return FocusNavigationScope(*downcast<LocalFrame>(frame.contentFrame())->document());
+}
+
+FocusNavigationScope FocusNavigationScope::scopeOwnedByPopoverInvoker(const HTMLFormControlElement& invoker)
+{
+    ASSERT(invokerForPopoverShowingState(&invoker));
+    HTMLElement* popover = invoker.popoverTargetElement();
+    ASSERT(isOpenPopoverWithInvoker(popover));
+    return FocusNavigationScope(*popover);
 }
 
 static inline void dispatchEventsOnWindowAndFocusedElement(Document* document, bool focused)
@@ -308,7 +365,7 @@ static inline void dispatchEventsOnWindowAndFocusedElement(Document* document, b
         document->focusedElement()->dispatchBlurEvent(nullptr);
     document->dispatchWindowEvent(Event::create(focused ? eventNames().focusEvent : eventNames().blurEvent, Event::CanBubble::No, Event::IsCancelable::No));
     if (focused && document->focusedElement())
-        document->focusedElement()->dispatchFocusEvent(nullptr, FocusDirectionNone);
+        document->focusedElement()->dispatchFocusEvent(nullptr, { });
 }
 
 static inline bool isFocusableElementOrScopeOwner(Element& element, KeyboardEvent* event)
@@ -332,10 +389,10 @@ static inline int shadowAdjustedTabIndex(Element& element, KeyboardEvent* event)
         if (!element.tabIndexSetExplicitly())
             return 0; // Treat a shadow host without tabindex if it has tabindex=0 even though HTMLElement::tabIndex returns -1 on such an element.
     }
-    return element.shouldBeIgnoredInSequentialFocusNavigation() ? -1 : element.tabIndexSetExplicitly().valueOr(0);
+    return element.shouldBeIgnoredInSequentialFocusNavigation() ? -1 : element.tabIndexSetExplicitly().value_or(0);
 }
 
-FocusController::FocusController(Page& page, OptionSet<ActivityState::Flag> activityState)
+FocusController::FocusController(Page& page, OptionSet<ActivityState> activityState)
     : m_page(page)
     , m_isChangingFocusedFrame(false)
     , m_activityState(activityState)
@@ -343,7 +400,7 @@ FocusController::FocusController(Page& page, OptionSet<ActivityState::Flag> acti
 {
 }
 
-void FocusController::setFocusedFrame(Frame* frame)
+void FocusController::setFocusedFrame(LocalFrame* frame)
 {
     ASSERT(!frame || frame->page() == &m_page);
     if (m_focusedFrame == frame || m_isChangingFocusedFrame)
@@ -351,20 +408,37 @@ void FocusController::setFocusedFrame(Frame* frame)
 
     m_isChangingFocusedFrame = true;
 
-    RefPtr<Frame> oldFrame = m_focusedFrame;
-    RefPtr<Frame> newFrame = frame;
+    RefPtr oldFrame { m_focusedFrame };
+    RefPtr newFrame { frame };
 
     m_focusedFrame = newFrame;
 
     // Now that the frame is updated, fire events and update the selection focused states of both frames.
-    if (oldFrame && oldFrame->view()) {
+    if (auto* oldFrameView = oldFrame ? oldFrame->view() : nullptr) {
+        oldFrameView->stopKeyboardScrollAnimation();
         oldFrame->selection().setFocused(false);
         oldFrame->document()->dispatchWindowEvent(Event::create(eventNames().blurEvent, Event::CanBubble::No, Event::IsCancelable::No));
+#if ENABLE(SERVICE_WORKER)
+        Frame* frame = oldFrame.get();
+        do {
+            if (auto* localFrame = dynamicDowncast<LocalFrame>(frame))
+                localFrame->document()->updateServiceWorkerClientData();
+            frame = frame->tree().parent();
+        } while (frame);
+#endif
     }
 
     if (newFrame && newFrame->view() && isFocused()) {
         newFrame->selection().setFocused(true);
         newFrame->document()->dispatchWindowEvent(Event::create(eventNames().focusEvent, Event::CanBubble::No, Event::IsCancelable::No));
+#if ENABLE(SERVICE_WORKER)
+        Frame* frame = newFrame.get();
+        do {
+            if (auto* localFrame = dynamicDowncast<LocalFrame>(frame))
+                localFrame->document()->updateServiceWorkerClientData();
+            frame = frame->tree().parent();
+        } while (frame);
+#endif
     }
 
     m_page.chrome().focusedFrameChanged(newFrame.get());
@@ -372,11 +446,16 @@ void FocusController::setFocusedFrame(Frame* frame)
     m_isChangingFocusedFrame = false;
 }
 
-Frame& FocusController::focusedOrMainFrame() const
+LocalFrame& FocusController::focusedOrMainFrame() const
 {
-    if (Frame* frame = focusedFrame())
+    if (auto* frame = focusedFrame())
         return *frame;
-    return m_page.mainFrame();
+    if (auto* localMainFrame = dynamicDowncast<LocalFrame>(m_page.mainFrame()))
+        return *localMainFrame;
+
+    // FIXME: Do something better here in the site isolated case.
+    ASSERT(m_page.settings().siteIsolationEnabled());
+    return *m_page.rootFrames().begin();
 }
 
 void FocusController::setFocused(bool focused)
@@ -389,10 +468,12 @@ void FocusController::setFocusedInternal(bool focused)
     if (!isFocused())
         focusedOrMainFrame().eventHandler().stopAutoscrollTimer();
 
-    if (!m_focusedFrame)
-        setFocusedFrame(&m_page.mainFrame());
+    if (!m_focusedFrame) {
+        if (auto* localMainFrame = dynamicDowncast<LocalFrame>(m_page.mainFrame()))
+            setFocusedFrame(localMainFrame);
+    }
 
-    if (m_focusedFrame->view()) {
+    if (m_focusedFrame && m_focusedFrame->view()) {
         m_focusedFrame->selection().setFocused(focused);
         dispatchEventsOnWindowAndFocusedElement(m_focusedFrame->document(), focused);
     }
@@ -405,9 +486,10 @@ Element* FocusController::findFocusableElementDescendingIntoSubframes(FocusDirec
     // 2) the deepest-nested HTMLFrameOwnerElement.
     while (is<HTMLFrameOwnerElement>(element)) {
         HTMLFrameOwnerElement& owner = downcast<HTMLFrameOwnerElement>(*element);
-        if (!owner.contentFrame() || !owner.contentFrame()->document())
+        auto* localContentFrame = dynamicDowncast<LocalFrame>(owner.contentFrame());
+        if (!localContentFrame || !localContentFrame->document())
             break;
-        owner.contentFrame()->document()->updateLayoutIgnorePendingStylesheets();
+        localContentFrame->document()->updateLayoutIgnorePendingStylesheets();
         Element* foundElement = findFocusableElementWithinScope(direction, FocusNavigationScope::scopeOwnedByIFrame(owner), nullptr, event);
         if (!foundElement)
             break;
@@ -433,13 +515,13 @@ bool FocusController::setInitialFocus(FocusDirection direction, KeyboardEvent* p
 bool FocusController::advanceFocus(FocusDirection direction, KeyboardEvent* event, bool initialFocus)
 {
     switch (direction) {
-    case FocusDirectionForward:
-    case FocusDirectionBackward:
+    case FocusDirection::Forward:
+    case FocusDirection::Backward:
         return advanceFocusInDocumentOrder(direction, event, initialFocus);
-    case FocusDirectionLeft:
-    case FocusDirectionRight:
-    case FocusDirectionUp:
-    case FocusDirectionDown:
+    case FocusDirection::Left:
+    case FocusDirection::Right:
+    case FocusDirection::Up:
+    case FocusDirection::Down:
         return advanceFocusDirectionally(direction, event);
     default:
         ASSERT_NOT_REACHED();
@@ -465,7 +547,7 @@ bool FocusController::relinquishFocusToChrome(FocusDirection direction)
 
 bool FocusController::advanceFocusInDocumentOrder(FocusDirection direction, KeyboardEvent* event, bool initialFocus)
 {
-    Frame& frame = focusedOrMainFrame();
+    LocalFrame& frame = focusedOrMainFrame();
     Document* document = frame.document();
 
     Node* currentNode = document->focusNavigationStartingNode(direction);
@@ -487,7 +569,10 @@ bool FocusController::advanceFocusInDocumentOrder(FocusDirection direction, Keyb
         }
 
         // Chrome doesn't want focus, so we should wrap focus.
-        element = findFocusableElementAcrossFocusScope(direction, FocusNavigationScope::scopeOf(*m_page.mainFrame().document()), nullptr, event);
+        auto* localMainFrame = dynamicDowncast<LocalFrame>(m_page.mainFrame());
+        if (!localMainFrame)
+            return false;
+        element = findFocusableElementAcrossFocusScope(direction, FocusNavigationScope::scopeOf(*localMainFrame->document()), nullptr, event);
 
         if (!element)
             return false;
@@ -508,7 +593,7 @@ bool FocusController::advanceFocusInDocumentOrder(FocusDirection direction, Keyb
             return false;
 
         document->setFocusedElement(nullptr);
-        setFocusedFrame(owner.contentFrame());
+        setFocusedFrame(dynamicDowncast<LocalFrame>(owner.contentFrame()));
         return true;
     }
     
@@ -526,15 +611,14 @@ bool FocusController::advanceFocusInDocumentOrder(FocusDirection direction, Keyb
     setFocusedFrame(newDocument.frame());
 
     if (caretBrowsing) {
-        Position position = firstPositionInOrBeforeNode(element.get());
-        VisibleSelection newSelection(position, position, DOWNSTREAM);
+        VisibleSelection newSelection(firstPositionInOrBeforeNode(element.get()), Affinity::Downstream);
         if (frame.selection().shouldChangeSelection(newSelection)) {
             AXTextStateChangeIntent intent(AXTextStateChangeTypeSelectionMove, AXTextSelection { AXTextSelectionDirectionDiscontiguous, AXTextSelectionGranularityUnknown, true });
-            frame.selection().setSelection(newSelection, FrameSelection::defaultSetSelectionOptions(UserTriggered), intent);
+            frame.selection().setSelection(newSelection, FrameSelection::defaultSetSelectionOptions(UserTriggered::Yes), intent);
         }
     }
 
-    element->focus(false, direction);
+    element->focus({ SelectionRestorationMode::SelectAll, direction, { }, { }, FocusVisibility::Visible });
     return true;
 }
 
@@ -542,18 +626,28 @@ Element* FocusController::findFocusableElementAcrossFocusScope(FocusDirection di
 {
     ASSERT(!is<Element>(currentNode) || !isNonFocusableScopeOwner(downcast<Element>(*currentNode), event));
 
-    if (currentNode && direction == FocusDirectionForward && is<Element>(currentNode) && isFocusableScopeOwner(downcast<Element>(*currentNode), event)) {
-        if (Element* candidateInInnerScope = findFocusableElementWithinScope(direction, FocusNavigationScope::scopeOwnedByScopeOwner(downcast<Element>(*currentNode)), 0, event))
-            return candidateInInnerScope;
+    if (currentNode && direction == FocusDirection::Forward && is<Element>(currentNode)) {
+        if (isFocusableScopeOwner(downcast<Element>(*currentNode), event)) {
+            if (Element* candidateInInnerScope = findFocusableElementWithinScope(direction, FocusNavigationScope::scopeOwnedByScopeOwner(downcast<Element>(*currentNode)), 0, event))
+                return candidateInInnerScope;
+        } else if (auto* invoker = invokerForPopoverShowingState(currentNode)) {
+            if (Element* candidateInInnerScope = findFocusableElementWithinScope(direction, FocusNavigationScope::scopeOwnedByPopoverInvoker(*invoker), nullptr, event))
+                return candidateInInnerScope;
+        }
     }
 
-    if (Element* candidateInCurrentScope = findFocusableElementWithinScope(direction, scope, currentNode, event))
+    if (Element* candidateInCurrentScope = findFocusableElementWithinScope(direction, scope, currentNode, event)) {
+        if (direction == FocusDirection::Backward) {
+            while (auto* invoker = invokerForPopoverShowingState(candidateInCurrentScope))
+                candidateInCurrentScope = findFocusableElementWithinScope(direction, FocusNavigationScope::scopeOwnedByPopoverInvoker(*invoker), nullptr, event);
+        }
         return candidateInCurrentScope;
+    }
 
     // If there's no focusable node to advance to, move up the focus scopes until we find one.
     Element* owner = scope.owner();
     while (owner) {
-        if (direction == FocusDirectionBackward && isFocusableScopeOwner(*owner, event))
+        if (direction == FocusDirection::Backward && isFocusableScopeOwner(*owner, event))
             return findFocusableElementDescendingIntoSubframes(direction, owner, event);
 
         auto outerScope = FocusNavigationScope::scopeOf(*owner);
@@ -567,7 +661,7 @@ Element* FocusController::findFocusableElementAcrossFocusScope(FocusDirection di
 Element* FocusController::findFocusableElementWithinScope(FocusDirection direction, const FocusNavigationScope& scope, Node* start, KeyboardEvent* event)
 {
     // Starting node is exclusive.
-    Element* candidate = direction == FocusDirectionForward
+    Element* candidate = direction == FocusDirection::Forward
         ? nextFocusableElementWithinScope(scope, start, event)
         : previousFocusableElementWithinScope(scope, start, event);
     return findFocusableElementDescendingIntoSubframes(direction, candidate, event);
@@ -607,7 +701,7 @@ Element* FocusController::previousFocusableElementWithinScope(const FocusNavigat
 
 Element* FocusController::findFocusableElementOrScopeOwner(FocusDirection direction, const FocusNavigationScope& scope, Node* node, KeyboardEvent* event)
 {
-    return (direction == FocusDirectionForward)
+    return (direction == FocusDirection::Forward)
         ? nextFocusableElementOrScopeOwner(scope, node, event)
         : previousFocusableElementOrScopeOwner(scope, node, event);
 }
@@ -615,7 +709,7 @@ Element* FocusController::findFocusableElementOrScopeOwner(FocusDirection direct
 Element* FocusController::findElementWithExactTabIndex(const FocusNavigationScope& scope, Node* start, int tabIndex, KeyboardEvent* event, FocusDirection direction)
 {
     // Search is inclusive of start
-    for (Node* node = start; node; node = direction == FocusDirectionForward ? scope.nextInScope(node) : scope.previousInScope(node)) {
+    for (Node* node = start; node; node = direction == FocusDirection::Forward ? scope.nextInScope(node) : scope.previousInScope(node)) {
         if (!is<Element>(*node))
             continue;
         Element& element = downcast<Element>(*node);
@@ -666,14 +760,14 @@ Element* FocusController::nextFocusableElement(Node& start)
 {
     // FIXME: This can return a non-focusable shadow host.
     // FIXME: This can't give the correct answer that takes modifier keys into account since it doesn't pass an event.
-    return nextFocusableElementOrScopeOwner(FocusNavigationScope::scopeOf(start), &start, nullptr);
+    return findFocusableElementAcrossFocusScope(FocusDirection::Forward, FocusNavigationScope::scopeOf(start), &start, nullptr);
 }
 
 Element* FocusController::previousFocusableElement(Node& start)
 {
     // FIXME: This can return a non-focusable shadow host.
     // FIXME: This can't give the correct answer that takes modifier keys into account since it doesn't pass an event.
-    return previousFocusableElementOrScopeOwner(FocusNavigationScope::scopeOf(start), &start, nullptr);
+    return findFocusableElementAcrossFocusScope(FocusDirection::Backward, FocusNavigationScope::scopeOf(start), &start, nullptr);
 }
 
 Element* FocusController::nextFocusableElementOrScopeOwner(const FocusNavigationScope& scope, Node* start, KeyboardEvent* event)
@@ -695,7 +789,7 @@ Element* FocusController::nextFocusableElementOrScopeOwner(const FocusNavigation
         }
 
         // First try to find a node with the same tabindex as start that comes after start in the scope.
-        if (Element* winner = findElementWithExactTabIndex(scope, scope.nextInScope(start), startTabIndex, event, FocusDirectionForward))
+        if (Element* winner = findElementWithExactTabIndex(scope, scope.nextInScope(start), startTabIndex, event, FocusDirection::Forward))
             return winner;
 
         if (!startTabIndex)
@@ -710,7 +804,7 @@ Element* FocusController::nextFocusableElementOrScopeOwner(const FocusNavigation
 
     // There are no nodes with a tabindex greater than start's tabindex,
     // so find the first node with a tabindex of 0.
-    return findElementWithExactTabIndex(scope, scope.firstNodeInScope(), 0, event, FocusDirectionForward);
+    return findElementWithExactTabIndex(scope, scope.firstNodeInScope(), 0, event, FocusDirection::Forward);
 }
 
 Element* FocusController::previousFocusableElementOrScopeOwner(const FocusNavigationScope& scope, Node* start, KeyboardEvent* event)
@@ -742,7 +836,7 @@ Element* FocusController::previousFocusableElementOrScopeOwner(const FocusNaviga
         }
     }
 
-    if (Element* winner = findElementWithExactTabIndex(scope, startingNode, startingTabIndex, event, FocusDirectionBackward))
+    if (Element* winner = findElementWithExactTabIndex(scope, startingNode, startingTabIndex, event, FocusDirection::Backward))
         return winner;
 
     // There are no nodes before start with the same tabindex as start, so look for a node that:
@@ -764,7 +858,7 @@ static bool relinquishesEditingFocus(Element& element)
     return frame->editor().shouldEndEditing(makeRangeSelectingNodeContents(*root));
 }
 
-static void clearSelectionIfNeeded(Frame* oldFocusedFrame, Frame* newFocusedFrame, Node* newFocusedNode)
+static void clearSelectionIfNeeded(LocalFrame* oldFocusedFrame, LocalFrame* newFocusedFrame, Node* newFocusedNode)
 {
     if (!oldFocusedFrame || !newFocusedFrame)
         return;
@@ -814,7 +908,10 @@ static bool shouldClearSelectionWhenChangingFocusedElement(const Page& page, Ref
     if (!oldFocusedElement->isRootEditableElement() && !is<HTMLInputElement>(oldFocusedElement) && !is<HTMLTextAreaElement>(oldFocusedElement))
         return true;
 
-    for (auto ancestor = page.mainFrame().eventHandler().draggedElement(); ancestor; ancestor = ancestor->parentOrShadowHostElement()) {
+    auto* localMainFrame = dynamicDowncast<LocalFrame>(page.mainFrame());
+    if (!localMainFrame)
+        return false;
+    for (auto ancestor = localMainFrame->eventHandler().draggedElement(); ancestor; ancestor = ancestor->parentOrShadowHostElement()) {
         if (ancestor == oldFocusedElement)
             return false;
     }
@@ -826,16 +923,16 @@ static bool shouldClearSelectionWhenChangingFocusedElement(const Page& page, Ref
     return true;
 }
 
-bool FocusController::setFocusedElement(Element* element, Frame& newFocusedFrame, FocusDirection direction)
+bool FocusController::setFocusedElement(Element* element, LocalFrame& newFocusedFrame, const FocusOptions& options)
 {
-    Ref<Frame> protectedNewFocusedFrame = newFocusedFrame;
-    RefPtr<Frame> oldFocusedFrame = focusedFrame();
-    RefPtr<Document> oldDocument = oldFocusedFrame ? oldFocusedFrame->document() : nullptr;
+    Ref protectedNewFocusedFrame { newFocusedFrame };
+    RefPtr oldFocusedFrame { focusedFrame() };
+    RefPtr oldDocument = oldFocusedFrame ? oldFocusedFrame->document() : nullptr;
     
     Element* oldFocusedElement = oldDocument ? oldDocument->focusedElement() : nullptr;
     if (oldFocusedElement == element) {
         if (element)
-            m_page.chrome().client().elementDidRefocus(*element);
+            m_page.chrome().client().elementDidRefocus(*element, options);
         return true;
     }
 
@@ -873,7 +970,7 @@ bool FocusController::setFocusedElement(Element* element, Frame& newFocusedFrame
 
     Ref<Element> protect(*element);
 
-    bool successfullyFocused = newDocument->setFocusedElement(element, direction);
+    bool successfullyFocused = newDocument->setFocusedElement(element, options);
     if (!successfullyFocused)
         return false;
 
@@ -886,7 +983,7 @@ bool FocusController::setFocusedElement(Element* element, Frame& newFocusedFrame
     return true;
 }
 
-void FocusController::setActivityState(OptionSet<ActivityState::Flag> activityState)
+void FocusController::setActivityState(OptionSet<ActivityState> activityState)
 {
     auto changed = m_activityState ^ activityState;
     m_activityState = activityState;
@@ -907,7 +1004,10 @@ void FocusController::setActive(bool active)
 
 void FocusController::setActiveInternal(bool active)
 {
-    if (FrameView* view = m_page.mainFrame().view()) {
+    auto* localMainFrame = dynamicDowncast<LocalFrame>(m_page.mainFrame());
+    if (!localMainFrame)
+        return;
+    if (auto* view = localMainFrame->view()) {
         if (!view->platformWidget()) {
             view->updateLayoutAndStyleIfNeededRecursive();
             view->updateControlTints();
@@ -930,25 +1030,32 @@ static void contentAreaDidShowOrHide(ScrollableArea* scrollableArea, bool didSho
 
 void FocusController::setIsVisibleAndActiveInternal(bool contentIsVisible)
 {
-    FrameView* view = m_page.mainFrame().view();
+    auto* localMainFrame = dynamicDowncast<LocalFrame>(m_page.mainFrame());
+    if (!localMainFrame)
+        return;
+
+    auto* view = localMainFrame->view();
     if (!view)
         return;
 
     contentAreaDidShowOrHide(view, contentIsVisible);
 
-    for (Frame* frame = &m_page.mainFrame(); frame; frame = frame->tree().traverseNext()) {
-        FrameView* frameView = frame->view();
+    for (Frame* frame = localMainFrame; frame; frame = frame->tree().traverseNext()) {
+        auto* localFrame = dynamicDowncast<LocalFrame>(frame);
+        if (!localFrame)
+            continue;
+        auto* frameView = localFrame->view();
         if (!frameView)
             continue;
 
-        const HashSet<ScrollableArea*>* scrollableAreas = frameView->scrollableAreas();
+        auto scrollableAreas = frameView->scrollableAreas();
         if (!scrollableAreas)
             continue;
 
-        for (auto& scrollableArea : *scrollableAreas) {
+        for (auto& area : *scrollableAreas) {
+            CheckedPtr<ScrollableArea> scrollableArea(area);
             ASSERT(scrollableArea->scrollbarsCanBeActive() || m_page.shouldSuppressScrollbarAnimations());
-
-            contentAreaDidShowOrHide(scrollableArea, contentIsVisible);
+            contentAreaDidShowOrHide(scrollableArea.get(), contentIsVisible);
         }
     }
 }
@@ -970,7 +1077,7 @@ static void updateFocusCandidateIfNeeded(FocusDirection direction, const FocusCa
     if (candidate.distance == maxDistance())
         return;
 
-    if (candidate.isOffscreenAfterScrolling && candidate.alignment < Full)
+    if (candidate.isOffscreenAfterScrolling && candidate.alignment < RectsAlignment::Full)
         return;
 
     if (closest.isNull()) {
@@ -982,8 +1089,11 @@ static void updateFocusCandidateIfNeeded(FocusDirection direction, const FocusCa
     if (!intersectionRect.isEmpty() && !areElementsOnSameLine(closest, candidate)) {
         // If 2 nodes are intersecting, do hit test to find which node in on top.
         auto center = flooredIntPoint(intersectionRect.center()); // FIXME: Would roundedIntPoint be better?
-        constexpr OptionSet<HitTestRequest::RequestType> hitType { HitTestRequest::ReadOnly, HitTestRequest::Active, HitTestRequest::IgnoreClipping, HitTestRequest::DisallowUserAgentShadowContent, HitTestRequest::AllowChildFrameContent };
-        HitTestResult result = candidate.visibleNode->document().page()->mainFrame().eventHandler().hitTestResultAtPoint(center, hitType);
+        constexpr OptionSet<HitTestRequest::Type> hitType { HitTestRequest::Type::ReadOnly, HitTestRequest::Type::Active, HitTestRequest::Type::IgnoreClipping, HitTestRequest::Type::DisallowUserAgentShadowContent, HitTestRequest::Type::AllowChildFrameContent };
+        auto* localMainFrame = dynamicDowncast<LocalFrame>(candidate.visibleNode->document().page()->mainFrame());
+        if (!localMainFrame)
+            return;
+        HitTestResult result = localMainFrame->eventHandler().hitTestResultAtPoint(center, hitType);
         if (candidate.visibleNode->contains(result.innerNode())) {
             closest = candidate;
             return;
@@ -1067,7 +1177,7 @@ bool FocusController::advanceFocusDirectionallyInContainer(Node* container, cons
         // If we have an iframe without the src attribute, it will not have a contentFrame().
         // We ASSERT here to make sure that
         // updateFocusCandidateIfNeeded() will never consider such an iframe as a candidate.
-        ASSERT(frameElement->contentFrame());
+        ASSERT(is<LocalFrame>(frameElement->contentFrame()));
 
         if (focusCandidate.isOffscreenAfterScrolling) {
             scrollInDirection(&focusCandidate.visibleNode->document(), direction);
@@ -1078,8 +1188,8 @@ bool FocusController::advanceFocusDirectionallyInContainer(Node* container, cons
         Element* focusedElement = focusedOrMainFrame().document()->focusedElement();
         if (focusedElement && !hasOffscreenRect(focusedElement))
             rect = nodeRectInAbsoluteCoordinates(focusedElement, true /* ignore border */);
-        frameElement->contentFrame()->document()->updateLayoutIgnorePendingStylesheets();
-        if (!advanceFocusDirectionallyInContainer(frameElement->contentFrame()->document(), rect, direction, event)) {
+        dynamicDowncast<LocalFrame>(frameElement->contentFrame())->document()->updateLayoutIgnorePendingStylesheets();
+        if (!advanceFocusDirectionallyInContainer(dynamicDowncast<LocalFrame>(frameElement->contentFrame())->document(), rect, direction, event)) {
             // The new frame had nothing interesting, need to find another candidate.
             return advanceFocusDirectionallyInContainer(container, nodeRectInAbsoluteCoordinates(focusCandidate.visibleNode, true), direction, event);
         }
@@ -1108,7 +1218,7 @@ bool FocusController::advanceFocusDirectionallyInContainer(Node* container, cons
     Element* element = downcast<Element>(focusCandidate.focusableNode);
     ASSERT(element);
 
-    element->focus(false, direction);
+    element->focus({ SelectionRestorationMode::SelectAll, direction });
     return true;
 }
 
