@@ -28,31 +28,41 @@
 
 #include "BackForwardController.h"
 #include "Document.h"
-#include "Frame.h"
 #include "FrameLoader.h"
-#include "FrameLoaderClient.h"
 #include "HistoryController.h"
 #include "HistoryItem.h"
+#include "LocalFrame.h"
+#include "LocalFrameLoaderClient.h"
 #include "Logging.h"
 #include "NavigationScheduler.h"
+#include "OriginAccessPatterns.h"
 #include "Page.h"
 #include "ScriptController.h"
 #include "SecurityOrigin.h"
 #include <wtf/CheckedArithmetic.h>
+#include <wtf/IsoMallocInlines.h>
 #include <wtf/MainThread.h>
+#include <wtf/text/StringConcatenateNumbers.h>
+
+#if PLATFORM(COCOA)
+#include <wtf/cocoa/RuntimeApplicationChecksCocoa.h>
+#endif
 
 namespace WebCore {
 
-History::History(Frame& frame)
-    : DOMWindowProperty(&frame)
+WTF_MAKE_ISO_ALLOCATED_IMPL(History);
+
+History::History(LocalDOMWindow& window)
+    : LocalDOMWindowProperty(&window)
 {
 }
 
 unsigned History::length() const
 {
-    if (!m_frame)
+    auto* frame = this->frame();
+    if (!frame)
         return 0;
-    auto* page = m_frame->page();
+    auto* page = frame->page();
     if (!page)
         return 0;
     return page->backForward().count();
@@ -60,10 +70,11 @@ unsigned History::length() const
 
 ExceptionOr<History::ScrollRestoration> History::scrollRestoration() const
 {
-    if (!m_frame)
+    auto* frame = this->frame();
+    if (!frame)
         return Exception { SecurityError };
 
-    auto* historyItem = m_frame->loader().history().currentItem();
+    auto* historyItem = frame->loader().history().currentItem();
     if (!historyItem)
         return ScrollRestoration::Auto;
     
@@ -72,10 +83,11 @@ ExceptionOr<History::ScrollRestoration> History::scrollRestoration() const
 
 ExceptionOr<void> History::setScrollRestoration(ScrollRestoration scrollRestoration)
 {
-    if (!m_frame)
+    auto* frame = this->frame();
+    if (!frame)
         return Exception { SecurityError };
 
-    auto* historyItem = m_frame->loader().history().currentItem();
+    auto* historyItem = frame->loader().history().currentItem();
     if (historyItem)
         historyItem->setShouldRestoreScrollPosition(scrollRestoration == ScrollRestoration::Auto);
 
@@ -90,9 +102,10 @@ SerializedScriptValue* History::state()
 
 SerializedScriptValue* History::stateInternal() const
 {
-    if (!m_frame)
+    auto* frame = this->frame();
+    if (!frame)
         return nullptr;
-    auto* historyItem = m_frame->loader().history().currentItem();
+    auto* historyItem = frame->loader().history().currentItem();
     if (!historyItem)
         return nullptr;
     return historyItem->stateObject();
@@ -106,7 +119,7 @@ bool History::stateChanged() const
 JSValueInWrappedObject& History::cachedState()
 {
     if (m_cachedState && stateChanged())
-        m_cachedState = { };
+        m_cachedState.clear();
     return m_cachedState;
 }
 
@@ -137,89 +150,99 @@ void History::forward(Document& document)
 
 void History::go(int distance)
 {
-    LOG(History, "History %p go(%d) frame %p (main frame %d)", this, distance, m_frame, m_frame ? m_frame->isMainFrame() : false);
+    auto* frame = this->frame();
+    LOG(History, "History %p go(%d) frame %p (main frame %d)", this, distance, frame, frame ? frame->isMainFrame() : false);
 
-    if (!m_frame)
+    if (!frame)
         return;
 
-    m_frame->navigationScheduler().scheduleHistoryNavigation(distance);
+    frame->navigationScheduler().scheduleHistoryNavigation(distance);
 }
 
 void History::go(Document& document, int distance)
 {
-    LOG(History, "History %p go(%d) in document %p frame %p (main frame %d)", this, distance, &document, m_frame, m_frame ? m_frame->isMainFrame() : false);
+    auto* frame = this->frame();
+    LOG(History, "History %p go(%d) in document %p frame %p (main frame %d)", this, distance, &document, frame, frame ? frame->isMainFrame() : false);
 
-    if (!m_frame)
+    if (!frame)
         return;
 
     ASSERT(isMainThread());
 
-    if (!document.canNavigate(m_frame))
+    if (!document.canNavigate(frame))
         return;
 
-    m_frame->navigationScheduler().scheduleHistoryNavigation(distance);
+    frame->navigationScheduler().scheduleHistoryNavigation(distance);
 }
 
 URL History::urlForState(const String& urlString)
 {
+    auto* frame = this->frame();
     if (urlString.isNull())
-        return m_frame->document()->url();
-    return m_frame->document()->completeURL(urlString);
+        return frame->document()->url();
+    return frame->document()->completeURL(urlString);
 }
 
 ExceptionOr<void> History::stateObjectAdded(RefPtr<SerializedScriptValue>&& data, const String& title, const String& urlString, StateObjectType stateObjectType)
 {
-    m_cachedState = { };
+    m_cachedState.clear();
 
     // Each unique main-frame document is only allowed to send 64MB of state object payload to the UI client/process.
     static uint32_t totalStateObjectPayloadLimit = 0x4000000;
-    static Seconds stateObjectTimeSpan { 30_s };
+    static Seconds stateObjectTimeSpan { 10_s };
     static unsigned perStateObjectTimeSpanLimit = 100;
 
-    if (!m_frame || !m_frame->page())
+    auto* frame = this->frame();
+    if (!frame || !frame->page())
         return { };
 
     URL fullURL = urlForState(urlString);
     if (!fullURL.isValid())
         return Exception { SecurityError };
 
-    const URL& documentURL = m_frame->document()->url();
+    const URL& documentURL = frame->document()->url();
 
     auto createBlockedURLSecurityErrorWithMessageSuffix = [&] (const char* suffix) {
         const char* functionName = stateObjectType == StateObjectType::Replace ? "history.replaceState()" : "history.pushState()";
         return Exception { SecurityError, makeString("Blocked attempt to use ", functionName, " to change session history URL from ", documentURL.stringCenterEllipsizedToLength(), " to ", fullURL.stringCenterEllipsizedToLength(), ". ", suffix) };
     };
-    if (!protocolHostAndPortAreEqual(fullURL, documentURL) || fullURL.user() != documentURL.user() || fullURL.pass() != documentURL.pass())
+    if (!protocolHostAndPortAreEqual(fullURL, documentURL) || fullURL.user() != documentURL.user() || fullURL.password() != documentURL.password())
         return createBlockedURLSecurityErrorWithMessageSuffix("Protocols, domains, ports, usernames, and passwords must match.");
 
-    const auto& documentSecurityOrigin = m_frame->document()->securityOrigin();
+    if (fullURL.protocolIsFile()
+#if PLATFORM(COCOA)
+        && linkedOnOrAfterSDKWithBehavior(SDKAlignedBehavior::PushStateFilePathRestriction)
+#endif
+        && fullURL.fileSystemPath() != documentURL.fileSystemPath()) {
+        return createBlockedURLSecurityErrorWithMessageSuffix("Only differences in query and fragment are allowed for file: URLs.");
+    }
+
+    const auto& documentSecurityOrigin = frame->document()->securityOrigin();
     // We allow sandboxed documents, 'data:'/'file:' URLs, etc. to use 'pushState'/'replaceState' to modify the URL query and fragments.
     // See https://bugs.webkit.org/show_bug.cgi?id=183028 for the compatibility concerns.
-    bool allowSandboxException = (documentSecurityOrigin.isLocal() || documentSecurityOrigin.isUnique()) && equalIgnoringQueryAndFragment(documentURL, fullURL);
+    bool allowSandboxException = (documentSecurityOrigin.isLocal() || documentSecurityOrigin.isOpaque())
+        && documentURL.viewWithoutQueryOrFragmentIdentifier() == fullURL.viewWithoutQueryOrFragmentIdentifier();
 
-    if (!allowSandboxException && !documentSecurityOrigin.canRequest(fullURL) && (fullURL.path() != documentURL.path() || fullURL.query() != documentURL.query()))
+    if (!allowSandboxException && !documentSecurityOrigin.canRequest(fullURL, OriginAccessPatternsForWebProcess::singleton()) && (fullURL.path() != documentURL.path() || fullURL.query() != documentURL.query()))
         return createBlockedURLSecurityErrorWithMessageSuffix("Paths and fragments must match for a sandboxed document.");
 
-    Document* mainDocument = m_frame->page()->mainFrame().document();
-    History* mainHistory = nullptr;
-    if (mainDocument) {
-        if (auto* mainDOMWindow = mainDocument->domWindow())
-            mainHistory = mainDOMWindow->history();
-    }
-
-    if (!mainHistory)
+    auto* localMainFrame = dynamicDowncast<LocalFrame>(frame->page()->mainFrame());
+    auto* mainWindow = localMainFrame ? localMainFrame->window() : nullptr;
+    if (!mainWindow)
         return { };
 
+    auto& mainHistory = mainWindow->history();
+
     WallTime currentTimestamp = WallTime::now();
-    if (currentTimestamp - mainHistory->m_currentStateObjectTimeSpanStart > stateObjectTimeSpan) {
-        mainHistory->m_currentStateObjectTimeSpanStart = currentTimestamp;
-        mainHistory->m_currentStateObjectTimeSpanObjectsAdded = 0;
+    if (currentTimestamp - mainHistory.m_currentStateObjectTimeSpanStart > stateObjectTimeSpan) {
+        mainHistory.m_currentStateObjectTimeSpanStart = currentTimestamp;
+        mainHistory.m_currentStateObjectTimeSpanObjectsAdded = 0;
     }
     
-    if (mainHistory->m_currentStateObjectTimeSpanObjectsAdded >= perStateObjectTimeSpanLimit) {
+    if (mainHistory.m_currentStateObjectTimeSpanObjectsAdded >= perStateObjectTimeSpanLimit) {
         if (stateObjectType == StateObjectType::Replace)
-            return Exception { SecurityError, String::format("Attempt to use history.replaceState() more than %u times per %f seconds", perStateObjectTimeSpanLimit, stateObjectTimeSpan.seconds()) };
-        return Exception { SecurityError, String::format("Attempt to use history.pushState() more than %u times per %f seconds", perStateObjectTimeSpanLimit, stateObjectTimeSpan.seconds()) };
+            return Exception { SecurityError, makeString("Attempt to use history.replaceState() more than ", perStateObjectTimeSpanLimit, " times per ", stateObjectTimeSpan.seconds(), " seconds") };
+        return Exception { SecurityError, makeString("Attempt to use history.pushState() more than ", perStateObjectTimeSpanLimit, " times per ", stateObjectTimeSpan.seconds(), " seconds") };
     }
 
     Checked<unsigned> titleSize = title.length();
@@ -230,9 +253,9 @@ ExceptionOr<void> History::stateObjectAdded(RefPtr<SerializedScriptValue>&& data
 
     Checked<uint64_t> payloadSize = titleSize;
     payloadSize += urlSize;
-    payloadSize += data ? data->data().size() : 0;
+    payloadSize += data ? data->wireBytes().size() : 0;
 
-    Checked<uint64_t> newTotalUsage = mainHistory->m_totalStateObjectUsage;
+    Checked<uint64_t> newTotalUsage = mainHistory.m_totalStateObjectUsage;
 
     if (stateObjectType == StateObjectType::Replace)
         newTotalUsage -= m_mostRecentStateObjectUsage;
@@ -244,20 +267,20 @@ ExceptionOr<void> History::stateObjectAdded(RefPtr<SerializedScriptValue>&& data
         return Exception { QuotaExceededError, "Attempt to store more data than allowed using history.pushState()"_s };
     }
 
-    m_mostRecentStateObjectUsage = payloadSize.unsafeGet();
+    m_mostRecentStateObjectUsage = payloadSize;
 
-    mainHistory->m_totalStateObjectUsage = newTotalUsage.unsafeGet();
-    ++mainHistory->m_currentStateObjectTimeSpanObjectsAdded;
+    mainHistory.m_totalStateObjectUsage = newTotalUsage;
+    ++mainHistory.m_currentStateObjectTimeSpanObjectsAdded;
 
     if (!urlString.isEmpty())
-        m_frame->document()->updateURLForPushOrReplaceState(fullURL);
+        frame->document()->updateURLForPushOrReplaceState(fullURL);
 
     if (stateObjectType == StateObjectType::Push) {
-        m_frame->loader().history().pushState(WTFMove(data), title, fullURL.string());
-        m_frame->loader().client().dispatchDidPushStateWithinPage();
+        frame->loader().history().pushState(WTFMove(data), title, fullURL.string());
+        frame->loader().client().dispatchDidPushStateWithinPage();
     } else if (stateObjectType == StateObjectType::Replace) {
-        m_frame->loader().history().replaceState(WTFMove(data), title, fullURL.string());
-        m_frame->loader().client().dispatchDidReplaceStateWithinPage();
+        frame->loader().history().replaceState(WTFMove(data), title, fullURL.string());
+        frame->loader().client().dispatchDidReplaceStateWithinPage();
     }
 
     return { };
