@@ -718,6 +718,26 @@ bool Quirks::shouldOmitHTMLDocumentSupportedPropertyNames()
 #endif
 }
 
+// rdar://110097836
+bool Quirks::shouldSilenceResizeObservers() const
+{
+#if PLATFORM(IOS) || PLATFORM(VISION)
+    if (!needsQuirks())
+        return false;
+
+    // ResizeObservers are silenced on YouTube during the 'homing out' snapshout sequence to
+    // resolve rdar://109837319. This is due to a bug on the site that is causing unexpected
+    // content layout and can be removed when it is addressed.
+    auto* page = m_document->page();
+    if (!page || !page->isTakingSnapshotsForApplicationSuspension())
+        return false;
+
+    return RegistrableDomain(m_document->topDocument().url()) == "youtube.com"_s;
+#else
+    return false;
+#endif
+}
+
 bool Quirks::shouldSilenceWindowResizeEvents() const
 {
 #if PLATFORM(IOS) || PLATFORM(VISION)
@@ -733,7 +753,8 @@ bool Quirks::shouldSilenceWindowResizeEvents() const
 
     auto host = m_document->topDocument().url().host();
     return equalLettersIgnoringASCIICase(host, "nytimes.com"_s) || host.endsWithIgnoringASCIICase(".nytimes.com"_s)
-        || equalLettersIgnoringASCIICase(host, "twitter.com"_s) || host.endsWithIgnoringASCIICase(".twitter.com"_s);
+        || equalLettersIgnoringASCIICase(host, "twitter.com"_s) || host.endsWithIgnoringASCIICase(".twitter.com"_s)
+        || equalLettersIgnoringASCIICase(host, "zillow.com"_s) || host.endsWithIgnoringASCIICase(".zillow.com"_s);
 #else
     return false;
 #endif
@@ -1577,4 +1598,10 @@ bool Quirks::shouldDisablePopoverAttributeQuirk() const
     return equalLettersIgnoringASCIICase(host, "apple-console.lrn.com"_s);
 }
 
+// ungap/@custom-elements polyfill (rdar://problem/111008826).
+bool Quirks::needsConfigurableIndexedPropertiesQuirk() const
+{
+    return needsQuirks() && m_needsConfigurableIndexedPropertiesQuirk;
+}
+
 }
