@@ -35,18 +35,20 @@
 #include "AsyncFileStream.h"
 #include "BlobData.h"
 #include "FileStream.h"
-#include "FileSystem.h"
 #include "HTTPHeaderNames.h"
 #include "HTTPParsers.h"
 #include "ParsedContentRange.h"
-#include "URL.h"
 #include "ResourceError.h"
 #include "ResourceHandleClient.h"
 #include "ResourceRequest.h"
 #include "ResourceResponse.h"
+#include "SecurityOrigin.h"
 #include "SharedBuffer.h"
+#include <wtf/CompletionHandler.h>
+#include <wtf/FileSystem.h>
 #include <wtf/MainThread.h>
 #include <wtf/Ref.h>
+#include <wtf/URL.h>
 
 namespace WebCore {
 
@@ -54,16 +56,10 @@ static const unsigned bufferSize = 512 * 1024;
 
 static const int httpOK = 200;
 static const int httpPartialContent = 206;
-static const int httpNotAllowed = 403;
-static const int httpRequestedRangeNotSatisfiable = 416;
-static const int httpInternalError = 500;
-static const char* httpOKText = "OK";
-static const char* httpPartialContentText = "Partial Content";
-static const char* httpNotAllowedText = "Not Allowed";
-static const char* httpRequestedRangeNotSatisfiableText = "Requested Range Not Satisfiable";
-static const char* httpInternalErrorText = "Internal Server Error";
+static constexpr auto httpOKText = "OK"_s;
+static constexpr auto httpPartialContentText = "Partial Content"_s;
 
-static const char* const webKitBlobResourceDomain = "WebKitBlobResource";
+static constexpr auto webKitBlobResourceDomain = "WebKitBlobResource"_s;
 
 ///////////////////////////////////////////////////////////////////////////////
 // BlobResourceSynchronousLoader
@@ -72,29 +68,48 @@ namespace {
 
 class BlobResourceSynchronousLoader : public ResourceHandleClient {
 public:
-    BlobResourceSynchronousLoader(ResourceError&, ResourceResponse&, Vector<char>&);
+    BlobResourceSynchronousLoader(ResourceError&, ResourceResponse&, Vector<uint8_t>&);
 
-    void didReceiveResponse(ResourceHandle*, ResourceResponse&&) override;
-    void didFail(ResourceHandle*, const ResourceError&) override;
+    void didReceiveResponseAsync(ResourceHandle*, ResourceResponse&&, CompletionHandler<void()>&&) final;
+    void didFail(ResourceHandle*, const ResourceError&) final;
+    void willSendRequestAsync(ResourceHandle*, ResourceRequest&&, ResourceResponse&&, CompletionHandler<void(ResourceRequest&&)>&&) final;
+#if USE(PROTECTION_SPACE_AUTH_CALLBACK)
+    void canAuthenticateAgainstProtectionSpaceAsync(ResourceHandle*, const ProtectionSpace&, CompletionHandler<void(bool)>&&) final;
+#endif
 
 private:
     ResourceError& m_error;
     ResourceResponse& m_response;
-    Vector<char>& m_data;
+    Vector<uint8_t>& m_data;
 };
 
-BlobResourceSynchronousLoader::BlobResourceSynchronousLoader(ResourceError& error, ResourceResponse& response, Vector<char>& data)
+BlobResourceSynchronousLoader::BlobResourceSynchronousLoader(ResourceError& error, ResourceResponse& response, Vector<uint8_t>& data)
     : m_error(error)
     , m_response(response)
     , m_data(data)
 {
 }
 
-void BlobResourceSynchronousLoader::didReceiveResponse(ResourceHandle* handle, ResourceResponse&& response)
+void BlobResourceSynchronousLoader::willSendRequestAsync(ResourceHandle*, ResourceRequest&& request, ResourceResponse&&, CompletionHandler<void(ResourceRequest&&)>&& completionHandler)
+{
+    ASSERT_NOT_REACHED();
+    completionHandler(WTFMove(request));
+}
+
+#if USE(PROTECTION_SPACE_AUTH_CALLBACK)
+void BlobResourceSynchronousLoader::canAuthenticateAgainstProtectionSpaceAsync(ResourceHandle*, const ProtectionSpace&, CompletionHandler<void(bool)>&& completionHandler)
+{
+    ASSERT_NOT_REACHED();
+    completionHandler(false);
+}
+#endif
+
+void BlobResourceSynchronousLoader::didReceiveResponseAsync(ResourceHandle* handle, ResourceResponse&& response, CompletionHandler<void()>&& completionHandler)
 {
     // We cannot handle the size that is more than maximum integer.
     if (response.expectedContentLength() > INT_MAX) {
-        m_error = ResourceError(webKitBlobResourceDomain, static_cast<int>(BlobResourceHandle::Error::NotReadableError), response.url(), "File is too large");
+        m_error = ResourceError(webKitBlobResourceDomain, static_cast<int>(BlobResourceHandle::Error::NotReadableError), response.url(), "File is too large"_s);
+        completionHandler();
         return;
     }
 
@@ -103,6 +118,7 @@ void BlobResourceSynchronousLoader::didReceiveResponse(ResourceHandle* handle, R
     // Read all the data.
     m_data.resize(static_cast<size_t>(response.expectedContentLength()));
     static_cast<BlobResourceHandle*>(handle)->readSync(m_data.data(), static_cast<int>(m_data.size()));
+    completionHandler();
 }
 
 void BlobResourceSynchronousLoader::didFail(ResourceHandle*, const ResourceError& error)
@@ -120,10 +136,10 @@ Ref<BlobResourceHandle> BlobResourceHandle::createAsync(BlobData* blobData, cons
     return adoptRef(*new BlobResourceHandle(blobData, request, client, true));
 }
 
-void BlobResourceHandle::loadResourceSynchronously(BlobData* blobData, const ResourceRequest& request, ResourceError& error, ResourceResponse& response, Vector<char>& data)
+void BlobResourceHandle::loadResourceSynchronously(BlobData* blobData, const ResourceRequest& request, ResourceError& error, ResourceResponse& response, Vector<uint8_t>& data)
 {
-    if (!equalLettersIgnoringASCIICase(request.httpMethod(), "get")) {
-        error = ResourceError(webKitBlobResourceDomain, static_cast<int>(Error::MethodNotAllowed), response.url(), "Request method must be GET");
+    if (!equalLettersIgnoringASCIICase(request.httpMethod(), "get"_s)) {
+        error = ResourceError(webKitBlobResourceDomain, static_cast<int>(Error::MethodNotAllowed), response.url(), "Request method must be GET"_s);
         return;
     }
 
@@ -133,38 +149,28 @@ void BlobResourceHandle::loadResourceSynchronously(BlobData* blobData, const Res
 }
 
 BlobResourceHandle::BlobResourceHandle(BlobData* blobData, const ResourceRequest& request, ResourceHandleClient* client, bool async)
-    : ResourceHandle(nullptr, request, client, false, false)
-    , m_blobData(blobData)
-    , m_async(async)
+    : ResourceHandle { nullptr, request, client, false /* defersLoading */, false /* shouldContentSniff */, ContentEncodingSniffingPolicy::Default, nullptr /* sourceOrigin */, false /* isMainFrameNavigation */ }
+    , m_blobData { blobData }
+    , m_async { async }
 {
     if (m_async)
-        m_asyncStream = std::make_unique<AsyncFileStream>(*this);
+        m_asyncStream = makeUnique<AsyncFileStream>(*this);
     else
-        m_stream = std::make_unique<FileStream>();
+        m_stream = makeUnique<FileStream>();
 }
 
-BlobResourceHandle::~BlobResourceHandle()
-{
-}
+BlobResourceHandle::~BlobResourceHandle() = default;
 
 void BlobResourceHandle::cancel()
 {
     m_asyncStream = nullptr;
+    m_fileOpened = false;
 
     m_aborted = true;
 
     ResourceHandle::cancel();
 }
 
-void BlobResourceHandle::continueDidReceiveResponse()
-{
-    ASSERT(m_async);
-    ASSERT(usesAsyncCallbacks());
-
-    m_buffer.resize(bufferSize);
-    readAsync();
-}
-
 void BlobResourceHandle::start()
 {
     if (!m_async) {
@@ -173,7 +179,7 @@ void BlobResourceHandle::start()
     }
 
     // Finish this async call quickly and return.
-    callOnMainThread([protectedThis = makeRef(*this)]() mutable {
+    callOnMainThread([protectedThis = Ref { *this }]() mutable {
         protectedThis->doStart();
     });
 }
@@ -186,7 +192,7 @@ void BlobResourceHandle::doStart()
     if (erroredOrAborted())
         return;
 
-    if (!equalLettersIgnoringASCIICase(firstRequest().httpMethod(), "get")) {
+    if (!equalLettersIgnoringASCIICase(firstRequest().httpMethod(), "get"_s)) {
         notifyFail(Error::MethodNotAllowed);
         return;
     }
@@ -199,9 +205,9 @@ void BlobResourceHandle::doStart()
 
     // Parse the "Range" header we care about.
     String range = firstRequest().httpHeaderField(HTTPHeaderName::Range);
-    if (!range.isEmpty() && !parseRange(range, m_rangeOffset, m_rangeEnd, m_rangeSuffixLength)) {
-        m_errorCode = Error::RangeError;
-        notifyResponse();
+    m_isRangeRequest = !range.isNull();
+    if (m_isRangeRequest && !parseRange(range, RangeAllowWhitespace::Yes, m_rangeStart, m_rangeEnd)) {
+        notifyFail(Error::RangeError);
         return;
     }
 
@@ -227,10 +233,6 @@ void BlobResourceHandle::getSizeForNext()
         if (m_async) {
             Ref<BlobResourceHandle> protectedThis(*this);
             notifyResponse();
-            if (!usesAsyncCallbacks()) {
-                m_buffer.resize(bufferSize);
-                readAsync();
-            }
         }
         return;
     }
@@ -286,18 +288,25 @@ void BlobResourceHandle::seek()
 {
     ASSERT(isMainThread());
 
-    // Convert from the suffix length to the range.
-    if (m_rangeSuffixLength != kPositionNotSpecified) {
-        m_rangeOffset = m_totalRemainingSize - m_rangeSuffixLength;
-        m_rangeEnd = m_rangeOffset + m_rangeSuffixLength - 1;
-    }
-
     // Bail out if the range is not provided.
-    if (m_rangeOffset == kPositionNotSpecified)
+    if (!m_isRangeRequest)
         return;
 
+    // Adjust m_rangeStart / m_rangeEnd
+    if (m_rangeStart == kPositionNotSpecified) {
+        m_rangeStart = m_totalSize - m_rangeEnd;
+        m_rangeEnd = m_rangeStart + m_rangeEnd - 1;
+    } else {
+        if (m_rangeStart >= m_totalSize) {
+            notifyFail(Error::RangeError);
+            return;
+        }
+        if (m_rangeEnd == kPositionNotSpecified || m_rangeEnd >= m_totalSize)
+            m_rangeEnd = m_totalSize - 1;
+    }
+
     // Skip the initial items that are not in the range.
-    long long offset = m_rangeOffset;
+    long long offset = m_rangeStart;
     for (m_readItemCount = 0; m_readItemCount < m_blobData->items().size() && offset >= m_itemLengthList[m_readItemCount]; ++m_readItemCount)
         offset -= m_itemLengthList[m_readItemCount];
 
@@ -305,15 +314,12 @@ void BlobResourceHandle::seek()
     m_currentItemReadSize = offset;
 
     // Adjust the total remaining size in order not to go beyond the range.
-    if (m_rangeEnd != kPositionNotSpecified) {
-        long long rangeSize = m_rangeEnd - m_rangeOffset + 1;
-        if (m_totalRemainingSize > rangeSize)
-            m_totalRemainingSize = rangeSize;
-    } else
-        m_totalRemainingSize -= m_rangeOffset;
+    long long rangeSize = m_rangeEnd - m_rangeStart + 1;
+    if (m_totalRemainingSize > rangeSize)
+        m_totalRemainingSize = rangeSize;
 }
 
-int BlobResourceHandle::readSync(char* buf, int length)
+int BlobResourceHandle::readSync(uint8_t* buf, int length)
 {
     ASSERT(isMainThread());
 
@@ -361,7 +367,7 @@ int BlobResourceHandle::readSync(char* buf, int length)
     return result;
 }
 
-int BlobResourceHandle::readDataSync(const BlobDataItem& item, char* buf, int length)
+int BlobResourceHandle::readDataSync(const BlobDataItem& item, void* buf, int length)
 {
     ASSERT(isMainThread());
 
@@ -371,7 +377,7 @@ int BlobResourceHandle::readDataSync(const BlobDataItem& item, char* buf, int le
     int bytesToRead = (length > remaining) ? static_cast<int>(remaining) : length;
     if (bytesToRead > m_totalRemainingSize)
         bytesToRead = static_cast<int>(m_totalRemainingSize);
-    memcpy(buf, item.data().data() + item.offset() + m_currentItemReadSize, bytesToRead);
+    memcpy(buf, item.data()->data() + item.offset() + m_currentItemReadSize, bytesToRead);
     m_totalRemainingSize -= bytesToRead;
 
     m_currentItemReadSize += bytesToRead;
@@ -383,7 +389,7 @@ int BlobResourceHandle::readDataSync(const BlobDataItem& item, char* buf, int le
     return bytesToRead;
 }
 
-int BlobResourceHandle::readFileSync(const BlobDataItem& item, char* buf, int length)
+int BlobResourceHandle::readFileSync(const BlobDataItem& item, void* buf, int length)
 {
     ASSERT(isMainThread());
 
@@ -421,7 +427,6 @@ int BlobResourceHandle::readFileSync(const BlobDataItem& item, char* buf, int le
 void BlobResourceHandle::readAsync()
 {
     ASSERT(isMainThread());
-    ASSERT(m_async);
 
     // Do not continue if the request is aborted or an error occurs.
     if (erroredOrAborted())
@@ -445,22 +450,24 @@ void BlobResourceHandle::readAsync()
 void BlobResourceHandle::readDataAsync(const BlobDataItem& item)
 {
     ASSERT(isMainThread());
-    ASSERT(m_async);
-    ASSERT(item.data().data());
+    ASSERT(item.data());
 
     Ref<BlobResourceHandle> protectedThis(*this);
 
     long long bytesToRead = item.length() - m_currentItemReadSize;
+    ASSERT(bytesToRead >= 0);
     if (bytesToRead > m_totalRemainingSize)
         bytesToRead = m_totalRemainingSize;
-    consumeData(reinterpret_cast<const char*>(item.data().data()->data()) + item.offset() + m_currentItemReadSize, static_cast<int>(bytesToRead));
+
+    auto* data = item.data()->data() + item.offset() + m_currentItemReadSize;
     m_currentItemReadSize = 0;
+
+    consumeData(data, static_cast<int>(bytesToRead));
 }
 
 void BlobResourceHandle::readFileAsync(const BlobDataItem& item)
 {
     ASSERT(isMainThread());
-    ASSERT(m_async);
 
     if (m_fileOpened) {
         m_asyncStream->read(m_buffer.data(), m_buffer.size());
@@ -498,7 +505,7 @@ void BlobResourceHandle::didRead(int bytesRead)
     consumeData(m_buffer.data(), bytesRead);
 }
 
-void BlobResourceHandle::consumeData(const char* data, int bytesRead)
+void BlobResourceHandle::consumeData(const uint8_t* data, int bytesRead)
 {
     ASSERT(m_async);
     Ref<BlobResourceHandle> protectedThis(*this);
@@ -549,64 +556,40 @@ void BlobResourceHandle::notifyResponse()
         return;
 
     if (m_errorCode != Error::NoError) {
-        Ref<BlobResourceHandle> protectedThis(*this);
-        notifyResponseOnError();
-        notifyFinish();
-    } else
-        notifyResponseOnSuccess();
+        notifyFail(m_errorCode);
+        return;
+    }
+
+    notifyResponseOnSuccess();
 }
 
 void BlobResourceHandle::notifyResponseOnSuccess()
 {
     ASSERT(isMainThread());
 
-    bool isRangeRequest = m_rangeOffset != kPositionNotSpecified;
-    ResourceResponse response(firstRequest().url(), m_blobData->contentType(), m_totalRemainingSize, String());
-    response.setHTTPStatusCode(isRangeRequest ? httpPartialContent : httpOK);
-    response.setHTTPStatusText(isRangeRequest ? httpPartialContentText : httpOKText);
+    ResourceResponse response(firstRequest().url(), extractMIMETypeFromMediaType(m_blobData->contentType()), m_totalRemainingSize, String());
+    response.setHTTPStatusCode(m_isRangeRequest ? httpPartialContent : httpOK);
+    response.setHTTPStatusText(m_isRangeRequest ? httpPartialContentText : httpOKText);
 
     response.setHTTPHeaderField(HTTPHeaderName::ContentType, m_blobData->contentType());
+    response.setTextEncodingName(extractCharsetFromMediaType(m_blobData->contentType()).toAtomString());
     response.setHTTPHeaderField(HTTPHeaderName::ContentLength, String::number(m_totalRemainingSize));
+    addPolicyContainerHeaders(response, m_blobData->policyContainer());
+
+    if (m_isRangeRequest)
+        response.setHTTPHeaderField(HTTPHeaderName::ContentRange, ParsedContentRange(m_rangeStart, m_rangeEnd, m_totalSize).headerValue());
 
-    if (isRangeRequest)
-        response.setHTTPHeaderField(HTTPHeaderName::ContentRange, ParsedContentRange(m_rangeOffset, m_rangeEnd, m_totalSize).headerValue());
     // FIXME: If a resource identified with a blob: URL is a File object, user agents must use that file's name attribute,
     // as if the response had a Content-Disposition header with the filename parameter set to the File's name attribute.
     // Notably, this will affect a name suggested in "File Save As".
 
-    if (usesAsyncCallbacks())
-        client()->didReceiveResponseAsync(this, WTFMove(response));
-    else
-        client()->didReceiveResponse(this, WTFMove(response));
+    client()->didReceiveResponseAsync(this, WTFMove(response), [this, protectedThis = Ref { *this }] {
+        m_buffer.resize(bufferSize);
+        readAsync();
+    });
 }
 
-void BlobResourceHandle::notifyResponseOnError()
-{
-    ASSERT(m_errorCode != Error::NoError);
-
-    ResourceResponse response(firstRequest().url(), "text/plain", 0, String());
-    switch (m_errorCode) {
-    case Error::RangeError:
-        response.setHTTPStatusCode(httpRequestedRangeNotSatisfiable);
-        response.setHTTPStatusText(httpRequestedRangeNotSatisfiableText);
-        break;
-    case Error::SecurityError:
-        response.setHTTPStatusCode(httpNotAllowed);
-        response.setHTTPStatusText(httpNotAllowedText);
-        break;
-    default:
-        response.setHTTPStatusCode(httpInternalError);
-        response.setHTTPStatusText(httpInternalErrorText);
-        break;
-    }
-
-    if (usesAsyncCallbacks())
-        client()->didReceiveResponseAsync(this, WTFMove(response));
-    else
-        client()->didReceiveResponse(this, WTFMove(response));
-}
-
-void BlobResourceHandle::notifyReceiveData(const char* data, int bytesRead)
+void BlobResourceHandle::notifyReceiveData(const uint8_t* data, int bytesRead)
 {
     if (client())
         client()->didReceiveBuffer(this, SharedBuffer::create(data, bytesRead), bytesRead);
@@ -626,7 +609,7 @@ static void doNotifyFinish(BlobResourceHandle& handle)
     if (!handle.client())
         return;
 
-    handle.client()->didFinishLoading(&handle, 0);
+    handle.client()->didFinishLoading(&handle, { });
 }
 
 void BlobResourceHandle::notifyFinish()
@@ -638,7 +621,7 @@ void BlobResourceHandle::notifyFinish()
 
     // Schedule to notify the client from a standalone function because the client might dispose the handle immediately from the callback function
     // while we still have BlobResourceHandle calls in the stack.
-    callOnMainThread([protectedThis = makeRef(*this)]() mutable {
+    callOnMainThread([protectedThis = Ref { *this }]() mutable {
         doNotifyFinish(protectedThis);
     });
 
