@@ -34,11 +34,13 @@
 #include "GeometryUtilities.h"
 #include "GraphicsContext.h"
 #include "InlineIteratorInlineBox.h"
-#include "NinePieceImage.h"
 #include "PaintInfo.h"
+#include "RenderBoxModelObjectInlines.h"
+#include "RenderElementInlines.h"
 #include "RenderImage.h"
 #include "RenderLayer.h"
 #include "RenderLayerBacking.h"
+#include "RenderObjectInlines.h"
 #include "RenderTableCell.h"
 #include "RenderView.h"
 #include "TextBoxPainter.h"
@@ -102,10 +104,12 @@ bool BackgroundPainter::paintsOwnBackground(const RenderBoxModelObject& renderer
 {
     if (!renderer.isBody())
         return true;
+    if (renderer.shouldApplyAnyContainment())
+        return true;
     // The <body> only paints its background if the root element has defined a background independent of the body,
     // or if the <body>'s parent is not the document element's renderer (e.g. inside SVG foreignObject).
     auto documentElementRenderer = renderer.document().documentElement()->renderer();
-    return !documentElementRenderer || documentElementRenderer->hasBackground() || documentElementRenderer != renderer.parent();
+    return !documentElementRenderer || documentElementRenderer->shouldApplyAnyContainment() || documentElementRenderer->hasBackground() || documentElementRenderer != renderer.parent();
 }
 
 void BackgroundPainter::paintFillLayers(const Color& color, const FillLayer& fillLayer, const LayoutRect& rect, BackgroundBleedAvoidance bleedAvoidance, CompositeOperator op, RenderElement* backgroundObject)
@@ -155,7 +159,7 @@ static void applyBoxShadowForBackground(GraphicsContext& context, const RenderSt
         boxShadow = boxShadow->next();
 
     FloatSize shadowOffset(boxShadow->x().value(), boxShadow->y().value());
-    context.setShadow(shadowOffset, boxShadow->radius().value(), style.colorByApplyingColorFilter(boxShadow->color()), boxShadow->isWebkitBoxShadow() ? ShadowRadiusMode::Legacy : ShadowRadiusMode::Default);
+    context.setShadow(shadowOffset, boxShadow->radius().value(), style.colorWithColorFilter(boxShadow->color()), boxShadow->isWebkitBoxShadow() ? ShadowRadiusMode::Legacy : ShadowRadiusMode::Default);
 }
 
 void BackgroundPainter::paintFillLayer(const Color& color, const FillLayer& bgLayer, const LayoutRect& rect,
@@ -318,6 +322,8 @@ void BackgroundPainter::paintFillLayer(const Color& color, const FillLayer& bgLa
         maskRect = snapRectToDevicePixels(rect, deviceScaleFactor);
         maskRect.intersect(snapRectToDevicePixels(m_paintInfo.rect, deviceScaleFactor));
 
+        maskRect.inflate(1);
+
         // Now create the mask.
         maskImage = context.createAlignedImageBuffer(maskRect.size());
         if (!maskImage)
@@ -401,7 +407,7 @@ void BackgroundPainter::paintFillLayer(const Color& color, const FillLayer& bgLa
                 op == CompositeOperator::SourceOver ? bgLayer.compositeForPainting() : op,
                 bgLayer.blendMode(),
                 m_renderer.decodingModeForImageDraw(*image, m_paintInfo),
-                ImageOrientation::FromImage,
+                ImageOrientation::Orientation::FromImage,
                 m_renderer.chooseInterpolationQuality(context, *image, &bgLayer, geometry.tileSize)
             };
 
@@ -410,6 +416,9 @@ void BackgroundPainter::paintFillLayer(const Color& color, const FillLayer& bgLa
                 ASSERT(bgImage->hasCachedImage());
                 bgImage->cachedImage()->addClientWaitingForAsyncDecoding(m_renderer);
             }
+
+            if (m_renderer.element() && !context.paintingDisabled())
+                m_renderer.element()->setHasEverPaintedImages(true);
         }
     }
 
@@ -556,7 +565,7 @@ BackgroundImageGeometry BackgroundPainter::calculateBackgroundImageGeometry(cons
         if (renderer.settings().fixedBackgroundsPaintRelativeToDocument())
             viewportRect = view.unscaledDocumentRect();
         else {
-            FrameView& frameView = view.frameView();
+            LocalFrameView& frameView = view.frameView();
             bool useFixedLayout = frameView.useFixedLayout() && !frameView.fixedLayoutSize().isEmpty();
 
             if (useFixedLayout) {
@@ -811,7 +820,7 @@ void BackgroundPainter::paintBoxShadow(const LayoutRect& paintRect, const Render
         if (shadowOffset.isZero() && !shadowRadius && !shadowSpread)
             continue;
 
-        Color shadowColor = style.colorByApplyingColorFilter(shadow->color());
+        Color shadowColor = style.colorWithColorFilter(shadow->color());
 
         if (shadow->style() == ShadowStyle::Normal) {
             auto fillRect = borderRect;
