@@ -31,7 +31,7 @@
 #include "InlineIteratorLineBox.h"
 #include "InlineTextBoxStyle.h"
 #include "RenderBlock.h"
-#include "RenderStyle.h"
+#include "RenderStyleInlines.h"
 #include "RenderText.h"
 #include "ShadowData.h"
 #include "TextRun.h"
@@ -161,22 +161,22 @@ static DashArray translateIntersectionPointsToSkipInkBoundaries(const DashArray&
 
 static StrokeStyle textDecorationStyleToStrokeStyle(TextDecorationStyle decorationStyle)
 {
-    StrokeStyle strokeStyle = SolidStroke;
+    StrokeStyle strokeStyle = StrokeStyle::SolidStroke;
     switch (decorationStyle) {
     case TextDecorationStyle::Solid:
-        strokeStyle = SolidStroke;
+        strokeStyle = StrokeStyle::SolidStroke;
         break;
     case TextDecorationStyle::Double:
-        strokeStyle = DoubleStroke;
+        strokeStyle = StrokeStyle::DoubleStroke;
         break;
     case TextDecorationStyle::Dotted:
-        strokeStyle = DottedStroke;
+        strokeStyle = StrokeStyle::DottedStroke;
         break;
     case TextDecorationStyle::Dashed:
-        strokeStyle = DashedStroke;
+        strokeStyle = StrokeStyle::DashedStroke;
         break;
     case TextDecorationStyle::Wavy:
-        strokeStyle = WavyStroke;
+        strokeStyle = StrokeStyle::WavyStroke;
         break;
     }
 
@@ -200,7 +200,7 @@ TextDecorationPainter::TextDecorationPainter(GraphicsContext& context, const Fon
 }
 
 // Paint text-shadow, underline, overline
-void TextDecorationPainter::paintBackgroundDecorations(const TextRun& textRun, const BackgroundDecorationGeometry& decorationGeometry, OptionSet<TextDecorationLine> decorationType, const Styles& decorationStyle)
+void TextDecorationPainter::paintBackgroundDecorations(const RenderStyle& style, const TextRun& textRun, const BackgroundDecorationGeometry& decorationGeometry, OptionSet<TextDecorationLine> decorationType, const Styles& decorationStyle)
 {
     auto paintDecoration = [&] (auto decoration, auto style, auto& color, auto& rect) {
         m_context.setStrokeColor(color);
@@ -270,7 +270,7 @@ void TextDecorationPainter::paintBackgroundDecorations(const TextRun& textRun, c
                 boxOrigin.move(0, -extraOffset);
                 extraOffset = 0;
             }
-            auto shadowColor = shadow->color();
+            auto shadowColor = style.colorResolvingCurrentColor(shadow->color());
             if (m_shadowColorFilter)
                 m_shadowColorFilter->transformColor(shadowColor);
 
@@ -281,6 +281,7 @@ void TextDecorationPainter::paintBackgroundDecorations(const TextRun& textRun, c
         };
         applyShadowIfNeeded();
 
+        // FIXME: Add support to handle left/right case
         if (decorationType.contains(TextDecorationLine::Underline))
             paintDecoration(TextDecorationLine::Underline, decorationStyle.underline.decorationStyle, decorationStyle.underline.color, underlineRect);
         if (decorationType.contains(TextDecorationLine::Overline))
