@@ -25,6 +25,7 @@
 #include "FloatRect.h"
 #include "RenderReplaced.h"
 #include "SVGRenderSupport.h"
+#include <wtf/WeakHashSet.h>
 
 namespace WebCore {
 
@@ -86,9 +87,6 @@ private:
 
     const AffineTransform& localToParentTransform() const override;
 
-    bool fillContains(const FloatPoint&) const;
-    bool strokeContains(const FloatPoint&) const;
-
     FloatRect objectBoundingBox() const override { return m_objectBoundingBox; }
     FloatRect strokeBoundingBox() const override { return m_strokeBoundingBox; }
     FloatRect repaintRectInLocalCoordinates() const override { return m_repaintBoundingBox; }
@@ -116,7 +114,7 @@ private:
     FloatRect m_repaintBoundingBox;
     mutable AffineTransform m_localToParentTransform;
     AffineTransform m_localToBorderBoxTransform;
-    HashSet<RenderSVGResourceContainer*> m_resourcesNeedingToInvalidateClients;
+    WeakHashSet<RenderSVGResourceContainer> m_resourcesNeedingToInvalidateClients;
     bool m_isLayoutSizeChanged : 1;
     bool m_needsBoundariesOrTransformUpdate : 1;
     bool m_hasBoxDecorations : 1;
