@@ -1,6 +1,7 @@
 /*
  * Copyright 2005 Maksim Orlovich <maksim@kde.org>
  * Copyright (C) 2006, 2013 Apple Inc. All rights reserved.
+ * Copyright (C) 2019 Google Inc. All rights reserved.
  * Copyright (C) 2007 Alexey Proskuryakov <ap@webkit.org>
  *
  * Redistribution and use in source and binary forms, with or without
@@ -28,20 +29,17 @@
 #include "config.h"
 #include "XPathParser.h"
 
-#include "ExceptionCode.h"
 #include "XPathEvaluator.h"
-#include "XPathException.h"
 #include "XPathNSResolver.h"
 #include "XPathPath.h"
 #include "XPathStep.h"
+#include "XPathUtil.h"
 #include <wtf/NeverDestroyed.h>
+#include <wtf/RobinHoodHashMap.h>
 #include <wtf/StdLibExtras.h>
 #include <wtf/text/StringHash.h>
 
-using namespace WebCore;
-using namespace XPath;
-
-extern int xpathyyparse(Parser&);
+extern int xpathyyparse(WebCore::XPath::Parser&);
 
 #include "XPathGrammar.h"
 
@@ -79,37 +77,36 @@ static XMLCat charCat(UChar character)
     return NotPartOfName;
 }
 
-static void populateAxisNamesMap(HashMap<String, Step::Axis>& axisNames)
+static MemoryCompactLookupOnlyRobinHoodHashMap<String, Step::Axis> createAxisNamesMap()
 {
     struct AxisName {
-        const char* name;
+        ASCIILiteral name;
         Step::Axis axis;
     };
     const AxisName axisNameList[] = {
-        { "ancestor", Step::AncestorAxis },
-        { "ancestor-or-self", Step::AncestorOrSelfAxis },
-        { "attribute", Step::AttributeAxis },
-        { "child", Step::ChildAxis },
-        { "descendant", Step::DescendantAxis },
-        { "descendant-or-self", Step::DescendantOrSelfAxis },
-        { "following", Step::FollowingAxis },
-        { "following-sibling", Step::FollowingSiblingAxis },
-        { "namespace", Step::NamespaceAxis },
-        { "parent", Step::ParentAxis },
-        { "preceding", Step::PrecedingAxis },
-        { "preceding-sibling", Step::PrecedingSiblingAxis },
-        { "self", Step::SelfAxis }
+        { "ancestor"_s, Step::AncestorAxis },
+        { "ancestor-or-self"_s, Step::AncestorOrSelfAxis },
+        { "attribute"_s, Step::AttributeAxis },
+        { "child"_s, Step::ChildAxis },
+        { "descendant"_s, Step::DescendantAxis },
+        { "descendant-or-self"_s, Step::DescendantOrSelfAxis },
+        { "following"_s, Step::FollowingAxis },
+        { "following-sibling"_s, Step::FollowingSiblingAxis },
+        { "namespace"_s, Step::NamespaceAxis },
+        { "parent"_s, Step::ParentAxis },
+        { "preceding"_s, Step::PrecedingAxis },
+        { "preceding-sibling"_s, Step::PrecedingSiblingAxis },
+        { "self"_s, Step::SelfAxis }
     };
+    MemoryCompactLookupOnlyRobinHoodHashMap<String, Step::Axis> map;
     for (auto& axisName : axisNameList)
-        axisNames.add(axisName.name, axisName.axis);
+        map.add(axisName.name, axisName.axis);
+    return map;
 }
 
 static bool parseAxisName(const String& name, Step::Axis& type)
 {
-    static NeverDestroyed<HashMap<String, Step::Axis>> axisNames;
-    if (axisNames.get().isEmpty())
-        populateAxisNamesMap(axisNames);
-
+    static NeverDestroyed axisNames = createAxisNamesMap();
     auto it = axisNames.get().find(name);
     if (it == axisNames.get().end())
         return false;
@@ -135,9 +132,10 @@ bool Parser::isBinaryOperatorContext() const
     }
 }
 
+// See https://www.w3.org/TR/1999/REC-xpath-19991116/#NT-ExprWhitespace .
 void Parser::skipWS()
 {
-    while (m_nextPos < m_data.length() && isSpaceOrNewline(m_data[m_nextPos]))
+    while (m_nextPos < m_data.length() && isXMLSpace(m_data[m_nextPos]))
         ++m_nextPos;
 }
 
@@ -313,7 +311,7 @@ inline Parser::Token Parser::nextTokenInternal()
         if (isBinaryOperatorContext())
             return makeTokenAndAdvance(MULOP, NumericOp::OP_Mul);
         ++m_nextPos;
-        return Token(NAMETEST, "*");
+        return Token(NAMETEST, "*"_s);
     case '$': { // $ QName
         m_nextPos++;
         String name;
@@ -330,13 +328,13 @@ inline Parser::Token Parser::nextTokenInternal()
     skipWS();
     // If we're in an operator context, check for any operator names
     if (isBinaryOperatorContext()) {
-        if (name == "and") //### hash?
+        if (name == "and"_s) // ### hash?
             return Token(AND);
-        if (name == "or")
+        if (name == "or"_s)
             return Token(OR);
-        if (name == "mod")
+        if (name == "mod"_s)
             return Token(MULOP, NumericOp::OP_Mod);
-        if (name == "div")
+        if (name == "div"_s)
             return Token(MULOP, NumericOp::OP_Div);
     }
 
@@ -377,13 +375,13 @@ inline Parser::Token Parser::nextTokenInternal()
 
         // Either node type oor function name.
 
-        if (name == "processing-instruction")
+        if (name == "processing-instruction"_s)
             return Token(PI);
-        if (name == "node")
+        if (name == "node"_s)
             return Token(NODE);
-        if (name == "text")
+        if (name == "text"_s)
             return Token(TEXT_);
-        if (name == "comment")
+        if (name == "comment"_s)
             return Token(COMMENT);
 
         return Token(FUNCTIONNAME, name);
@@ -434,7 +432,7 @@ int Parser::lex(YYSTYPE& yylval)
     return token.type;
 }
 
-bool Parser::expandQualifiedName(const String& qualifiedName, String& localName, String& namespaceURI)
+bool Parser::expandQualifiedName(const String& qualifiedName, AtomString& localName, AtomString& namespaceURI)
 {
     size_t colon = qualifiedName.find(':');
     if (colon != notFound) {
@@ -442,14 +440,14 @@ bool Parser::expandQualifiedName(const String& qualifiedName, String& localName,
             m_sawNamespaceError = true;
             return false;
         }
-        namespaceURI = m_resolver->lookupNamespaceURI(qualifiedName.left(colon));
+        namespaceURI = m_resolver->lookupNamespaceURI(StringView(qualifiedName).left(colon).toAtomString());
         if (namespaceURI.isNull()) {
             m_sawNamespaceError = true;
             return false;
         }
-        localName = qualifiedName.substring(colon + 1);
+        localName = StringView(qualifiedName).substring(colon + 1).toAtomString();
     } else
-        localName = qualifiedName;
+        localName = AtomString { qualifiedName };
     return true;
 }
 
@@ -460,10 +458,10 @@ ExceptionOr<std::unique_ptr<Expression>> Parser::parseStatement(const String& st
     int parseError = xpathyyparse(parser);
 
     if (parser.m_sawNamespaceError)
-        return Exception { NAMESPACE_ERR };
+        return Exception { NamespaceError };
 
     if (parseError)
-        return Exception { XPathException::INVALID_EXPRESSION_ERR };
+        return Exception { SyntaxError };
 
     return WTFMove(parser.m_result);
 }
