@@ -34,21 +34,21 @@
 #include "CommonAtomStrings.h"
 #include "CustomElementReactionQueue.h"
 #include "CustomElementRegistry.h"
-#include "DOMWindow.h"
 #include "Document.h"
 #include "DocumentFragment.h"
 #include "DocumentType.h"
 #include "EventLoop.h"
-#include "Frame.h"
 #include "FrameDestructionObserverInlines.h"
 #include "FrameLoader.h"
 #include "HTMLEntityParser.h"
 #include "HTMLHtmlElement.h"
-#include "HTMLParserIdioms.h"
 #include "HTMLTemplateElement.h"
 #include "HTTPParsers.h"
 #include "InlineClassicScript.h"
+#include "LocalDOMWindow.h"
+#include "LocalFrame.h"
 #include "MIMETypeRegistry.h"
+#include "OriginAccessPatterns.h"
 #include "Page.h"
 #include "PageConsoleClient.h"
 #include "PendingScript.h"
@@ -406,7 +406,7 @@ static inline void setAttributes(Element* element, Vector<Attribute>& attributeV
 {
     if (!scriptingContentIsAllowed(parserContentPolicy))
         element->stripScriptingAttributes(attributeVector);
-    element->parserSetAttributes(attributeVector);
+    element->parserSetAttributes(attributeVector.span());
 }
 
 static void switchToUTF16(xmlParserCtxtPtr ctxt)
@@ -448,7 +448,7 @@ static bool shouldAllowExternalLoad(const URL& url)
     // retrieved content.  If we had more context, we could potentially allow
     // the parser to load a DTD.  As things stand, we take the conservative
     // route and allow same-origin requests only.
-    if (!XMLDocumentParserScope::currentCachedResourceLoader->document()->securityOrigin().canRequest(url)) {
+    if (!XMLDocumentParserScope::currentCachedResourceLoader->document()->securityOrigin().canRequest(url, OriginAccessPatternsForWebProcess::singleton())) {
         XMLDocumentParserScope::currentCachedResourceLoader->printAccessDeniedMessage(url);
         return false;
     }
@@ -465,7 +465,7 @@ static void* openFunc(const char* uri)
     Document* document = cachedResourceLoader.document();
     // Same logic as HTMLBaseElement::href(). Keep them in sync.
     auto* encoding = (document && document->decoder()) ? document->decoder()->encodingForURLParsing() : nullptr;
-    URL url(document ? document->fallbackBaseURL() : URL(), stripLeadingAndTrailingHTMLSpaces(String::fromLatin1(uri)), encoding);
+    URL url(document ? document->fallbackBaseURL() : URL(), String::fromLatin1(uri), encoding);
 
     if (!shouldAllowExternalLoad(url))
         return &globalDescriptor;
@@ -598,7 +598,7 @@ bool XMLDocumentParser::supportsXMLVersion(const String& version)
     return version == "1.0"_s;
 }
 
-XMLDocumentParser::XMLDocumentParser(Document& document, FrameView* frameView, OptionSet<ParserContentPolicy> policy)
+XMLDocumentParser::XMLDocumentParser(Document& document, LocalFrameView* frameView, OptionSet<ParserContentPolicy> policy)
     : ScriptableDocumentParser(document, policy)
     , m_view(frameView)
     , m_pendingCallbacks(makeUnique<PendingCallbacks>())
@@ -1187,10 +1187,12 @@ static xmlEntityPtr sharedXHTMLEntity()
     return &entity;
 }
 
-static size_t convertUTF16EntityToUTF8(const UChar* utf16Entity, size_t numberOfCodeUnits, char* target, size_t targetSize)
+static size_t convertUTF16EntityToUTF8(std::span<const UChar> utf16Entity, char* target, size_t targetSize)
 {
     const char* originalTarget = target;
-    auto conversionResult = WTF::Unicode::convertUTF16ToUTF8(&utf16Entity, utf16Entity + numberOfCodeUnits, &target, target + targetSize);
+    auto start = utf16Entity.data();
+    auto end = start + utf16Entity.size();
+    auto conversionResult = WTF::Unicode::convertUTF16ToUTF8(&start, end, &target, target + targetSize);
     if (conversionResult != WTF::Unicode::ConversionResult::Success)
         return 0;
 
@@ -1202,23 +1204,24 @@ static size_t convertUTF16EntityToUTF8(const UChar* utf16Entity, size_t numberOf
 
 static xmlEntityPtr getXHTMLEntity(const xmlChar* name)
 {
-    UChar utf16DecodedEntity[4];
-    size_t numberOfCodeUnits = decodeNamedEntityToUCharArray(reinterpret_cast<const char*>(name), utf16DecodedEntity);
-    if (!numberOfCodeUnits)
-        return 0;
+    auto decodedEntity = decodeNamedHTMLEntityForXMLParser(reinterpret_cast<const char*>(name));
+    if (decodedEntity.failed())
+        return nullptr;
+
+    auto utf16DecodedEntity = decodedEntity.span();
 
     constexpr size_t kSharedXhtmlEntityResultLength = std::size(sharedXHTMLEntityResult);
     size_t entityLengthInUTF8;
     // Unlike HTML parser, XML parser parses the content of named
     // entities. So we need to escape '&' and '<'.
-    if (numberOfCodeUnits == 1 && utf16DecodedEntity[0] == '&') {
+    if (utf16DecodedEntity.size() == 1 && utf16DecodedEntity[0] == '&') {
         sharedXHTMLEntityResult[0] = '&';
         sharedXHTMLEntityResult[1] = '#';
         sharedXHTMLEntityResult[2] = '3';
         sharedXHTMLEntityResult[3] = '8';
         sharedXHTMLEntityResult[4] = ';';
         entityLengthInUTF8 = 5;
-    } else if (numberOfCodeUnits == 1 && utf16DecodedEntity[0] == '<') {
+    } else if (utf16DecodedEntity.size() == 1 && utf16DecodedEntity[0] == '<') {
         sharedXHTMLEntityResult[0] = '&';
         sharedXHTMLEntityResult[1] = '#';
         sharedXHTMLEntityResult[2] = 'x';
@@ -1226,7 +1229,7 @@ static xmlEntityPtr getXHTMLEntity(const xmlChar* name)
         sharedXHTMLEntityResult[4] = 'C';
         sharedXHTMLEntityResult[5] = ';';
         entityLengthInUTF8 = 6;
-    } else if (numberOfCodeUnits == 2 && utf16DecodedEntity[0] == '<' && utf16DecodedEntity[1] == 0x20D2) {
+    } else if (utf16DecodedEntity.size() == 2 && utf16DecodedEntity[0] == '<' && utf16DecodedEntity[1] == 0x20D2) {
         sharedXHTMLEntityResult[0] = '&';
         sharedXHTMLEntityResult[1] = '#';
         sharedXHTMLEntityResult[2] = '6';
@@ -1237,8 +1240,8 @@ static xmlEntityPtr getXHTMLEntity(const xmlChar* name)
         sharedXHTMLEntityResult[7] = 0x92;
         entityLengthInUTF8 = 8;
     } else {
-        ASSERT(numberOfCodeUnits <= 4);
-        entityLengthInUTF8 = convertUTF16EntityToUTF8(utf16DecodedEntity, numberOfCodeUnits,
+        ASSERT(utf16DecodedEntity.size() <= 4);
+        entityLengthInUTF8 = convertUTF16EntityToUTF8(utf16DecodedEntity,
             reinterpret_cast<char*>(sharedXHTMLEntityResult), kSharedXhtmlEntityResultLength);
         if (!entityLengthInUTF8)
             return 0;
