@@ -333,7 +333,9 @@ Token Lexer<T>::lex()
                 { "asm", TokenType::ReservedWord },
                 { "bf16", TokenType::ReservedWord },
                 { "bool", TokenType::KeywordBool },
+                { "break", TokenType::KeywordBreak },
                 { "const", TokenType::KeywordConst },
+                { "continue", TokenType::KeywordContinue },
                 { "do", TokenType::ReservedWord },
                 { "else", TokenType::KeywordElse },
                 { "enum", TokenType::ReservedWord },
