@@ -57,14 +57,8 @@ static const unsigned bufferSize = 512 * 1024;
 
 static const int httpOK = 200;
 static const int httpPartialContent = 206;
-static const int httpNotAllowed = 403;
-static const int httpRequestedRangeNotSatisfiable = 416;
-static const int httpInternalError = 500;
 static constexpr auto httpOKText = "OK"_s;
 static constexpr auto httpPartialContentText = "Partial Content"_s;
-static constexpr auto httpNotAllowedText = "Not Allowed"_s;
-static constexpr auto httpRequestedRangeNotSatisfiableText = "Requested Range Not Satisfiable"_s;
-static constexpr auto httpInternalErrorText = "Internal Server Error"_s;
 
 static constexpr auto webKitBlobResourceDomain = "WebKitBlobResource"_s;
 
@@ -132,8 +126,9 @@ void NetworkDataTaskBlob::resume()
 
         // Parse the "Range" header we care about.
         String range = m_firstRequest.httpHeaderField(HTTPHeaderName::Range);
-        if (!range.isEmpty() && !parseRange(range, m_rangeOffset, m_rangeEnd, m_rangeSuffixLength)) {
-            dispatchDidReceiveResponse(Error::RangeError);
+        m_isRangeRequest = !range.isNull();
+        if (m_isRangeRequest && !parseRange(range, RangeAllowWhitespace::Yes, m_rangeStart, m_rangeEnd)) {
+            didFail(Error::RangeError);
             return;
         }
 
@@ -223,18 +218,25 @@ void NetworkDataTaskBlob::seek()
 {
     ASSERT(RunLoop::isMain());
 
-    // Convert from the suffix length to the range.
-    if (m_rangeSuffixLength != kPositionNotSpecified) {
-        m_rangeOffset = m_totalRemainingSize - m_rangeSuffixLength;
-        m_rangeEnd = m_rangeOffset + m_rangeSuffixLength - 1;
-    }
-
     // Bail out if the range is not provided.
-    if (m_rangeOffset == kPositionNotSpecified)
+    if (!m_isRangeRequest)
         return;
 
+    // Adjust m_rangeStart / m_rangeEnd
+    if (m_rangeStart == kPositionNotSpecified) {
+        m_rangeStart = m_totalSize - m_rangeEnd;
+        m_rangeEnd = m_rangeStart + m_rangeEnd - 1;
+    } else {
+        if (m_rangeStart >= m_totalSize) {
+            didFail(Error::RangeError);
+            return;
+        }
+        if (m_rangeEnd == kPositionNotSpecified || m_rangeEnd >= m_totalSize)
+            m_rangeEnd = m_totalSize - 1;
+    }
+
     // Skip the initial items that are not in the range.
-    long long offset = m_rangeOffset;
+    long long offset = m_rangeStart;
     for (m_readItemCount = 0; m_readItemCount < m_blobData->items().size() && offset >= m_itemLengthList[m_readItemCount]; ++m_readItemCount)
         offset -= m_itemLengthList[m_readItemCount];
 
@@ -242,58 +244,33 @@ void NetworkDataTaskBlob::seek()
     m_currentItemReadSize = offset;
 
     // Adjust the total remaining size in order not to go beyond the range.
-    if (m_rangeEnd != kPositionNotSpecified) {
-        long long rangeSize = m_rangeEnd - m_rangeOffset + 1;
-        if (m_totalRemainingSize > rangeSize)
-            m_totalRemainingSize = rangeSize;
-    } else
-        m_totalRemainingSize -= m_rangeOffset;
+    long long rangeSize = m_rangeEnd - m_rangeStart + 1;
+    if (m_totalRemainingSize > rangeSize)
+        m_totalRemainingSize = rangeSize;
 }
 
-void NetworkDataTaskBlob::dispatchDidReceiveResponse(Error errorCode)
+void NetworkDataTaskBlob::dispatchDidReceiveResponse()
 {
-    LOG(NetworkSession, "%p - NetworkDataTaskBlob::dispatchDidReceiveResponse(%u)", this, static_cast<unsigned>(errorCode));
+    LOG(NetworkSession, "%p - NetworkDataTaskBlob::dispatchDidReceiveResponse()", this);
 
     Ref<NetworkDataTaskBlob> protectedThis(*this);
-    ResourceResponse response(m_firstRequest.url(), errorCode != Error::NoError ? "text/plain"_s : extractMIMETypeFromMediaType(m_blobData->contentType()), errorCode != Error::NoError ? 0 : m_totalRemainingSize, String());
-    switch (errorCode) {
-    case Error::NoError: {
-        bool isRangeRequest = m_rangeOffset != kPositionNotSpecified;
-        response.setHTTPStatusCode(isRangeRequest ? httpPartialContent : httpOK);
-        response.setHTTPStatusText(isRangeRequest ? httpPartialContentText : httpOKText);
+    ResourceResponse response(m_firstRequest.url(), extractMIMETypeFromMediaType(m_blobData->contentType()), m_totalRemainingSize, String());
+    response.setHTTPStatusCode(m_isRangeRequest ? httpPartialContent : httpOK);
+    response.setHTTPStatusText(m_isRangeRequest ? httpPartialContentText : httpOKText);
 
-        response.setHTTPHeaderField(HTTPHeaderName::ContentType, m_blobData->contentType());
-        response.setTextEncodingName(extractCharsetFromMediaType(m_blobData->contentType()).toAtomString());
-        response.setHTTPHeaderField(HTTPHeaderName::ContentLength, String::number(m_totalRemainingSize));
-        addPolicyContainerHeaders(response, m_blobData->policyContainer());
+    response.setHTTPHeaderField(HTTPHeaderName::ContentType, m_blobData->contentType());
+    response.setTextEncodingName(extractCharsetFromMediaType(m_blobData->contentType()).toAtomString());
+    response.setHTTPHeaderField(HTTPHeaderName::ContentLength, String::number(m_totalRemainingSize));
+    addPolicyContainerHeaders(response, m_blobData->policyContainer());
 
-        if (isRangeRequest) {
-            auto rangeEnd = m_rangeEnd;
-            if (rangeEnd == kPositionNotSpecified)
-                rangeEnd = m_totalSize - 1;
+    if (m_isRangeRequest)
+        response.setHTTPHeaderField(HTTPHeaderName::ContentRange, ParsedContentRange(m_rangeStart, m_rangeEnd, m_totalSize).headerValue());
 
-            response.setHTTPHeaderField(HTTPHeaderName::ContentRange, ParsedContentRange(m_rangeOffset, rangeEnd, m_totalSize).headerValue());
-        }
-        // FIXME: If a resource identified with a blob: URL is a File object, user agents must use that file's name attribute,
-        // as if the response had a Content-Disposition header with the filename parameter set to the File's name attribute.
-        // Notably, this will affect a name suggested in "File Save As".
-        break;
-    }
-    case Error::RangeError:
-        response.setHTTPStatusCode(httpRequestedRangeNotSatisfiable);
-        response.setHTTPStatusText(httpRequestedRangeNotSatisfiableText);
-        break;
-    case Error::SecurityError:
-        response.setHTTPStatusCode(httpNotAllowed);
-        response.setHTTPStatusText(httpNotAllowedText);
-        break;
-    default:
-        response.setHTTPStatusCode(httpInternalError);
-        response.setHTTPStatusText(httpInternalErrorText);
-        break;
-    }
+    // FIXME: If a resource identified with a blob: URL is a File object, user agents must use that file's name attribute,
+    // as if the response had a Content-Disposition header with the filename parameter set to the File's name attribute.
+    // Notably, this will affect a name suggested in "File Save As".
 
-    didReceiveResponse(WTFMove(response), NegotiatedLegacyTLS::No, PrivateRelayed::No, [this, protectedThis = WTFMove(protectedThis), errorCode](PolicyAction policyAction) {
+    didReceiveResponse(WTFMove(response), NegotiatedLegacyTLS::No, PrivateRelayed::No, [this, protectedThis = WTFMove(protectedThis)](PolicyAction policyAction) {
         LOG(NetworkSession, "%p - NetworkDataTaskBlob::didReceiveResponse completionHandler (%u)", this, static_cast<unsigned>(policyAction));
 
         if (m_state == State::Canceling || m_state == State::Completed) {
@@ -301,11 +278,6 @@ void NetworkDataTaskBlob::dispatchDidReceiveResponse(Error errorCode)
             return;
         }
 
-        if (errorCode != Error::NoError) {
-            didFinish();
-            return;
-        }
-
         switch (policyAction) {
         case PolicyAction::Use:
             m_buffer.resize(bufferSize);
@@ -464,7 +436,7 @@ void NetworkDataTaskBlob::download()
 
     LOG(NetworkSession, "%p - NetworkDataTaskBlob::download to %s", this, m_pendingDownloadLocation.utf8().data());
 
-    m_downloadFile = FileSystem::openFile(m_pendingDownloadLocation, FileSystem::FileOpenMode::Write);
+    m_downloadFile = FileSystem::openFile(m_pendingDownloadLocation, FileSystem::FileOpenMode::Truncate);
     if (m_downloadFile == FileSystem::invalidPlatformFileHandle) {
         didFailDownload(cancelledError(m_firstRequest));
         return;
