@@ -28,16 +28,19 @@
 #include <WebCore/ResourceError.h>
 #include <WebCore/ResourceRequest.h>
 #include <WebCore/ResourceResponse.h>
-#include <wtf/Forward.h>
 
 namespace WebCore {
+class AuthenticationChallenge;
 class NetworkLoadMetrics;
-class ProtectionSpace;
-class SharedBuffer;
+class FragmentedSharedBuffer;
+enum class PolicyAction : uint8_t;
 }
 
 namespace WebKit {
 
+enum class PrivateRelayed : bool;
+using ResponseCompletionHandler = CompletionHandler<void(WebCore::PolicyAction)>;
+
 class NetworkLoadClient {
 public:
     virtual ~NetworkLoadClient() { }
@@ -46,14 +49,15 @@ public:
 
     virtual bool isAllowedToAskUserForCredentials() const = 0;
 
-    virtual void didSendData(unsigned long long bytesSent, unsigned long long totalBytesToBeSent) = 0;
+    virtual void didSendData(uint64_t bytesSent, uint64_t totalBytesToBeSent) = 0;
     virtual void willSendRedirectedRequest(WebCore::ResourceRequest&&, WebCore::ResourceRequest&& redirectRequest, WebCore::ResourceResponse&& redirectResponse) = 0;
-    enum class ShouldContinueDidReceiveResponse { No, Yes };
-    virtual ShouldContinueDidReceiveResponse didReceiveResponse(WebCore::ResourceResponse&&) = 0;
-    virtual void didReceiveBuffer(Ref<WebCore::SharedBuffer>&&, int reportedEncodedDataLength) = 0;
+    virtual void didReceiveInformationalResponse(WebCore::ResourceResponse&&) { };
+    virtual void didReceiveResponse(WebCore::ResourceResponse&&, PrivateRelayed, ResponseCompletionHandler&&) = 0;
+    virtual void didReceiveBuffer(const WebCore::FragmentedSharedBuffer&, uint64_t reportedEncodedDataLength) = 0;
     virtual void didFinishLoading(const WebCore::NetworkLoadMetrics&) = 0;
     virtual void didFailLoading(const WebCore::ResourceError&) = 0;
     virtual void didBlockAuthenticationChallenge() { };
+    virtual void didReceiveChallenge(const WebCore::AuthenticationChallenge&) { };
     virtual bool shouldCaptureExtraNetworkLoadMetrics() const { return false; }
 };
 
