@@ -180,6 +180,8 @@ public:
 
     void willSendServiceWorkerRedirectedRequest(WebCore::ResourceRequest&&, WebCore::ResourceRequest&& redirectRequest, WebCore::ResourceResponse&&);
 
+    void useRedirectionForCurrentNavigation(WebCore::ResourceResponse&&);
+
 private:
     NetworkResourceLoader(NetworkResourceLoadParameters&&, NetworkConnectionToWebProcess&, CompletionHandler<void(const WebCore::ResourceError&, const WebCore::ResourceResponse, Vector<uint8_t>&&)>&&);
 
@@ -328,6 +330,8 @@ private:
 
     PrivateRelayed m_privateRelayed { PrivateRelayed::No };
     MemoryCompactRobinHoodHashMap<String, String> m_reportingEndpoints;
+
+    std::unique_ptr<WebCore::ResourceResponse> m_redirectionForCurrentNavigation;
 };
 
 } // namespace WebKit
