@@ -29,6 +29,7 @@
 #include "CacheStorageRecord.h"
 #include "Logging.h"
 #include "NetworkCacheCoders.h"
+#include <WebCore/ResourceResponse.h>
 #include <wtf/PageBlock.h>
 #include <wtf/RefCounted.h>
 #include <wtf/Scope.h>
@@ -50,7 +51,7 @@ static bool shouldStoreBodyAsBlob(const Vector<uint8_t>& bodyData)
     return bodyData.size() > WTF::pageSize();
 }
 
-static SHA1::Digest computeSHA1(Span<const uint8_t> span, FileSystem::Salt salt)
+static SHA1::Digest computeSHA1(std::span<const uint8_t> span, FileSystem::Salt salt)
 {
     SHA1 sha1;
     sha1.addBytes(salt.data(), salt.size());
@@ -86,7 +87,7 @@ struct RecordHeader {
     WebCore::FetchOptions options;
     String referrer;
     WebCore::FetchHeaders::Guard responseHeadersGuard { WebCore::FetchHeaders::Guard::None };
-    WebCore::ResourceResponse response;
+    WebCore::ResourceResponse::CrossThreadData responseData;
     uint64_t responseBodySize { 0 };
 };
 
@@ -147,7 +148,7 @@ String CacheStorageDiskStore::blobFilePath(const String& blobFileName) const
     return FileSystem::pathByAppendingComponent(blobsDirectoryPath(), blobFileName);
 }
 
-static std::optional<RecordMetaData> decodeRecordMetaData(Span<const uint8_t> fileData)
+static std::optional<RecordMetaData> decodeRecordMetaData(std::span<const uint8_t> fileData)
 {
     WTF::Persistence::Decoder decoder(fileData);
     RecordMetaData metaData;
@@ -206,7 +207,7 @@ static std::optional<RecordMetaData> decodeRecordMetaData(Span<const uint8_t> fi
     return metaData;
 }
 
-static std::optional<RecordHeader> decodeRecordHeader(Span<const uint8_t> headerData)
+static std::optional<RecordHeader> decodeRecordHeader(std::span<const uint8_t> headerData)
 {
     WTF::Persistence::Decoder decoder(headerData);
     std::optional<double> insertionTime;
@@ -243,8 +244,14 @@ static std::optional<RecordHeader> decodeRecordHeader(Span<const uint8_t> header
     if (!responseHeadersGuard)
         return std::nullopt;
 
-    WebCore::ResourceResponse response;
-    if (!WebCore::ResourceResponse::decode(decoder, response))
+    std::optional<bool> isNull;
+    decoder >> isNull;
+    if (!isNull || *isNull)
+        return std::nullopt;
+
+    std::optional<WebCore::ResourceResponse::CrossThreadData> responseData;
+    decoder >> responseData;
+    if (!responseData)
         return std::nullopt;
 
     std::optional<uint64_t> responseBodySize;
@@ -259,11 +266,11 @@ static std::optional<RecordHeader> decodeRecordHeader(Span<const uint8_t> header
         *insertionTime,
         *size,
         *requestHeadersGuard,
-        *request,
+        WTFMove(*request),
         WTFMove(options),
         WTFMove(*referrer),
         *responseHeadersGuard,
-        WTFMove(response),
+        WTFMove(*responseData),
         WTFMove(*responseBodySize)
     };
 }
@@ -273,7 +280,7 @@ static std::optional<StoredRecordInformation> readRecordInfoFromFileData(const F
     if (buffer.isEmpty())
         return std::nullopt;
 
-    auto fileData = Span { buffer.data(), buffer.size() };
+    auto fileData = std::span(buffer.data(), buffer.size());
     auto metaData = decodeRecordMetaData(fileData);
     if (!metaData)
         return std::nullopt;
@@ -290,8 +297,8 @@ static std::optional<StoredRecordInformation> readRecordInfoFromFileData(const F
         return std::nullopt;
 
     CacheStorageRecordInformation info { metaData->key, header->insertionTime, 0, 0, header->responseBodySize, header->request.url(), false, { } };
-    info.updateVaryHeaders(header->request, header->response.httpHeaderField(WebCore::HTTPHeaderName::Vary));
-    return StoredRecordInformation { info, *metaData, *header };
+    info.updateVaryHeaders(header->request, header->responseData);
+    return StoredRecordInformation { info, WTFMove(*metaData), WTFMove(*header) };
 }
 
 std::optional<CacheStorageRecord> CacheStorageDiskStore::readRecordFromFileData(const Vector<uint8_t>& buffer, const Vector<uint8_t>& blobBuffer)
@@ -307,7 +314,7 @@ std::optional<CacheStorageRecord> CacheStorageDiskStore::readRecordFromFileData(
         if (bodyOffset + bodySize != buffer.size())
             return std::nullopt;
 
-        auto bodyData = Span { buffer.data()  + bodyOffset, bodySize };
+        auto bodyData = std::span(buffer.data() + bodyOffset, bodySize);
         if (storedInfo->metaData.bodyHash != computeSHA1(bodyData, m_salt))
             return std::nullopt;
 
@@ -317,7 +324,7 @@ std::optional<CacheStorageRecord> CacheStorageDiskStore::readRecordFromFileData(
             return std::nullopt;
 
         auto sharedBuffer = WebCore::SharedBuffer::create(blobBuffer.data(), blobBuffer.size());
-        auto bodyData = Span { sharedBuffer->data(), sharedBuffer->size() };
+        auto bodyData = std::span(sharedBuffer->data(), sharedBuffer->size());
         if (storedInfo->metaData.bodyHash != computeSHA1(bodyData, m_salt))
             return std::nullopt;
 
@@ -327,26 +334,14 @@ std::optional<CacheStorageRecord> CacheStorageDiskStore::readRecordFromFileData(
     if (!responseBody)
         return std::nullopt;
 
-    return CacheStorageRecord { storedInfo->info, storedInfo->header.requestHeadersGuard, storedInfo->header.request, storedInfo->header.options, storedInfo->header.referrer, storedInfo->header.responseHeadersGuard, storedInfo->header.response.crossThreadData(), storedInfo->header.responseBodySize, WTFMove(*responseBody) };
+    return CacheStorageRecord { storedInfo->info, storedInfo->header.requestHeadersGuard, storedInfo->header.request, storedInfo->header.options, storedInfo->header.referrer, storedInfo->header.responseHeadersGuard, WTFMove(storedInfo->header.responseData), storedInfo->header.responseBodySize, WTFMove(*responseBody) };
 }
 
-void CacheStorageDiskStore::readAllRecordInfos(ReadAllRecordInfosCallback&& callback)
+void CacheStorageDiskStore::readAllRecordInfosInternal(CompletionHandler<void(FileDatas&&)>&& callback)
 {
-    auto didReadRecordFiles = [this, protectedThis = Ref { *this }, callback = WTFMove(callback)](auto fileDatas) mutable {
-        Vector<CacheStorageRecordInformation> result;
-        result.reserveInitialCapacity(fileDatas.size());
-        for (auto& fileData : fileDatas) {
-            if (auto storedInfo = readRecordInfoFromFileData(m_salt, fileData))
-                result.uncheckedAppend(WTFMove(storedInfo->info));
-            else
-                RELEASE_LOG(CacheStorage, "%p - CacheStorageDiskStore::readAllRecordInfos fails to decode record from file", this);
-        }
-        callback(WTFMove(result));
-    };
-
-    m_ioQueue->dispatch([this, protectedThis = Ref { *this }, recordsDirectory = recordsDirectoryPath().isolatedCopy(), cacheName = m_cacheName.isolatedCopy(), didReadRecordFiles = WTFMove(didReadRecordFiles)]() mutable {
-        Vector<Vector<uint8_t>> fileDatas;
-        Vector<Vector<uint8_t>> blobDatas;
+    m_ioQueue->dispatch([this, protectedThis = Ref { *this }, recordsDirectory = recordsDirectoryPath().isolatedCopy(), cacheName = m_cacheName.isolatedCopy(), callback = WTFMove(callback)]() mutable {
+        FileDatas fileDatas;
+        FileDatas blobDatas;
         auto partitionNames = FileSystem::listDirectory(recordsDirectory);
         for (auto& partitionName : partitionNames) {
             auto partitionDirectoryPath = FileSystem::pathByAppendingComponent(recordsDirectory, partitionName);
@@ -362,8 +357,41 @@ void CacheStorageDiskStore::readAllRecordInfos(ReadAllRecordInfosCallback&& call
             }
         }
 
-        m_callbackQueue->dispatch([protectedThis = WTFMove(protectedThis), fileDatas = crossThreadCopy(WTFMove(fileDatas)), didReadRecordFiles = WTFMove(didReadRecordFiles)]() mutable {
-            didReadRecordFiles(WTFMove(fileDatas));
+        m_callbackQueue->dispatch([protectedThis = WTFMove(protectedThis), fileDatas = crossThreadCopy(WTFMove(fileDatas)), callback = WTFMove(callback)]() mutable {
+            callback(WTFMove(fileDatas));
+        });
+    });
+}
+
+void CacheStorageDiskStore::readAllRecordInfos(ReadAllRecordInfosCallback&& callback)
+{
+    readAllRecordInfosInternal([this, protectedThis = Ref { *this }, callback = WTFMove(callback)](auto fileDatas) mutable {
+        Vector<CacheStorageRecordInformation> result;
+        result.reserveInitialCapacity(fileDatas.size());
+        for (auto& fileData : fileDatas) {
+            if (auto storedInfo = readRecordInfoFromFileData(m_salt, fileData))
+                result.uncheckedAppend(WTFMove(storedInfo->info));
+            else
+                RELEASE_LOG(CacheStorage, "%p - CacheStorageDiskStore::readAllRecordInfos fails to decode record from file", this);
+        }
+        callback(WTFMove(result));
+    });
+}
+
+void CacheStorageDiskStore::readRecordsInternal(Vector<String>&& recordFiles, CompletionHandler<void(FileDatas&&, FileDatas&&)>&& callback)
+{
+    m_ioQueue->dispatch([this, protectedThis = Ref { *this }, recordFiles = crossThreadCopy(WTFMove(recordFiles)), callback = WTFMove(callback)]() mutable {
+        FileDatas fileDatas;
+        FileDatas blobDatas;
+        for (auto& recordFile : recordFiles) {
+            auto fileData = valueOrDefault(FileSystem::readEntireFile(recordFile));
+            auto blobData = fileData.isEmpty()? Vector<uint8_t> { } : valueOrDefault(FileSystem::readEntireFile(recordBlobFilePath(recordFile)));
+            fileDatas.append(WTFMove(fileData));
+            blobDatas.append(WTFMove(blobData));
+        }
+
+        m_callbackQueue->dispatch([protectedThis = WTFMove(protectedThis), fileDatas = crossThreadCopy(WTFMove(fileDatas)), blobDatas = crossThreadCopy(WTFMove(blobDatas)), callback = WTFMove(callback)]() mutable {
+            callback(WTFMove(fileDatas), WTFMove(blobDatas));
         });
     });
 }
@@ -374,7 +402,7 @@ void CacheStorageDiskStore::readRecords(const Vector<CacheStorageRecordInformati
         return recordFilePath(recordInfo.key);
     });
 
-    auto didReadRecordFiles = [this, protectedThis = Ref { *this }, recordInfos, callback = WTFMove(callback)](auto fileDatas, auto blobDatas) mutable {
+    readRecordsInternal(WTFMove(recordFiles), [this, protectedThis = Ref { *this }, recordInfos, callback = WTFMove(callback)](auto fileDatas, auto blobDatas) mutable {
         ASSERT(recordInfos.size() == fileDatas.size());
         ASSERT(recordInfos.size() == blobDatas.size());
 
@@ -397,21 +425,6 @@ void CacheStorageDiskStore::readRecords(const Vector<CacheStorageRecordInformati
             result.append(WTFMove(record));
         }
         callback(WTFMove(result));
-    };
-
-    m_ioQueue->dispatch([this, protectedThis = Ref { *this }, recordFiles = crossThreadCopy(WTFMove(recordFiles)), didReadRecordFiles = WTFMove(didReadRecordFiles)]() mutable {
-        Vector<Vector<uint8_t>> fileDatas;
-        Vector<Vector<uint8_t>> blobDatas;
-        for (auto& recordFile : recordFiles) {
-            auto fileData = valueOrDefault(FileSystem::readEntireFile(recordFile));
-            auto blobData = fileData.isEmpty()? Vector<uint8_t> { } : valueOrDefault(FileSystem::readEntireFile(recordBlobFilePath(recordFile)));
-            fileDatas.append(WTFMove(fileData));
-            blobDatas.append(WTFMove(blobData));
-        }
-
-        m_callbackQueue->dispatch([protectedThis = WTFMove(protectedThis), fileDatas = crossThreadCopy(WTFMove(fileDatas)), blobDatas = crossThreadCopy(WTFMove(blobDatas)), didReadRecordFiles = WTFMove(didReadRecordFiles)]() mutable {
-            didReadRecordFiles(WTFMove(fileDatas), WTFMove(blobDatas));
-        });
     });
 }
 
@@ -421,7 +434,7 @@ void CacheStorageDiskStore::deleteRecords(const Vector<CacheStorageRecordInforma
         return recordFilePath(recordInfo.key);
     });
 
-    m_ioQueue->dispatch([this, protectedThis = Ref { *this }, recordFiles = WTFMove(recordFiles), callback = WTFMove(callback)]() mutable {
+    m_ioQueue->dispatch([this, protectedThis = Ref { *this }, recordFiles = crossThreadCopy(WTFMove(recordFiles)), callback = WTFMove(callback)]() mutable {
         bool result = true;
         for (auto recordFile : recordFiles) {
             FileSystem::deleteFile(recordBlobFilePath(recordFile));
@@ -447,7 +460,11 @@ static Vector<uint8_t> encodeRecordHeader(CacheStorageRecord&& record)
     record.options.encodePersistent(encoder);
     encoder << record.referrer;
     encoder << record.responseHeadersGuard;
-    encoder << WebCore::ResourceResponse::fromCrossThreadData(WTFMove(record.responseData));
+    // isNull is needed as we switched from encoding ResourceResponse to encoding ResourceResponse::CrossThreadData,
+    // and we don't want to change storage format on disk.
+    bool isNull = false;
+    encoder << isNull;
+    encoder << record.responseData;
     encoder << record.responseBodySize;
     encoder.encodeChecksum();
 
@@ -506,15 +523,16 @@ void CacheStorageDiskStore::writeRecords(Vector<CacheStorageRecord>&& records, W
         auto bodyData = encodeRecordBody(record);
         auto bodyHash = computeSHA1(bodyData.span(), m_salt);
         bool shouldCreateBlob = shouldStoreBodyAsBlob(bodyData);
+        auto recordInfoKey = record.info.key;
         auto headerData = encodeRecordHeader(WTFMove(record));
-        auto recordData = encodeRecord(record.info.key, headerData, !shouldCreateBlob, bodyData, bodyHash, m_salt);
+        auto recordData = encodeRecord(recordInfoKey, headerData, !shouldCreateBlob, bodyData, bodyHash, m_salt);
         recordDatas.append(WTFMove(recordData));
         if (!shouldCreateBlob)
             bodyData = { };
         recordBlobDatas.append(WTFMove(bodyData));
     }
 
-    m_ioQueue->dispatch([this, protectedThis = Ref { *this }, recordFiles = WTFMove(recordFiles), recordDatas = WTFMove(recordDatas), recordBlobDatas = WTFMove(recordBlobDatas), callback = WTFMove(callback)]() mutable {
+    m_ioQueue->dispatch([this, protectedThis = Ref { *this }, recordFiles = crossThreadCopy(WTFMove(recordFiles)), recordDatas = crossThreadCopy(WTFMove(recordDatas)), recordBlobDatas = crossThreadCopy(WTFMove(recordBlobDatas)), callback = WTFMove(callback)]() mutable {
         bool result = true;
         // FIXME: we should probably stop writing and revert changes when result becomes false.
         for (size_t index = 0; index < recordFiles.size(); ++index) {
@@ -523,12 +541,12 @@ void CacheStorageDiskStore::writeRecords(Vector<CacheStorageRecord>&& records, W
             auto recordBlobData = recordBlobDatas[index];
             FileSystem::makeAllDirectories(FileSystem::parentPath(recordFile));
             if (!recordBlobData.isEmpty())  {
-                if (FileSystem::overwriteEntireFile(recordBlobFilePath(recordFile), Span { recordBlobData.data(), recordBlobData.size() }) == -1) {
+                if (FileSystem::overwriteEntireFile(recordBlobFilePath(recordFile), std::span(recordBlobData.data(), recordBlobData.size())) == -1) {
                     result = false;
                     continue;
                 }
             }
-            if (FileSystem::overwriteEntireFile(recordFile, Span { recordData.data(), recordData.size() }) == -1)
+            if (FileSystem::overwriteEntireFile(recordFile, std::span(recordData.data(), recordData.size())) == -1)
                 result = false;
         }
 
