@@ -76,6 +76,7 @@ void SQLiteStorageArea::close()
     ASSERT(!isMainRunLoop());
 
     m_cache = std::nullopt;
+    m_cacheSize = std::nullopt;
     commitTransactionIfNecessary();
     for (size_t i = 0; i < static_cast<size_t>(StatementType::Invalid); ++i)
         m_cachedStatements[i] = nullptr;
@@ -155,14 +156,21 @@ bool SQLiteStorageArea::prepareDatabase(ShouldCreateIfNotExists shouldCreateIfNo
         return true;
 
     m_database = nullptr;
-    if (shouldCreateIfNotExists == ShouldCreateIfNotExists::No && !FileSystem::fileExists(m_path))
+    bool databaseExists = FileSystem::fileExists(m_path);
+    if (shouldCreateIfNotExists == ShouldCreateIfNotExists::No && !databaseExists)
         return true;
 
     m_database = makeUnique<WebCore::SQLiteDatabase>();
     FileSystem::makeAllDirectories(FileSystem::parentPath(m_path));
     auto openResult  = m_database->open(m_path);
-    if (!openResult && handleDatabaseCorruptionIfNeeded(m_database->lastError()))
+    if (!openResult && handleDatabaseCorruptionIfNeeded(m_database->lastError())) {
+        databaseExists = false;
+        if (shouldCreateIfNotExists == ShouldCreateIfNotExists::No)
+            return true;
+
+        m_database = makeUnique<WebCore::SQLiteDatabase>();
         openResult = m_database->open(m_path);
+    }
 
     if (!openResult) {
         RELEASE_LOG_ERROR(Storage, "SQLiteStorageArea::prepareDatabase failed to open database at '%s'", m_path.utf8().data());
@@ -179,10 +187,9 @@ bool SQLiteStorageArea::prepareDatabase(ShouldCreateIfNotExists shouldCreateIfNo
         return false;
     }
 
-    if (quota() != WebCore::StorageMap::noQuota) {
-        // Value is upconverted and stored as blob in database, so we need to make database file limit
-        // bigger than quota.
-        m_database->setMaximumSize(quota() * 2);
+    if (!databaseExists) {
+        m_cache = HashMap<String, Value> { };
+        m_cacheSize = 0;
     }
 
     return true;
@@ -192,7 +199,7 @@ void SQLiteStorageArea::startTransactionIfNecessary()
 {
     ASSERT(m_database);
 
-    if (!m_transaction)
+    if (!m_transaction || m_transaction->wasRolledBackBySqlite())
         m_transaction = makeUnique<WebCore::SQLiteTransaction>(*m_database);
 
     if (m_transaction->inProgress())
@@ -224,10 +231,12 @@ Expected<String, StorageError> SQLiteStorageArea::getItem(const String& key)
     if (m_cache) {
         auto iterator = m_cache->find(key);
         if (iterator == m_cache->end())
-            return String();
+            return makeUnexpected(StorageError::ItemNotFound);
 
-        if (!iterator->value.isNull())
-            return iterator->value;
+        if (auto* valueString = std::get_if<String>(&iterator->value)) {
+            ASSERT(!valueString->isNull());
+            return *valueString;
+        }
     }
 
     return getItemFromDatabase(key);
@@ -271,8 +280,9 @@ HashMap<String, String> SQLiteStorageArea::allItems()
     if (m_cache) {
         items.reserveInitialCapacity(m_cache->size());
         for (auto& [key, value] : *m_cache) {
-            if (!value.isNull()) {
-                items.add(key, value);
+            if (auto* valueString = std::get_if<String>(&value)) {
+                ASSERT(!valueString->isNull());
+                items.add(key, *valueString);
                 continue;
             }
 
@@ -289,14 +299,15 @@ HashMap<String, String> SQLiteStorageArea::allItems()
         return { };
     }
 
-    m_cache = HashMap<String, String> { };
+    m_cache = HashMap<String, Value> { };
+    m_cacheSize = 0;
     auto result = statement->step();
     while (result == SQLITE_ROW) {
         String key = statement->columnText(0);
         String value = statement->columnBlobAsString(1);
         if (!key.isNull() && !value.isNull()) {
-            m_cache->add(key, value.sizeInBytes() > maximumSizeForValuesKeptInMemory ? String() : value);
-            items.add(WTFMove(key), WTFMove(value));
+            items.add(key, value);
+            updateCacheIfNeeded(WTFMove(key), WTFMove(value));
         }
 
         result = statement->step();
@@ -317,6 +328,9 @@ Expected<void, StorageError> SQLiteStorageArea::setItem(IPC::Connection::UniqueI
     if (!prepareDatabase(ShouldCreateIfNotExists::Yes))
         return makeUnexpected(StorageError::Database);
 
+    if (!requestSpace(key, value))
+        return makeUnexpected(StorageError::QuotaExceeded);
+
     startTransactionIfNecessary();
     String oldValue;
     if (auto valueOrError = getItem(key))
@@ -329,18 +343,14 @@ Expected<void, StorageError> SQLiteStorageArea::setItem(IPC::Connection::UniqueI
     }
 
     const auto result = statement->step();
-    if (result == SQLITE_FULL)
-        return makeUnexpected(StorageError::QuotaExceeded);
     if (result != SQLITE_DONE) {
         RELEASE_LOG_ERROR(Storage, "SQLiteStorageArea::setItem failed on stepping statement (%d) - %s", m_database->lastError(), m_database->lastErrorMsg());
         handleDatabaseCorruptionIfNeeded(result);
-
         return makeUnexpected(StorageError::Database);
     }
 
     dispatchEvents(connection, storageAreaImplID, key, oldValue, value, urlString);
-    if (m_cache)
-        m_cache->set(WTFMove(key), value.sizeInBytes() > maximumSizeForValuesKeptInMemory ? String() : WTFMove(value));
+    updateCacheIfNeeded(key, value);
 
     return { };
 }
@@ -377,8 +387,7 @@ Expected<void, StorageError> SQLiteStorageArea::removeItem(IPC::Connection::Uniq
     }
 
     dispatchEvents(connection, storageAreaImplID, key, oldValue, String(), urlString);
-    if (m_cache)
-        m_cache->remove(key);
+    updateCacheIfNeeded(key, { });
 
     return { };
 }
@@ -393,8 +402,10 @@ Expected<void, StorageError> SQLiteStorageArea::clear(IPC::Connection::UniqueID
     if (m_cache && m_cache->isEmpty())
         return makeUnexpected(StorageError::ItemNotFound);
 
-    if (m_cache)
+    if (m_cache) {
         m_cache->clear();
+        m_cacheSize = 0;
+    }
 
     if (!m_database)
         return makeUnexpected(StorageError::ItemNotFound);
@@ -442,9 +453,93 @@ bool SQLiteStorageArea::handleDatabaseCorruptionIfNeeded(int databaseError)
         return false;
 
     m_database = nullptr;
+    m_cache = std::nullopt;
+    m_cacheSize = std::nullopt;
     RELEASE_LOG(Storage, "SQLiteStorageArea::handleDatabaseCorruption deletes corrupted database file '%s'", m_path.utf8().data());
     WebCore::SQLiteFileSystem::deleteDatabaseFile(m_path);
     return true;
 }
 
+void SQLiteStorageArea::updateCacheIfNeeded(const String& key, const String& value)
+{
+    if (!m_cache)
+        return;
+
+    ASSERT(m_cacheSize);
+    auto iter = m_cache->find(key);
+    bool itemExists = iter != m_cache->end();
+    unsigned oldKeySize = 0;
+    unsigned oldValueSize = 0;
+    unsigned keySize = key.sizeInBytes();
+    unsigned valueSize = value.sizeInBytes();
+    if (itemExists) {
+        oldKeySize = iter->key.sizeInBytes();
+        WTF::switchOn(iter->value, [&](unsigned valueSize) {
+            oldValueSize = valueSize;
+        }, [&](const String& value) {
+            oldValueSize = value.sizeInBytes();
+        });
+    }
+
+    CheckedUint32 newCacheSize = *m_cacheSize;
+    // Null value means to remove.
+    if (value.isNull()) {
+        m_cache->remove(key);
+        newCacheSize -= oldKeySize;
+        newCacheSize -= oldValueSize;
+    } else {
+        if (valueSize > maximumSizeForValuesKeptInMemory)
+            m_cache->set(key, valueSize);
+        else
+            m_cache->set(key, value);
+        newCacheSize -= oldKeySize;
+        newCacheSize -= oldValueSize;
+        newCacheSize += itemExists ? oldKeySize : keySize;
+        newCacheSize += valueSize;
+    }
+
+    if (newCacheSize.hasOverflowed()) {
+        RELEASE_LOG_ERROR(Storage, "SQLiteStorageArea::updateCacheIfNeeded newCacheSize has overflowed: cacheSize - %u, oldKeySize - %u, oldValueSize - %u, keySize - %u, valueSize - %u, will recompute", *m_cacheSize, oldKeySize, oldValueSize, keySize, valueSize);
+        newCacheSize = 0;
+        for (auto& value : m_cache->values()) {
+            WTF::switchOn(value, [&](unsigned size) {
+                newCacheSize += size;
+            }, [&](const String& value) {
+                newCacheSize += value.sizeInBytes();
+            });
+        }
+    }
+
+    m_cacheSize = newCacheSize;
+}
+
+bool SQLiteStorageArea::requestSpace(const String& key, const String& value)
+{
+    ASSERT(m_database && m_database->isOpen());
+    if (!m_cache)
+        return key.sizeInBytes() + value.sizeInBytes() <= quota();
+
+    if (value.isNull())
+        return true;
+
+    ASSERT(m_cacheSize);
+    CheckedUint32 newCacheSize = *m_cacheSize;
+    auto iter = m_cache->find(key);
+    if (iter == m_cache->end())
+        newCacheSize += key.sizeInBytes();
+    else {
+        auto oldValueSize = WTF::switchOn(iter->value, [](unsigned valueSize) {
+            return valueSize;
+        }, [](const String& value) {
+            return value.sizeInBytes();
+        });
+        newCacheSize -= oldValueSize;
+    }
+    newCacheSize += value.sizeInBytes();
+    if (newCacheSize.hasOverflowed())
+        return false;
+
+    return newCacheSize <= quota();
+}
+
 } // namespace WebKit
