@@ -511,10 +511,10 @@ UniqueRef<Encoder> Connection::createSyncMessageEncoder(MessageName messageName,
     return encoder;
 }
 
-bool Connection::sendMessage(UniqueRef<Encoder>&& encoder, OptionSet<SendOption> sendOptions, std::optional<Thread::QOS> qos)
+Error Connection::sendMessage(UniqueRef<Encoder>&& encoder, OptionSet<SendOption> sendOptions, std::optional<Thread::QOS> qos)
 {
     if (!isValid())
-        return false;
+        return Error::InvalidConnection;
 
 #if ENABLE(IPC_TESTING_API)
     if (isMainRunLoop()) {
@@ -535,7 +535,7 @@ bool Connection::sendMessage(UniqueRef<Encoder>&& encoder, OptionSet<SendOption>
         auto wrappedMessage = createSyncMessageEncoder(MessageName::WrappedAsyncMessageForTesting, encoder->destinationID(), syncRequestID);
         wrappedMessage->setFullySynchronousModeForTesting();
         wrappedMessage->wrapForTesting(WTFMove(encoder));
-        return static_cast<bool>(sendSyncMessage(syncRequestID, WTFMove(wrappedMessage), Timeout::infinity(), { }));
+        return sendSyncMessage(syncRequestID, WTFMove(wrappedMessage), Timeout::infinity(), { }).error;
     }
 
 #if ENABLE(IPC_TESTING_API)
@@ -560,8 +560,10 @@ bool Connection::sendMessage(UniqueRef<Encoder>&& encoder, OptionSet<SendOption>
 
     size_t outgoingMessagesCount;
     bool shouldNotifyOfQueueGrowingLarge;
+    bool shouldDispatchMessageSend;
     {
         Locker locker { m_outgoingMessagesLock };
+        shouldDispatchMessageSend = m_outgoingMessages.isEmpty();
         m_outgoingMessages.append(WTFMove(encoder));
         outgoingMessagesCount = m_outgoingMessages.size();
         shouldNotifyOfQueueGrowingLarge = m_outgoingMessageQueueIsGrowingLargeCallback && outgoingMessagesCount > largeOutgoingMessageQueueCountThreshold && (MonotonicTime::now() - m_lastOutgoingMessageQueueIsGrowingLargeCallbackCallTime) >= largeOutgoingMessageQueueTimeThreshold;
@@ -578,28 +580,32 @@ bool Connection::sendMessage(UniqueRef<Encoder>&& encoder, OptionSet<SendOption>
         m_outgoingMessageQueueIsGrowingLargeCallback();
     }
 
-    // FIXME: We should add a boolean flag so we don't call this when work has already been scheduled.
-    auto sendOutgoingMessages = [protectedThis = Ref { *this }]() mutable {
-        protectedThis->sendOutgoingMessages();
-    };
+    // It's not clear if calling dispatchWithQOS() will do anything if Connection::sendOutgoingMessages() is already running.
+    if (shouldDispatchMessageSend || qos) {
+        auto sendOutgoingMessages = [protectedThis = Ref { *this }]() mutable {
+            protectedThis->sendOutgoingMessages();
+        };
 
-    if (qos)
-        m_connectionQueue->dispatchWithQOS(WTFMove(sendOutgoingMessages), *qos);
-    else
-        m_connectionQueue->dispatch(WTFMove(sendOutgoingMessages));
+        if (qos)
+            m_connectionQueue->dispatchWithQOS(WTFMove(sendOutgoingMessages), *qos);
+        else
+            m_connectionQueue->dispatch(WTFMove(sendOutgoingMessages));
+    }
 
-    return true;
+    return Error::NoError;
 }
 
-bool Connection::sendMessageWithAsyncReply(UniqueRef<Encoder>&& encoder, AsyncReplyHandler replyHandler, OptionSet<SendOption> sendOptions, std::optional<Thread::QOS> qos)
+Error Connection::sendMessageWithAsyncReply(UniqueRef<Encoder>&& encoder, AsyncReplyHandler replyHandler, OptionSet<SendOption> sendOptions, std::optional<Thread::QOS> qos)
 {
     ASSERT(replyHandler.replyID);
     ASSERT(replyHandler.completionHandler);
     auto replyID = replyHandler.replyID;
     encoder.get() << replyID;
     addAsyncReplyHandler(WTFMove(replyHandler));
-    if (sendMessage(WTFMove(encoder), sendOptions, qos))
-        return true;
+    auto error = sendMessage(WTFMove(encoder), sendOptions, qos);
+    if (error == Error::NoError)
+        return Error::NoError;
+
     // replyHandlerToCancel might be already cancelled if invalidate() happened in-between.
     if (auto replyHandlerToCancel = takeAsyncReplyHandler(replyID)) {
         // FIXME: Current contract is that completionHandler is called on the connection run loop.
@@ -608,10 +614,10 @@ bool Connection::sendMessageWithAsyncReply(UniqueRef<Encoder>&& encoder, AsyncRe
             completionHandler(nullptr);
         });
     }
-    return false;
+    return error;
 }
 
-bool Connection::sendSyncReply(UniqueRef<Encoder>&& encoder)
+Error Connection::sendSyncReply(UniqueRef<Encoder>&& encoder)
 {
     return sendMessage(WTFMove(encoder), { });
 }
@@ -621,10 +627,11 @@ Timeout Connection::timeoutRespectingIgnoreTimeoutsForTesting(Timeout timeout) c
     return m_ignoreTimeoutsForTesting ? Timeout::infinity() : timeout;
 }
 
-std::unique_ptr<Decoder> Connection::waitForMessage(MessageName messageName, uint64_t destinationID, Timeout timeout, OptionSet<WaitForOption> waitForOptions)
+auto Connection::waitForMessage(MessageName messageName, uint64_t destinationID, Timeout timeout, OptionSet<WaitForOption> waitForOptions) -> DecoderOrError
 {
     if (!isValid())
-        return nullptr;
+        return Error::InvalidConnection;
+
     assertIsCurrent(dispatcher());
     Ref protectedThis { *this };
 
@@ -638,12 +645,12 @@ std::unique_ptr<Decoder> Connection::waitForMessage(MessageName messageName, uin
         // We don't support having multiple clients waiting for messages.
         ASSERT(!m_waitingForMessage);
         if (m_waitingForMessage)
-            return nullptr;
+            return Error::MultipleWaitingClients;
 
         // If the connection is already invalidated, don't even start waiting.
         // Once m_waitingForMessage is set, messageWaitingInterrupted will cover this instead.
         if (!m_shouldWaitForMessages)
-            return nullptr;
+            return Error::AttemptingToWaitOnClosedConnection;
 
         bool hasIncomingSynchronousMessage = false;
 
@@ -657,7 +664,7 @@ std::unique_ptr<Decoder> Connection::waitForMessage(MessageName messageName, uin
                     std::unique_ptr<Decoder> returnedMessage = WTFMove(message);
 
                     m_incomingMessages.remove(it);
-                    return returnedMessage;
+                    return { WTFMove(returnedMessage) };
                 }
 
                 if (message->isSyncMessage())
@@ -666,13 +673,8 @@ std::unique_ptr<Decoder> Connection::waitForMessage(MessageName messageName, uin
         }
 
         // Don't even start waiting if we have InterruptWaitingIfSyncMessageArrives and there's a sync message already in the queue.
-        if (hasIncomingSynchronousMessage && waitForOptions.contains(WaitForOption::InterruptWaitingIfSyncMessageArrives)) {
-#if ASSERT_ENABLED
-            // We don't support having multiple clients waiting for messages.
-            ASSERT(!m_waitingForMessage);
-#endif
-            return nullptr;
-        }
+        if (hasIncomingSynchronousMessage && waitForOptions.contains(WaitForOption::InterruptWaitingIfSyncMessageArrives))
+            return { Error::SyncMessageInterruptedWait };
 
         m_waitingForMessage = &waitingForMessage;
     }
@@ -689,31 +691,38 @@ std::unique_ptr<Decoder> Connection::waitForMessage(MessageName messageName, uin
 
         if (wasMessageToWaitForAlreadyDispatched) {
             m_waitingForMessage = nullptr;
-            break;
+            return { Error::WaitingOnAlreadyDispatchedMessage };
         }
 
         if (UNLIKELY(m_inDispatchSyncMessageCount && !timeout.isInfinity())) {
             RELEASE_LOG_ERROR(IPC, "Connection::waitForMessage(%" PUBLIC_LOG_STRING "): Exiting immediately, since we're handling a sync message already", description(messageName));
             m_waitingForMessage = nullptr;
-            break;
+            return { Error::AttemptingToWaitInsideSyncMessageHandling };
         }
 
         if (m_waitingForMessage->decoder) {
             auto decoder = WTFMove(m_waitingForMessage->decoder);
             m_waitingForMessage = nullptr;
-            return decoder;
+            return { WTFMove(decoder) };
         }
 
-        // Now we wait.
-        bool didTimeout = !m_waitForMessageCondition.waitUntil(m_waitForMessageLock, timeout.deadline());
-        // We timed out, lost our connection, or a sync message came in with InterruptWaitingIfSyncMessageArrives, so stop waiting.
-        if (didTimeout || m_waitingForMessage->messageWaitingInterrupted) {
+        if (!isValid()) {
             m_waitingForMessage = nullptr;
-            break;
+            return Error::InvalidConnection;
+        }
+
+        bool didTimeout = !m_waitForMessageCondition.waitUntil(m_waitForMessageLock, timeout.deadline());
+        if (didTimeout) {
+            m_waitingForMessage = nullptr;
+            return Error::Timeout;
+        }
+        if (m_waitingForMessage->messageWaitingInterrupted) {
+            m_waitingForMessage = nullptr;
+            return Error::SyncMessageInterruptedWait;
         }
     }
 
-    return nullptr;
+    return Error::Unspecified;
 }
 
 bool Connection::pushPendingSyncRequestID(SyncRequestID syncRequestID)
@@ -736,17 +745,17 @@ void Connection::popPendingSyncRequestID(SyncRequestID syncRequestID)
     m_pendingSyncReplies.removeLast();
 }
 
-std::unique_ptr<Decoder> Connection::sendSyncMessage(SyncRequestID syncRequestID, UniqueRef<Encoder>&& encoder, Timeout timeout, OptionSet<SendSyncOption> sendSyncOptions)
+auto Connection::sendSyncMessage(SyncRequestID syncRequestID, UniqueRef<Encoder>&& encoder, Timeout timeout, OptionSet<SendSyncOption> sendSyncOptions) -> DecoderOrError
 {
     ASSERT(syncRequestID);
     if (!isValid()) {
-        didFailToSendSyncMessage();
-        return nullptr;
+        didFailToSendSyncMessage(Error::InvalidConnection);
+        return Error::InvalidConnection;
     }
     assertIsCurrent(dispatcher());
     if (!pushPendingSyncRequestID(syncRequestID)) {
-        didFailToSendSyncMessage();
-        return nullptr;
+        didFailToSendSyncMessage(Error::CantWaitForSyncReplies);
+        return { Error::CantWaitForSyncReplies };
     }
 
     // First send the message.
@@ -766,17 +775,20 @@ std::unique_ptr<Decoder> Connection::sendSyncMessage(SyncRequestID syncRequestID
     // Then wait for a reply. Waiting for a reply could involve dispatching incoming sync messages, so
     // keep an extra reference to the connection here in case it's invalidated.
     Ref<Connection> protect(*this);
-    std::unique_ptr<Decoder> reply = waitForSyncReply(syncRequestID, messageName, timeout, sendSyncOptions);
+    auto replyOrError = waitForSyncReply(syncRequestID, messageName, timeout, sendSyncOptions);
 
     popPendingSyncRequestID(syncRequestID);
 
-    if (!reply)
-        didFailToSendSyncMessage();
+    if (!replyOrError.decoder) {
+        if (replyOrError.error == Error::NoError)
+            replyOrError.error = Error::Unspecified;
+        didFailToSendSyncMessage(replyOrError.error);
+    }
 
-    return reply;
+    return replyOrError;
 }
 
-std::unique_ptr<Decoder> Connection::waitForSyncReply(SyncRequestID syncRequestID, MessageName messageName, Timeout timeout, OptionSet<SendSyncOption> sendSyncOptions)
+auto Connection::waitForSyncReply(SyncRequestID syncRequestID, MessageName messageName, Timeout timeout, OptionSet<SendSyncOption> sendSyncOptions) -> DecoderOrError
 {
     timeout = timeoutRespectingIgnoreTimeoutsForTesting(timeout);
 
@@ -793,13 +805,19 @@ std::unique_ptr<Decoder> Connection::waitForSyncReply(SyncRequestID syncRequestI
             // Second, check if there is a sync reply at the top of the stack.
             ASSERT(!m_pendingSyncReplies.isEmpty());
 
-            PendingSyncReply& pendingSyncReply = m_pendingSyncReplies.last();
+            auto& pendingSyncReply = m_pendingSyncReplies.last();
             ASSERT_UNUSED(syncRequestID, pendingSyncReply.syncRequestID == syncRequestID);
 
-            // We found the sync reply, or the connection was closed.
-            if (pendingSyncReply.didReceiveReply || !m_shouldWaitForSyncReplies) {
+            // We found the sync reply.
+            if (pendingSyncReply.didReceiveReply) {
                 didReceiveSyncReply(sendSyncOptions);
-                return WTFMove(pendingSyncReply.replyDecoder);
+                return { WTFMove(pendingSyncReply.replyDecoder) };
+            }
+
+            // The connection was closed.
+            if (!m_shouldWaitForSyncReplies) {
+                didReceiveSyncReply(sendSyncOptions);
+                return Error::InvalidConnection;
             }
         }
 
@@ -810,7 +828,7 @@ std::unique_ptr<Decoder> Connection::waitForSyncReply(SyncRequestID syncRequestI
         if (!isValid()) {
             RELEASE_LOG_ERROR(IPC, "Connection::waitForSyncReply: Connection no longer valid, id=%" PRIu64, syncRequestID.toUInt64());
             didReceiveSyncReply(sendSyncOptions);
-            return nullptr;
+            return Error::InvalidConnection;
         }
 
         // We didn't find a sync reply yet, keep waiting.
@@ -827,7 +845,7 @@ std::unique_ptr<Decoder> Connection::waitForSyncReply(SyncRequestID syncRequestI
 
     didReceiveSyncReply(sendSyncOptions);
 
-    return nullptr;
+    return Error::Timeout;
 }
 
 void Connection::processIncomingSyncReply(std::unique_ptr<Decoder> decoder)
@@ -1144,7 +1162,7 @@ void Connection::dispatchOnReceiveQueueForTesting(Function<void()>&& completionH
     m_connectionQueue->dispatch(WTFMove(completionHandler));
 }
 
-void Connection::didFailToSendSyncMessage()
+void Connection::didFailToSendSyncMessage(Error)
 {
     if (!m_shouldExitOnSyncMessageSendFailure)
         return;
@@ -1464,5 +1482,30 @@ std::optional<Connection::Handle> Connection::Handle::decode(Decoder& decoder)
     return handle;
 }
 
+const char* errorAsString(Error error)
+{
+    switch (error) {
+    case Error::NoError: return "NoError";
+    case Error::InvalidConnection: return "InvalidConnection";
+    case Error::NoConnectionForIdentifier: return "NoConnectionForIdentifier";
+    case Error::NoMessageSenderConnection: return "NoMessageSenderConnection";
+    case Error::Timeout: return "Timeout";
+    case Error::Unspecified: return "Unspecified";
+    case Error::MultipleWaitingClients: return "MultipleWaitingClients";
+    case Error::AttemptingToWaitOnClosedConnection: return "AttemptingToWaitOnClosedConnection";
+    case Error::WaitingOnAlreadyDispatchedMessage: return "WaitingOnAlreadyDispatchedMessage";
+    case Error::AttemptingToWaitInsideSyncMessageHandling: return "AttemptingToWaitInsideSyncMessageHandling";
+    case Error::SyncMessageInterruptedWait: return "SyncMessageInterruptedWait";
+    case Error::CantWaitForSyncReplies: return "CantWaitForSyncReplies";
+    case Error::FailedToEncodeMessageArguments: return "FailedToEncodeMessageArguments";
+    case Error::FailedToDecodeReplyArguments: return "FailedToDecodeReplyArguments";
+    case Error::FailedToFindReplyHandler: return "FailedToFindReplyHandler";
+    case Error::FailedToAcquireBufferSpan: return "FailedToAcquireBufferSpan";
+    case Error::FailedToAcquireReplyBufferSpan: return "FailedToAcquireReplyBufferSpan";
+    case Error::StreamConnectionEncodingError: return "StreamConnectionEncodingError";
+    }
+
+    return "";
+}
 
 } // namespace IPC
