@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2015 Apple Inc. All rights reserved.
+ * Copyright (C) 2014-2023 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -92,78 +92,8 @@ template<> struct ArgumentCoder<ScrollingStatePositionedNode> {
     static WARN_UNUSED_RETURN bool decode(Decoder&, ScrollingStatePositionedNode&);
 };
 
-template<> struct ArgumentCoder<RequestedScrollData> {
-    static void encode(Encoder&, const RequestedScrollData&);
-    static WARN_UNUSED_RETURN bool decode(Decoder&, RequestedScrollData&);
-};
-
-template<> struct ArgumentCoder<FloatScrollSnapOffsetsInfo> {
-    static void encode(Encoder&, const FloatScrollSnapOffsetsInfo&);
-    static WARN_UNUSED_RETURN bool decode(Decoder&, FloatScrollSnapOffsetsInfo&);
-};
-
-template<> struct ArgumentCoder<SnapOffset<float>> {
-    static void encode(Encoder&, const SnapOffset<float>&);
-    static WARN_UNUSED_RETURN bool decode(Decoder&, SnapOffset<float>&);
-};
-
 } // namespace IPC
 
-namespace WTF {
-
-template<> struct EnumTraits<WebCore::ScrollingStateNode::Property> {
-    using values = EnumValues<
-        WebCore::ScrollingStateNode::Property,
-        WebCore::ScrollingStateNode::Property::Layer,
-        WebCore::ScrollingStateNode::Property::ChildNodes,
-        WebCore::ScrollingStateNode::Property::ScrollableAreaSize,
-        WebCore::ScrollingStateNode::Property::TotalContentsSize,
-        WebCore::ScrollingStateNode::Property::ReachableContentsSize,
-        WebCore::ScrollingStateNode::Property::ScrollPosition,
-        WebCore::ScrollingStateNode::Property::ScrollOrigin,
-        WebCore::ScrollingStateNode::Property::ScrollableAreaParams,
-        WebCore::ScrollingStateNode::Property::ReasonsForSynchronousScrolling,
-        WebCore::ScrollingStateNode::Property::RequestedScrollPosition,
-        WebCore::ScrollingStateNode::Property::SnapOffsetsInfo,
-        WebCore::ScrollingStateNode::Property::CurrentHorizontalSnapOffsetIndex,
-        WebCore::ScrollingStateNode::Property::CurrentVerticalSnapOffsetIndex,
-        WebCore::ScrollingStateNode::Property::IsMonitoringWheelEvents,
-        WebCore::ScrollingStateNode::Property::ScrollContainerLayer,
-        WebCore::ScrollingStateNode::Property::ScrolledContentsLayer,
-        WebCore::ScrollingStateNode::Property::HorizontalScrollbarLayer,
-        WebCore::ScrollingStateNode::Property::VerticalScrollbarLayer,
-        WebCore::ScrollingStateNode::Property::PainterForScrollbar,
-        WebCore::ScrollingStateNode::Property::FrameScaleFactor,
-        WebCore::ScrollingStateNode::Property::EventTrackingRegion,
-        WebCore::ScrollingStateNode::Property::RootContentsLayer,
-        WebCore::ScrollingStateNode::Property::CounterScrollingLayer,
-        WebCore::ScrollingStateNode::Property::InsetClipLayer,
-        WebCore::ScrollingStateNode::Property::ContentShadowLayer,
-        WebCore::ScrollingStateNode::Property::HeaderHeight,
-        WebCore::ScrollingStateNode::Property::FooterHeight,
-        WebCore::ScrollingStateNode::Property::HeaderLayer,
-        WebCore::ScrollingStateNode::Property::FooterLayer,
-        WebCore::ScrollingStateNode::Property::BehaviorForFixedElements,
-        WebCore::ScrollingStateNode::Property::TopContentInset,
-        WebCore::ScrollingStateNode::Property::FixedElementsLayoutRelativeToFrame,
-        WebCore::ScrollingStateNode::Property::VisualViewportIsSmallerThanLayoutViewport,
-        WebCore::ScrollingStateNode::Property::AsyncFrameOrOverflowScrollingEnabled,
-        WebCore::ScrollingStateNode::Property::WheelEventGesturesBecomeNonBlocking,
-        WebCore::ScrollingStateNode::Property::ScrollingPerformanceTestingEnabled,
-        WebCore::ScrollingStateNode::Property::LayoutViewport,
-        WebCore::ScrollingStateNode::Property::MinLayoutViewportOrigin,
-        WebCore::ScrollingStateNode::Property::MaxLayoutViewportOrigin,
-        WebCore::ScrollingStateNode::Property::OverrideVisualViewportSize,
-        WebCore::ScrollingStateNode::Property::RelatedOverflowScrollingNodes,
-        WebCore::ScrollingStateNode::Property::LayoutConstraintData,
-        WebCore::ScrollingStateNode::Property::ViewportConstraints,
-        WebCore::ScrollingStateNode::Property::OverflowScrollingNode,
-        WebCore::ScrollingStateNode::Property::KeyboardScrollData
-    >;
-};
-
-} // namespace WTF
-
 using namespace IPC;
 
 void ArgumentCoder<ScrollingStateNode>::encode(Encoder& encoder, const ScrollingStateNode& node)
@@ -186,10 +116,10 @@ bool ArgumentCoder<ScrollingStateNode>::decode(Decoder& decoder, ScrollingStateN
 
     node.setChangedProperties(changedProperties);
     if (node.hasChangedProperty(ScrollingStateNode::Property::Layer)) {
-        std::optional<GraphicsLayer::PlatformLayerID> layerID;
+        std::optional<PlatformLayerIdentifier> layerID;
         if (!decoder.decode(layerID))
             return false;
-        node.setLayer(layerID.value_or(GraphicsLayer::PlatformLayerID { }));
+        node.setLayer(layerID.value_or(PlatformLayerIdentifier { }));
     }
 
     return true;
@@ -222,6 +152,7 @@ void ArgumentCoder<ScrollingStateScrollingNode>::encode(Encoder& encoder, const
     SCROLLING_NODE_ENCODE(ScrollingStateNode::Property::ScrollableAreaParams, scrollableAreaParameters)
     SCROLLING_NODE_ENCODE(ScrollingStateNode::Property::RequestedScrollPosition, requestedScrollData)
     SCROLLING_NODE_ENCODE(ScrollingStateNode::Property::KeyboardScrollData, keyboardScrollData)
+    SCROLLING_NODE_ENCODE(ScrollingStateNode::Property::ContentAreaHoverState, mouseIsOverContentArea)
 
     if (node.hasChangedProperty(ScrollingStateNode::Property::ScrollContainerLayer))
         encoder << node.scrollContainerLayer().layerIDForEncoding();
@@ -234,6 +165,18 @@ void ArgumentCoder<ScrollingStateScrollingNode>::encode(Encoder& encoder, const
 
     if (node.hasChangedProperty(ScrollingStateNode::Property::VerticalScrollbarLayer))
         encoder << node.verticalScrollbarLayer().layerIDForEncoding();
+    
+    if (node.hasChangedProperty(ScrollingStateNode::Property::ScrollbarHoverState)) {
+        auto mouseIsInScrollbar = node.scrollbarHoverState();
+        encoder << mouseIsInScrollbar.mouseIsOverHorizontalScrollbar;
+        encoder << mouseIsInScrollbar.mouseIsOverVerticalScrollbar;
+    }
+    
+    if (node.hasChangedProperty(ScrollingStateNode::Property::MouseActivityState)) {
+        auto mouseLocationState = node.mouseLocationState();
+        encoder << mouseLocationState.locationInHorizontalScrollbar;
+        encoder << mouseLocationState.locationInVerticalScrollbar;
+    }
 }
 
 void ArgumentCoder<ScrollingStateFrameScrollingNode>::encode(Encoder& encoder, const ScrollingStateFrameScrollingNode& node)
@@ -255,7 +198,8 @@ void ArgumentCoder<ScrollingStateFrameScrollingNode>::encode(Encoder& encoder, c
     SCROLLING_NODE_ENCODE(ScrollingStateNode::Property::MinLayoutViewportOrigin, minLayoutViewportOrigin)
     SCROLLING_NODE_ENCODE(ScrollingStateNode::Property::MaxLayoutViewportOrigin, maxLayoutViewportOrigin)
     SCROLLING_NODE_ENCODE(ScrollingStateNode::Property::OverrideVisualViewportSize, overrideVisualViewportSize)
-
+    SCROLLING_NODE_ENCODE(ScrollingStateNode::Property::OverlayScrollbarsEnabled, overlayScrollbarsEnabled)
+    
     if (node.hasChangedProperty(ScrollingStateNode::Property::CounterScrollingLayer))
         encoder << node.counterScrollingLayer().layerIDForEncoding();
 
@@ -290,7 +234,7 @@ void ArgumentCoder<ScrollingStateOverflowScrollProxyNode>::encode(Encoder& encod
         type decodedValue; \
         if (!decoder.decode(decodedValue)) \
             return false; \
-        node.setter(decodedValue); \
+        node.setter(WTFMove(decodedValue)); \
     }
 
 #define SCROLLING_NODE_DECODE_ENUM(property, type, setter) \
@@ -319,35 +263,63 @@ bool ArgumentCoder<ScrollingStateScrollingNode>::decode(Decoder& decoder, Scroll
 #endif
     SCROLLING_NODE_DECODE(ScrollingStateNode::Property::IsMonitoringWheelEvents, bool, setIsMonitoringWheelEvents);
     SCROLLING_NODE_DECODE(ScrollingStateNode::Property::ScrollableAreaParams, ScrollableAreaParameters, setScrollableAreaParameters);
-    SCROLLING_NODE_DECODE(ScrollingStateNode::Property::RequestedScrollPosition, RequestedScrollData, setRequestedScrollData);
+    if (node.hasChangedProperty(ScrollingStateNode::Property::RequestedScrollPosition)) {
+        RequestedScrollData requestedScrollData;
+        if (!decoder.decode(requestedScrollData))
+            return false;
+        node.setRequestedScrollData(WTFMove(requestedScrollData), ScrollingStateScrollingNode::CanMergeScrollData::No);
+    }
     SCROLLING_NODE_DECODE(ScrollingStateNode::Property::KeyboardScrollData, RequestedKeyboardScrollData, setKeyboardScrollData);
+    SCROLLING_NODE_DECODE(ScrollingStateNode::Property::ContentAreaHoverState, bool, setMouseIsOverContentArea);
 
     if (node.hasChangedProperty(ScrollingStateNode::Property::ScrollContainerLayer)) {
-        std::optional<GraphicsLayer::PlatformLayerID> layerID;
+        std::optional<PlatformLayerIdentifier> layerID;
         if (!decoder.decode(layerID))
             return false;
-        node.setScrollContainerLayer(layerID.value_or(GraphicsLayer::PlatformLayerID { }));
+        node.setScrollContainerLayer(layerID.value_or(PlatformLayerIdentifier { }));
     }
 
     if (node.hasChangedProperty(ScrollingStateNode::Property::ScrolledContentsLayer)) {
-        std::optional<GraphicsLayer::PlatformLayerID> layerID;
+        std::optional<PlatformLayerIdentifier> layerID;
         if (!decoder.decode(layerID))
             return false;
-        node.setScrolledContentsLayer(layerID.value_or(GraphicsLayer::PlatformLayerID { }));
+        node.setScrolledContentsLayer(layerID.value_or(PlatformLayerIdentifier { }));
     }
 
     if (node.hasChangedProperty(ScrollingStateNode::Property::HorizontalScrollbarLayer)) {
-        std::optional<GraphicsLayer::PlatformLayerID> layerID;
+        std::optional<PlatformLayerIdentifier> layerID;
         if (!decoder.decode(layerID))
             return false;
-        node.setHorizontalScrollbarLayer(layerID.value_or(GraphicsLayer::PlatformLayerID { }));
+        node.setHorizontalScrollbarLayer(layerID.value_or(PlatformLayerIdentifier { }));
     }
 
     if (node.hasChangedProperty(ScrollingStateNode::Property::VerticalScrollbarLayer)) {
-        std::optional<GraphicsLayer::PlatformLayerID> layerID;
+        std::optional<PlatformLayerIdentifier> layerID;
         if (!decoder.decode(layerID))
             return false;
-        node.setVerticalScrollbarLayer(layerID.value_or(GraphicsLayer::PlatformLayerID { }));
+        node.setVerticalScrollbarLayer(layerID.value_or(PlatformLayerIdentifier { }));
+    }
+    
+    if (node.hasChangedProperty(ScrollingStateNode::Property::ScrollbarHoverState)) {
+        bool didEnterScrollbarHorizontal;
+        if (!decoder.decode(didEnterScrollbarHorizontal))
+            return false;
+
+        bool didEnterScrollbarVertical;
+        if (!decoder.decode(didEnterScrollbarVertical))
+            return false;
+        node.setScrollbarHoverState({ didEnterScrollbarHorizontal, didEnterScrollbarVertical });
+    }
+    
+    if (node.hasChangedProperty(ScrollingStateNode::Property::MouseActivityState)) {
+        IntPoint locationInHorizontalScrollbar;
+        if (!decoder.decode(locationInHorizontalScrollbar))
+            return false;
+
+        IntPoint locationInVerticalScrollbar;
+        if (!decoder.decode(locationInVerticalScrollbar))
+            return false;
+        node.setMouseMovedInContentArea({ locationInHorizontalScrollbar, locationInVerticalScrollbar });
     }
 
     return true;
@@ -374,33 +346,34 @@ bool ArgumentCoder<ScrollingStateFrameScrollingNode>::decode(Decoder& decoder, S
     SCROLLING_NODE_DECODE(ScrollingStateNode::Property::MinLayoutViewportOrigin, FloatPoint, setMinLayoutViewportOrigin)
     SCROLLING_NODE_DECODE(ScrollingStateNode::Property::MaxLayoutViewportOrigin, FloatPoint, setMaxLayoutViewportOrigin)
     SCROLLING_NODE_DECODE(ScrollingStateNode::Property::OverrideVisualViewportSize, std::optional<FloatSize>, setOverrideVisualViewportSize)
+    SCROLLING_NODE_DECODE(ScrollingStateNode::Property::OverlayScrollbarsEnabled, bool, setOverlayScrollbarsEnabled)
 
     if (node.hasChangedProperty(ScrollingStateNode::Property::CounterScrollingLayer)) {
-        std::optional<GraphicsLayer::PlatformLayerID> layerID;
+        std::optional<PlatformLayerIdentifier> layerID;
         if (!decoder.decode(layerID))
             return false;
-        node.setCounterScrollingLayer(layerID.value_or(GraphicsLayer::PlatformLayerID { }));
+        node.setCounterScrollingLayer(layerID.value_or(PlatformLayerIdentifier { }));
     }
 
     if (node.hasChangedProperty(ScrollingStateNode::Property::InsetClipLayer)) {
-        std::optional<GraphicsLayer::PlatformLayerID> layerID;
+        std::optional<PlatformLayerIdentifier> layerID;
         if (!decoder.decode(layerID))
             return false;
-        node.setInsetClipLayer(layerID.value_or(GraphicsLayer::PlatformLayerID { }));
+        node.setInsetClipLayer(layerID.value_or(PlatformLayerIdentifier { }));
     }
 
     if (node.hasChangedProperty(ScrollingStateNode::Property::ContentShadowLayer)) {
-        std::optional<GraphicsLayer::PlatformLayerID> layerID;
+        std::optional<PlatformLayerIdentifier> layerID;
         if (!decoder.decode(layerID))
             return false;
-        node.setContentShadowLayer(layerID.value_or(GraphicsLayer::PlatformLayerID { }));
+        node.setContentShadowLayer(layerID.value_or(PlatformLayerIdentifier { }));
     }
 
     if (node.hasChangedProperty(ScrollingStateNode::Property::RootContentsLayer)) {
-        std::optional<GraphicsLayer::PlatformLayerID> layerID;
+        std::optional<PlatformLayerIdentifier> layerID;
         if (!decoder.decode(layerID))
             return false;
-        node.setRootContentsLayer(layerID.value_or(GraphicsLayer::PlatformLayerID { }));
+        node.setRootContentsLayer(layerID.value_or(PlatformLayerIdentifier { }));
     }
 
     return true;
@@ -510,81 +483,6 @@ bool ArgumentCoder<ScrollingStatePositionedNode>::decode(Decoder& decoder, Scrol
     return true;
 }
 
-
-void ArgumentCoder<RequestedScrollData>::encode(Encoder& encoder, const RequestedScrollData& scrollData)
-{
-    encoder << scrollData.requestType;
-    encoder << scrollData.scrollPosition;
-    encoder << scrollData.scrollType;
-    encoder << scrollData.clamping;
-    encoder << scrollData.animated;
-}
-
-bool ArgumentCoder<RequestedScrollData>::decode(Decoder& decoder, RequestedScrollData& scrollData)
-{
-    if (!decoder.decode(scrollData.requestType))
-        return false;
-
-    if (!decoder.decode(scrollData.scrollPosition))
-        return false;
-
-    if (!decoder.decode(scrollData.scrollType))
-        return false;
-
-    if (!decoder.decode(scrollData.clamping))
-        return false;
-
-    if (!decoder.decode(scrollData.animated))
-        return false;
-
-    return true;
-}
-
-void ArgumentCoder<SnapOffset<float>>::encode(Encoder& encoder, const SnapOffset<float>& offset)
-{
-    encoder << offset.offset;
-    encoder << offset.stop;
-    encoder << offset.hasSnapAreaLargerThanViewport;
-    encoder << offset.snapTargetID;
-    encoder << offset.snapAreaIndices;
-}
-
-bool ArgumentCoder<SnapOffset<float>>::decode(Decoder& decoder, SnapOffset<float>& offset)
-{
-    if (!decoder.decode(offset.offset))
-        return false;
-    if (!decoder.decode(offset.stop))
-        return false;
-    if (!decoder.decode(offset.hasSnapAreaLargerThanViewport))
-        return false;
-    if (!decoder.decode(offset.snapTargetID))
-        return false;
-    if (!decoder.decode(offset.snapAreaIndices))
-        return false;
-    return true;
-}
-
-void ArgumentCoder<FloatScrollSnapOffsetsInfo>::encode(Encoder& encoder, const FloatScrollSnapOffsetsInfo& info)
-{
-    encoder << info.horizontalSnapOffsets;
-    encoder << info.verticalSnapOffsets;
-    encoder << info.strictness;
-    encoder << info.snapAreas;
-}
-
-bool ArgumentCoder<FloatScrollSnapOffsetsInfo>::decode(Decoder& decoder, FloatScrollSnapOffsetsInfo& info)
-{
-    if (!decoder.decode(info.horizontalSnapOffsets))
-        return false;
-    if (!decoder.decode(info.verticalSnapOffsets))
-        return false;
-    if (!decoder.decode(info.strictness))
-        return false;
-    if (!decoder.decode(info.snapAreas))
-        return false;
-    return true;
-}
-
 namespace WebKit {
 
 static void encodeNodeAndDescendants(IPC::Encoder& encoder, const ScrollingStateNode& stateNode, int& encodedNodeCount)
@@ -684,9 +582,15 @@ bool RemoteScrollingCoordinatorTransaction::decode(IPC::Decoder& decoder)
         if (!decoder.decode(parentNodeID))
             return false;
 
-        m_scrollingStateTree->insertNode(nodeType, nodeID, parentNodeID, notFound);
+        auto createdNodeID = m_scrollingStateTree->insertNode(nodeType, nodeID, parentNodeID, notFound);
+        if (createdNodeID != nodeID)
+            return false;
+
         auto newNode = m_scrollingStateTree->stateNodeForID(nodeID);
         ASSERT(newNode);
+        if (newNode->nodeType() != nodeType)
+            return false;
+
         ASSERT(!parentNodeID || newNode->parent());
         
         switch (nodeType) {
@@ -759,10 +663,10 @@ static void dump(TextStream& ts, const ScrollingStateScrollingNode& node, bool c
     }
 
     if (!changedPropertiesOnly || node.hasChangedProperty(ScrollingStateNode::Property::ScrollContainerLayer))
-        ts.dumpProperty("scroll-container-layer", static_cast<GraphicsLayer::PlatformLayerID>(node.scrollContainerLayer()));
+        ts.dumpProperty("scroll-container-layer", static_cast<PlatformLayerIdentifier>(node.scrollContainerLayer()));
 
     if (!changedPropertiesOnly || node.hasChangedProperty(ScrollingStateNode::Property::ScrolledContentsLayer))
-        ts.dumpProperty("scrolled-contents-layer", static_cast<GraphicsLayer::PlatformLayerID>(node.scrolledContentsLayer()));
+        ts.dumpProperty("scrolled-contents-layer", static_cast<PlatformLayerIdentifier>(node.scrolledContentsLayer()));
 
     if (!changedPropertiesOnly || node.hasChangedProperty(ScrollingStateNode::Property::SnapOffsetsInfo)) {
         ts.dumpProperty("horizontal snap offsets", node.snapOffsetsInfo().horizontalSnapOffsets);
@@ -845,16 +749,16 @@ static void dump(TextStream& ts, const ScrollingStateFrameScrollingNode& node, b
         ts.dumpProperty("frame-scale-factor", node.frameScaleFactor());
 
     if (!changedPropertiesOnly || node.hasChangedProperty(ScrollingStateNode::Property::InsetClipLayer))
-        ts.dumpProperty("clip-inset-layer", static_cast<GraphicsLayer::PlatformLayerID>(node.insetClipLayer()));
+        ts.dumpProperty("clip-inset-layer", static_cast<PlatformLayerIdentifier>(node.insetClipLayer()));
 
     if (!changedPropertiesOnly || node.hasChangedProperty(ScrollingStateNode::Property::ContentShadowLayer))
-        ts.dumpProperty("content-shadow-layer", static_cast<GraphicsLayer::PlatformLayerID>(node.contentShadowLayer()));
+        ts.dumpProperty("content-shadow-layer", static_cast<PlatformLayerIdentifier>(node.contentShadowLayer()));
 
     if (!changedPropertiesOnly || node.hasChangedProperty(ScrollingStateNode::Property::HeaderLayer))
-        ts.dumpProperty("header-layer", static_cast<GraphicsLayer::PlatformLayerID>(node.headerLayer()));
+        ts.dumpProperty("header-layer", static_cast<PlatformLayerIdentifier>(node.headerLayer()));
 
     if (!changedPropertiesOnly || node.hasChangedProperty(ScrollingStateNode::Property::FooterLayer))
-        ts.dumpProperty("footer-layer", static_cast<GraphicsLayer::PlatformLayerID>(node.footerLayer()));
+        ts.dumpProperty("footer-layer", static_cast<PlatformLayerIdentifier>(node.footerLayer()));
 }
     
 static void dump(TextStream& ts, const ScrollingStateOverflowScrollingNode& node, bool changedPropertiesOnly)
@@ -894,7 +798,7 @@ static void dump(TextStream& ts, const ScrollingStateNode& node, bool changedPro
     ts.dumpProperty("type", node.nodeType());
 
     if (!changedPropertiesOnly || node.hasChangedProperty(ScrollingStateNode::Property::Layer))
-        ts.dumpProperty("layer", static_cast<GraphicsLayer::PlatformLayerID>(node.layer()));
+        ts.dumpProperty("layer", static_cast<PlatformLayerIdentifier>(node.layer()));
     
     switch (node.nodeType()) {
     case ScrollingNodeType::MainFrame:
