@@ -119,6 +119,7 @@ class Font;
 class FontPlatformData;
 class FragmentedSharedBuffer;
 class LightSource;
+class Path;
 class PaymentInstallmentConfiguration;
 class PixelBuffer;
 class ResourceError;
@@ -139,6 +140,9 @@ struct SoupNetworkProxySettings;
 struct TextRecognitionDataDetector;
 struct ViewportArguments;
 
+template <class>
+struct PathCommand;
+
 namespace DOMCacheEngine {
 struct Record;
 }
@@ -453,6 +457,12 @@ template<> struct ArgumentCoder<WebCore::Filter> {
     static std::optional<Ref<WebCore::Filter>> decode(Decoder&);
 };
 
+template<> struct ArgumentCoder<WebCore::Path> {
+    template<typename Encoder>
+    static void encode(Encoder&, const WebCore::Path&);
+    static std::optional<WebCore::Path> decode(Decoder&);
+};
+
 #if ENABLE(DATA_DETECTION)
 
 template<> struct ArgumentCoder<WebCore::DataDetectorElementInfo> {
@@ -505,14 +515,14 @@ template<> struct ArgumentCoder<WebCore::PixelBuffer> {
 
 #if PLATFORM(COCOA) && ENABLE(GPU_PROCESS) && ENABLE(WEBGL)
 
-template<> struct ArgumentCoder<WebCore::GraphicsContextGL::ExternalImageSourceIOSurfaceHandle> {
-    static void encode(Encoder&, const WebCore::GraphicsContextGL::ExternalImageSourceIOSurfaceHandle&);
-    static std::optional<WebCore::GraphicsContextGL::ExternalImageSourceIOSurfaceHandle> decode(Decoder&);
+template<> struct ArgumentCoder<WebCore::GraphicsContextGL::EGLImageSourceIOSurfaceHandle> {
+    static void encode(Encoder&, const WebCore::GraphicsContextGL::EGLImageSourceIOSurfaceHandle&);
+    static std::optional<WebCore::GraphicsContextGL::EGLImageSourceIOSurfaceHandle> decode(Decoder&);
 };
 
-template<> struct ArgumentCoder<WebCore::GraphicsContextGL::ExternalImageSourceMTLSharedTextureHandle> {
-    static void encode(Encoder&, const WebCore::GraphicsContextGL::ExternalImageSourceMTLSharedTextureHandle&);
-    static std::optional<WebCore::GraphicsContextGL::ExternalImageSourceMTLSharedTextureHandle> decode(Decoder&);
+template<> struct ArgumentCoder<WebCore::GraphicsContextGL::EGLImageSourceMTLSharedTextureHandle> {
+    static void encode(Encoder&, const WebCore::GraphicsContextGL::EGLImageSourceMTLSharedTextureHandle&);
+    static std::optional<WebCore::GraphicsContextGL::EGLImageSourceMTLSharedTextureHandle> decode(Decoder&);
 };
 
 #endif
