@@ -54,7 +54,7 @@ HTTPCookieStore::HTTPCookieStore(WebKit::WebsiteDataStore& websiteDataStore)
 
 HTTPCookieStore::~HTTPCookieStore()
 {
-    ASSERT(m_observers.computesEmpty());
+    ASSERT(m_observers.isEmptyIgnoringNullReferences());
 }
 
 void HTTPCookieStore::filterAppBoundCookies(Vector<WebCore::Cookie>&& cookies, CompletionHandler<void(Vector<WebCore::Cookie>&&)>&& completionHandler)
@@ -64,7 +64,7 @@ void HTTPCookieStore::filterAppBoundCookies(Vector<WebCore::Cookie>&& cookies, C
         return completionHandler({ });
     m_owningDataStore->getAppBoundDomains([cookies = WTFMove(cookies), completionHandler = WTFMove(completionHandler)] (auto& domains) mutable {
         Vector<WebCore::Cookie> appBoundCookies;
-        if (!domains.isEmpty() && !isFullWebBrowser()) {
+        if (!domains.isEmpty() && !isFullWebBrowserOrRunningTest()) {
             for (auto& cookie : WTFMove(cookies)) {
                 if (domains.contains(WebCore::RegistrableDomain::uncheckedCreateFromHost(cookie.domain)))
                     appBoundCookies.append(WTFMove(cookie));
@@ -144,7 +144,7 @@ void HTTPCookieStore::deleteCookiesForHostnames(const Vector<WTF::String>& hostn
 void HTTPCookieStore::setHTTPCookieAcceptPolicy(WebCore::HTTPCookieAcceptPolicy policy, CompletionHandler<void()>&& completionHandler)
 {
     if (auto* networkProcess = networkProcessLaunchingIfNecessary())
-        networkProcess->sendWithAsyncReply(Messages::WebCookieManager::SetHTTPCookieAcceptPolicy(policy), WTFMove(completionHandler));
+        networkProcess->sendWithAsyncReply(Messages::WebCookieManager::SetHTTPCookieAcceptPolicy(m_sessionID, policy), WTFMove(completionHandler));
     else
         completionHandler();
 }
@@ -167,7 +167,7 @@ void HTTPCookieStore::flushCookies(CompletionHandler<void()>&& completionHandler
 
 void HTTPCookieStore::registerObserver(Observer& observer)
 {
-    bool wasObserving = !m_observers.computesEmpty();
+    bool wasObserving = !m_observers.isEmptyIgnoringNullReferences();
     m_observers.add(observer);
     if (wasObserving)
         return;
@@ -179,7 +179,7 @@ void HTTPCookieStore::registerObserver(Observer& observer)
 void HTTPCookieStore::unregisterObserver(Observer& observer)
 {
     m_observers.remove(observer);
-    if (!m_observers.computesEmpty())
+    if (!m_observers.isEmptyIgnoringNullReferences())
         return;
 
     if (auto* networkProcess = networkProcessIfExists())
