@@ -26,7 +26,6 @@
 #include "config.h"
 #include "AcceleratedBackingStoreDMABuf.h"
 
-#if USE(GBM)
 #include "AcceleratedBackingStoreDMABufMessages.h"
 #include "AcceleratedSurfaceDMABufMessages.h"
 #include "LayerTreeContext.h"
@@ -38,9 +37,12 @@
 #include <WebCore/IntRect.h>
 #include <WebCore/PlatformDisplay.h>
 #include <epoxy/egl.h>
-#include <gbm.h>
 #include <wtf/glib/GUniquePtr.h>
 
+#if USE(GBM)
+#include <gbm.h>
+#endif
+
 #if PLATFORM(X11) && USE(GTK4)
 #include <gdk/x11/gdkx.h>
 #endif
@@ -92,7 +94,7 @@ AcceleratedBackingStoreDMABuf::RenderSource::RenderSource(const WebCore::IntSize
 {
 }
 
-AcceleratedBackingStoreDMABuf::Texture::Texture(GdkGLContext* glContext, const UnixFileDescriptor& backFD, const UnixFileDescriptor& frontFD, const WebCore::IntSize& size, uint32_t format, uint32_t offset, uint32_t stride, uint64_t modifier, float deviceScaleFactor)
+AcceleratedBackingStoreDMABuf::Texture::Texture(GdkGLContext* glContext, const UnixFileDescriptor& backFD, const UnixFileDescriptor& frontFD, const UnixFileDescriptor& displayFD, const WebCore::IntSize& size, uint32_t format, uint32_t offset, uint32_t stride, uint64_t modifier, float deviceScaleFactor)
     : RenderSource(size, deviceScaleFactor)
     , m_context(glContext)
 {
@@ -122,14 +124,18 @@ AcceleratedBackingStoreDMABuf::Texture::Texture(GdkGLContext* glContext, const U
 
     m_backImage = createImage(backFD);
     m_frontImage = createImage(frontFD);
-    if (!m_backImage || !m_frontImage) {
-        WTFLogAlways("Failed to create EGL image from DMABufs with file descriptors %d and %d", backFD.value(), frontFD.value());
+    m_displayImage = createImage(displayFD);
+    if (!m_backImage || !m_frontImage || !m_displayImage) {
+        WTFLogAlways("Failed to create EGL image from DMABufs with file descriptors %d, %d and %d", backFD.value(), frontFD.value(), displayFD.value());
         if (m_backImage)
             display.destroyEGLImage(m_backImage);
         m_backImage = nullptr;
         if (m_frontImage)
             display.destroyEGLImage(m_frontImage);
         m_frontImage = nullptr;
+        if (m_displayImage)
+            display.destroyEGLImage(m_displayImage);
+        m_displayImage = nullptr;
         return;
     }
 
@@ -156,15 +162,22 @@ AcceleratedBackingStoreDMABuf::Texture::~Texture()
         display.destroyEGLImage(m_backImage);
     if (m_frontImage)
         display.destroyEGLImage(m_frontImage);
+    if (m_displayImage)
+        display.destroyEGLImage(m_displayImage);
     if (m_textureID)
         glDeleteTextures(1, &m_textureID);
 }
 
-void AcceleratedBackingStoreDMABuf::Texture::swap()
+void AcceleratedBackingStoreDMABuf::Texture::frame()
 {
     std::swap(m_backImage, m_frontImage);
 }
 
+void AcceleratedBackingStoreDMABuf::Texture::willDisplayFrame()
+{
+    std::swap(m_frontImage, m_displayImage);
+}
+
 bool AcceleratedBackingStoreDMABuf::Texture::prepareForRendering()
 {
     if (!m_textureID)
@@ -172,10 +185,11 @@ bool AcceleratedBackingStoreDMABuf::Texture::prepareForRendering()
 
     gdk_gl_context_make_current(m_context.get());
     glBindTexture(GL_TEXTURE_2D, m_textureID);
-    glEGLImageTargetTexture2DOES(GL_TEXTURE_2D, m_frontImage);
+    glEGLImageTargetTexture2DOES(GL_TEXTURE_2D, m_displayImage);
 #if USE(GTK4)
     m_textureIndex = !m_textureIndex;
 #endif
+
     return true;
 }
 
@@ -202,7 +216,8 @@ void AcceleratedBackingStoreDMABuf::Texture::paint(GtkWidget* widget, cairo_t* c
 }
 #endif
 
-AcceleratedBackingStoreDMABuf::Surface::Surface(const UnixFileDescriptor& backFD, const UnixFileDescriptor& frontFD, const WebCore::IntSize& size, uint32_t format, uint32_t offset, uint32_t stride, float deviceScaleFactor)
+#if USE(GBM)
+AcceleratedBackingStoreDMABuf::Surface::Surface(const UnixFileDescriptor& backFD, const UnixFileDescriptor& frontFD, const UnixFileDescriptor& displayFD, const WebCore::IntSize& size, uint32_t format, uint32_t offset, uint32_t stride, float deviceScaleFactor)
     : RenderSource(size, deviceScaleFactor)
 {
     auto* device = WebCore::PlatformDisplay::sharedDisplay().gbmDevice();
@@ -224,12 +239,23 @@ AcceleratedBackingStoreDMABuf::Surface::Surface(const UnixFileDescriptor& backFD
         gbm_bo_destroy(m_backBuffer);
         m_backBuffer = nullptr;
     }
+    fdData.fd = displayFD.value();
+    m_displayBuffer = gbm_bo_import(device, GBM_BO_IMPORT_FD, &fdData, GBM_BO_USE_RENDERING);
+    if (!m_displayBuffer) {
+        WTFLogAlways("Failed to import DMABuf with file descriptor %d", fdData.fd);
+        gbm_bo_destroy(m_backBuffer);
+        m_backBuffer = nullptr;
+        gbm_bo_destroy(m_frontBuffer);
+        m_frontBuffer = nullptr;
+    }
 }
+#endif
 
-AcceleratedBackingStoreDMABuf::Surface::Surface(RefPtr<ShareableBitmap>& backBitmap, RefPtr<ShareableBitmap>& frontBitmap, float deviceScaleFactor)
+AcceleratedBackingStoreDMABuf::Surface::Surface(RefPtr<ShareableBitmap>& backBitmap, RefPtr<ShareableBitmap>& frontBitmap, RefPtr<ShareableBitmap>& displayBitmap, float deviceScaleFactor)
     : RenderSource(backBitmap ? backBitmap->size() : WebCore::IntSize(), deviceScaleFactor)
     , m_backBitmap(backBitmap)
     , m_frontBitmap(frontBitmap)
+    , m_displayBitmap(displayBitmap)
 {
 }
 
@@ -237,12 +263,17 @@ AcceleratedBackingStoreDMABuf::Surface::~Surface()
 {
     m_surface = nullptr;
 
+#if USE(GBM)
     if (m_backBuffer)
         gbm_bo_destroy(m_backBuffer);
     if (m_frontBuffer)
         gbm_bo_destroy(m_frontBuffer);
+    if (m_displayBuffer)
+        gbm_bo_destroy(m_displayBuffer);
+#endif
 }
 
+#if USE(GBM)
 RefPtr<cairo_surface_t> AcceleratedBackingStoreDMABuf::Surface::map(struct gbm_bo* buffer) const
 {
     if (!buffer)
@@ -269,6 +300,7 @@ RefPtr<cairo_surface_t> AcceleratedBackingStoreDMABuf::Surface::map(struct gbm_b
     });
     return surface;
 }
+#endif
 
 RefPtr<cairo_surface_t> AcceleratedBackingStoreDMABuf::Surface::map(RefPtr<ShareableBitmap>& bitmap) const
 {
@@ -278,20 +310,34 @@ RefPtr<cairo_surface_t> AcceleratedBackingStoreDMABuf::Surface::map(RefPtr<Share
     return bitmap->createCairoSurface();
 }
 
-void AcceleratedBackingStoreDMABuf::Surface::swap()
+void AcceleratedBackingStoreDMABuf::Surface::frame()
 {
     if (m_backBitmap && m_frontBitmap)
         std::swap(m_backBitmap, m_frontBitmap);
+#if USE(GBM)
     else
         std::swap(m_backBuffer, m_frontBuffer);
+#endif
+}
+
+void AcceleratedBackingStoreDMABuf::Surface::willDisplayFrame()
+{
+    if (m_frontBitmap && m_displayBitmap)
+        std::swap(m_frontBitmap, m_displayBitmap);
+#if USE(GBM)
+    else
+        std::swap(m_frontBuffer, m_displayBuffer);
+#endif
 }
 
 bool AcceleratedBackingStoreDMABuf::Surface::prepareForRendering()
 {
-    if (m_backBitmap && m_frontBitmap)
-        m_surface = map(m_frontBitmap);
+    if (m_displayBitmap)
+        m_surface = map(m_displayBitmap);
+#if USE(GBM)
     else
-        m_surface = map(m_frontBuffer);
+        m_surface = map(m_displayBuffer);
+#endif
 
     if (m_surface) {
         cairo_surface_mark_dirty(m_surface.get());
@@ -330,11 +376,12 @@ void AcceleratedBackingStoreDMABuf::Surface::paint(GtkWidget*, cairo_t* cr, cons
 }
 #endif
 
-void AcceleratedBackingStoreDMABuf::configure(UnixFileDescriptor&& backFD, UnixFileDescriptor&& frontFD, const WebCore::IntSize& size, uint32_t format, uint32_t offset, uint32_t stride, uint64_t modifier)
+void AcceleratedBackingStoreDMABuf::configure(UnixFileDescriptor&& backFD, UnixFileDescriptor&& frontFD, UnixFileDescriptor&& displayFD, const WebCore::IntSize& size, uint32_t format, uint32_t offset, uint32_t stride, uint64_t modifier)
 {
     m_isSoftwareRast = false;
     m_surface.backFD = WTFMove(backFD);
     m_surface.frontFD = WTFMove(frontFD);
+    m_surface.displayFD = WTFMove(displayFD);
     m_surface.size = size;
     m_surface.format = format;
     m_surface.offset = offset;
@@ -344,11 +391,12 @@ void AcceleratedBackingStoreDMABuf::configure(UnixFileDescriptor&& backFD, UnixF
         m_pendingSource = createSource();
 }
 
-void AcceleratedBackingStoreDMABuf::configureSHM(ShareableBitmap::Handle&& backBufferHandle, ShareableBitmap::Handle&& frontBufferHandle)
+void AcceleratedBackingStoreDMABuf::configureSHM(ShareableBitmap::Handle&& backBufferHandle, ShareableBitmap::Handle&& frontBufferHandle, ShareableBitmap::Handle&& displayBufferHandle)
 {
     m_isSoftwareRast = true;
     m_surface.backBitmap = ShareableBitmap::create(WTFMove(backBufferHandle), SharedMemory::Protection::ReadOnly);
     m_surface.frontBitmap = ShareableBitmap::create(WTFMove(frontBufferHandle), SharedMemory::Protection::ReadOnly);
+    m_surface.displayBitmap = ShareableBitmap::create(WTFMove(displayBufferHandle), SharedMemory::Protection::ReadOnly);
     if (gtk_widget_get_realized(m_webPage.viewWidget()))
         m_pendingSource = createSource();
 }
@@ -356,13 +404,15 @@ void AcceleratedBackingStoreDMABuf::configureSHM(ShareableBitmap::Handle&& backB
 std::unique_ptr<AcceleratedBackingStoreDMABuf::RenderSource> AcceleratedBackingStoreDMABuf::createSource()
 {
     if (m_isSoftwareRast)
-        return makeUnique<Surface>(m_surface.backBitmap, m_surface.frontBitmap, m_webPage.deviceScaleFactor());
+        return makeUnique<Surface>(m_surface.backBitmap, m_surface.frontBitmap, m_surface.displayBitmap, m_webPage.deviceScaleFactor());
 
+#if USE(GBM)
     if (!WebCore::PlatformDisplay::sharedDisplay().gtkEGLDisplay())
-        return makeUnique<Surface>(m_surface.backFD, m_surface.frontFD, m_surface.size, m_surface.format, m_surface.offset, m_surface.stride, m_webPage.deviceScaleFactor());
+        return makeUnique<Surface>(m_surface.backFD, m_surface.frontFD, m_surface.displayFD, m_surface.size, m_surface.format, m_surface.offset, m_surface.stride, m_webPage.deviceScaleFactor());
+#endif
 
     ensureGLContext();
-    return makeUnique<Texture>(m_gdkGLContext.get(), m_surface.backFD, m_surface.frontFD, m_surface.size, m_surface.format, m_surface.offset, m_surface.stride, m_surface.modifier, m_webPage.deviceScaleFactor());
+    return makeUnique<Texture>(m_gdkGLContext.get(), m_surface.backFD, m_surface.frontFD, m_surface.displayFD, m_surface.size, m_surface.format, m_surface.offset, m_surface.stride, m_surface.modifier, m_webPage.deviceScaleFactor());
 }
 
 void AcceleratedBackingStoreDMABuf::frame()
@@ -381,14 +431,27 @@ void AcceleratedBackingStoreDMABuf::frame()
         std::swap(m_surface.backFD, m_surface.frontFD);
 
     if (m_committedSource) {
-        m_committedSource->swap();
-        if (m_committedSource->prepareForRendering()) {
-            gtk_widget_queue_draw(m_webPage.viewWidget());
-            return;
-        }
+        m_committedSource->frame();
+        gtk_widget_queue_draw(m_webPage.viewWidget());
+    } else {
+        if (m_isSoftwareRast)
+            std::swap(m_surface.frontBitmap, m_surface.displayBitmap);
+        else
+            std::swap(m_surface.frontFD, m_surface.displayFD);
+        frameDone();
     }
+}
 
-    frameDone();
+void AcceleratedBackingStoreDMABuf::willDisplayFrame()
+{
+    ASSERT(m_frameCompletionHandler);
+    if (m_isSoftwareRast)
+        std::swap(m_surface.frontBitmap, m_surface.displayBitmap);
+    else
+        std::swap(m_surface.frontFD, m_surface.displayFD);
+
+    if (m_committedSource)
+        m_committedSource->willDisplayFrame();
 }
 
 void AcceleratedBackingStoreDMABuf::frameDone()
@@ -472,18 +535,30 @@ void AcceleratedBackingStoreDMABuf::update(const LayerTreeContext& context)
 #if USE(GTK4)
 void AcceleratedBackingStoreDMABuf::snapshot(GtkSnapshot* gtkSnapshot)
 {
+    if (m_frameCompletionHandler)
+        willDisplayFrame();
+
     if (!m_committedSource)
         return;
 
+    if (m_frameCompletionHandler)
+        m_committedSource->prepareForRendering();
+
     m_committedSource->snapshot(gtkSnapshot);
     frameDone();
 }
 #else
 bool AcceleratedBackingStoreDMABuf::paint(cairo_t* cr, const WebCore::IntRect& clipRect)
 {
+    if (m_frameCompletionHandler)
+        willDisplayFrame();
+
     if (!m_committedSource)
         return false;
 
+    if (m_frameCompletionHandler)
+        m_committedSource->prepareForRendering();
+
     m_committedSource->paint(m_webPage.viewWidget(), cr, clipRect);
     frameDone();
 
@@ -492,5 +567,3 @@ bool AcceleratedBackingStoreDMABuf::paint(cairo_t* cr, const WebCore::IntRect& c
 #endif
 
 } // namespace WebKit
-
-#endif // USE(GBM)
