@@ -65,10 +65,13 @@ public:
 
     void clearBackend();
     void backingStoreWillChange();
-    void didCreateImageBufferBackend(ImageBufferBackendHandle&&);
-
     std::unique_ptr<WebCore::SerializedImageBuffer> sinkIntoSerializedImageBuffer() final;
 
+    void didReceiveMessage(IPC::Connection&, IPC::Decoder&);
+
+    // Messages
+    void didCreateBackend(ImageBufferBackendHandle&&);
+
 private:
     RemoteImageBufferProxy(const WebCore::ImageBufferBackend::Parameters&, const WebCore::ImageBufferBackend::Info&, RemoteRenderingBackendProxy&, std::unique_ptr<WebCore::ImageBufferBackend>&& = nullptr, WebCore::RenderingResourceIdentifier = WebCore::RenderingResourceIdentifier::generate());
 
@@ -101,6 +104,8 @@ private:
 
     void assertDispatcherIsCurrent() const;
 
+    IPC::StreamClientConnection& streamConnection() const;
+
     RefPtr<RemoteImageBufferProxyFlushFence> m_pendingFlush;
     WeakPtr<RemoteRenderingBackendProxy> m_remoteRenderingBackendProxy;
     RemoteDisplayListRecorderProxy m_remoteDisplayList;
