@@ -141,16 +141,19 @@ MediaPlayerPrivateRemote::~MediaPlayerPrivateRemote()
     if (m_audioSourceProvider)
         m_audioSourceProvider->close();
 #endif
+
+    for (auto& request : std::exchange(m_layerHostingContextIDRequests, { }))
+        request({ });
 }
 
 void MediaPlayerPrivateRemote::prepareForPlayback(bool privateMode, MediaPlayer::Preload preload, bool preservesPitch, bool prepare)
 {
-    RefPtr player = m_player.get();
+    auto player = m_player.get();
     if (!player)
         return;
 
     auto scale = player->playerContentsScale();
-    auto preferredDynamicRangeMode = m_player->preferredDynamicRangeMode();
+    auto preferredDynamicRangeMode = player->preferredDynamicRangeMode();
     auto presentationSize = player->presentationSize();
     auto pitchCorrectionAlgorithm = player->pitchCorrectionAlgorithm();
 
@@ -184,7 +187,7 @@ void MediaPlayerPrivateRemote::load(const URL& url, const ContentType& contentTy
         if (!createExtension()) {
             WTFLogAlways("Unable to create sandbox extension handle for GPUProcess url.\n");
             m_cachedState.networkState = MediaPlayer::NetworkState::FormatError;
-            if (RefPtr player = m_player.get())
+            if (auto player = m_player.get())
                 player->networkStateChanged();
             return;
         }
@@ -192,11 +195,11 @@ void MediaPlayerPrivateRemote::load(const URL& url, const ContentType& contentTy
         sandboxExtensionHandle = WTFMove(handle);
     }
 
-    connection().sendWithAsyncReply(Messages::RemoteMediaPlayerProxy::Load(url, sandboxExtensionHandle, contentType, keySystem, m_player->requiresRemotePlayback()), [weakThis = WeakPtr { *this }, this](auto&& configuration) {
+    connection().sendWithAsyncReply(Messages::RemoteMediaPlayerProxy::Load(url, sandboxExtensionHandle, contentType, keySystem, m_player.get()->requiresRemotePlayback()), [weakThis = WeakPtr { *this }, this](auto&& configuration) {
         if (!weakThis)
             return;
 
-        RefPtr player = m_player.get();
+        auto player = m_player.get();
         if (!player)
             return;
 
@@ -329,21 +332,21 @@ MediaPlayer::MovieLoadType MediaPlayerPrivateRemote::movieLoadType() const
 void MediaPlayerPrivateRemote::networkStateChanged(RemoteMediaPlayerState&& state)
 {
     updateCachedState(WTFMove(state));
-    if (RefPtr player = m_player.get())
+    if (auto player = m_player.get())
         player->networkStateChanged();
 }
 
 void MediaPlayerPrivateRemote::setReadyState(MediaPlayer::ReadyState readyState)
 {
     m_cachedState.readyState = readyState;
-    if (RefPtr player = m_player.get())
+    if (auto player = m_player.get())
         player->readyStateChanged();
 }
 
 void MediaPlayerPrivateRemote::readyStateChanged(RemoteMediaPlayerState&& state)
 {
     updateCachedState(WTFMove(state));
-    if (RefPtr player = m_player.get()) {
+    if (auto player = m_player.get()) {
         player->readyStateChanged();
         checkAcceleratedRenderingState();
     }
@@ -352,14 +355,14 @@ void MediaPlayerPrivateRemote::readyStateChanged(RemoteMediaPlayerState&& state)
 void MediaPlayerPrivateRemote::volumeChanged(double volume)
 {
     m_volume = volume;
-    if (RefPtr player = m_player.get())
+    if (auto player = m_player.get())
         player->volumeChanged(volume);
 }
 
 void MediaPlayerPrivateRemote::muteChanged(bool muted)
 {
     m_muted = muted;
-    if (RefPtr player = m_player.get())
+    if (auto player = m_player.get())
         player->muteChanged(muted);
 }
 
@@ -367,21 +370,21 @@ void MediaPlayerPrivateRemote::timeChanged(RemoteMediaPlayerState&& state)
 {
     m_seeking = false;
     updateCachedState(WTFMove(state));
-    if (RefPtr player = m_player.get())
+    if (auto player = m_player.get())
         player->timeChanged();
 }
 
 void MediaPlayerPrivateRemote::durationChanged(RemoteMediaPlayerState&& state)
 {
     updateCachedState(WTFMove(state));
-    if (RefPtr player = m_player.get())
+    if (auto player = m_player.get())
         player->durationChanged();
 }
 
 void MediaPlayerPrivateRemote::rateChanged(double rate)
 {
     m_rate = rate;
-    if (RefPtr player = m_player.get())
+    if (auto player = m_player.get())
         player->rateChanged();
 }
 
@@ -390,28 +393,28 @@ void MediaPlayerPrivateRemote::playbackStateChanged(bool paused, MediaTime&& med
     m_cachedState.paused = paused;
     m_cachedMediaTime = mediaTime;
     m_cachedMediaTimeQueryTime = wallTime;
-    if (RefPtr player = m_player.get())
+    if (auto player = m_player.get())
         player->playbackStateChanged();
 }
 
 void MediaPlayerPrivateRemote::engineFailedToLoad(int64_t platformErrorCode)
 {
     m_platformErrorCode = platformErrorCode;
-    if (RefPtr player = m_player.get())
+    if (auto player = m_player.get())
         player->remoteEngineFailedToLoad();
 }
 
 void MediaPlayerPrivateRemote::characteristicChanged(RemoteMediaPlayerState&& state)
 {
     updateCachedState(WTFMove(state));
-    if (RefPtr player = m_player.get())
+    if (auto player = m_player.get())
         player->characteristicChanged();
 }
 
 void MediaPlayerPrivateRemote::sizeChanged(WebCore::FloatSize naturalSize)
 {
     m_cachedState.naturalSize = naturalSize;
-    if (RefPtr player = m_player.get())
+    if (auto player = m_player.get())
         player->sizeChanged();
 }
 
@@ -427,7 +430,7 @@ void MediaPlayerPrivateRemote::currentTimeChanged(const MediaTime& mediaTime, co
 
     if (reverseJump
         || (timeIsProgressing != m_timeIsProgressing && m_cachedMediaTime != oldCachedTime && !m_cachedState.paused)) {
-        if (RefPtr player = m_player.get())
+        if (auto player = m_player.get())
             player->timeChanged();
     }
 }
@@ -435,14 +438,14 @@ void MediaPlayerPrivateRemote::currentTimeChanged(const MediaTime& mediaTime, co
 void MediaPlayerPrivateRemote::firstVideoFrameAvailable()
 {
     ALWAYS_LOG(LOGIDENTIFIER);
-    if (RefPtr player = m_player.get())
+    if (auto player = m_player.get())
         player->firstVideoFrameAvailable();
 }
 
 void MediaPlayerPrivateRemote::renderingModeChanged()
 {
     ALWAYS_LOG(LOGIDENTIFIER);
-    if (RefPtr player = m_player.get())
+    if (auto player = m_player.get())
         player->renderingModeChanged();
 }
 
@@ -473,8 +476,8 @@ bool MediaPlayerPrivateRemote::supportsAcceleratedRendering() const
 
 void MediaPlayerPrivateRemote::acceleratedRenderingStateChanged()
 {
-    if (m_player) {
-        m_renderingCanBeAccelerated = m_player->renderingCanBeAccelerated();
+    if (auto player = m_player.get()) {
+        m_renderingCanBeAccelerated = player->renderingCanBeAccelerated();
         connection().send(Messages::RemoteMediaPlayerProxy::AcceleratedRenderingStateChanged(m_renderingCanBeAccelerated), m_id);
     }
     renderingModeChanged();
@@ -482,8 +485,8 @@ void MediaPlayerPrivateRemote::acceleratedRenderingStateChanged()
 
 void MediaPlayerPrivateRemote::checkAcceleratedRenderingState()
 {
-    if (m_player) {
-        bool renderingCanBeAccelerated = m_player->renderingCanBeAccelerated();
+    if (auto player = m_player.get()) {
+        bool renderingCanBeAccelerated = player->renderingCanBeAccelerated();
         if (m_renderingCanBeAccelerated != renderingCanBeAccelerated)
             acceleratedRenderingStateChanged();
     }
@@ -600,8 +603,8 @@ void MediaPlayerPrivateRemote::addRemoteAudioTrack(TrackPrivateRemoteIdentifier
         return;
 #endif
 
-    if (RefPtr player = m_player.get())
-        m_player->addAudioTrack(addResult.iterator->value);
+    if (auto player = m_player.get())
+        player->addAudioTrack(addResult.iterator->value);
 }
 
 void MediaPlayerPrivateRemote::removeRemoteAudioTrack(TrackPrivateRemoteIdentifier identifier)
@@ -609,7 +612,7 @@ void MediaPlayerPrivateRemote::removeRemoteAudioTrack(TrackPrivateRemoteIdentifi
     ASSERT(m_audioTracks.contains(identifier));
 
     if (auto* track = m_audioTracks.get(identifier)) {
-        if (RefPtr player = m_player.get())
+        if (auto player = m_player.get())
             player->removeAudioTrack(*track);
         m_audioTracks.remove(identifier);
     }
@@ -638,7 +641,7 @@ void MediaPlayerPrivateRemote::addRemoteTextTrack(TrackPrivateRemoteIdentifier i
         return;
 #endif
 
-    if (RefPtr player = m_player.get())
+    if (auto player = m_player.get())
         player->addTextTrack(addResult.iterator->value);
 }
 
@@ -647,7 +650,7 @@ void MediaPlayerPrivateRemote::removeRemoteTextTrack(TrackPrivateRemoteIdentifie
     ASSERT(m_textTracks.contains(identifier));
 
     if (auto* track = m_textTracks.get(identifier)) {
-        if (RefPtr player = m_player.get())
+        if (auto player = m_player.get())
             player->removeTextTrack(*track);
         m_textTracks.remove(identifier);
     }
@@ -758,7 +761,7 @@ void MediaPlayerPrivateRemote::addRemoteVideoTrack(TrackPrivateRemoteIdentifier
         return;
 #endif
 
-    if (RefPtr player = m_player.get())
+    if (auto player = m_player.get())
         player->addVideoTrack(addResult.iterator->value);
 }
 
@@ -767,7 +770,7 @@ void MediaPlayerPrivateRemote::removeRemoteVideoTrack(TrackPrivateRemoteIdentifi
     ASSERT(m_videoTracks.contains(identifier));
 
     if (auto* track = m_videoTracks.get(identifier)) {
-        if (RefPtr player = m_player.get())
+        if (auto player = m_player.get())
             player->removeVideoTrack(*track);
         m_videoTracks.remove(identifier);
     }
@@ -791,7 +794,7 @@ void MediaPlayerPrivateRemote::load(const URL& url, const ContentType& contentTy
             if (!weakThis)
                 return;
 
-            RefPtr player = m_player.get();
+            auto player = m_player.get();
             if (!player)
                 return;
 
@@ -807,7 +810,7 @@ void MediaPlayerPrivateRemote::load(const URL& url, const ContentType& contentTy
         if (!weakThis)
             return;
 
-        RefPtr player = m_player.get();
+        auto player = m_player.get();
         if (!player)
             return;
 
@@ -824,7 +827,7 @@ void MediaPlayerPrivateRemote::load(MediaStreamPrivate&)
         if (!weakThis)
             return;
 
-        RefPtr player = m_player.get();
+        auto player = m_player.get();
         if (!player)
             return;
 
@@ -884,7 +887,7 @@ void MediaPlayerPrivateRemote::setVideoFullscreenMode(MediaPlayer::VideoFullscre
 
 void MediaPlayerPrivateRemote::videoFullscreenStandbyChanged()
 {
-    RefPtr player = m_player.get();
+    auto player = m_player.get();
     if (!player)
         return;
 
@@ -1036,7 +1039,7 @@ RefPtr<WebCore::VideoFrame> MediaPlayerPrivateRemote::videoFrameForCurrentTime()
         return { };
 
     auto sendResult = connection().sendSync(Messages::RemoteMediaPlayerProxy::VideoFrameForCurrentTimeIfChanged(), m_id);
-    if (!sendResult)
+    if (!sendResult.succeeded())
         return nullptr;
 
     auto [result, changed] = sendResult.takeReply();
@@ -1095,7 +1098,7 @@ void MediaPlayerPrivateRemote::setWirelessVideoPlaybackDisabled(bool disabled)
 void MediaPlayerPrivateRemote::currentPlaybackTargetIsWirelessChanged(bool isCurrentPlaybackTargetWireless)
 {
     m_isCurrentPlaybackTargetWireless = isCurrentPlaybackTargetWireless;
-    if (RefPtr player = m_player.get())
+    if (auto player = m_player.get())
         player->currentPlaybackTargetIsWirelessChanged(isCurrentPlaybackTargetWireless);
 }
 
@@ -1222,7 +1225,7 @@ void MediaPlayerPrivateRemote::keyAdded()
 
 void MediaPlayerPrivateRemote::mediaPlayerKeyNeeded(IPC::DataReference&& message)
 {
-    if (RefPtr player = m_player.get())
+    if (auto player = m_player.get())
         player->keyNeeded(SharedBuffer::create(message));
 }
 #endif
@@ -1249,14 +1252,14 @@ void MediaPlayerPrivateRemote::attemptToDecryptWithInstance(CDMInstance& instanc
 void MediaPlayerPrivateRemote::waitingForKeyChanged(bool waitingForKey)
 {
     m_waitingForKey = waitingForKey;
-    if (RefPtr player = m_player.get())
+    if (auto player = m_player.get())
         player->waitingForKeyChanged();
 }
 
 void MediaPlayerPrivateRemote::initializationDataEncountered(const String& initDataType, IPC::DataReference&& initData)
 {
     auto initDataBuffer = ArrayBuffer::create(initData.data(), initData.size());
-    if (RefPtr player = m_player.get())
+    if (auto player = m_player.get())
         player->initializationDataEncountered(initDataType, WTFMove(initDataBuffer));
 }
 
@@ -1475,20 +1478,20 @@ void MediaPlayerPrivateRemote::removeResource(RemoteMediaResourceIdentifier remo
 
 void MediaPlayerPrivateRemote::resourceNotSupported()
 {
-    if (RefPtr player = m_player.get())
+    if (auto player = m_player.get())
         player->resourceNotSupported();
 }
 
 void MediaPlayerPrivateRemote::activeSourceBuffersChanged()
 {
-    if (RefPtr player = m_player.get())
+    if (auto player = m_player.get())
         player->activeSourceBuffersChanged();
 }
 
 #if PLATFORM(IOS_FAMILY)
 void MediaPlayerPrivateRemote::getRawCookies(const URL& url, WebCore::MediaPlayerClient::GetRawCookiesCallback&& completionHandler) const
 {
-    if (RefPtr player = m_player.get())
+    if (auto player = m_player.get())
         player->getRawCookies(url, WTFMove(completionHandler));
 }
 #endif
@@ -1500,6 +1503,20 @@ WTFLogChannel& MediaPlayerPrivateRemote::logChannel() const
 }
 #endif
 
+void MediaPlayerPrivateRemote::requestHostingContextID(LayerHostingContextIDCallback&& completionHandler)
+{
+    if (m_layerHostingContextID) {
+        completionHandler(m_layerHostingContextID);
+        return;
+    }
+
+    m_layerHostingContextIDRequests.append(WTFMove(completionHandler));
+    connection().sendWithAsyncReply(Messages::RemoteMediaPlayerProxy::RequestHostingContextID(), [weakThis = WeakPtr { this }] (auto contextID) {
+        if (weakThis)
+            weakThis->setLayerHostingContextID(contextID);
+    }, m_id);
+}
+
 LayerHostingContextID MediaPlayerPrivateRemote::hostingContextID() const
 {
     return m_layerHostingContextID;
@@ -1514,6 +1531,9 @@ void MediaPlayerPrivateRemote::setLayerHostingContextID(LayerHostingContextID in
 #if PLATFORM(COCOA)
     m_videoLayer = nullptr;
 #endif
+
+    for (auto& request : std::exchange(m_layerHostingContextIDRequests, { }))
+        request(inID);
 }
 
 } // namespace WebKit
