@@ -78,17 +78,17 @@ void WebRemoteFrameClient::changeLocation(WebCore::FrameLoadRequest&& request)
     // FIXME: Get more parameters correct and add tests for each one.
     dispatchDecidePolicyForNavigationAction(action, action.resourceRequest(), WebCore::ResourceResponse(), nullptr, WebCore::PolicyDecisionMode::Asynchronous, WebCore::PolicyCheckIdentifier::generate(), [protectedFrame = Ref { m_frame }, request = WTFMove(request)] (WebCore::PolicyAction policyAction, WebCore::PolicyCheckIdentifier responseIdentifier) mutable {
         // FIXME: Check responseIdentifier.
-        // WebPage::transitionFrameToLocalAndLoadRequest will make this load happen if needed.
+        // WebPage::loadRequest will make this load happen if needed.
     });
 }
 
 String WebRemoteFrameClient::renderTreeAsText(WebCore::ProcessIdentifier processIdentifier, WebCore::FrameIdentifier frameIdentifier, size_t baseIndent, OptionSet<WebCore::RenderAsTextFlag> behavior)
 {
-    auto reply = WebProcess::singleton().parentProcessConnection()->sendSync(Messages::WebProcessProxy::RenderTreeAsText(processIdentifier, frameIdentifier, baseIndent, behavior), 0);
-    if (!reply)
+    auto sendResult = WebProcess::singleton().parentProcessConnection()->sendSync(Messages::WebProcessProxy::RenderTreeAsText(processIdentifier, frameIdentifier, baseIndent, behavior), 0);
+    if (!sendResult.succeeded())
         return { };
 
-    auto [result] = reply.takeReply();
+    auto [result] = sendResult.takeReply();
     return result;
 }
 
