@@ -30,18 +30,22 @@
 
 #include "DrawingAreaProxyMessages.h"
 #include "LayerTreeHost.h"
+#include "MessageSenderInlines.h"
 #include "ShareableBitmap.h"
 #include "UpdateInfo.h"
 #include "WebPage.h"
 #include "WebPageCreationParameters.h"
+#include "WebPageInlines.h"
 #include "WebPreferencesKeys.h"
-#include <WebCore/Frame.h>
-#include <WebCore/FrameView.h>
+#include "WebProcess.h"
 #include <WebCore/GraphicsContext.h>
+#include <WebCore/LocalFrame.h>
+#include <WebCore/LocalFrameView.h>
 #include <WebCore/Page.h>
 #include <WebCore/PageOverlayController.h>
 #include <WebCore/Region.h>
 #include <WebCore/Settings.h>
+#include <wtf/SetForScope.h>
 
 #if USE(GLIB_EVENT_LOOP)
 #include <wtf/glib/RunLoopSourcePriority.h>
@@ -54,7 +58,6 @@ DrawingAreaCoordinatedGraphics::DrawingAreaCoordinatedGraphics(WebPage& webPage,
     : DrawingArea(DrawingAreaType::CoordinatedGraphics, parameters.drawingAreaIdentifier, webPage)
     , m_exitCompositingTimer(RunLoop::main(), this, &DrawingAreaCoordinatedGraphics::exitAcceleratedCompositingMode)
     , m_discardPreviousLayerTreeHostTimer(RunLoop::main(), this, &DrawingAreaCoordinatedGraphics::discardPreviousLayerTreeHost)
-    , m_supportsAsyncScrolling(parameters.store.getBoolValueForKey(WebPreferencesKey::threadedScrollingEnabledKey()))
     , m_displayTimer(RunLoop::main(), this, &DrawingAreaCoordinatedGraphics::displayTimerFired)
 {
 #if USE(GLIB_EVENT_LOOP)
@@ -64,13 +67,13 @@ DrawingAreaCoordinatedGraphics::DrawingAreaCoordinatedGraphics(WebPage& webPage,
 #endif
 #endif
 
-#if ENABLE(DEVELOPER_MODE)
-    if (m_supportsAsyncScrolling) {
-        auto* disableAsyncScrolling = getenv("WEBKIT_DISABLE_ASYNC_SCROLLING");
-        if (disableAsyncScrolling && strcmp(disableAsyncScrolling, "0"))
-            m_supportsAsyncScrolling = false;
+    updatePreferences(parameters.store);
+
+    if (m_alwaysUseCompositing) {
+        enterAcceleratedCompositingMode(nullptr);
+        if (!parameters.isProcessSwap)
+            sendEnterAcceleratedCompositingModeIfNeeded();
     }
-#endif
 }
 
 DrawingAreaCoordinatedGraphics::~DrawingAreaCoordinatedGraphics() = default;
@@ -95,9 +98,6 @@ void DrawingAreaCoordinatedGraphics::setNeedsDisplayInRect(const IntRect& rect)
         return;
     }
 
-    if (!m_isPaintingEnabled)
-        return;
-
     IntRect dirtyRect = rect;
     dirtyRect.intersect(m_webPage.bounds());
     if (dirtyRect.isEmpty())
@@ -109,9 +109,6 @@ void DrawingAreaCoordinatedGraphics::setNeedsDisplayInRect(const IntRect& rect)
 
 void DrawingAreaCoordinatedGraphics::scroll(const IntRect& scrollRect, const IntSize& scrollDelta)
 {
-    if (!m_isPaintingEnabled)
-        return;
-
     if (m_layerTreeHost) {
         ASSERT(m_scrollRect.isEmpty());
         ASSERT(m_scrollOffset.isEmpty());
@@ -169,18 +166,10 @@ void DrawingAreaCoordinatedGraphics::scroll(const IntRect& scrollRect, const Int
 
 void DrawingAreaCoordinatedGraphics::forceRepaint()
 {
-    if (m_inUpdateBackingStoreState) {
-        m_forceRepaintAfterBackingStoreStateUpdate = true;
-        return;
-    }
-    m_forceRepaintAfterBackingStoreStateUpdate = false;
-
     if (!m_layerTreeHost) {
         m_isWaitingForDidUpdate = false;
-        if (m_isPaintingEnabled) {
-            m_dirtyRegion = m_webPage.bounds();
-            display();
-        }
+        m_dirtyRegion = m_webPage.bounds();
+        display();
         return;
     }
 
@@ -255,6 +244,15 @@ void DrawingAreaCoordinatedGraphics::updatePreferences(const WebPreferencesStore
 
     m_alwaysUseCompositing = settings.acceleratedCompositingEnabled() && settings.forceCompositingMode();
 
+    m_supportsAsyncScrolling = store.getBoolValueForKey(WebPreferencesKey::threadedScrollingEnabledKey());
+#if ENABLE(DEVELOPER_MODE)
+    if (m_supportsAsyncScrolling) {
+        auto* disableAsyncScrolling = getenv("WEBKIT_DISABLE_ASYNC_SCROLLING");
+        if (disableAsyncScrolling && strcmp(disableAsyncScrolling, "0"))
+            m_supportsAsyncScrolling = false;
+    }
+#endif
+
     // If async scrolling is disabled, we have to force-disable async frame and overflow scrolling
     // to keep the non-async scrolling on those elements working.
     if (!m_supportsAsyncScrolling) {
@@ -263,15 +261,7 @@ void DrawingAreaCoordinatedGraphics::updatePreferences(const WebPreferencesStore
     }
 }
 
-void DrawingAreaCoordinatedGraphics::enablePainting()
-{
-    m_isPaintingEnabled = true;
-
-    if (m_alwaysUseCompositing && !m_layerTreeHost)
-        enterAcceleratedCompositingMode(nullptr);
-}
-
-void DrawingAreaCoordinatedGraphics::mainFrameContentSizeChanged(const IntSize& size)
+void DrawingAreaCoordinatedGraphics::mainFrameContentSizeChanged(WebCore::FrameIdentifier, const IntSize& size)
 {
     if (m_layerTreeHost)
         m_layerTreeHost->contentsSizeChanged(size);
@@ -297,11 +287,32 @@ void DrawingAreaCoordinatedGraphics::didChangeViewportAttributes(ViewportAttribu
 }
 #endif
 
+void DrawingAreaCoordinatedGraphics::setDeviceScaleFactor(float deviceScaleFactor)
+{
+    m_webPage.setDeviceScaleFactor(deviceScaleFactor);
+}
+
 bool DrawingAreaCoordinatedGraphics::supportsAsyncScrolling() const
 {
     return m_supportsAsyncScrolling;
 }
 
+void DrawingAreaCoordinatedGraphics::registerScrollingTree()
+{
+#if ENABLE(SCROLLING_THREAD)
+    if (m_supportsAsyncScrolling)
+        WebProcess::singleton().eventDispatcher().addScrollingTreeForPage(m_webPage);
+#endif
+}
+
+void DrawingAreaCoordinatedGraphics::unregisterScrollingTree()
+{
+#if ENABLE(SCROLLING_THREAD)
+    if (m_supportsAsyncScrolling)
+        WebProcess::singleton().eventDispatcher().removeScrollingTreeForPage(m_webPage);
+#endif
+}
+
 GraphicsLayerFactory* DrawingAreaCoordinatedGraphics::graphicsLayerFactory()
 {
     if (!m_layerTreeHost)
@@ -309,7 +320,7 @@ GraphicsLayerFactory* DrawingAreaCoordinatedGraphics::graphicsLayerFactory()
     return m_layerTreeHost ? m_layerTreeHost->graphicsLayerFactory() : nullptr;
 }
 
-void DrawingAreaCoordinatedGraphics::setRootCompositingLayer(GraphicsLayer* graphicsLayer)
+void DrawingAreaCoordinatedGraphics::setRootCompositingLayer(WebCore::Frame&, GraphicsLayer* graphicsLayer)
 {
     if (m_layerTreeHost) {
         if (graphicsLayer) {
@@ -329,7 +340,7 @@ void DrawingAreaCoordinatedGraphics::setRootCompositingLayer(GraphicsLayer* grap
             // compositing code via display() and layout.
             // If we're leaving compositing mode because of a setSize, it is safe to
             // exit accelerated compositing mode right away.
-            if (m_inUpdateBackingStoreState)
+            if (m_inUpdateGeometry)
                 exitAcceleratedCompositingMode();
             else
                 exitAcceleratedCompositingModeSoon();
@@ -360,17 +371,6 @@ void DrawingAreaCoordinatedGraphics::layerHostDidFlushLayers()
 {
     ASSERT(m_layerTreeHost);
     m_layerTreeHost->forceRepaint();
-
-    if (m_shouldSendDidUpdateBackingStoreState && !exitAcceleratedCompositingModePending()) {
-        sendDidUpdateBackingStoreState();
-        return;
-    }
-
-    ASSERT(!m_compositingAccordingToProxyMessages);
-    if (!exitAcceleratedCompositingModePending()) {
-        send(Messages::DrawingAreaProxy::EnterAcceleratedCompositingMode(m_backingStoreStateID, m_layerTreeHost->layerTreeContext()));
-        m_compositingAccordingToProxyMessages = true;
-    }
 }
 #endif
 
@@ -381,7 +381,7 @@ RefPtr<DisplayRefreshMonitor> DrawingAreaCoordinatedGraphics::createDisplayRefre
     return m_layerTreeHost->createDisplayRefreshMonitor(displayID);
 }
 
-void DrawingAreaCoordinatedGraphics::activityStateDidChange(OptionSet<ActivityState::Flag> changed, ActivityStateChangeID, CompletionHandler<void()>&& completionHandler)
+void DrawingAreaCoordinatedGraphics::activityStateDidChange(OptionSet<ActivityState> changed, ActivityStateChangeID, CompletionHandler<void()>&& completionHandler)
 {
     if (changed & ActivityState::IsVisible) {
         if (m_webPage.isVisible())
@@ -392,7 +392,7 @@ void DrawingAreaCoordinatedGraphics::activityStateDidChange(OptionSet<ActivitySt
     completionHandler();
 }
 
-void DrawingAreaCoordinatedGraphics::attachViewOverlayGraphicsLayer(GraphicsLayer* viewOverlayRootLayer)
+void DrawingAreaCoordinatedGraphics::attachViewOverlayGraphicsLayer(WebCore::FrameIdentifier, GraphicsLayer* viewOverlayRootLayer)
 {
     if (m_layerTreeHost)
         m_layerTreeHost->setViewOverlayRootLayer(viewOverlayRootLayer);
@@ -400,65 +400,37 @@ void DrawingAreaCoordinatedGraphics::attachViewOverlayGraphicsLayer(GraphicsLaye
         m_previousLayerTreeHost->setViewOverlayRootLayer(viewOverlayRootLayer);
 }
 
-void DrawingAreaCoordinatedGraphics::updateBackingStoreState(uint64_t stateID, bool respondImmediately, float deviceScaleFactor, const IntSize& size, const IntSize& scrollOffset)
+void DrawingAreaCoordinatedGraphics::updateGeometry(const IntSize& size, CompletionHandler<void()>&& completionHandler)
 {
-    if (stateID != m_backingStoreStateID && !m_layerTreeHost)
+    SetForScope inUpdateGeometry(m_inUpdateGeometry, true);
+    m_webPage.setSize(size);
+    m_webPage.layoutIfNeeded();
+
+    if (!m_layerTreeHost)
         m_dirtyRegion = IntRect(IntPoint(), size);
 
-    ASSERT(!m_inUpdateBackingStoreState);
-    m_inUpdateBackingStoreState = true;
+    LayerTreeContext previousLayerTreeContext;
+    if (m_layerTreeHost) {
+        previousLayerTreeContext = m_layerTreeHost->layerTreeContext();
+        m_layerTreeHost->sizeDidChange(m_webPage.size());
+    } else if (m_previousLayerTreeHost)
+        m_previousLayerTreeHost->sizeDidChange(m_webPage.size());
 
-    ASSERT_ARG(stateID, stateID >= m_backingStoreStateID);
-    if (stateID != m_backingStoreStateID) {
-        m_backingStoreStateID = stateID;
-        m_shouldSendDidUpdateBackingStoreState = true;
-
-        m_webPage.setDeviceScaleFactor(deviceScaleFactor);
-        m_webPage.setSize(size);
-        m_webPage.updateRendering();
-        m_webPage.finalizeRenderingUpdate({ });
-        m_webPage.flushPendingEditorStateUpdate();
-        m_webPage.scrollMainFrameIfNotAtMaxScrollPosition(scrollOffset);
-        m_webPage.didUpdateRendering();
-
-        if (m_layerTreeHost)
-            m_layerTreeHost->sizeDidChange(m_webPage.size());
-        else if (m_previousLayerTreeHost)
-            m_previousLayerTreeHost->sizeDidChange(m_webPage.size());
+    if (m_layerTreeHost) {
+        auto layerTreeContext = m_layerTreeHost->layerTreeContext();
+        m_layerTreeHost->forceRepaint();
+        if (layerTreeContext != previousLayerTreeContext)
+            send(Messages::DrawingAreaProxy::UpdateAcceleratedCompositingMode(0, layerTreeContext));
     } else {
-        ASSERT(size == m_webPage.size());
-        if (!m_shouldSendDidUpdateBackingStoreState) {
-            // We've already sent a DidUpdateBackingStoreState message for this state. We have nothing more to do.
-            m_inUpdateBackingStoreState = false;
-            if (m_forceRepaintAfterBackingStoreStateUpdate)
-                forceRepaint();
-            return;
-        }
+        UpdateInfo updateInfo;
+        updateInfo.viewSize = m_webPage.size();
+        updateInfo.deviceScaleFactor = m_webPage.corePage()->deviceScaleFactor();
+        display(updateInfo);
+        if (!m_layerTreeHost)
+            send(Messages::DrawingAreaProxy::Update(0, WTFMove(updateInfo)));
     }
 
-    // The UI process has updated to a new backing store state. Any Update messages we sent before
-    // this point will be ignored. We wait to set this to false until after updating the page's
-    // size so that any displays triggered by the relayout will be ignored. If we're supposed to
-    // respond to the UpdateBackingStoreState message immediately, we'll do a display anyway in
-    // sendDidUpdateBackingStoreState; otherwise we shouldn't do one right now.
-    m_isWaitingForDidUpdate = false;
-
-    if (respondImmediately) {
-        // Make sure to resume painting if we're supposed to respond immediately, otherwise we'll just
-        // send back an empty UpdateInfo struct.
-        bool wasSuspended = m_isPaintingSuspended;
-        if (m_isPaintingSuspended)
-            resumePainting();
-
-        sendDidUpdateBackingStoreState();
-        if (wasSuspended)
-            suspendPainting();
-    }
-
-    m_inUpdateBackingStoreState = false;
-
-    if (m_forceRepaintAfterBackingStoreStateUpdate)
-        forceRepaint();
+    completionHandler();
 }
 
 void DrawingAreaCoordinatedGraphics::targetRefreshRateDidChange(unsigned rate)
@@ -491,7 +463,7 @@ void DrawingAreaCoordinatedGraphics::displayDidRefresh()
 void DrawingAreaCoordinatedGraphics::adjustTransientZoom(double scale, FloatPoint origin)
 {
     if (!m_transientZoom) {
-        FrameView& frameView = *m_webPage.mainFrameView();
+        auto& frameView = *m_webPage.mainFrameView();
         FloatRect unobscuredContentRect = frameView.unobscuredContentRectIncludingScrollbars();
 
         m_transientZoom = true;
@@ -525,47 +497,6 @@ void DrawingAreaCoordinatedGraphics::commitTransientZoom(double scale, FloatPoin
 }
 #endif
 
-void DrawingAreaCoordinatedGraphics::sendDidUpdateBackingStoreState()
-{
-    ASSERT(!m_isWaitingForDidUpdate);
-    ASSERT(m_shouldSendDidUpdateBackingStoreState);
-
-    if (!m_isPaintingSuspended && !m_layerTreeHost) {
-        UpdateInfo updateInfo;
-        display(updateInfo);
-        if (!m_layerTreeHost) {
-            m_shouldSendDidUpdateBackingStoreState = false;
-
-            LayerTreeContext layerTreeContext;
-            send(Messages::DrawingAreaProxy::DidUpdateBackingStoreState(m_backingStoreStateID, updateInfo, layerTreeContext));
-            m_compositingAccordingToProxyMessages = false;
-            return;
-        }
-    }
-
-    ASSERT(m_shouldSendDidUpdateBackingStoreState);
-    m_shouldSendDidUpdateBackingStoreState = false;
-
-    UpdateInfo updateInfo;
-    updateInfo.viewSize = m_webPage.size();
-    updateInfo.deviceScaleFactor = m_webPage.corePage()->deviceScaleFactor();
-
-    LayerTreeContext layerTreeContext;
-    if (m_layerTreeHost) {
-        layerTreeContext = m_layerTreeHost->layerTreeContext();
-
-        // We don't want the layer tree host to notify after the next scheduled
-        // layer flush because that might end up sending an EnterAcceleratedCompositingMode
-        // message back to the UI process, but the updated layer tree context
-        // will be sent back in the DidUpdateBackingStoreState message.
-        m_layerTreeHost->setShouldNotifyAfterNextScheduledLayerFlush(false);
-        m_layerTreeHost->forceRepaint();
-    }
-
-    send(Messages::DrawingAreaProxy::DidUpdateBackingStoreState(m_backingStoreStateID, updateInfo, layerTreeContext));
-    m_compositingAccordingToProxyMessages = !layerTreeContext.isEmpty();
-}
-
 void DrawingAreaCoordinatedGraphics::exitAcceleratedCompositingModeSoon()
 {
     if (m_layerTreeStateIsFrozen) {
@@ -633,7 +564,7 @@ void DrawingAreaCoordinatedGraphics::enterAcceleratedCompositingMode(GraphicsLay
 
     auto changeWindowScreen = [&] {
         // In order to ensure that we get a unique DisplayRefreshMonitor per-DrawingArea (necessary because ThreadedDisplayRefreshMonitor
-        // is driven by the ThreadedCompositor of the drawing area), give each page a unique DisplayID derived from WebPage's unique ID.
+        // is driven by the ThreadedCompositor of the drawing area), give each page a unique DisplayID derived from DrawingArea's unique ID.
         m_webPage.windowScreenDidChange(m_layerTreeHost->displayID(), std::nullopt);
     };
 
@@ -646,24 +577,29 @@ void DrawingAreaCoordinatedGraphics::enterAcceleratedCompositingMode(GraphicsLay
         if (!m_layerTreeStateIsFrozen)
             m_layerTreeHost->setLayerFlushSchedulingEnabled(true);
     } else {
-#if USE(COORDINATED_GRAPHICS) || USE(GRAPHICS_LAYER_TEXTURE_MAPPER)
+#if USE(GRAPHICS_LAYER_TEXTURE_MAPPER)
         m_layerTreeHost = makeUnique<LayerTreeHost>(m_webPage);
-        changeWindowScreen();
+#elif USE(COORDINATED_GRAPHICS)
+        m_layerTreeHost = makeUnique<LayerTreeHost>(m_webPage, std::numeric_limits<uint32_t>::max() - m_identifier.toUInt64());
 #else
         m_layerTreeHost = nullptr;
         return;
 #endif
+        changeWindowScreen();
         if (m_layerTreeStateIsFrozen)
             m_layerTreeHost->setLayerFlushSchedulingEnabled(false);
         if (m_isPaintingSuspended)
             m_layerTreeHost->pauseRendering();
     }
 
-    if (!m_inUpdateBackingStoreState)
+    if (!m_inUpdateGeometry)
         m_layerTreeHost->setShouldNotifyAfterNextScheduledLayerFlush(true);
 
     m_layerTreeHost->setRootCompositingLayer(graphicsLayer);
 
+    if (m_shouldSendEnterAcceleratedCompositingMode)
+        sendEnterAcceleratedCompositingModeIfNeeded();
+
     // Non-composited content will now be handled exclusively by the layer tree host.
     m_dirtyRegion = WebCore::Region();
     m_scrollRect = IntRect();
@@ -672,6 +608,20 @@ void DrawingAreaCoordinatedGraphics::enterAcceleratedCompositingMode(GraphicsLay
     m_isWaitingForDidUpdate = false;
 }
 
+void DrawingAreaCoordinatedGraphics::sendEnterAcceleratedCompositingModeIfNeeded()
+{
+    if (m_compositingAccordingToProxyMessages)
+        return;
+
+    if (!m_layerTreeHost) {
+        m_shouldSendEnterAcceleratedCompositingMode = true;
+        return;
+    }
+
+    send(Messages::DrawingAreaProxy::EnterAcceleratedCompositingMode(0, m_layerTreeHost->layerTreeContext()));
+    m_compositingAccordingToProxyMessages = true;
+}
+
 void DrawingAreaCoordinatedGraphics::exitAcceleratedCompositingMode()
 {
     if (m_alwaysUseCompositing)
@@ -694,14 +644,9 @@ void DrawingAreaCoordinatedGraphics::exitAcceleratedCompositingMode()
 
     m_dirtyRegion = m_webPage.bounds();
 
-    if (m_inUpdateBackingStoreState)
+    if (m_inUpdateGeometry)
         return;
 
-    if (m_shouldSendDidUpdateBackingStoreState) {
-        sendDidUpdateBackingStoreState();
-        return;
-    }
-
     UpdateInfo updateInfo;
     if (m_isPaintingSuspended) {
         updateInfo.viewSize = m_webPage.size();
@@ -712,12 +657,12 @@ void DrawingAreaCoordinatedGraphics::exitAcceleratedCompositingMode()
     // Send along a complete update of the page so we can paint the contents right after we exit the
     // accelerated compositing mode, eliminiating flicker.
     if (m_compositingAccordingToProxyMessages) {
-        send(Messages::DrawingAreaProxy::ExitAcceleratedCompositingMode(m_backingStoreStateID, updateInfo));
+        send(Messages::DrawingAreaProxy::ExitAcceleratedCompositingMode(0, WTFMove(updateInfo)));
         m_compositingAccordingToProxyMessages = false;
     } else {
         // If we left accelerated compositing mode before we sent an EnterAcceleratedCompositingMode message to the
         // UI process, we still need to let it know about the new contents, so send an Update message.
-        send(Messages::DrawingAreaProxy::Update(m_backingStoreStateID, updateInfo));
+        send(Messages::DrawingAreaProxy::Update(0, WTFMove(updateInfo)));
     }
 }
 
@@ -748,7 +693,7 @@ void DrawingAreaCoordinatedGraphics::display()
 {
     ASSERT(!m_layerTreeHost);
     ASSERT(!m_isWaitingForDidUpdate);
-    ASSERT(!m_inUpdateBackingStoreState);
+    ASSERT(!m_inUpdateGeometry);
 
     if (m_layerTreeStateIsFrozen)
         return;
@@ -756,11 +701,6 @@ void DrawingAreaCoordinatedGraphics::display()
     if (m_isPaintingSuspended)
         return;
 
-    if (m_shouldSendDidUpdateBackingStoreState) {
-        sendDidUpdateBackingStoreState();
-        return;
-    }
-
     UpdateInfo updateInfo;
     display(updateInfo);
 
@@ -774,10 +714,10 @@ void DrawingAreaCoordinatedGraphics::display()
     }
 
     if (m_compositingAccordingToProxyMessages) {
-        send(Messages::DrawingAreaProxy::ExitAcceleratedCompositingMode(m_backingStoreStateID, updateInfo));
+        send(Messages::DrawingAreaProxy::ExitAcceleratedCompositingMode(0, WTFMove(updateInfo)));
         m_compositingAccordingToProxyMessages = false;
     } else
-        send(Messages::DrawingAreaProxy::Update(m_backingStoreStateID, updateInfo));
+        send(Messages::DrawingAreaProxy::Update(0, WTFMove(updateInfo)));
     m_isWaitingForDidUpdate = true;
     m_scheduledWhileWaitingForDidUpdate = false;
 }
@@ -805,7 +745,7 @@ static bool shouldPaintBoundsRect(const IntRect& bounds, const Vector<IntRect, 1
 
 void DrawingAreaCoordinatedGraphics::display(UpdateInfo& updateInfo)
 {
-    ASSERT(!m_isPaintingSuspended);
+    ASSERT(!m_isPaintingSuspended || m_inUpdateGeometry);
     ASSERT(!m_layerTreeHost);
 
     m_webPage.updateRendering();
@@ -829,7 +769,7 @@ void DrawingAreaCoordinatedGraphics::display(UpdateInfo& updateInfo)
     IntSize bitmapSize = bounds.size();
     float deviceScaleFactor = m_webPage.corePage()->deviceScaleFactor();
     bitmapSize.scale(deviceScaleFactor);
-    auto bitmap = ShareableBitmap::create(bitmapSize, { });
+    auto bitmap = ShareableBitmap::create({ bitmapSize });
     if (!bitmap)
         return;
 
