@@ -1167,8 +1167,8 @@ static NetworkProcessConnectionInfo getNetworkProcessConnection(IPC::Connection&
     NetworkProcessConnectionInfo connectionInfo;
     auto requestConnection = [&]() -> bool {
         auto sendResult = connection.sendSync(Messages::WebProcessProxy::GetNetworkProcessConnection(), 0);
-        if (!sendResult) {
-            RELEASE_LOG_ERROR(Process, "getNetworkProcessConnection: Failed to send message or receive invalid message");
+        if (!sendResult.succeeded()) {
+            RELEASE_LOG_ERROR(Process, "getNetworkProcessConnection: Failed to send message or receive invalid message: error %" PUBLIC_LOG_STRING, IPC::errorAsString(sendResult.error));
             failedToGetNetworkProcessConnection();
         }
         std::tie(connectionInfo) = sendResult.takeReply();
@@ -1330,14 +1330,14 @@ GPUProcessConnection& WebProcess::ensureGPUProcessConnection()
     // If we've lost our connection to the GPU process (e.g. it crashed) try to re-establish it.
     if (!m_gpuProcessConnection) {
         m_gpuProcessConnection = GPUProcessConnection::create(*parentProcessConnection());
-
+        if (!m_gpuProcessConnection)
+            CRASH();
         for (auto& page : m_pageMap.values()) {
             // If page is null, then it is currently being constructed.
             if (page)
                 page->gpuProcessConnectionDidBecomeAvailable(*m_gpuProcessConnection);
         }
     }
-    
     return *m_gpuProcessConnection;
 }
 
@@ -1848,11 +1848,6 @@ void WebProcess::seedResourceLoadStatisticsForTesting(const RegistrableDomain& f
 RefPtr<API::Object> WebProcess::transformHandlesToObjects(API::Object* object)
 {
     struct Transformer final : UserData::Transformer {
-        Transformer(WebProcess& webProcess)
-            : m_webProcess(webProcess)
-        {
-        }
-
         bool shouldTransformObject(const API::Object& object) const override
         {
             switch (object.type()) {
@@ -1876,24 +1871,22 @@ RefPtr<API::Object> WebProcess::transformHandlesToObjects(API::Object* object)
         {
             switch (object.type()) {
             case API::Object::Type::FrameHandle:
-                return m_webProcess.webFrame(static_cast<const API::FrameHandle&>(object).frameID());
+                return WebProcess::singleton().webFrame(static_cast<const API::FrameHandle&>(object).frameID());
 
             case API::Object::Type::PageHandle:
-                return m_webProcess.webPage(static_cast<const API::PageHandle&>(object).webPageID());
+                return WebProcess::singleton().webPage(static_cast<const API::PageHandle&>(object).webPageID());
 
 #if PLATFORM(COCOA)
             case API::Object::Type::ObjCObjectGraph:
-                return m_webProcess.transformHandlesToObjects(static_cast<ObjCObjectGraph&>(object));
+                return WebProcess::singleton().transformHandlesToObjects(static_cast<ObjCObjectGraph&>(object));
 #endif
             default:
                 return &object;
             }
         }
-
-        WebProcess& m_webProcess;
     };
 
-    return UserData::transform(object, Transformer(*this));
+    return UserData::transform(object, Transformer());
 }
 
 RefPtr<API::Object> WebProcess::transformObjectsToHandles(API::Object* object)
@@ -2217,14 +2210,14 @@ void WebProcess::setUseGPUProcessForMedia(bool useGPUProcessForMedia)
 
 #if USE(AUDIO_SESSION)
     if (useGPUProcessForMedia)
-        AudioSession::setSharedSession(RemoteAudioSession::create(*this));
+        AudioSession::setSharedSession(RemoteAudioSession::create());
     else
         AudioSession::setSharedSession(AudioSession::create());
 #endif
 
 #if PLATFORM(IOS_FAMILY)
     if (useGPUProcessForMedia)
-        MediaSessionHelper::setSharedHelper(adoptRef(*new RemoteMediaSessionHelper(*this)));
+        MediaSessionHelper::setSharedHelper(adoptRef(*new RemoteMediaSessionHelper()));
     else
         MediaSessionHelper::resetSharedHelper();
 #endif
@@ -2241,14 +2234,18 @@ void WebProcess::setUseGPUProcessForMedia(bool useGPUProcessForMedia)
     else
         MediaEngineConfigurationFactory::resetFactories();
 
-    if (useGPUProcessForMedia)
-        WebCore::AudioHardwareListener::setCreationFunction([this] (WebCore::AudioHardwareListener::Client& client) { return RemoteAudioHardwareListener::create(client, *this); });
-    else
+    if (useGPUProcessForMedia) {
+        WebCore::AudioHardwareListener::setCreationFunction([] (WebCore::AudioHardwareListener::Client& client) {
+            return RemoteAudioHardwareListener::create(client);
+        });
+    } else
         WebCore::AudioHardwareListener::resetCreationFunction();
 
-    if (useGPUProcessForMedia)
-        WebCore::RemoteCommandListener::setCreationFunction([this] (WebCore::RemoteCommandListenerClient& client) { return RemoteRemoteCommandListener::create(client, *this); });
-    else
+    if (useGPUProcessForMedia) {
+        WebCore::RemoteCommandListener::setCreationFunction([] (WebCore::RemoteCommandListenerClient& client) {
+            return RemoteRemoteCommandListener::create(client);
+        });
+    } else
         WebCore::RemoteCommandListener::resetCreationFunction();
 
 #if PLATFORM(COCOA)
@@ -2346,6 +2343,21 @@ void WebProcess::setNetworkProcessConnectionID(IPC::Connection::UniqueID uniqueI
 
 }
 
+void WebProcess::addAllowedFirstPartyForCookies(WebCore::RegistrableDomain&& firstPartyForCookies)
+{
+    if (!HashSet<WebCore::RegistrableDomain>::isValidValue(firstPartyForCookies))
+        return;
+
+    m_allowedFirstPartiesForCookies.add(WTFMove(firstPartyForCookies));
+}
+
+bool WebProcess::allowsFirstPartyForCookies(const URL& firstParty)
+{
+    return AuxiliaryProcess::allowsFirstPartyForCookies(firstParty, [&] {
+        return AuxiliaryProcess::allowsFirstPartyForCookies(WebCore::RegistrableDomain { firstParty }, m_allowedFirstPartiesForCookies);
+    });
+}
+
 } // namespace WebKit
 
 #undef RELEASE_LOG_SESSION_ID
